/* Generated by Cython 0.23.4 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "src/sophia.h"
        ]
    }
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_4"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__sophy
#define __PYX_HAVE_API__sophy
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "stdint.h"
#include "src/sophia.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "sophy.pyx",
  "type.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5sophy__ConfigManager;
struct __pyx_obj_5sophy__SophiaObject;
struct __pyx_obj_5sophy_Sophia;
struct __pyx_obj_5sophy__BaseDBObject;
struct __pyx_obj_5sophy_Database;
struct __pyx_obj_5sophy_View;
struct __pyx_obj_5sophy_Transaction;
struct __pyx_obj_5sophy_Cursor;
struct __pyx_obj_5sophy__Index;
struct __pyx_obj_5sophy__UInt32Index;
struct __pyx_obj_5sophy__UInt32RevIndex;
struct __pyx_obj_5sophy__UInt64Index;
struct __pyx_obj_5sophy__UInt64RevIndex;
struct __pyx_obj_5sophy__MultiIndex;
struct __pyx_obj_5sophy_SimpleDatabase;
struct __pyx_obj_5sophy___pyx_scope_struct___sp;
struct __pyx_obj_5sophy___pyx_scope_struct_1__sm;
struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp;
struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm;
struct __pyx_obj_5sophy___pyx_scope_struct_4___call__;
struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range;
struct __pyx_obj_5sophy___pyx_scope_struct_6_keys;
struct __pyx_obj_5sophy___pyx_scope_struct_7_values;
struct __pyx_obj_5sophy___pyx_scope_struct_8_items;
struct __pyx_opt_args_5sophy_14_ConfigManager_get_option;
struct __pyx_opt_args_5sophy_6Sophia__add_db;
struct __pyx_opt_args_5sophy_13_BaseDBObject_open;
struct __pyx_opt_args_5sophy_13_BaseDBObject__update;
struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor;
struct __pyx_opt_args_5sophy_8Database_cursor;
struct __pyx_opt_args_5sophy_4View_cursor;
struct __pyx_opt_args_5sophy_11Transaction_commit;
struct __pyx_opt_args_5sophy_11Transaction_rollback;

/* "sophy.pyx":124
 *             self.apply_config(key, value)
 * 
 *     cpdef get_option(self, key, string=True):             # <<<<<<<<<<<<<<
 *         if string:
 *             return _getstring(self.sophia.handle, key)
 */
struct __pyx_opt_args_5sophy_14_ConfigManager_get_option {
  int __pyx_n;
  PyObject *string;
};

/* "sophy.pyx":233
 *         return True
 * 
 *     cdef _add_db(self, Database db_obj, open_database=False):             # <<<<<<<<<<<<<<
 *         self.dbs[db_obj.name] = db_obj
 *         db_obj.configure()
 */
struct __pyx_opt_args_5sophy_6Sophia__add_db {
  int __pyx_n;
  PyObject *open_database;
};

/* "sophy.pyx":383
 *         self.index = self._get_index()
 * 
 *     cpdef bint open(self, bint configure=True):             # <<<<<<<<<<<<<<
 *         if self.handle:
 *             return False
 */
struct __pyx_opt_args_5sophy_13_BaseDBObject_open {
  int __pyx_n;
  int configure;
};

/* "sophy.pyx":436
 *         return self.index.exists(self, key)
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):             # <<<<<<<<<<<<<<
 *         with db.transaction() as txn:
 *             if _data:
 */
struct __pyx_opt_args_5sophy_13_BaseDBObject__update {
  int __pyx_n;
  PyObject *_data;
  PyObject *k;
};

/* "sophy.pyx":500
 *         return i
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         raise NotImplemented
 */
struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor {
  int __pyx_n;
  PyObject *order;
  PyObject *key;
  PyObject *prefix;
  PyObject *keys;
  PyObject *values;
};

/* "sophy.pyx":580
 *         return View(self.sophia, self, name)
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */
struct __pyx_opt_args_5sophy_8Database_cursor {
  int __pyx_n;
  PyObject *order;
  PyObject *key;
  PyObject *prefix;
  PyObject *keys;
  PyObject *values;
};

/* "sophy.pyx":666
 *         raise ValueError('Views are read-only.')
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */
struct __pyx_opt_args_5sophy_4View_cursor {
  int __pyx_n;
  PyObject *order;
  PyObject *key;
  PyObject *prefix;
  PyObject *keys;
  PyObject *values;
};

/* "sophy.pyx":706
 *                 raise
 * 
 *     cpdef commit(self, begin=True):             # <<<<<<<<<<<<<<
 *         cdef int rc = sp_commit(self.handle)
 *         _check(self.sophia.handle, rc)
 */
struct __pyx_opt_args_5sophy_11Transaction_commit {
  int __pyx_n;
  PyObject *begin;
};

/* "sophy.pyx":719
 *             self.open()
 * 
 *     cpdef rollback(self, begin=True):             # <<<<<<<<<<<<<<
 *         self.destroy()
 *         if begin:
 */
struct __pyx_opt_args_5sophy_11Transaction_rollback {
  int __pyx_n;
  PyObject *begin;
};

/* "sophy.pyx":109
 * 
 * 
 * cdef class _ConfigManager(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         dict _config
 */
struct __pyx_obj_5sophy__ConfigManager {
  PyObject_HEAD
  struct __pyx_vtabstruct_5sophy__ConfigManager *__pyx_vtab;
  PyObject *_config;
  struct __pyx_obj_5sophy_Sophia *sophia;
  void *handle;
};


/* "sophy.pyx":157
 * 
 * 
 * cdef class _SophiaObject(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         void *handle
 */
struct __pyx_obj_5sophy__SophiaObject {
  PyObject_HEAD
  struct __pyx_vtabstruct_5sophy__SophiaObject *__pyx_vtab;
  void *handle;
};


/* "sophy.pyx":169
 * 
 * 
 * cdef class Sophia(_SophiaObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         bint is_open
 */
struct __pyx_obj_5sophy_Sophia {
  struct __pyx_obj_5sophy__SophiaObject __pyx_base;
  int is_open;
  PyObject *b_path;
  PyObject *dbs;
  PyObject *db_defs;
  int auto_open;
  struct __pyx_obj_5sophy__ConfigManager *config;
  PyObject *path;
};


/* "sophy.pyx":372
 * 
 * 
 * cdef class _BaseDBObject(_SophiaObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly Sophia sophia
 */
struct __pyx_obj_5sophy__BaseDBObject {
  struct __pyx_obj_5sophy__SophiaObject __pyx_base;
  struct __pyx_obj_5sophy_Sophia *sophia;
  struct __pyx_obj_5sophy__Index *index;
};


/* "sophy.pyx":508
 * 
 * 
 * cdef class Database(_BaseDBObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly bytes name
 */
struct __pyx_obj_5sophy_Database {
  struct __pyx_obj_5sophy__BaseDBObject __pyx_base;
  PyObject *name;
  struct __pyx_obj_5sophy__ConfigManager *config;
  PyObject *index_type;
};


/* "sophy.pyx":632
 * 
 * 
 * cdef class View(_BaseDBObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Database db
 */
struct __pyx_obj_5sophy_View {
  struct __pyx_obj_5sophy__BaseDBObject __pyx_base;
  struct __pyx_obj_5sophy_Database *db;
  PyObject *name;
};


/* "sophy.pyx":679
 * 
 * 
 * cdef class Transaction(_BaseDBObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Database db
 */
struct __pyx_obj_5sophy_Transaction {
  struct __pyx_obj_5sophy__BaseDBObject __pyx_base;
  struct __pyx_obj_5sophy_Database *db;
};


/* "sophy.pyx":725
 * 
 * 
 * cdef class Cursor(_SophiaObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Sophia sophia
 */
struct __pyx_obj_5sophy_Cursor {
  struct __pyx_obj_5sophy__SophiaObject __pyx_base;
  struct __pyx_obj_5sophy_Sophia *sophia;
  struct __pyx_obj_5sophy_Database *db;
  struct __pyx_obj_5sophy__SophiaObject *target;
  PyObject *order;
  PyObject *prefix;
  int keys;
  int values;
  PyObject *key;
  PyObject *indexes;
  void *current_item;
};


/* "sophy.pyx":799
 * 
 * 
 * cdef class _Index(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Sophia sophia
 */
struct __pyx_obj_5sophy__Index {
  PyObject_HEAD
  struct __pyx_vtabstruct_5sophy__Index *__pyx_vtab;
  struct __pyx_obj_5sophy_Sophia *sophia;
  struct __pyx_obj_5sophy_Database *db;
  PyObject *b_path;
  PyObject *b_type;
  PyObject *key;
  PyObject *empty_value;
  PyObject *keys;
};


/* "sophy.pyx":891
 * 
 * 
 * cdef class _UInt32Index(_Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u32'
 * 
 */
struct __pyx_obj_5sophy__UInt32Index {
  struct __pyx_obj_5sophy__Index __pyx_base;
};


/* "sophy.pyx":915
 * 
 * 
 * cdef class _UInt32RevIndex(_UInt32Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u32rev'
 * 
 */
struct __pyx_obj_5sophy__UInt32RevIndex {
  struct __pyx_obj_5sophy__UInt32Index __pyx_base;
};


/* "sophy.pyx":919
 * 
 * 
 * cdef class _UInt64Index(_Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u64'
 * 
 */
struct __pyx_obj_5sophy__UInt64Index {
  struct __pyx_obj_5sophy__Index __pyx_base;
};


/* "sophy.pyx":943
 * 
 * 
 * cdef class _UInt64RevIndex(_UInt64Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u64rev'
 * 
 */
struct __pyx_obj_5sophy__UInt64RevIndex {
  struct __pyx_obj_5sophy__UInt64Index __pyx_base;
};


/* "sophy.pyx":947
 * 
 * 
 * cdef class _MultiIndex(_Index):             # <<<<<<<<<<<<<<
 *     cdef:
 *         tuple indexes
 */
struct __pyx_obj_5sophy__MultiIndex {
  struct __pyx_obj_5sophy__Index __pyx_base;
  PyObject *indexes;
};


/* "sophy.pyx":1029
 * 
 * 
 * cdef class SimpleDatabase(Sophia):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly db_name
 */
struct __pyx_obj_5sophy_SimpleDatabase {
  struct __pyx_obj_5sophy_Sophia __pyx_base;
  PyObject *db_name;
};


/* "sophy.pyx":78
 * 
 * 
 * def _sp(propname, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 */
struct __pyx_obj_5sophy___pyx_scope_struct___sp {
  PyObject_HEAD
  PyObject *__pyx_v_propname;
  PyObject *__pyx_v_string;
};


/* "sophy.pyx":88
 *         return property(_getter, _setter)
 * 
 * def _sm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 */
struct __pyx_obj_5sophy___pyx_scope_struct_1__sm {
  PyObject_HEAD
  PyObject *__pyx_v_propname;
};


/* "sophy.pyx":93
 *     return _method
 * 
 * def _dbp(prop, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 */
struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp {
  PyObject_HEAD
  PyObject *__pyx_v_prop;
  PyObject *__pyx_v_string;
};


/* "sophy.pyx":103
 *         return property(_getter, _setter)
 * 
 * def _dbm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 */
struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm {
  PyObject_HEAD
  PyObject *__pyx_v_propname;
};


/* "sophy.pyx":415
 *         self.close()
 * 
 *     def __call__(self, fn):             # <<<<<<<<<<<<<<
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 */
struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ {
  PyObject_HEAD
  PyObject *__pyx_v_fn;
  struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self;
};


/* "sophy.pyx":445
 *                     txn[key] = k[key]
 * 
 *     def get_range(self, start=None, stop=None, reverse=False):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Cursor cursor
 */
struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range {
  PyObject_HEAD
  struct __pyx_obj_5sophy_Cursor *__pyx_v_cursor;
  PyObject *__pyx_v_first;
  PyObject *__pyx_v_key;
  PyObject *__pyx_v_last;
  PyObject *__pyx_v_order;
  PyObject *__pyx_v_reverse;
  struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self;
  PyObject *__pyx_v_start;
  PyObject *__pyx_v_stop;
  PyObject *__pyx_v_value;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "sophy.pyx":472
 *             yield (key, value)
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor(values=False)
 *         for item in cursor:
 */
struct __pyx_obj_5sophy___pyx_scope_struct_6_keys {
  PyObject_HEAD
  struct __pyx_obj_5sophy_Cursor *__pyx_v_cursor;
  PyObject *__pyx_v_item;
  struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "sophy.pyx":477
 *             yield item
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor(keys=False)
 *         for item in cursor:
 */
struct __pyx_obj_5sophy___pyx_scope_struct_7_values {
  PyObject_HEAD
  struct __pyx_obj_5sophy_Cursor *__pyx_v_cursor;
  PyObject *__pyx_v_item;
  struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "sophy.pyx":482
 *             yield item
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor()
 *         for item in cursor:
 */
struct __pyx_obj_5sophy___pyx_scope_struct_8_items {
  PyObject_HEAD
  struct __pyx_obj_5sophy_Cursor *__pyx_v_cursor;
  PyObject *__pyx_v_item;
  struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};



/* "sophy.pyx":109
 * 
 * 
 * cdef class _ConfigManager(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         dict _config
 */

struct __pyx_vtabstruct_5sophy__ConfigManager {
  PyObject *(*set_option)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_option)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_14_ConfigManager_get_option *__pyx_optional_args);
  PyObject *(*clear_option)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*apply_config)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, PyObject *);
  PyObject *(*apply_all)(struct __pyx_obj_5sophy__ConfigManager *);
};
static struct __pyx_vtabstruct_5sophy__ConfigManager *__pyx_vtabptr_5sophy__ConfigManager;


/* "sophy.pyx":157
 * 
 * 
 * cdef class _SophiaObject(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         void *handle
 */

struct __pyx_vtabstruct_5sophy__SophiaObject {
  PyObject *(*destroy)(struct __pyx_obj_5sophy__SophiaObject *);
};
static struct __pyx_vtabstruct_5sophy__SophiaObject *__pyx_vtabptr_5sophy__SophiaObject;


/* "sophy.pyx":169
 * 
 * 
 * cdef class Sophia(_SophiaObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         bint is_open
 */

struct __pyx_vtabstruct_5sophy_Sophia {
  struct __pyx_vtabstruct_5sophy__SophiaObject __pyx_base;
  int (*open)(struct __pyx_obj_5sophy_Sophia *, int __pyx_skip_dispatch);
  PyObject *(*_add_db)(struct __pyx_obj_5sophy_Sophia *, struct __pyx_obj_5sophy_Database *, struct __pyx_opt_args_5sophy_6Sophia__add_db *__pyx_optional_args);
  int (*close)(struct __pyx_obj_5sophy_Sophia *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5sophy_Sophia *__pyx_vtabptr_5sophy_Sophia;


/* "sophy.pyx":372
 * 
 * 
 * cdef class _BaseDBObject(_SophiaObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly Sophia sophia
 */

struct __pyx_vtabstruct_5sophy__BaseDBObject {
  struct __pyx_vtabstruct_5sophy__SophiaObject __pyx_base;
  PyObject *(*configure)(struct __pyx_obj_5sophy__BaseDBObject *);
  int (*open)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_open *__pyx_optional_args);
  int (*close)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch);
  void *(*_create_handle)(struct __pyx_obj_5sophy__BaseDBObject *);
  struct __pyx_obj_5sophy__Index *(*_get_index)(struct __pyx_obj_5sophy__BaseDBObject *);
  PyObject *(*_update)(struct __pyx_obj_5sophy__BaseDBObject *, struct __pyx_obj_5sophy_Database *, struct __pyx_opt_args_5sophy_13_BaseDBObject__update *__pyx_optional_args);
  PyObject *(*cursor)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor *__pyx_optional_args);
};
static struct __pyx_vtabstruct_5sophy__BaseDBObject *__pyx_vtabptr_5sophy__BaseDBObject;


/* "sophy.pyx":508
 * 
 * 
 * cdef class Database(_BaseDBObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly bytes name
 */

struct __pyx_vtabstruct_5sophy_Database {
  struct __pyx_vtabstruct_5sophy__BaseDBObject __pyx_base;
  PyObject *(*transaction)(struct __pyx_obj_5sophy_Database *, int __pyx_skip_dispatch);
  PyObject *(*view)(struct __pyx_obj_5sophy_Database *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5sophy_Database *__pyx_vtabptr_5sophy_Database;


/* "sophy.pyx":632
 * 
 * 
 * cdef class View(_BaseDBObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Database db
 */

struct __pyx_vtabstruct_5sophy_View {
  struct __pyx_vtabstruct_5sophy__BaseDBObject __pyx_base;
};
static struct __pyx_vtabstruct_5sophy_View *__pyx_vtabptr_5sophy_View;


/* "sophy.pyx":679
 * 
 * 
 * cdef class Transaction(_BaseDBObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Database db
 */

struct __pyx_vtabstruct_5sophy_Transaction {
  struct __pyx_vtabstruct_5sophy__BaseDBObject __pyx_base;
  PyObject *(*commit)(struct __pyx_obj_5sophy_Transaction *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_commit *__pyx_optional_args);
  PyObject *(*rollback)(struct __pyx_obj_5sophy_Transaction *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_rollback *__pyx_optional_args);
};
static struct __pyx_vtabstruct_5sophy_Transaction *__pyx_vtabptr_5sophy_Transaction;


/* "sophy.pyx":725
 * 
 * 
 * cdef class Cursor(_SophiaObject):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Sophia sophia
 */

struct __pyx_vtabstruct_5sophy_Cursor {
  struct __pyx_vtabstruct_5sophy__SophiaObject __pyx_base;
};
static struct __pyx_vtabstruct_5sophy_Cursor *__pyx_vtabptr_5sophy_Cursor;


/* "sophy.pyx":799
 * 
 * 
 * cdef class _Index(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Sophia sophia
 */

struct __pyx_vtabstruct_5sophy__Index {
  PyObject *(*_get_empty_value)(struct __pyx_obj_5sophy__Index *);
  PyObject *(*configure)(struct __pyx_obj_5sophy__Index *);
  PyObject *(*set)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *, PyObject *);
  PyObject *(*set_key)(struct __pyx_obj_5sophy__Index *, void *, PyObject *);
  PyObject *(*extract_key)(struct __pyx_obj_5sophy__Index *, void *);
  PyObject *(*get)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *);
  PyObject *(*delete)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *);
  PyObject *(*exists)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *);
};
static struct __pyx_vtabstruct_5sophy__Index *__pyx_vtabptr_5sophy__Index;


/* "sophy.pyx":891
 * 
 * 
 * cdef class _UInt32Index(_Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u32'
 * 
 */

struct __pyx_vtabstruct_5sophy__UInt32Index {
  struct __pyx_vtabstruct_5sophy__Index __pyx_base;
};
static struct __pyx_vtabstruct_5sophy__UInt32Index *__pyx_vtabptr_5sophy__UInt32Index;


/* "sophy.pyx":915
 * 
 * 
 * cdef class _UInt32RevIndex(_UInt32Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u32rev'
 * 
 */

struct __pyx_vtabstruct_5sophy__UInt32RevIndex {
  struct __pyx_vtabstruct_5sophy__UInt32Index __pyx_base;
};
static struct __pyx_vtabstruct_5sophy__UInt32RevIndex *__pyx_vtabptr_5sophy__UInt32RevIndex;


/* "sophy.pyx":919
 * 
 * 
 * cdef class _UInt64Index(_Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u64'
 * 
 */

struct __pyx_vtabstruct_5sophy__UInt64Index {
  struct __pyx_vtabstruct_5sophy__Index __pyx_base;
};
static struct __pyx_vtabstruct_5sophy__UInt64Index *__pyx_vtabptr_5sophy__UInt64Index;


/* "sophy.pyx":943
 * 
 * 
 * cdef class _UInt64RevIndex(_UInt64Index):             # <<<<<<<<<<<<<<
 *     index_type = 'u64rev'
 * 
 */

struct __pyx_vtabstruct_5sophy__UInt64RevIndex {
  struct __pyx_vtabstruct_5sophy__UInt64Index __pyx_base;
};
static struct __pyx_vtabstruct_5sophy__UInt64RevIndex *__pyx_vtabptr_5sophy__UInt64RevIndex;


/* "sophy.pyx":947
 * 
 * 
 * cdef class _MultiIndex(_Index):             # <<<<<<<<<<<<<<
 *     cdef:
 *         tuple indexes
 */

struct __pyx_vtabstruct_5sophy__MultiIndex {
  struct __pyx_vtabstruct_5sophy__Index __pyx_base;
  PyObject *(*initialize_indexes)(struct __pyx_obj_5sophy__MultiIndex *, PyObject *);
};
static struct __pyx_vtabstruct_5sophy__MultiIndex *__pyx_vtabptr_5sophy__MultiIndex;


/* "sophy.pyx":1029
 * 
 * 
 * cdef class SimpleDatabase(Sophia):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly db_name
 */

struct __pyx_vtabstruct_5sophy_SimpleDatabase {
  struct __pyx_vtabstruct_5sophy_Sophia __pyx_base;
  PyObject *(*transaction)(struct __pyx_obj_5sophy_SimpleDatabase *, int __pyx_skip_dispatch);
  PyObject *(*view)(struct __pyx_obj_5sophy_SimpleDatabase *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5sophy_SimpleDatabase *__pyx_vtabptr_5sophy_SimpleDatabase;

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    return __Pyx_decode_c_bytes(
        PyBytes_AS_STRING(string), PyBytes_GET_SIZE(string),
        start, stop, encoding, errors, decode_func);
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb);
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    ((likely((cfunc)->func)) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
             ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) : __Pyx__CallUnboundCMethod0(cfunc, self)))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE int __Pyx_IterFinish(void);

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return __Pyx_PyObject_GetAttrStr(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#else
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

static CYTHON_INLINE int __Pyx_unpack_tuple2(PyObject* tuple, PyObject** value1, PyObject** value2,
                                             int is_tuple, int has_known_size, int decref_tuple);

static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_GetSlice(PyObject* src, Py_ssize_t start, Py_ssize_t stop);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_GetSlice(PyObject* src, Py_ssize_t start, Py_ssize_t stop);
#else
#define __Pyx_PyList_GetSlice(seq, start, stop)   PySequence_GetSlice(seq, start, stop)
#define __Pyx_PyTuple_GetSlice(seq, start, stop)  PySequence_GetSlice(seq, start, stop)
#endif

static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

#include <string.h>

static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64_t(int64_t value);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value);

static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);

static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyObject *);
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(PyTypeObject *type, __pyx_coroutine_body_t body,
                                                   PyObject *closure, PyObject *name, PyObject *qualname);
static int __Pyx_Coroutine_clear(PyObject *self);
#if 1 || PY_VERSION_HEX < 0x030300B0
static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue);
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue) PyGen_FetchStopIterationValue(pvalue)
#endif

static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

static int __Pyx_patch_abc(void);

#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, closure, name, qualname)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, closure, name, qualname)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_5sophy_14_ConfigManager_set_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_get_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_14_ConfigManager_get_option *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_clear_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_apply_config(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_apply_all(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_13_SophiaObject_destroy(struct __pyx_obj_5sophy__SophiaObject *__pyx_v_self); /* proto*/
static int __pyx_f_5sophy_6Sophia_open(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_6Sophia__add_db(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, struct __pyx_obj_5sophy_Database *__pyx_v_db_obj, struct __pyx_opt_args_5sophy_6Sophia__add_db *__pyx_optional_args); /* proto*/
static int __pyx_f_5sophy_6Sophia_close(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_13_BaseDBObject_configure(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto*/
static int __pyx_f_5sophy_13_BaseDBObject_open(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_open *__pyx_optional_args); /* proto*/
static int __pyx_f_5sophy_13_BaseDBObject_close(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void *__pyx_f_5sophy_13_BaseDBObject__create_handle(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto*/
static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_13_BaseDBObject__get_index(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_13_BaseDBObject__update(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, struct __pyx_obj_5sophy_Database *__pyx_v_db, struct __pyx_opt_args_5sophy_13_BaseDBObject__update *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_5sophy_13_BaseDBObject_cursor(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor *__pyx_optional_args); /* proto*/
static int __pyx_f_5sophy_8Database_close(struct __pyx_obj_5sophy_Database *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_8Database_destroy(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_8Database_configure(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto*/
static void *__pyx_f_5sophy_8Database__create_handle(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto*/
static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_8Database__get_index(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_8Database_transaction(struct __pyx_obj_5sophy_Database *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_8Database_view(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_8Database_cursor(struct __pyx_obj_5sophy_Database *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_8Database_cursor *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_5sophy_4View_destroy(struct __pyx_obj_5sophy_View *__pyx_v_self); /* proto*/
static void *__pyx_f_5sophy_4View__create_handle(struct __pyx_obj_5sophy_View *__pyx_v_self); /* proto*/
static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_4View__get_index(struct __pyx_obj_5sophy_View *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_4View_cursor(struct __pyx_obj_5sophy_View *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_4View_cursor *__pyx_optional_args); /* proto*/
static void *__pyx_f_5sophy_11Transaction__create_handle(struct __pyx_obj_5sophy_Transaction *__pyx_v_self); /* proto*/
static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_11Transaction__get_index(struct __pyx_obj_5sophy_Transaction *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_11Transaction_commit(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_commit *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_5sophy_11Transaction_rollback(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_rollback *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index__get_empty_value(CYTHON_UNUSED struct __pyx_obj_5sophy__Index *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_configure(struct __pyx_obj_5sophy__Index *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_set(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_set_key(struct __pyx_obj_5sophy__Index *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_extract_key(struct __pyx_obj_5sophy__Index *__pyx_v_self, void *__pyx_v_obj); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_get(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_delete(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_6_Index_exists(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_12_UInt32Index__get_empty_value(CYTHON_UNUSED struct __pyx_obj_5sophy__UInt32Index *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_12_UInt32Index_set_key(struct __pyx_obj_5sophy__UInt32Index *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_12_UInt32Index_extract_key(struct __pyx_obj_5sophy__UInt32Index *__pyx_v_self, void *__pyx_v_obj); /* proto*/
static PyObject *__pyx_f_5sophy_12_UInt64Index__get_empty_value(CYTHON_UNUSED struct __pyx_obj_5sophy__UInt64Index *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_12_UInt64Index_set_key(struct __pyx_obj_5sophy__UInt64Index *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_12_UInt64Index_extract_key(struct __pyx_obj_5sophy__UInt64Index *__pyx_v_self, void *__pyx_v_obj); /* proto*/
static PyObject *__pyx_f_5sophy_11_MultiIndex_initialize_indexes(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, PyObject *__pyx_v_index_types); /* proto*/
static PyObject *__pyx_f_5sophy_11_MultiIndex__get_empty_value(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_11_MultiIndex_configure(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5sophy_11_MultiIndex_set_key(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key); /* proto*/
static PyObject *__pyx_f_5sophy_11_MultiIndex_extract_key(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, void *__pyx_v_obj); /* proto*/
static PyObject *__pyx_f_5sophy_14SimpleDatabase_transaction(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5sophy_14SimpleDatabase_view(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'sophy' */
static PyTypeObject *__pyx_ptype_5sophy__ConfigManager = 0;
static PyTypeObject *__pyx_ptype_5sophy__SophiaObject = 0;
static PyTypeObject *__pyx_ptype_5sophy_Sophia = 0;
static PyTypeObject *__pyx_ptype_5sophy__BaseDBObject = 0;
static PyTypeObject *__pyx_ptype_5sophy_Database = 0;
static PyTypeObject *__pyx_ptype_5sophy_View = 0;
static PyTypeObject *__pyx_ptype_5sophy_Transaction = 0;
static PyTypeObject *__pyx_ptype_5sophy_Cursor = 0;
static PyTypeObject *__pyx_ptype_5sophy__Index = 0;
static PyTypeObject *__pyx_ptype_5sophy__UInt32Index = 0;
static PyTypeObject *__pyx_ptype_5sophy__UInt32RevIndex = 0;
static PyTypeObject *__pyx_ptype_5sophy__UInt64Index = 0;
static PyTypeObject *__pyx_ptype_5sophy__UInt64RevIndex = 0;
static PyTypeObject *__pyx_ptype_5sophy__MultiIndex = 0;
static PyTypeObject *__pyx_ptype_5sophy_SimpleDatabase = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct___sp = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_1__sm = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_2__dbp = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_3__dbm = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_4___call__ = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_5_get_range = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_6_keys = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_7_values = 0;
static PyTypeObject *__pyx_ptype_5sophy___pyx_scope_struct_8_items = 0;
static int __pyx_v_5sophy_IS_PY3K;
static PyObject *__pyx_v_5sophy_INDEX_TYPE_MAP = 0;
static PyObject *__pyx_f_5sophy_encode(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_5sophy__getstring(void *, char const *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_5sophy__check(void *, int); /*proto*/
#define __Pyx_MODULE_NAME "sophy"
int __pyx_module_is_main_sophy = 0;

/* Implementation of 'sophy' */
static PyObject *__pyx_builtin_Exception;
static PyObject *__pyx_builtin_UnicodeDecodeError;
static PyObject *__pyx_builtin_property;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_NotImplementedError;
static PyObject *__pyx_builtin_StopIteration;
static PyObject *__pyx_builtin_NotImplemented;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_super;
static char __pyx_k_db[] = "db";
static char __pyx_k_fn[] = "fn";
static char __pyx_k_id[] = "id";
static char __pyx_k_os[] = "os";
static char __pyx_k_sm[] = "_sm";
static char __pyx_k_sp[] = "_sp";
static char __pyx_k__20[] = "<=";
static char __pyx_k__21[] = ">=";
static char __pyx_k__23[] = ", ";
static char __pyx_k__28[] = "";
static char __pyx_k__30[] = ".";
static char __pyx_k_dbm[] = "_dbm";
static char __pyx_k_dbp[] = "_dbp";
static char __pyx_k_doc[] = "__doc__";
static char __pyx_k_key[] = "key";
static char __pyx_k_lru[] = "lru";
static char __pyx_k_sys[] = "sys";
static char __pyx_k_u32[] = "u32";
static char __pyx_k_u64[] = "u64";
static char __pyx_k_zip[] = "zip";
static char __pyx_k_amqf[] = "amqf";
static char __pyx_k_args[] = "args";
static char __pyx_k_data[] = "_data";
static char __pyx_k_db_s[] = "db.%s";
static char __pyx_k_exit[] = "__exit__";
static char __pyx_k_init[] = "__init__";
static char __pyx_k_join[] = "join";
static char __pyx_k_keys[] = "keys";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_mmap[] = "mmap";
static char __pyx_k_name[] = "name";
static char __pyx_k_open[] = "open";
static char __pyx_k_path[] = "path";
static char __pyx_k_prop[] = "prop";
static char __pyx_k_self[] = "self";
static char __pyx_k_send[] = "send";
static char __pyx_k_step[] = "step";
static char __pyx_k_stop[] = "stop";
static char __pyx_k_sync[] = "sync";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_view[] = "view";
static char __pyx_k_begin[] = "begin";
static char __pyx_k_build[] = "build";
static char __pyx_k_cache[] = "cache";
static char __pyx_k_close[] = "close";
static char __pyx_k_enter[] = "__enter__";
static char __pyx_k_error[] = "error";
static char __pyx_k_items[] = "items";
static char __pyx_k_order[] = "order";
static char __pyx_k_sophy[] = "sophy";
static char __pyx_k_split[] = "split";
static char __pyx_k_start[] = "start";
static char __pyx_k_super[] = "super";
static char __pyx_k_throw[] = "throw";
static char __pyx_k_utf_8[] = "utf-8";
static char __pyx_k_value[] = "value";
static char __pyx_k_wraps[] = "wraps";
static char __pyx_k_branch[] = "branch";
static char __pyx_k_commit[] = "commit";
static char __pyx_k_config[] = "config";
static char __pyx_k_cursor[] = "cursor";
static char __pyx_k_db_s_s[] = "db.%s.%s";
static char __pyx_k_encode[] = "encode";
static char __pyx_k_exc_tb[] = "exc_tb";
static char __pyx_k_format[] = "format";
static char __pyx_k_getter[] = "_getter";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_key__s[] = "key_%s";
static char __pyx_k_kwargs[] = "kwargs";
static char __pyx_k_log_gc[] = "log.gc";
static char __pyx_k_method[] = "_method";
static char __pyx_k_module[] = "__module__";
static char __pyx_k_prefix[] = "prefix";
static char __pyx_k_setter[] = "_setter";
static char __pyx_k_sophia[] = "sophia";
static char __pyx_k_status[] = "status";
static char __pyx_k_string[] = "string";
static char __pyx_k_target[] = "target";
static char __pyx_k_u32rev[] = "u32rev";
static char __pyx_k_u64rev[] = "u64rev";
static char __pyx_k_update[] = "update";
static char __pyx_k_values[] = "values";
static char __pyx_k_view_s[] = "view.%s";
static char __pyx_k_bcdefgh[] = " bcdefgh";
static char __pyx_k_compact[] = "compact";
static char __pyx_k_connect[] = "connect";
static char __pyx_k_db_name[] = "db_name";
static char __pyx_k_default[] = "default";
static char __pyx_k_dirname[] = "dirname";
static char __pyx_k_exc_val[] = "exc_val";
static char __pyx_k_prepare[] = "__prepare__";
static char __pyx_k_recover[] = "recover";
static char __pyx_k_reverse[] = "reverse";
static char __pyx_k_storage[] = "storage";
static char __pyx_k_threads[] = "threads";
static char __pyx_k_version[] = "version";
static char __pyx_k_wrapper[] = "wrapper";
static char __pyx_k_KeyError[] = "KeyError";
static char __pyx_k_basename[] = "basename";
static char __pyx_k_data_dir[] = "data_dir";
static char __pyx_k_exc_type[] = "exc_type";
static char __pyx_k_filename[] = "filename";
static char __pyx_k_log_gc_2[] = "log_gc";
static char __pyx_k_log_path[] = "log_path";
static char __pyx_k_log_sync[] = "log.sync";
static char __pyx_k_lru_step[] = "lru_step";
static char __pyx_k_property[] = "property";
static char __pyx_k_propname[] = "propname";
static char __pyx_k_qualname[] = "__qualname__";
static char __pyx_k_readonly[] = "readonly";
static char __pyx_k_rollback[] = "rollback";
static char __pyx_k_Exception[] = "Exception";
static char __pyx_k_auto_open[] = "auto_open";
static char __pyx_k_configure[] = "configure";
static char __pyx_k_databases[] = "databases";
static char __pyx_k_enumerate[] = "enumerate";
static char __pyx_k_functools[] = "functools";
static char __pyx_k_get_range[] = "get_range";
static char __pyx_k_log_files[] = "log.files";
static char __pyx_k_metaclass[] = "__metaclass__";
static char __pyx_k_node_size[] = "node_size";
static char __pyx_k_page_size[] = "page_size";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_backup_run[] = "backup.run";
static char __pyx_k_cache_mode[] = "cache_mode";
static char __pyx_k_db_s_index[] = "db.%s.index";
static char __pyx_k_get_option[] = "get_option";
static char __pyx_k_index_size[] = "index.size";
static char __pyx_k_index_type[] = "index_type";
static char __pyx_k_log_enable[] = "log.enable";
static char __pyx_k_log_path_2[] = "log.path";
static char __pyx_k_log_rotate[] = "log.rotate";
static char __pyx_k_log_sync_2[] = "log_sync";
static char __pyx_k_metric_asn[] = "metric.asn";
static char __pyx_k_metric_bsn[] = "metric.bsn";
static char __pyx_k_metric_dsn[] = "metric.dsn";
static char __pyx_k_metric_lsn[] = "metric.lsn";
static char __pyx_k_metric_nsn[] = "metric.nsn";
static char __pyx_k_metric_ssn[] = "metric.ssn";
static char __pyx_k_metric_tsn[] = "metric.tsn";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_set_option[] = "set_option";
static char __pyx_k_MemoryError[] = "MemoryError";
static char __pyx_k_backup_last[] = "backup.last";
static char __pyx_k_backup_path[] = "backup_path";
static char __pyx_k_compression[] = "compression";
static char __pyx_k_database_id[] = "database_id";
static char __pyx_k_index_count[] = "index.count";
static char __pyx_k_index_types[] = "index_types";
static char __pyx_k_log_files_2[] = "log_files";
static char __pyx_k_memory_used[] = "memory.used";
static char __pyx_k_metric_lfsn[] = "metric.lfsn";
static char __pyx_k_sophia_path[] = "sophia.path";
static char __pyx_k_transaction[] = "transaction";
static char __pyx_k_backup_run_2[] = "backup_run";
static char __pyx_k_clear_option[] = "clear_option";
static char __pyx_k_db_s_index_s[] = "db.%s.index.%s";
static char __pyx_k_index_size_2[] = "index_size";
static char __pyx_k_log_enable_2[] = "log_enable";
static char __pyx_k_log_rotate_2[] = "log_rotate";
static char __pyx_k_memory_limit[] = "memory_limit";
static char __pyx_k_metric_asn_2[] = "metric_asn";
static char __pyx_k_metric_bsn_2[] = "metric_bsn";
static char __pyx_k_metric_dsn_2[] = "metric_dsn";
static char __pyx_k_metric_lsn_2[] = "metric_lsn";
static char __pyx_k_metric_nsn_2[] = "metric_nsn";
static char __pyx_k_metric_ssn_2[] = "metric_ssn";
static char __pyx_k_metric_tsn_2[] = "metric_tsn";
static char __pyx_k_node_preload[] = "node_preload";
static char __pyx_k_scheduler_gc[] = "scheduler.gc";
static char __pyx_k_sophia_build[] = "sophia.build";
static char __pyx_k_sophia_error[] = "sophia.error";
static char __pyx_k_version_info[] = "version_info";
static char __pyx_k_StopIteration[] = "StopIteration";
static char __pyx_k_backup_active[] = "backup.active";
static char __pyx_k_backup_last_2[] = "backup_last";
static char __pyx_k_backup_path_2[] = "backup.path";
static char __pyx_k_compact_index[] = "compact_index";
static char __pyx_k_database_path[] = "database_path";
static char __pyx_k_index_count_2[] = "index_count";
static char __pyx_k_log_rotate_wm[] = "log.rotate_wm";
static char __pyx_k_memory_used_2[] = "memory_used";
static char __pyx_k_metric_lfsn_2[] = "metric_lfsn";
static char __pyx_k_page_checksum[] = "page_checksum";
static char __pyx_k_scheduler_lru[] = "scheduler.lru";
static char __pyx_k_scheduler_run[] = "scheduler.run";
static char __pyx_k_sophia_path_2[] = "sophia_path";
static char __pyx_k_NotImplemented[] = "NotImplemented";
static char __pyx_k_memory_limit_2[] = "memory.limit";
static char __pyx_k_performance_tx[] = "performance.tx";
static char __pyx_k_scheduler_gc_2[] = "scheduler_gc";
static char __pyx_k_scheduler_zone[] = "scheduler.zone";
static char __pyx_k_sophia_recover[] = "sophia.recover";
static char __pyx_k_sophia_version[] = "sophia.version";
static char __pyx_k_backup_active_2[] = "backup_active";
static char __pyx_k_compression_key[] = "compression_key";
static char __pyx_k_index_count_dup[] = "index.count_dup";
static char __pyx_k_index_read_disk[] = "index.read_disk";
static char __pyx_k_index_size_amqf[] = "index.size_amqf";
static char __pyx_k_log_rotate_sync[] = "log.rotate_sync";
static char __pyx_k_log_rotate_wm_2[] = "log_rotate_wm";
static char __pyx_k_performance_get[] = "performance.get";
static char __pyx_k_performance_key[] = "performance.key";
static char __pyx_k_performance_set[] = "performance.set";
static char __pyx_k_scheduler_lru_2[] = "scheduler_lru";
static char __pyx_k_scheduler_run_2[] = "scheduler_run";
static char __pyx_k_version_storage[] = "version_storage";
static char __pyx_k_index_branch_max[] = "index.branch_max";
static char __pyx_k_index_node_count[] = "index.node_count";
static char __pyx_k_index_page_count[] = "index.page_count";
static char __pyx_k_index_read_cache[] = "index.read_cache";
static char __pyx_k_memory_anticache[] = "memory.anticache";
static char __pyx_k_performance_tx_2[] = "performance_tx";
static char __pyx_k_scheduler_zone_2[] = "scheduler_zone";
static char __pyx_k_BaseDBObject_keys[] = "_BaseDBObject.keys";
static char __pyx_k_index_count_dup_2[] = "index_count_dup";
static char __pyx_k_index_memory_used[] = "index.memory_used";
static char __pyx_k_index_read_disk_2[] = "index_read_disk";
static char __pyx_k_index_size_amqf_2[] = "index_size_amqf";
static char __pyx_k_log_rotate_sync_2[] = "log_rotate_sync";
static char __pyx_k_path_fail_on_drop[] = "path_fail_on_drop";
static char __pyx_k_performance_get_2[] = "performance_get";
static char __pyx_k_performance_key_2[] = "performance_key";
static char __pyx_k_performance_set_2[] = "performance_set";
static char __pyx_k_performance_value[] = "performance.value";
static char __pyx_k_scheduler_threads[] = "scheduler_threads";
static char __pyx_k_sm_locals__method[] = "_sm.<locals>._method";
static char __pyx_k_sp_locals__getter[] = "_sp.<locals>._getter";
static char __pyx_k_sp_locals__setter[] = "_sp.<locals>._setter";
static char __pyx_k_BaseDBObject_items[] = "_BaseDBObject.items";
static char __pyx_k_UnicodeDecodeError[] = "UnicodeDecodeError";
static char __pyx_k_compaction_redzone[] = "compaction.redzone";
static char __pyx_k_compression_branch[] = "compression_branch";
static char __pyx_k_dbm_locals__method[] = "_dbm.<locals>._method";
static char __pyx_k_dbp_locals__getter[] = "_dbp.<locals>._getter";
static char __pyx_k_dbp_locals__setter[] = "_dbp.<locals>._setter";
static char __pyx_k_index_branch_count[] = "index.branch_count";
static char __pyx_k_index_branch_max_2[] = "index_branch_max";
static char __pyx_k_index_node_count_2[] = "index_node_count";
static char __pyx_k_index_page_count_2[] = "index_page_count";
static char __pyx_k_index_read_cache_2[] = "index_read_cache";
static char __pyx_k_memory_anticache_2[] = "memory_anticache";
static char __pyx_k_memory_pager_pools[] = "memory.pager_pools";
static char __pyx_k_performance_cursor[] = "performance.cursor";
static char __pyx_k_performance_delete[] = "performance.delete";
static char __pyx_k_performance_tx_ops[] = "performance.tx_ops";
static char __pyx_k_scheduler_snapshot[] = "scheduler.snapshot";
static char __pyx_k_sophia_path_create[] = "sophia.path_create";
static char __pyx_k_BaseDBObject_values[] = "_BaseDBObject.values";
static char __pyx_k_NotImplementedError[] = "NotImplementedError";
static char __pyx_k_Views_are_read_only[] = "Views are read-only.";
static char __pyx_k_index_memory_used_2[] = "index_memory_used";
static char __pyx_k_index_size_snapshot[] = "index.size_snapshot";
static char __pyx_k_path_fail_on_exists[] = "path_fail_on_exists";
static char __pyx_k_performance_tx_lock[] = "performance.tx_lock";
static char __pyx_k_performance_value_2[] = "performance_value";
static char __pyx_k_scheduler_anticache[] = "scheduler.anticache";
static char __pyx_k_scheduler_gc_active[] = "scheduler.gc_active";
static char __pyx_k_scheduler_threads_2[] = "scheduler.threads";
static char __pyx_k_CannotCloseException[] = "CannotCloseException";
static char __pyx_k_backup_last_complete[] = "backup.last_complete";
static char __pyx_k_compaction_redzone_2[] = "compaction_redzone";
static char __pyx_k_index_branch_count_2[] = "index_branch_count";
static char __pyx_k_memory_pager_pools_2[] = "memory_pager_pools";
static char __pyx_k_performance_cursor_2[] = "performance_cursor";
static char __pyx_k_performance_delete_2[] = "performance_delete";
static char __pyx_k_performance_tx_ops_2[] = "performance_tx_ops";
static char __pyx_k_scheduler_checkpoint[] = "scheduler.checkpoint";
static char __pyx_k_scheduler_lru_active[] = "scheduler.lru_active";
static char __pyx_k_scheduler_snapshot_2[] = "scheduler_snapshot";
static char __pyx_k_sophia_path_create_2[] = "sophia_path_create";
static char __pyx_k_call___locals_wrapper[] = "__call__.<locals>.wrapper";
static char __pyx_k_index_size_snapshot_2[] = "index_size_snapshot";
static char __pyx_k_performance_documents[] = "performance.documents";
static char __pyx_k_performance_tx_lock_2[] = "performance_tx_lock";
static char __pyx_k_scheduler_anticache_2[] = "scheduler_anticache";
static char __pyx_k_scheduler_gc_active_2[] = "scheduler_gc_active";
static char __pyx_k_BaseDBObject_get_range[] = "_BaseDBObject.get_range";
static char __pyx_k_backup_last_complete_2[] = "backup_last_complete";
static char __pyx_k_memory_pager_page_size[] = "memory.pager_page_size";
static char __pyx_k_memory_pager_pool_size[] = "memory.pager_pool_size";
static char __pyx_k_performance_cursor_ops[] = "performance.cursor_ops";
static char __pyx_k_performance_tx_latency[] = "performance.tx_latency";
static char __pyx_k_scheduler_checkpoint_2[] = "scheduler_checkpoint";
static char __pyx_k_scheduler_lru_active_2[] = "scheduler_lru_active";
static char __pyx_k_scheduler_snapshot_ssn[] = "scheduler.snapshot_ssn";
static char __pyx_k_sophia_version_storage[] = "sophia.version_storage";
static char __pyx_k_unknown_error_occurred[] = "unknown error occurred.";
static char __pyx_k_compaction_redzone_mode[] = "compaction.redzone.mode";
static char __pyx_k_index_size_uncompressed[] = "index.size_uncompressed";
static char __pyx_k_performance_documents_2[] = "performance_documents";
static char __pyx_k_performance_get_latency[] = "performance.get_latency";
static char __pyx_k_performance_set_latency[] = "performance.set_latency";
static char __pyx_k_performance_tx_conflict[] = "performance.tx_conflict";
static char __pyx_k_performance_tx_gc_queue[] = "performance.tx_gc_queue";
static char __pyx_k_performance_tx_rollback[] = "performance.tx_rollback";
static char __pyx_k_scheduler_anticache_asn[] = "scheduler.anticache_asn";
static char __pyx_k_compaction_redzone_async[] = "compaction.redzone.async";
static char __pyx_k_compaction_redzone_gc_wm[] = "compaction.redzone.gc_wm";
static char __pyx_k_memory_pager_page_size_2[] = "memory_pager_page_size";
static char __pyx_k_memory_pager_pool_size_2[] = "memory_pager_pool_size";
static char __pyx_k_performance_cursor_ops_2[] = "performance_cursor_ops";
static char __pyx_k_performance_tx_active_ro[] = "performance.tx_active_ro";
static char __pyx_k_performance_tx_active_rw[] = "performance.tx_active_rw";
static char __pyx_k_performance_tx_latency_2[] = "performance_tx_latency";
static char __pyx_k_scheduler_checkpoint_lsn[] = "scheduler.checkpoint_lsn";
static char __pyx_k_scheduler_snapshot_ssn_2[] = "scheduler_snapshot_ssn";
static char __pyx_k_compaction_redzone_mode_2[] = "compaction_redzone_mode";
static char __pyx_k_index_size_uncompressed_2[] = "index_size_uncompressed";
static char __pyx_k_performance_get_latency_2[] = "performance_get_latency";
static char __pyx_k_performance_get_read_disk[] = "performance.get_read_disk";
static char __pyx_k_performance_set_latency_2[] = "performance_set_latency";
static char __pyx_k_performance_tx_conflict_2[] = "performance_tx_conflict";
static char __pyx_k_performance_tx_gc_queue_2[] = "performance_tx_gc_queue";
static char __pyx_k_performance_tx_rollback_2[] = "performance_tx_rollback";
static char __pyx_k_scheduler_anticache_asn_2[] = "scheduler_anticache_asn";
static char __pyx_k_scheduler_snapshot_active[] = "scheduler.snapshot_active";
static char __pyx_k_compaction_redzone_async_2[] = "compaction_redzone_async";
static char __pyx_k_compaction_redzone_gc_wm_2[] = "compaction_redzone_gc_wm";
static char __pyx_k_performance_cursor_latency[] = "performance.cursor_latency";
static char __pyx_k_performance_delete_latency[] = "performance.delete_latency";
static char __pyx_k_performance_documents_used[] = "performance.documents_used";
static char __pyx_k_performance_get_read_cache[] = "performance.get_read_cache";
static char __pyx_k_performance_tx_active_ro_2[] = "performance_tx_active_ro";
static char __pyx_k_performance_tx_active_rw_2[] = "performance_tx_active_rw";
static char __pyx_k_scheduler_anticache_active[] = "scheduler.anticache_active";
static char __pyx_k_scheduler_checkpoint_lsn_2[] = "scheduler_checkpoint_lsn";
static char __pyx_k_Unable_to_allocate_object_s[] = "Unable to allocate object: %s.";
static char __pyx_k_compaction_redzone_lru_prio[] = "compaction.redzone.lru_prio";
static char __pyx_k_performance_get_read_disk_2[] = "performance_get_read_disk";
static char __pyx_k_scheduler_checkpoint_active[] = "scheduler.checkpoint_active";
static char __pyx_k_scheduler_snapshot_active_2[] = "scheduler_snapshot_active";
static char __pyx_k_scheduler_snapshot_ssn_last[] = "scheduler.snapshot_ssn_last";
static char __pyx_k_compaction_redzone_branch_wm[] = "compaction.redzone.branch_wm";
static char __pyx_k_compaction_redzone_gc_period[] = "compaction.redzone.gc_period";
static char __pyx_k_performance_cursor_latency_2[] = "performance_cursor_latency";
static char __pyx_k_performance_cursor_read_disk[] = "performance.cursor_read_disk";
static char __pyx_k_performance_delete_latency_2[] = "performance_delete_latency";
static char __pyx_k_performance_documents_used_2[] = "performance_documents_used";
static char __pyx_k_performance_get_read_cache_2[] = "performance_get_read_cache";
static char __pyx_k_scheduler_anticache_active_2[] = "scheduler_anticache_active";
static char __pyx_k_scheduler_anticache_asn_last[] = "scheduler.anticache_asn_last";
static char __pyx_k_compaction_redzone_branch_age[] = "compaction.redzone.branch_age";
static char __pyx_k_compaction_redzone_compact_wm[] = "compaction.redzone.compact_wm";
static char __pyx_k_compaction_redzone_gc_db_prio[] = "compaction.redzone.gc_db_prio";
static char __pyx_k_compaction_redzone_lru_period[] = "compaction.redzone.lru_period";
static char __pyx_k_compaction_redzone_lru_prio_2[] = "compaction_redzone_lru_prio";
static char __pyx_k_performance_cursor_read_cache[] = "performance.cursor_read_cache";
static char __pyx_k_scheduler_checkpoint_active_2[] = "scheduler_checkpoint_active";
static char __pyx_k_scheduler_checkpoint_lsn_last[] = "scheduler.checkpoint_lsn_last";
static char __pyx_k_scheduler_snapshot_ssn_last_2[] = "scheduler_snapshot_ssn_last";
static char __pyx_k_compaction_redzone_backup_prio[] = "compaction.redzone.backup_prio";
static char __pyx_k_compaction_redzone_branch_prio[] = "compaction.redzone.branch_prio";
static char __pyx_k_compaction_redzone_branch_wm_2[] = "compaction_redzone_branch_wm";
static char __pyx_k_compaction_redzone_gc_period_2[] = "compaction_redzone_gc_period";
static char __pyx_k_performance_cursor_read_disk_2[] = "performance_cursor_read_disk";
static char __pyx_k_scheduler_anticache_asn_last_2[] = "scheduler_anticache_asn_last";
static char __pyx_k_Unrecognized_index_type_must_be[] = "Unrecognized index type, must be one of: %s";
static char __pyx_k_compaction_redzone_branch_age_2[] = "compaction_redzone_branch_age";
static char __pyx_k_compaction_redzone_compact_mode[] = "compaction.redzone.compact_mode";
static char __pyx_k_compaction_redzone_compact_wm_2[] = "compaction_redzone_compact_wm";
static char __pyx_k_compaction_redzone_gc_db_prio_2[] = "compaction_redzone_gc_db_prio";
static char __pyx_k_compaction_redzone_lru_period_2[] = "compaction_redzone_lru_period";
static char __pyx_k_home_charles_Dropbox_code_sophy[] = "/home/charles/Dropbox/code/sophy/sophy.pyx";
static char __pyx_k_performance_cursor_read_cache_2[] = "performance_cursor_read_cache";
static char __pyx_k_scheduler_checkpoint_lsn_last_2[] = "scheduler_checkpoint_lsn_last";
static char __pyx_k_compaction_redzone_anticache_per[] = "compaction.redzone.anticache_period";
static char __pyx_k_compaction_redzone_backup_prio_2[] = "compaction_redzone_backup_prio";
static char __pyx_k_compaction_redzone_branch_age_pe[] = "compaction.redzone.branch_age_period";
static char __pyx_k_compaction_redzone_branch_age_wm[] = "compaction.redzone.branch_age_wm";
static char __pyx_k_compaction_redzone_branch_prio_2[] = "compaction_redzone_branch_prio";
static char __pyx_k_index_types_is_a_required_parame[] = "index_types is a required parameter.";
static char __pyx_k_transaction_is_not_finished_wait[] = "transaction is not finished, waiting for a concurrent transaction to finish.";
static char __pyx_k_transaction_was_rolled_back_by_a[] = "transaction was rolled back by another concurrent transaction.";
static char __pyx_k_compaction_redzone_compact_mode_2[] = "compaction_redzone_compact_mode";
static char __pyx_k_compaction_redzone_anticache_per_2[] = "compaction_redzone_anticache_period";
static char __pyx_k_compaction_redzone_branch_age_pe_2[] = "compaction_redzone_branch_age_period";
static char __pyx_k_compaction_redzone_branch_age_wm_2[] = "compaction_redzone_branch_age_wm";
static PyObject *__pyx_n_s_BaseDBObject_get_range;
static PyObject *__pyx_n_s_BaseDBObject_items;
static PyObject *__pyx_n_s_BaseDBObject_keys;
static PyObject *__pyx_n_s_BaseDBObject_values;
static PyObject *__pyx_n_s_CannotCloseException;
static PyObject *__pyx_n_s_Exception;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_s_NotImplemented;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_StopIteration;
static PyObject *__pyx_kp_s_Unable_to_allocate_object_s;
static PyObject *__pyx_n_s_UnicodeDecodeError;
static PyObject *__pyx_kp_s_Unrecognized_index_type_must_be;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s_Views_are_read_only;
static PyObject *__pyx_kp_s__20;
static PyObject *__pyx_kp_s__21;
static PyObject *__pyx_kp_s__23;
static PyObject *__pyx_kp_s__28;
static PyObject *__pyx_kp_s__30;
static PyObject *__pyx_n_s_amqf;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_auto_open;
static PyObject *__pyx_kp_s_backup_active;
static PyObject *__pyx_n_s_backup_active_2;
static PyObject *__pyx_kp_s_backup_last;
static PyObject *__pyx_n_s_backup_last_2;
static PyObject *__pyx_kp_s_backup_last_complete;
static PyObject *__pyx_n_s_backup_last_complete_2;
static PyObject *__pyx_n_s_backup_path;
static PyObject *__pyx_kp_s_backup_path_2;
static PyObject *__pyx_kp_s_backup_run;
static PyObject *__pyx_n_s_backup_run_2;
static PyObject *__pyx_n_s_basename;
static PyObject *__pyx_kp_s_bcdefgh;
static PyObject *__pyx_n_s_begin;
static PyObject *__pyx_n_s_branch;
static PyObject *__pyx_n_s_build;
static PyObject *__pyx_n_s_cache;
static PyObject *__pyx_n_s_cache_mode;
static PyObject *__pyx_n_s_call___locals_wrapper;
static PyObject *__pyx_n_s_clear_option;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_commit;
static PyObject *__pyx_n_s_compact;
static PyObject *__pyx_n_s_compact_index;
static PyObject *__pyx_kp_s_compaction_redzone;
static PyObject *__pyx_n_s_compaction_redzone_2;
static PyObject *__pyx_kp_s_compaction_redzone_anticache_per;
static PyObject *__pyx_n_s_compaction_redzone_anticache_per_2;
static PyObject *__pyx_kp_s_compaction_redzone_async;
static PyObject *__pyx_n_s_compaction_redzone_async_2;
static PyObject *__pyx_kp_s_compaction_redzone_backup_prio;
static PyObject *__pyx_n_s_compaction_redzone_backup_prio_2;
static PyObject *__pyx_kp_s_compaction_redzone_branch_age;
static PyObject *__pyx_n_s_compaction_redzone_branch_age_2;
static PyObject *__pyx_kp_s_compaction_redzone_branch_age_pe;
static PyObject *__pyx_n_s_compaction_redzone_branch_age_pe_2;
static PyObject *__pyx_kp_s_compaction_redzone_branch_age_wm;
static PyObject *__pyx_n_s_compaction_redzone_branch_age_wm_2;
static PyObject *__pyx_kp_s_compaction_redzone_branch_prio;
static PyObject *__pyx_n_s_compaction_redzone_branch_prio_2;
static PyObject *__pyx_kp_s_compaction_redzone_branch_wm;
static PyObject *__pyx_n_s_compaction_redzone_branch_wm_2;
static PyObject *__pyx_kp_s_compaction_redzone_compact_mode;
static PyObject *__pyx_n_s_compaction_redzone_compact_mode_2;
static PyObject *__pyx_kp_s_compaction_redzone_compact_wm;
static PyObject *__pyx_n_s_compaction_redzone_compact_wm_2;
static PyObject *__pyx_kp_s_compaction_redzone_gc_db_prio;
static PyObject *__pyx_n_s_compaction_redzone_gc_db_prio_2;
static PyObject *__pyx_kp_s_compaction_redzone_gc_period;
static PyObject *__pyx_n_s_compaction_redzone_gc_period_2;
static PyObject *__pyx_kp_s_compaction_redzone_gc_wm;
static PyObject *__pyx_n_s_compaction_redzone_gc_wm_2;
static PyObject *__pyx_kp_s_compaction_redzone_lru_period;
static PyObject *__pyx_n_s_compaction_redzone_lru_period_2;
static PyObject *__pyx_kp_s_compaction_redzone_lru_prio;
static PyObject *__pyx_n_s_compaction_redzone_lru_prio_2;
static PyObject *__pyx_kp_s_compaction_redzone_mode;
static PyObject *__pyx_n_s_compaction_redzone_mode_2;
static PyObject *__pyx_n_s_compression;
static PyObject *__pyx_n_s_compression_branch;
static PyObject *__pyx_n_s_compression_key;
static PyObject *__pyx_n_s_config;
static PyObject *__pyx_n_s_configure;
static PyObject *__pyx_n_s_connect;
static PyObject *__pyx_n_s_cursor;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_data_dir;
static PyObject *__pyx_n_s_database_id;
static PyObject *__pyx_n_s_database_path;
static PyObject *__pyx_n_s_databases;
static PyObject *__pyx_n_s_db;
static PyObject *__pyx_n_s_db_name;
static PyObject *__pyx_kp_s_db_s;
static PyObject *__pyx_kp_s_db_s_index;
static PyObject *__pyx_kp_s_db_s_index_s;
static PyObject *__pyx_kp_s_db_s_s;
static PyObject *__pyx_n_s_dbm;
static PyObject *__pyx_n_s_dbm_locals__method;
static PyObject *__pyx_n_s_dbp;
static PyObject *__pyx_n_s_dbp_locals__getter;
static PyObject *__pyx_n_s_dbp_locals__setter;
static PyObject *__pyx_n_s_default;
static PyObject *__pyx_n_s_dirname;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enter;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_exc_tb;
static PyObject *__pyx_n_s_exc_type;
static PyObject *__pyx_n_s_exc_val;
static PyObject *__pyx_n_s_exit;
static PyObject *__pyx_n_s_filename;
static PyObject *__pyx_n_s_fn;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_functools;
static PyObject *__pyx_n_s_get_option;
static PyObject *__pyx_n_s_get_range;
static PyObject *__pyx_n_s_getter;
static PyObject *__pyx_kp_s_home_charles_Dropbox_code_sophy;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_kp_s_index_branch_count;
static PyObject *__pyx_n_s_index_branch_count_2;
static PyObject *__pyx_kp_s_index_branch_max;
static PyObject *__pyx_n_s_index_branch_max_2;
static PyObject *__pyx_kp_s_index_count;
static PyObject *__pyx_n_s_index_count_2;
static PyObject *__pyx_kp_s_index_count_dup;
static PyObject *__pyx_n_s_index_count_dup_2;
static PyObject *__pyx_kp_s_index_memory_used;
static PyObject *__pyx_n_s_index_memory_used_2;
static PyObject *__pyx_kp_s_index_node_count;
static PyObject *__pyx_n_s_index_node_count_2;
static PyObject *__pyx_kp_s_index_page_count;
static PyObject *__pyx_n_s_index_page_count_2;
static PyObject *__pyx_kp_s_index_read_cache;
static PyObject *__pyx_n_s_index_read_cache_2;
static PyObject *__pyx_kp_s_index_read_disk;
static PyObject *__pyx_n_s_index_read_disk_2;
static PyObject *__pyx_kp_s_index_size;
static PyObject *__pyx_n_s_index_size_2;
static PyObject *__pyx_kp_s_index_size_amqf;
static PyObject *__pyx_n_s_index_size_amqf_2;
static PyObject *__pyx_kp_s_index_size_snapshot;
static PyObject *__pyx_n_s_index_size_snapshot_2;
static PyObject *__pyx_kp_s_index_size_uncompressed;
static PyObject *__pyx_n_s_index_size_uncompressed_2;
static PyObject *__pyx_n_s_index_type;
static PyObject *__pyx_n_s_index_types;
static PyObject *__pyx_kp_s_index_types_is_a_required_parame;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_kp_s_key__s;
static PyObject *__pyx_n_s_keys;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_kp_s_log_enable;
static PyObject *__pyx_n_s_log_enable_2;
static PyObject *__pyx_kp_s_log_files;
static PyObject *__pyx_n_s_log_files_2;
static PyObject *__pyx_kp_s_log_gc;
static PyObject *__pyx_n_s_log_gc_2;
static PyObject *__pyx_n_s_log_path;
static PyObject *__pyx_kp_s_log_path_2;
static PyObject *__pyx_kp_s_log_rotate;
static PyObject *__pyx_n_s_log_rotate_2;
static PyObject *__pyx_kp_s_log_rotate_sync;
static PyObject *__pyx_n_s_log_rotate_sync_2;
static PyObject *__pyx_kp_s_log_rotate_wm;
static PyObject *__pyx_n_s_log_rotate_wm_2;
static PyObject *__pyx_kp_s_log_sync;
static PyObject *__pyx_n_s_log_sync_2;
static PyObject *__pyx_n_s_lru;
static PyObject *__pyx_n_s_lru_step;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_kp_s_memory_anticache;
static PyObject *__pyx_n_s_memory_anticache_2;
static PyObject *__pyx_n_s_memory_limit;
static PyObject *__pyx_kp_s_memory_limit_2;
static PyObject *__pyx_kp_s_memory_pager_page_size;
static PyObject *__pyx_n_s_memory_pager_page_size_2;
static PyObject *__pyx_kp_s_memory_pager_pool_size;
static PyObject *__pyx_n_s_memory_pager_pool_size_2;
static PyObject *__pyx_kp_s_memory_pager_pools;
static PyObject *__pyx_n_s_memory_pager_pools_2;
static PyObject *__pyx_kp_s_memory_used;
static PyObject *__pyx_n_s_memory_used_2;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_method;
static PyObject *__pyx_kp_s_metric_asn;
static PyObject *__pyx_n_s_metric_asn_2;
static PyObject *__pyx_kp_s_metric_bsn;
static PyObject *__pyx_n_s_metric_bsn_2;
static PyObject *__pyx_kp_s_metric_dsn;
static PyObject *__pyx_n_s_metric_dsn_2;
static PyObject *__pyx_kp_s_metric_lfsn;
static PyObject *__pyx_n_s_metric_lfsn_2;
static PyObject *__pyx_kp_s_metric_lsn;
static PyObject *__pyx_n_s_metric_lsn_2;
static PyObject *__pyx_kp_s_metric_nsn;
static PyObject *__pyx_n_s_metric_nsn_2;
static PyObject *__pyx_kp_s_metric_ssn;
static PyObject *__pyx_n_s_metric_ssn_2;
static PyObject *__pyx_kp_s_metric_tsn;
static PyObject *__pyx_n_s_metric_tsn_2;
static PyObject *__pyx_n_s_mmap;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_node_preload;
static PyObject *__pyx_n_s_node_size;
static PyObject *__pyx_n_s_open;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_s_os;
static PyObject *__pyx_n_s_page_checksum;
static PyObject *__pyx_n_s_page_size;
static PyObject *__pyx_n_s_path;
static PyObject *__pyx_n_s_path_fail_on_drop;
static PyObject *__pyx_n_s_path_fail_on_exists;
static PyObject *__pyx_kp_s_performance_cursor;
static PyObject *__pyx_n_s_performance_cursor_2;
static PyObject *__pyx_kp_s_performance_cursor_latency;
static PyObject *__pyx_n_s_performance_cursor_latency_2;
static PyObject *__pyx_kp_s_performance_cursor_ops;
static PyObject *__pyx_n_s_performance_cursor_ops_2;
static PyObject *__pyx_kp_s_performance_cursor_read_cache;
static PyObject *__pyx_n_s_performance_cursor_read_cache_2;
static PyObject *__pyx_kp_s_performance_cursor_read_disk;
static PyObject *__pyx_n_s_performance_cursor_read_disk_2;
static PyObject *__pyx_kp_s_performance_delete;
static PyObject *__pyx_n_s_performance_delete_2;
static PyObject *__pyx_kp_s_performance_delete_latency;
static PyObject *__pyx_n_s_performance_delete_latency_2;
static PyObject *__pyx_kp_s_performance_documents;
static PyObject *__pyx_n_s_performance_documents_2;
static PyObject *__pyx_kp_s_performance_documents_used;
static PyObject *__pyx_n_s_performance_documents_used_2;
static PyObject *__pyx_kp_s_performance_get;
static PyObject *__pyx_n_s_performance_get_2;
static PyObject *__pyx_kp_s_performance_get_latency;
static PyObject *__pyx_n_s_performance_get_latency_2;
static PyObject *__pyx_kp_s_performance_get_read_cache;
static PyObject *__pyx_n_s_performance_get_read_cache_2;
static PyObject *__pyx_kp_s_performance_get_read_disk;
static PyObject *__pyx_n_s_performance_get_read_disk_2;
static PyObject *__pyx_kp_s_performance_key;
static PyObject *__pyx_n_s_performance_key_2;
static PyObject *__pyx_kp_s_performance_set;
static PyObject *__pyx_n_s_performance_set_2;
static PyObject *__pyx_kp_s_performance_set_latency;
static PyObject *__pyx_n_s_performance_set_latency_2;
static PyObject *__pyx_kp_s_performance_tx;
static PyObject *__pyx_n_s_performance_tx_2;
static PyObject *__pyx_kp_s_performance_tx_active_ro;
static PyObject *__pyx_n_s_performance_tx_active_ro_2;
static PyObject *__pyx_kp_s_performance_tx_active_rw;
static PyObject *__pyx_n_s_performance_tx_active_rw_2;
static PyObject *__pyx_kp_s_performance_tx_conflict;
static PyObject *__pyx_n_s_performance_tx_conflict_2;
static PyObject *__pyx_kp_s_performance_tx_gc_queue;
static PyObject *__pyx_n_s_performance_tx_gc_queue_2;
static PyObject *__pyx_kp_s_performance_tx_latency;
static PyObject *__pyx_n_s_performance_tx_latency_2;
static PyObject *__pyx_kp_s_performance_tx_lock;
static PyObject *__pyx_n_s_performance_tx_lock_2;
static PyObject *__pyx_kp_s_performance_tx_ops;
static PyObject *__pyx_n_s_performance_tx_ops_2;
static PyObject *__pyx_kp_s_performance_tx_rollback;
static PyObject *__pyx_n_s_performance_tx_rollback_2;
static PyObject *__pyx_kp_s_performance_value;
static PyObject *__pyx_n_s_performance_value_2;
static PyObject *__pyx_n_s_prefix;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_prop;
static PyObject *__pyx_n_s_property;
static PyObject *__pyx_n_s_propname;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_readonly;
static PyObject *__pyx_n_s_recover;
static PyObject *__pyx_n_s_reverse;
static PyObject *__pyx_n_s_rollback;
static PyObject *__pyx_kp_s_scheduler_anticache;
static PyObject *__pyx_n_s_scheduler_anticache_2;
static PyObject *__pyx_kp_s_scheduler_anticache_active;
static PyObject *__pyx_n_s_scheduler_anticache_active_2;
static PyObject *__pyx_kp_s_scheduler_anticache_asn;
static PyObject *__pyx_n_s_scheduler_anticache_asn_2;
static PyObject *__pyx_kp_s_scheduler_anticache_asn_last;
static PyObject *__pyx_n_s_scheduler_anticache_asn_last_2;
static PyObject *__pyx_kp_s_scheduler_checkpoint;
static PyObject *__pyx_n_s_scheduler_checkpoint_2;
static PyObject *__pyx_kp_s_scheduler_checkpoint_active;
static PyObject *__pyx_n_s_scheduler_checkpoint_active_2;
static PyObject *__pyx_kp_s_scheduler_checkpoint_lsn;
static PyObject *__pyx_n_s_scheduler_checkpoint_lsn_2;
static PyObject *__pyx_kp_s_scheduler_checkpoint_lsn_last;
static PyObject *__pyx_n_s_scheduler_checkpoint_lsn_last_2;
static PyObject *__pyx_kp_s_scheduler_gc;
static PyObject *__pyx_n_s_scheduler_gc_2;
static PyObject *__pyx_kp_s_scheduler_gc_active;
static PyObject *__pyx_n_s_scheduler_gc_active_2;
static PyObject *__pyx_kp_s_scheduler_lru;
static PyObject *__pyx_n_s_scheduler_lru_2;
static PyObject *__pyx_kp_s_scheduler_lru_active;
static PyObject *__pyx_n_s_scheduler_lru_active_2;
static PyObject *__pyx_kp_s_scheduler_run;
static PyObject *__pyx_n_s_scheduler_run_2;
static PyObject *__pyx_kp_s_scheduler_snapshot;
static PyObject *__pyx_n_s_scheduler_snapshot_2;
static PyObject *__pyx_kp_s_scheduler_snapshot_active;
static PyObject *__pyx_n_s_scheduler_snapshot_active_2;
static PyObject *__pyx_kp_s_scheduler_snapshot_ssn;
static PyObject *__pyx_n_s_scheduler_snapshot_ssn_2;
static PyObject *__pyx_kp_s_scheduler_snapshot_ssn_last;
static PyObject *__pyx_n_s_scheduler_snapshot_ssn_last_2;
static PyObject *__pyx_n_s_scheduler_threads;
static PyObject *__pyx_kp_s_scheduler_threads_2;
static PyObject *__pyx_kp_s_scheduler_zone;
static PyObject *__pyx_n_s_scheduler_zone_2;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_set_option;
static PyObject *__pyx_n_s_setter;
static PyObject *__pyx_n_s_sm;
static PyObject *__pyx_n_s_sm_locals__method;
static PyObject *__pyx_n_s_sophia;
static PyObject *__pyx_kp_s_sophia_build;
static PyObject *__pyx_kp_s_sophia_error;
static PyObject *__pyx_kp_s_sophia_path;
static PyObject *__pyx_n_s_sophia_path_2;
static PyObject *__pyx_kp_s_sophia_path_create;
static PyObject *__pyx_n_s_sophia_path_create_2;
static PyObject *__pyx_kp_s_sophia_recover;
static PyObject *__pyx_kp_s_sophia_version;
static PyObject *__pyx_kp_s_sophia_version_storage;
static PyObject *__pyx_n_s_sophy;
static PyObject *__pyx_n_s_sp;
static PyObject *__pyx_n_s_sp_locals__getter;
static PyObject *__pyx_n_s_sp_locals__setter;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_status;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_n_s_storage;
static PyObject *__pyx_n_s_string;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_sync;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_target;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_threads;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_transaction;
static PyObject *__pyx_kp_s_transaction_is_not_finished_wait;
static PyObject *__pyx_kp_s_transaction_was_rolled_back_by_a;
static PyObject *__pyx_n_s_u32;
static PyObject *__pyx_n_s_u32rev;
static PyObject *__pyx_n_s_u64;
static PyObject *__pyx_n_s_u64rev;
static PyObject *__pyx_kp_s_unknown_error_occurred;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_kp_s_utf_8;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_values;
static PyObject *__pyx_n_s_version;
static PyObject *__pyx_n_s_version_info;
static PyObject *__pyx_n_s_version_storage;
static PyObject *__pyx_n_s_view;
static PyObject *__pyx_kp_s_view_s;
static PyObject *__pyx_n_s_wrapper;
static PyObject *__pyx_n_s_wraps;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_pf_5sophy_3_sp__getter(PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_3_sp_2_setter(PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5sophy__sp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_propname, PyObject *__pyx_v_string, PyObject *__pyx_v_readonly); /* proto */
static PyObject *__pyx_pf_5sophy_3_sm__method(PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_2_sm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_propname); /* proto */
static PyObject *__pyx_pf_5sophy_4_dbp__getter(PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_4_dbp_2_setter(PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5sophy_4_dbp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_prop, PyObject *__pyx_v_string, PyObject *__pyx_v_readonly); /* proto */
static PyObject *__pyx_pf_5sophy_4_dbm__method(PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6_dbm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_propname); /* proto */
static int __pyx_pf_5sophy_14_ConfigManager___cinit__(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia); /* proto */
static PyObject *__pyx_pf_5sophy_14_ConfigManager_2set_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5sophy_14_ConfigManager_4get_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_5sophy_14_ConfigManager_6clear_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_5sophy_13_SophiaObject___cinit__(struct __pyx_obj_5sophy__SophiaObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__, CYTHON_UNUSED PyObject *__pyx_v___); /* proto */
static int __pyx_pf_5sophy_6Sophia___cinit__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static int __pyx_pf_5sophy_6Sophia_2__init__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, PyObject *__pyx_v_path, PyObject *__pyx_v_databases, PyObject *__pyx_v_auto_open, PyObject *__pyx_v_backup_path, PyObject *__pyx_v_memory_limit, PyObject *__pyx_v_threads, PyObject *__pyx_v_log_path); /* proto */
static void __pyx_pf_5sophy_6Sophia_4__dealloc__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_6open(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_8close(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_10__getitem__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_12create_database(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_index_type); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_3dbs___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_9auto_open___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_6config___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Sophia_4path___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self); /* proto */
static int __pyx_pf_5sophy_13_BaseDBObject___cinit__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED PyObject *__pyx_v__); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_2open(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_v_configure); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_4close(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_6__enter__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_8__exit__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_8__call___wrapper(PyObject *__pyx_self, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_10__call__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_fn); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_12__getitem__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_5sophy_13_BaseDBObject_14__setitem__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5sophy_13_BaseDBObject_16__delitem__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_5sophy_13_BaseDBObject_18__contains__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_20get_range(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_reverse); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_23keys(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_26values(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_29items(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_32__iter__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_5sophy_13_BaseDBObject_34__len__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_36cursor(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values); /* proto */
static PyObject *__pyx_pf_5sophy_13_BaseDBObject_6sophia___get__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self); /* proto */
static int __pyx_pf_5sophy_8Database___cinit__(struct __pyx_obj_5sophy_Database *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, PyObject *__pyx_v_name, PyObject *__pyx_v_index_type); /* proto */
static void __pyx_pf_5sophy_8Database_2__dealloc__(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_4close(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_6update(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v__data, PyObject *__pyx_v_k); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_8transaction(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_10view(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_12cursor(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_4name___get__(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_8Database_6config___get__(struct __pyx_obj_5sophy_Database *__pyx_v_self); /* proto */
static int __pyx_pf_5sophy_4View___cinit__(struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db, PyObject *__pyx_v_name); /* proto */
static int __pyx_pf_5sophy_4View_2__init__(struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db, CYTHON_UNUSED PyObject *__pyx_v_name); /* proto */
static void __pyx_pf_5sophy_4View_4__dealloc__(struct __pyx_obj_5sophy_View *__pyx_v_self); /* proto */
static int __pyx_pf_5sophy_4View_6__setitem__(CYTHON_UNUSED struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_key, CYTHON_UNUSED PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5sophy_4View_8__delitem__(CYTHON_UNUSED struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_5sophy_4View_10cursor(struct __pyx_obj_5sophy_View *__pyx_v_self, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values); /* proto */
static int __pyx_pf_5sophy_11Transaction___cinit__(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db); /* proto */
static void __pyx_pf_5sophy_11Transaction_2__dealloc__(struct __pyx_obj_5sophy_Transaction *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_11Transaction_4__exit__(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_5sophy_11Transaction_6commit(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, PyObject *__pyx_v_begin); /* proto */
static PyObject *__pyx_pf_5sophy_11Transaction_8rollback(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, PyObject *__pyx_v_begin); /* proto */
static int __pyx_pf_5sophy_6Cursor___cinit__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db, struct __pyx_obj_5sophy__SophiaObject *__pyx_v_target, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values); /* proto */
static void __pyx_pf_5sophy_6Cursor_2__dealloc__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_4__iter__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_6__next__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_5order___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_6prefix___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_4keys___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_6values___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_6Cursor_3key___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self); /* proto */
static int __pyx_pf_5sophy_6_Index___cinit__(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db, PyObject *__pyx_v_key, CYTHON_UNUSED PyObject *__pyx_v__); /* proto */
static int __pyx_pf_5sophy_6_Index_2__init__(struct __pyx_obj_5sophy__Index *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db, CYTHON_UNUSED PyObject *__pyx_v_key, CYTHON_UNUSED PyObject *__pyx_v__); /* proto */
static int __pyx_pf_5sophy_11_MultiIndex___cinit__(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db, CYTHON_UNUSED PyObject *__pyx_v_key, PyObject *__pyx_v_index_types, CYTHON_UNUSED PyObject *__pyx_v__); /* proto */
static int __pyx_pf_5sophy_14SimpleDatabase___init__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_filename, PyObject *__pyx_v_index_type, PyObject *__pyx_v_auto_open); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_2__getitem__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_5sophy_14SimpleDatabase_4__setitem__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5sophy_14SimpleDatabase_6__delitem__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_5sophy_14SimpleDatabase_8__contains__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_10keys(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_12values(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_14items(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_16__iter__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_5sophy_14SimpleDatabase_18__len__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_20cursor(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_22update(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v__data, PyObject *__pyx_v_k); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_24transaction(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_26view(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5sophy_14SimpleDatabase_7db_name___get__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5sophy_8connect(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data_dir, PyObject *__pyx_v_db_name, PyObject *__pyx_v_index_type); /* proto */
static PyObject *__pyx_tp_new_5sophy__ConfigManager(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__SophiaObject(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy_Sophia(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__BaseDBObject(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy_Database(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy_View(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy_Transaction(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy_Cursor(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__Index(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__UInt32Index(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__UInt32RevIndex(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__UInt64Index(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__UInt64RevIndex(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy__MultiIndex(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy_SimpleDatabase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct___sp(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_1__sm(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_2__dbp(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_3__dbm(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_4___call__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_5_get_range(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_6_keys(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_7_values(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_8_items(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items = {0, &__pyx_n_s_items, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values = {0, &__pyx_n_s_values, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_3;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__91;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__93;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__95;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__97;
static PyObject *__pyx_tuple__98;
static PyObject *__pyx_tuple__99;
static PyObject *__pyx_codeobj__4;
static PyObject *__pyx_codeobj__6;
static PyObject *__pyx_codeobj__8;
static PyObject *__pyx_tuple__100;
static PyObject *__pyx_tuple__101;
static PyObject *__pyx_tuple__102;
static PyObject *__pyx_tuple__103;
static PyObject *__pyx_tuple__104;
static PyObject *__pyx_tuple__105;
static PyObject *__pyx_tuple__106;
static PyObject *__pyx_tuple__107;
static PyObject *__pyx_tuple__108;
static PyObject *__pyx_tuple__109;
static PyObject *__pyx_tuple__110;
static PyObject *__pyx_tuple__111;
static PyObject *__pyx_tuple__112;
static PyObject *__pyx_tuple__113;
static PyObject *__pyx_tuple__114;
static PyObject *__pyx_tuple__115;
static PyObject *__pyx_tuple__116;
static PyObject *__pyx_tuple__117;
static PyObject *__pyx_tuple__118;
static PyObject *__pyx_tuple__119;
static PyObject *__pyx_tuple__120;
static PyObject *__pyx_tuple__121;
static PyObject *__pyx_tuple__122;
static PyObject *__pyx_tuple__123;
static PyObject *__pyx_tuple__124;
static PyObject *__pyx_tuple__125;
static PyObject *__pyx_tuple__126;
static PyObject *__pyx_tuple__127;
static PyObject *__pyx_tuple__128;
static PyObject *__pyx_tuple__129;
static PyObject *__pyx_tuple__130;
static PyObject *__pyx_tuple__131;
static PyObject *__pyx_tuple__132;
static PyObject *__pyx_tuple__133;
static PyObject *__pyx_tuple__134;
static PyObject *__pyx_tuple__135;
static PyObject *__pyx_tuple__136;
static PyObject *__pyx_tuple__137;
static PyObject *__pyx_tuple__138;
static PyObject *__pyx_tuple__139;
static PyObject *__pyx_tuple__140;
static PyObject *__pyx_tuple__141;
static PyObject *__pyx_tuple__142;
static PyObject *__pyx_tuple__143;
static PyObject *__pyx_tuple__144;
static PyObject *__pyx_tuple__145;
static PyObject *__pyx_tuple__146;
static PyObject *__pyx_tuple__147;
static PyObject *__pyx_tuple__148;
static PyObject *__pyx_tuple__149;
static PyObject *__pyx_tuple__150;
static PyObject *__pyx_tuple__151;
static PyObject *__pyx_tuple__152;
static PyObject *__pyx_tuple__153;
static PyObject *__pyx_tuple__154;
static PyObject *__pyx_tuple__155;
static PyObject *__pyx_tuple__156;
static PyObject *__pyx_tuple__157;
static PyObject *__pyx_tuple__158;
static PyObject *__pyx_tuple__159;
static PyObject *__pyx_tuple__160;
static PyObject *__pyx_tuple__161;
static PyObject *__pyx_tuple__162;
static PyObject *__pyx_tuple__163;
static PyObject *__pyx_tuple__164;
static PyObject *__pyx_tuple__165;
static PyObject *__pyx_tuple__166;
static PyObject *__pyx_tuple__167;
static PyObject *__pyx_tuple__168;
static PyObject *__pyx_tuple__169;
static PyObject *__pyx_tuple__170;
static PyObject *__pyx_tuple__171;
static PyObject *__pyx_tuple__172;
static PyObject *__pyx_tuple__173;
static PyObject *__pyx_codeobj__10;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__174;

/* "sophy.pyx":43
 * cdef bint IS_PY3K = sys.version_info[0] == 3
 * 
 * cdef bytes encode(obj):             # <<<<<<<<<<<<<<
 *     if isinstance(obj, unicode):
 *         return obj.encode('utf-8')
 */

static PyObject *__pyx_f_5sophy_encode(PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode", 0);

  /* "sophy.pyx":44
 * 
 * cdef bytes encode(obj):
 *     if isinstance(obj, unicode):             # <<<<<<<<<<<<<<
 *         return obj.encode('utf-8')
 *     elif isinstance(obj, bytes):
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_obj); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":45
 * cdef bytes encode(obj):
 *     if isinstance(obj, unicode):
 *         return obj.encode('utf-8')             # <<<<<<<<<<<<<<
 *     elif isinstance(obj, bytes):
 *         return obj
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":44
 * 
 * cdef bytes encode(obj):
 *     if isinstance(obj, unicode):             # <<<<<<<<<<<<<<
 *         return obj.encode('utf-8')
 *     elif isinstance(obj, bytes):
 */
  }

  /* "sophy.pyx":46
 *     if isinstance(obj, unicode):
 *         return obj.encode('utf-8')
 *     elif isinstance(obj, bytes):             # <<<<<<<<<<<<<<
 *         return obj
 *     elif obj is None:
 */
  __pyx_t_2 = PyBytes_Check(__pyx_v_obj); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":47
 *         return obj.encode('utf-8')
 *     elif isinstance(obj, bytes):
 *         return obj             # <<<<<<<<<<<<<<
 *     elif obj is None:
 *         return obj
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyBytes_CheckExact(__pyx_v_obj))||((__pyx_v_obj) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_obj)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_INCREF(__pyx_v_obj);
    __pyx_r = ((PyObject*)__pyx_v_obj);
    goto __pyx_L0;

    /* "sophy.pyx":46
 *     if isinstance(obj, unicode):
 *         return obj.encode('utf-8')
 *     elif isinstance(obj, bytes):             # <<<<<<<<<<<<<<
 *         return obj
 *     elif obj is None:
 */
  }

  /* "sophy.pyx":48
 *     elif isinstance(obj, bytes):
 *         return obj
 *     elif obj is None:             # <<<<<<<<<<<<<<
 *         return obj
 *     elif IS_PY3K:
 */
  __pyx_t_1 = (__pyx_v_obj == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":49
 *         return obj
 *     elif obj is None:
 *         return obj             # <<<<<<<<<<<<<<
 *     elif IS_PY3K:
 *         return bytes(str(obj), 'utf-8')
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyBytes_CheckExact(__pyx_v_obj))||((__pyx_v_obj) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_obj)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_INCREF(__pyx_v_obj);
    __pyx_r = ((PyObject*)__pyx_v_obj);
    goto __pyx_L0;

    /* "sophy.pyx":48
 *     elif isinstance(obj, bytes):
 *         return obj
 *     elif obj is None:             # <<<<<<<<<<<<<<
 *         return obj
 *     elif IS_PY3K:
 */
  }

  /* "sophy.pyx":50
 *     elif obj is None:
 *         return obj
 *     elif IS_PY3K:             # <<<<<<<<<<<<<<
 *         return bytes(str(obj), 'utf-8')
 *     return bytes(obj)
 */
  __pyx_t_2 = (__pyx_v_5sophy_IS_PY3K != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":51
 *         return obj
 *     elif IS_PY3K:
 *         return bytes(str(obj), 'utf-8')             # <<<<<<<<<<<<<<
 *     return bytes(obj)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_obj);
    __Pyx_GIVEREF(__pyx_v_obj);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_obj);
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_INCREF(__pyx_kp_s_utf_8);
    __Pyx_GIVEREF(__pyx_kp_s_utf_8);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_kp_s_utf_8);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)(&PyBytes_Type)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":50
 *     elif obj is None:
 *         return obj
 *     elif IS_PY3K:             # <<<<<<<<<<<<<<
 *         return bytes(str(obj), 'utf-8')
 *     return bytes(obj)
 */
  }

  /* "sophy.pyx":52
 *     elif IS_PY3K:
 *         return bytes(str(obj), 'utf-8')
 *     return bytes(obj)             # <<<<<<<<<<<<<<
 * 
 * cdef inline _getstring(void *obj, const char *key):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_obj);
  __Pyx_GIVEREF(__pyx_v_obj);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_obj);
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)(&PyBytes_Type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":43
 * cdef bint IS_PY3K = sys.version_info[0] == 3
 * 
 * cdef bytes encode(obj):             # <<<<<<<<<<<<<<
 *     if isinstance(obj, unicode):
 *         return obj.encode('utf-8')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sophy.encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":54
 *     return bytes(obj)
 * 
 * cdef inline _getstring(void *obj, const char *key):             # <<<<<<<<<<<<<<
 *     cdef:
 *         char *buf
 */

static CYTHON_INLINE PyObject *__pyx_f_5sophy__getstring(void *__pyx_v_obj, char const *__pyx_v_key) {
  char *__pyx_v_buf;
  int __pyx_v_nlen;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_getstring", 0);

  /* "sophy.pyx":59
 *         int nlen
 * 
 *     buf = <char *>sp_getstring(obj, key, &nlen)             # <<<<<<<<<<<<<<
 *     if buf:
 *         value = buf[:nlen - 1]
 */
  __pyx_v_buf = ((char *)sp_getstring(__pyx_v_obj, __pyx_v_key, (&__pyx_v_nlen)));

  /* "sophy.pyx":60
 * 
 *     buf = <char *>sp_getstring(obj, key, &nlen)
 *     if buf:             # <<<<<<<<<<<<<<
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:
 */
  __pyx_t_1 = (__pyx_v_buf != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":61
 *     buf = <char *>sp_getstring(obj, key, &nlen)
 *     if buf:
 *         value = buf[:nlen - 1]             # <<<<<<<<<<<<<<
 *         if IS_PY3K:
 *             try:
 */
    __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_buf + 0, (__pyx_v_nlen - 1) - 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_value = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "sophy.pyx":62
 *     if buf:
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:             # <<<<<<<<<<<<<<
 *             try:
 *                 return value.decode('utf-8')
 */
    __pyx_t_1 = (__pyx_v_5sophy_IS_PY3K != 0);
    if (__pyx_t_1) {

      /* "sophy.pyx":63
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:
 *             try:             # <<<<<<<<<<<<<<
 *                 return value.decode('utf-8')
 *             except UnicodeDecodeError:
 */
      {
        __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_5);
        /*try:*/ {

          /* "sophy.pyx":64
 *         if IS_PY3K:
 *             try:
 *                 return value.decode('utf-8')             # <<<<<<<<<<<<<<
 *             except UnicodeDecodeError:
 *                 pass
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = __Pyx_decode_bytes(__pyx_v_value, 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L5_error;}
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L9_try_return;

          /* "sophy.pyx":63
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:
 *             try:             # <<<<<<<<<<<<<<
 *                 return value.decode('utf-8')
 *             except UnicodeDecodeError:
 */
        }
        __pyx_L5_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sophy.pyx":65
 *             try:
 *                 return value.decode('utf-8')
 *             except UnicodeDecodeError:             # <<<<<<<<<<<<<<
 *                 pass
 *         return value
 */
        __pyx_t_6 = PyErr_ExceptionMatches(__pyx_builtin_UnicodeDecodeError);
        if (__pyx_t_6) {
          PyErr_Restore(0,0,0);
          goto __pyx_L6_exception_handled;
        }
        goto __pyx_L7_except_error;
        __pyx_L7_except_error:;

        /* "sophy.pyx":63
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:
 *             try:             # <<<<<<<<<<<<<<
 *                 return value.decode('utf-8')
 *             except UnicodeDecodeError:
 */
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
        goto __pyx_L1_error;
        __pyx_L9_try_return:;
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
        goto __pyx_L0;
        __pyx_L6_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      }

      /* "sophy.pyx":62
 *     if buf:
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:             # <<<<<<<<<<<<<<
 *             try:
 *                 return value.decode('utf-8')
 */
    }

    /* "sophy.pyx":67
 *             except UnicodeDecodeError:
 *                 pass
 *         return value             # <<<<<<<<<<<<<<
 * 
 * cdef inline _check(void *env, int rc):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "sophy.pyx":60
 * 
 *     buf = <char *>sp_getstring(obj, key, &nlen)
 *     if buf:             # <<<<<<<<<<<<<<
 *         value = buf[:nlen - 1]
 *         if IS_PY3K:
 */
  }

  /* "sophy.pyx":54
 *     return bytes(obj)
 * 
 * cdef inline _getstring(void *obj, const char *key):             # <<<<<<<<<<<<<<
 *     cdef:
 *         char *buf
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy._getstring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":69
 *         return value
 * 
 * cdef inline _check(void *env, int rc):             # <<<<<<<<<<<<<<
 *     if rc == -1:
 *         error = _getstring(env, 'sophia.error')
 */

static CYTHON_INLINE PyObject *__pyx_f_5sophy__check(void *__pyx_v_env, int __pyx_v_rc) {
  PyObject *__pyx_v_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_check", 0);

  /* "sophy.pyx":70
 * 
 * cdef inline _check(void *env, int rc):
 *     if rc == -1:             # <<<<<<<<<<<<<<
 *         error = _getstring(env, 'sophia.error')
 *         if error:
 */
  __pyx_t_1 = ((__pyx_v_rc == -1L) != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":71
 * cdef inline _check(void *env, int rc):
 *     if rc == -1:
 *         error = _getstring(env, 'sophia.error')             # <<<<<<<<<<<<<<
 *         if error:
 *             raise Exception(error)
 */
    __pyx_t_2 = __pyx_f_5sophy__getstring(__pyx_v_env, __pyx_k_sophia_error); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_error = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "sophy.pyx":72
 *     if rc == -1:
 *         error = _getstring(env, 'sophia.error')
 *         if error:             # <<<<<<<<<<<<<<
 *             raise Exception(error)
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_error); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 72; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_1) {

      /* "sophy.pyx":73
 *         error = _getstring(env, 'sophia.error')
 *         if error:
 *             raise Exception(error)             # <<<<<<<<<<<<<<
 *         else:
 *             raise Exception('unknown error occurred.')
 */
      __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_v_error);
      __Pyx_GIVEREF(__pyx_v_error);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_error);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "sophy.pyx":72
 *     if rc == -1:
 *         error = _getstring(env, 'sophia.error')
 *         if error:             # <<<<<<<<<<<<<<
 *             raise Exception(error)
 *         else:
 */
    }

    /* "sophy.pyx":75
 *             raise Exception(error)
 *         else:
 *             raise Exception('unknown error occurred.')             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 75; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 75; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "sophy.pyx":70
 * 
 * cdef inline _check(void *env, int rc):
 *     if rc == -1:             # <<<<<<<<<<<<<<
 *         error = _getstring(env, 'sophia.error')
 *         if error:
 */
  }

  /* "sophy.pyx":69
 *         return value
 * 
 * cdef inline _check(void *env, int rc):             # <<<<<<<<<<<<<<
 *     if rc == -1:
 *         error = _getstring(env, 'sophia.error')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._check", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":78
 * 
 * 
 * def _sp(propname, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_1_sp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_1_sp = {"_sp", (PyCFunction)__pyx_pw_5sophy_1_sp, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5sophy_1_sp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_propname = 0;
  PyObject *__pyx_v_string = 0;
  PyObject *__pyx_v_readonly = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_propname,&__pyx_n_s_string,&__pyx_n_s_readonly,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_False);
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_propname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_string);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_readonly);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_sp") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_propname = values[0];
    __pyx_v_string = values[1];
    __pyx_v_readonly = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_sp", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._sp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy__sp(__pyx_self, __pyx_v_propname, __pyx_v_string, __pyx_v_readonly);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":79
 * 
 * def _sp(propname, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option(propname, string)
 *     if readonly:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_3_sp_1_getter(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_3_sp_1_getter = {"_getter", (PyCFunction)__pyx_pw_5sophy_3_sp_1_getter, METH_O, 0};
static PyObject *__pyx_pw_5sophy_3_sp_1_getter(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_getter (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_3_sp__getter(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_3_sp__getter(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_getter", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct___sp *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":80
 * def _sp(propname, string=False, readonly=False):
 *     def _getter(self):
 *         return self.config.get_option(propname, string)             # <<<<<<<<<<<<<<
 *     if readonly:
 *         return property(_getter)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get_option); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_cur_scope->__pyx_v_propname)) { __Pyx_RaiseClosureNameError("propname"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  if (unlikely(!__pyx_cur_scope->__pyx_v_string)) { __Pyx_RaiseClosureNameError("string"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_2) {
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
  }
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_cur_scope->__pyx_v_propname);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_string);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_string);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_cur_scope->__pyx_v_string);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":79
 * 
 * def _sp(propname, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option(propname, string)
 *     if readonly:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy._sp._getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":84
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option(propname, value)
 *         return property(_getter, _setter)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_3_sp_3_setter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_3_sp_3_setter = {"_setter", (PyCFunction)__pyx_pw_5sophy_3_sp_3_setter, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5sophy_3_sp_3_setter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_setter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_setter", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_setter") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_setter", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._sp._setter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_3_sp_2_setter(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_3_sp_2_setter(PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_setter", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct___sp *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":85
 *     else:
 *         def _setter(self, value):
 *             self.config.set_option(propname, value)             # <<<<<<<<<<<<<<
 *         return property(_getter, _setter)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_set_option); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_cur_scope->__pyx_v_propname)) { __Pyx_RaiseClosureNameError("propname"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_2) {
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
  }
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_cur_scope->__pyx_v_propname);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_value);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":84
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option(propname, value)
 *         return property(_getter, _setter)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy._sp._setter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":78
 * 
 * 
 * def _sp(propname, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 */

static PyObject *__pyx_pf_5sophy__sp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_propname, PyObject *__pyx_v_string, PyObject *__pyx_v_readonly) {
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *__pyx_cur_scope;
  PyObject *__pyx_v__getter = 0;
  PyObject *__pyx_v__setter = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sp", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct___sp *)__pyx_tp_new_5sophy___pyx_scope_struct___sp(__pyx_ptype_5sophy___pyx_scope_struct___sp, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_propname = __pyx_v_propname;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);
  __pyx_cur_scope->__pyx_v_string = __pyx_v_string;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_string);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_string);

  /* "sophy.pyx":79
 * 
 * def _sp(propname, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option(propname, string)
 *     if readonly:
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_3_sp_1_getter, 0, __pyx_n_s_sp_locals__getter, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__4)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__getter = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":81
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 *     if readonly:             # <<<<<<<<<<<<<<
 *         return property(_getter)
 *     else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_readonly); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 81; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "sophy.pyx":82
 *         return self.config.get_option(propname, string)
 *     if readonly:
 *         return property(_getter)             # <<<<<<<<<<<<<<
 *     else:
 *         def _setter(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 82; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__getter);
    __Pyx_GIVEREF(__pyx_v__getter);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__getter);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 82; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":81
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 *     if readonly:             # <<<<<<<<<<<<<<
 *         return property(_getter)
 *     else:
 */
  }

  /* "sophy.pyx":84
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option(propname, value)
 *         return property(_getter, _setter)
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_3_sp_3_setter, 0, __pyx_n_s_sp_locals__setter, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__6)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v__setter = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "sophy.pyx":86
 *         def _setter(self, value):
 *             self.config.set_option(propname, value)
 *         return property(_getter, _setter)             # <<<<<<<<<<<<<<
 * 
 * def _sm(propname):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 86; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__getter);
    __Pyx_GIVEREF(__pyx_v__getter);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__getter);
    __Pyx_INCREF(__pyx_v__setter);
    __Pyx_GIVEREF(__pyx_v__setter);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v__setter);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 86; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "sophy.pyx":78
 * 
 * 
 * def _sp(propname, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._sp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__getter);
  __Pyx_XDECREF(__pyx_v__setter);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":88
 *         return property(_getter, _setter)
 * 
 * def _sm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_3_sm(PyObject *__pyx_self, PyObject *__pyx_v_propname); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_3_sm = {"_sm", (PyCFunction)__pyx_pw_5sophy_3_sm, METH_O, 0};
static PyObject *__pyx_pw_5sophy_3_sm(PyObject *__pyx_self, PyObject *__pyx_v_propname) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sm (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_2_sm(__pyx_self, ((PyObject *)__pyx_v_propname));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":89
 * 
 * def _sm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option(propname, 0)
 *     return _method
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_3_sm_1_method(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_3_sm_1_method = {"_method", (PyCFunction)__pyx_pw_5sophy_3_sm_1_method, METH_O, 0};
static PyObject *__pyx_pw_5sophy_3_sm_1_method(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_method (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_3_sm__method(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_3_sm__method(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_method", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":90
 * def _sm(propname):
 *     def _method(self):
 *         self.config.set_option(propname, 0)             # <<<<<<<<<<<<<<
 *     return _method
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_set_option); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_cur_scope->__pyx_v_propname)) { __Pyx_RaiseClosureNameError("propname"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_2) {
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
  }
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_cur_scope->__pyx_v_propname);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_int_0);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":89
 * 
 * def _sm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option(propname, 0)
 *     return _method
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy._sm._method", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":88
 *         return property(_getter, _setter)
 * 
 * def _sm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 */

static PyObject *__pyx_pf_5sophy_2_sm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_propname) {
  struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *__pyx_cur_scope;
  PyObject *__pyx_v__method = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sm", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *)__pyx_tp_new_5sophy___pyx_scope_struct_1__sm(__pyx_ptype_5sophy___pyx_scope_struct_1__sm, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_propname = __pyx_v_propname;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);

  /* "sophy.pyx":89
 * 
 * def _sm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option(propname, 0)
 *     return _method
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_3_sm_1_method, 0, __pyx_n_s_sm_locals__method, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__8)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__method = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":91
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 *     return _method             # <<<<<<<<<<<<<<
 * 
 * def _dbp(prop, string=False, readonly=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v__method);
  __pyx_r = __pyx_v__method;
  goto __pyx_L0;

  /* "sophy.pyx":88
 *         return property(_getter, _setter)
 * 
 * def _sm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._sm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__method);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":93
 *     return _method
 * 
 * def _dbp(prop, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_5_dbp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_5_dbp = {"_dbp", (PyCFunction)__pyx_pw_5sophy_5_dbp, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5sophy_5_dbp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_prop = 0;
  PyObject *__pyx_v_string = 0;
  PyObject *__pyx_v_readonly = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_dbp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_prop,&__pyx_n_s_string,&__pyx_n_s_readonly,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_False);
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prop)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_string);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_readonly);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_dbp") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_prop = values[0];
    __pyx_v_string = values[1];
    __pyx_v_readonly = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_dbp", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._dbp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_4_dbp(__pyx_self, __pyx_v_prop, __pyx_v_string, __pyx_v_readonly);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":94
 * 
 * def _dbp(prop, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_4_dbp_1_getter(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_4_dbp_1_getter = {"_getter", (PyCFunction)__pyx_pw_5sophy_4_dbp_1_getter, METH_O, 0};
static PyObject *__pyx_pw_5sophy_4_dbp_1_getter(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_getter (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_4_dbp__getter(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_4_dbp__getter(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_getter", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":95
 * def _dbp(prop, string=False, readonly=False):
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)             # <<<<<<<<<<<<<<
 *     if readonly:
 *         return property(_getter)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get_option); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(!__pyx_cur_scope->__pyx_v_prop)) { __Pyx_RaiseClosureNameError("prop"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_prop);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_prop);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_cur_scope->__pyx_v_prop);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_db_s_s, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_cur_scope->__pyx_v_string)) { __Pyx_RaiseClosureNameError("string"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_2);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_string);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_string);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_cur_scope->__pyx_v_string);
  __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":94
 * 
 * def _dbp(prop, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy._dbp._getter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":99
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option('db.%s.%s' % (self.name, prop), value)
 *         return property(_getter, _setter)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_4_dbp_3_setter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_4_dbp_3_setter = {"_setter", (PyCFunction)__pyx_pw_5sophy_4_dbp_3_setter, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5sophy_4_dbp_3_setter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_setter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_setter", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_setter") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_setter", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._dbp._setter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_4_dbp_2_setter(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_4_dbp_2_setter(PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_setter", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":100
 *     else:
 *         def _setter(self, value):
 *             self.config.set_option('db.%s.%s' % (self.name, prop), value)             # <<<<<<<<<<<<<<
 *         return property(_getter, _setter)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_set_option); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(!__pyx_cur_scope->__pyx_v_prop)) { __Pyx_RaiseClosureNameError("prop"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_prop);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_prop);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_cur_scope->__pyx_v_prop);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_db_s_s, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_value);
  __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":99
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option('db.%s.%s' % (self.name, prop), value)
 *         return property(_getter, _setter)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy._dbp._setter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":93
 *     return _method
 * 
 * def _dbp(prop, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 */

static PyObject *__pyx_pf_5sophy_4_dbp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_prop, PyObject *__pyx_v_string, PyObject *__pyx_v_readonly) {
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *__pyx_cur_scope;
  PyObject *__pyx_v__getter = 0;
  PyObject *__pyx_v__setter = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_dbp", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *)__pyx_tp_new_5sophy___pyx_scope_struct_2__dbp(__pyx_ptype_5sophy___pyx_scope_struct_2__dbp, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_prop = __pyx_v_prop;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_prop);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_prop);
  __pyx_cur_scope->__pyx_v_string = __pyx_v_string;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_string);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_string);

  /* "sophy.pyx":94
 * 
 * def _dbp(prop, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_4_dbp_1_getter, 0, __pyx_n_s_dbp_locals__getter, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__10)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__getter = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":96
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:             # <<<<<<<<<<<<<<
 *         return property(_getter)
 *     else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_readonly); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "sophy.pyx":97
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:
 *         return property(_getter)             # <<<<<<<<<<<<<<
 *     else:
 *         def _setter(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__getter);
    __Pyx_GIVEREF(__pyx_v__getter);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__getter);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":96
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:             # <<<<<<<<<<<<<<
 *         return property(_getter)
 *     else:
 */
  }

  /* "sophy.pyx":99
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option('db.%s.%s' % (self.name, prop), value)
 *         return property(_getter, _setter)
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_4_dbp_3_setter, 0, __pyx_n_s_dbp_locals__setter, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v__setter = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "sophy.pyx":101
 *         def _setter(self, value):
 *             self.config.set_option('db.%s.%s' % (self.name, prop), value)
 *         return property(_getter, _setter)             # <<<<<<<<<<<<<<
 * 
 * def _dbm(propname):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__getter);
    __Pyx_GIVEREF(__pyx_v__getter);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__getter);
    __Pyx_INCREF(__pyx_v__setter);
    __Pyx_GIVEREF(__pyx_v__setter);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v__setter);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "sophy.pyx":93
 *     return _method
 * 
 * def _dbp(prop, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._dbp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__getter);
  __Pyx_XDECREF(__pyx_v__setter);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":103
 *         return property(_getter, _setter)
 * 
 * def _dbm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_7_dbm(PyObject *__pyx_self, PyObject *__pyx_v_propname); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_7_dbm = {"_dbm", (PyCFunction)__pyx_pw_5sophy_7_dbm, METH_O, 0};
static PyObject *__pyx_pw_5sophy_7_dbm(PyObject *__pyx_self, PyObject *__pyx_v_propname) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_dbm (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6_dbm(__pyx_self, ((PyObject *)__pyx_v_propname));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":104
 * 
 * def _dbm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 *     return _method
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_4_dbm_1_method(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_4_dbm_1_method = {"_method", (PyCFunction)__pyx_pw_5sophy_4_dbm_1_method, METH_O, 0};
static PyObject *__pyx_pw_5sophy_4_dbm_1_method(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_method (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_4_dbm__method(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_4_dbm__method(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_method", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":105
 * def _dbm(propname):
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)             # <<<<<<<<<<<<<<
 *     return _method
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_set_option); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(!__pyx_cur_scope->__pyx_v_propname)) { __Pyx_RaiseClosureNameError("propname"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_cur_scope->__pyx_v_propname);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_db_s_s, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_0);
  __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":104
 * 
 * def _dbm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 *     return _method
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy._dbm._method", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":103
 *         return property(_getter, _setter)
 * 
 * def _dbm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 */

static PyObject *__pyx_pf_5sophy_6_dbm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_propname) {
  struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *__pyx_cur_scope;
  PyObject *__pyx_v__method = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_dbm", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *)__pyx_tp_new_5sophy___pyx_scope_struct_3__dbm(__pyx_ptype_5sophy___pyx_scope_struct_3__dbm, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_propname = __pyx_v_propname;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_propname);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_propname);

  /* "sophy.pyx":104
 * 
 * def _dbm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 *     return _method
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_4_dbm_1_method, 0, __pyx_n_s_dbm_locals__method, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 104; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__method = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":106
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 *     return _method             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v__method);
  __pyx_r = __pyx_v__method;
  goto __pyx_L0;

  /* "sophy.pyx":103
 *         return property(_getter, _setter)
 * 
 * def _dbm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._dbm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__method);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":115
 *         void *handle
 * 
 *     def __cinit__(self, Sophia sophia):             # <<<<<<<<<<<<<<
 *         self._config = {}
 *         self.sophia = sophia
 */

/* Python wrapper */
static int __pyx_pw_5sophy_14_ConfigManager_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_14_ConfigManager_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._ConfigManager.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_14_ConfigManager___cinit__(((struct __pyx_obj_5sophy__ConfigManager *)__pyx_v_self), __pyx_v_sophia);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_14_ConfigManager___cinit__(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":116
 * 
 *     def __cinit__(self, Sophia sophia):
 *         self._config = {}             # <<<<<<<<<<<<<<
 *         self.sophia = sophia
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_config);
  __Pyx_DECREF(__pyx_v_self->_config);
  __pyx_v_self->_config = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":117
 *     def __cinit__(self, Sophia sophia):
 *         self._config = {}
 *         self.sophia = sophia             # <<<<<<<<<<<<<<
 * 
 *     cpdef set_option(self, key, value):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GOTREF(__pyx_v_self->sophia);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->sophia));
  __pyx_v_self->sophia = __pyx_v_sophia;

  /* "sophy.pyx":115
 *         void *handle
 * 
 *     def __cinit__(self, Sophia sophia):             # <<<<<<<<<<<<<<
 *         self._config = {}
 *         self.sophia = sophia
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._ConfigManager.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":119
 *         self.sophia = sophia
 * 
 *     cpdef set_option(self, key, value):             # <<<<<<<<<<<<<<
 *         self._config[key] = value
 *         if self.sophia.is_open:
 */

static PyObject *__pyx_pw_5sophy_14_ConfigManager_3set_option(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_set_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_option", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_set_option); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_14_ConfigManager_3set_option)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_key);
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_value);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":120
 * 
 *     cpdef set_option(self, key, value):
 *         self._config[key] = value             # <<<<<<<<<<<<<<
 *         if self.sophia.is_open:
 *             self.apply_config(key, value)
 */
  if (unlikely(__pyx_v_self->_config == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  if (unlikely(PyDict_SetItem(__pyx_v_self->_config, __pyx_v_key, __pyx_v_value) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":121
 *     cpdef set_option(self, key, value):
 *         self._config[key] = value
 *         if self.sophia.is_open:             # <<<<<<<<<<<<<<
 *             self.apply_config(key, value)
 * 
 */
  __pyx_t_7 = (__pyx_v_self->sophia->is_open != 0);
  if (__pyx_t_7) {

    /* "sophy.pyx":122
 *         self._config[key] = value
 *         if self.sophia.is_open:
 *             self.apply_config(key, value)             # <<<<<<<<<<<<<<
 * 
 *     cpdef get_option(self, key, string=True):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__ConfigManager *)__pyx_v_self->__pyx_vtab)->apply_config(__pyx_v_self, __pyx_v_key, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 122; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sophy.pyx":121
 *     cpdef set_option(self, key, value):
 *         self._config[key] = value
 *         if self.sophia.is_open:             # <<<<<<<<<<<<<<
 *             self.apply_config(key, value)
 * 
 */
  }

  /* "sophy.pyx":119
 *         self.sophia = sophia
 * 
 *     cpdef set_option(self, key, value):             # <<<<<<<<<<<<<<
 *         self._config[key] = value
 *         if self.sophia.is_open:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy._ConfigManager.set_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14_ConfigManager_3set_option(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_14_ConfigManager_3set_option(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_option (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_option", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_option") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_key = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_option", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._ConfigManager.set_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_14_ConfigManager_2set_option(((struct __pyx_obj_5sophy__ConfigManager *)__pyx_v_self), __pyx_v_key, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14_ConfigManager_2set_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_option", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5sophy_14_ConfigManager_set_option(__pyx_v_self, __pyx_v_key, __pyx_v_value, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._ConfigManager.set_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":124
 *             self.apply_config(key, value)
 * 
 *     cpdef get_option(self, key, string=True):             # <<<<<<<<<<<<<<
 *         if string:
 *             return _getstring(self.sophia.handle, key)
 */

static PyObject *__pyx_pw_5sophy_14_ConfigManager_5get_option(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_get_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_14_ConfigManager_get_option *__pyx_optional_args) {
  PyObject *__pyx_v_string = ((PyObject *)Py_True);
  PyObject *__pyx_v_bkey = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  char const *__pyx_t_8;
  char const *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_option", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_string = __pyx_optional_args->string;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_option); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_14_ConfigManager_5get_option)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_key);
      __Pyx_INCREF(__pyx_v_string);
      __Pyx_GIVEREF(__pyx_v_string);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_string);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":125
 * 
 *     cpdef get_option(self, key, string=True):
 *         if string:             # <<<<<<<<<<<<<<
 *             return _getstring(self.sophia.handle, key)
 *         else:
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_string); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_7) {

    /* "sophy.pyx":126
 *     cpdef get_option(self, key, string=True):
 *         if string:
 *             return _getstring(self.sophia.handle, key)             # <<<<<<<<<<<<<<
 *         else:
 *             bkey = encode(key)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_8 = __Pyx_PyObject_AsString(__pyx_v_key); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 126; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = __pyx_f_5sophy__getstring(__pyx_v_self->sophia->__pyx_base.handle, __pyx_t_8); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 126; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":125
 * 
 *     cpdef get_option(self, key, string=True):
 *         if string:             # <<<<<<<<<<<<<<
 *             return _getstring(self.sophia.handle, key)
 *         else:
 */
  }

  /* "sophy.pyx":128
 *             return _getstring(self.sophia.handle, key)
 *         else:
 *             bkey = encode(key)             # <<<<<<<<<<<<<<
 *             return sp_getint(self.sophia.handle, <const char *>bkey)
 * 
 */
  /*else*/ {
    __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_bkey = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "sophy.pyx":129
 *         else:
 *             bkey = encode(key)
 *             return sp_getint(self.sophia.handle, <const char *>bkey)             # <<<<<<<<<<<<<<
 * 
 *     cpdef clear_option(self, key):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_9 = __Pyx_PyObject_AsString(__pyx_v_bkey); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = __Pyx_PyInt_From_int64_t(sp_getint(__pyx_v_self->sophia->__pyx_base.handle, ((char const *)__pyx_t_9))); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "sophy.pyx":124
 *             self.apply_config(key, value)
 * 
 *     cpdef get_option(self, key, string=True):             # <<<<<<<<<<<<<<
 *         if string:
 *             return _getstring(self.sophia.handle, key)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy._ConfigManager.get_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bkey);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14_ConfigManager_5get_option(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_14_ConfigManager_5get_option(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_string = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_option (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_string,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_string);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_option") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = values[0];
    __pyx_v_string = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_option", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._ConfigManager.get_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_14_ConfigManager_4get_option(((struct __pyx_obj_5sophy__ConfigManager *)__pyx_v_self), __pyx_v_key, __pyx_v_string);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14_ConfigManager_4get_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_14_ConfigManager_get_option __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_option", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.string = __pyx_v_string;
  __pyx_t_1 = __pyx_vtabptr_5sophy__ConfigManager->get_option(__pyx_v_self, __pyx_v_key, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._ConfigManager.get_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":131
 *             return sp_getint(self.sophia.handle, <const char *>bkey)
 * 
 *     cpdef clear_option(self, key):             # <<<<<<<<<<<<<<
 *         del self._config[key]
 * 
 */

static PyObject *__pyx_pw_5sophy_14_ConfigManager_7clear_option(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_f_5sophy_14_ConfigManager_clear_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clear_option", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clear_option); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_14_ConfigManager_7clear_option)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_key); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_key);
        __Pyx_GIVEREF(__pyx_v_key);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_key);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":132
 * 
 *     cpdef clear_option(self, key):
 *         del self._config[key]             # <<<<<<<<<<<<<<
 * 
 *     cdef apply_config(self, key, value):
 */
  if (unlikely(__pyx_v_self->_config == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  if (PyDict_DelItem(__pyx_v_self->_config, __pyx_v_key) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":131
 *             return sp_getint(self.sophia.handle, <const char *>bkey)
 * 
 *     cpdef clear_option(self, key):             # <<<<<<<<<<<<<<
 *         del self._config[key]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy._ConfigManager.clear_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14_ConfigManager_7clear_option(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_5sophy_14_ConfigManager_7clear_option(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clear_option (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14_ConfigManager_6clear_option(((struct __pyx_obj_5sophy__ConfigManager *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14_ConfigManager_6clear_option(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clear_option", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5sophy_14_ConfigManager_clear_option(__pyx_v_self, __pyx_v_key, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._ConfigManager.clear_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":134
 *         del self._config[key]
 * 
 *     cdef apply_config(self, key, value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes bkey = encode(key)
 */

static PyObject *__pyx_f_5sophy_14_ConfigManager_apply_config(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_bkey = 0;
  int __pyx_v_rc;
  PyObject *__pyx_v_bvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  char const *__pyx_t_5;
  int __pyx_t_6;
  char const *__pyx_t_7;
  char const *__pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_config", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "sophy.pyx":136
 *     cdef apply_config(self, key, value):
 *         cdef:
 *             bytes bkey = encode(key)             # <<<<<<<<<<<<<<
 *             int rc
 * 
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bkey = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":139
 *             int rc
 * 
 *         if isinstance(value, bool):             # <<<<<<<<<<<<<<
 *             value = value and 1 or 0
 *         if isinstance(value, int):
 */
  __pyx_t_1 = ((PyObject*)&PyBool_Type);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_value, __pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 139; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "sophy.pyx":140
 * 
 *         if isinstance(value, bool):
 *             value = value and 1 or 0             # <<<<<<<<<<<<<<
 *         if isinstance(value, int):
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!__pyx_t_3) {
      goto __pyx_L5_next_or;
    } else {
    }
    if (!1) {
    } else {
      __pyx_t_4 = __Pyx_PyInt_From_long(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_L5_next_or:;
    __pyx_t_4 = __Pyx_PyInt_From_long(0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __pyx_t_4 = 0;
    __pyx_L4_bool_binop_done:;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sophy.pyx":139
 *             int rc
 * 
 *         if isinstance(value, bool):             # <<<<<<<<<<<<<<
 *             value = value and 1 or 0
 *         if isinstance(value, int):
 */
  }

  /* "sophy.pyx":141
 *         if isinstance(value, bool):
 *             value = value and 1 or 0
 *         if isinstance(value, int):             # <<<<<<<<<<<<<<
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)
 *         elif isinstance(value, basestring):
 */
  __pyx_t_3 = PyInt_Check(__pyx_v_value); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":142
 *             value = value and 1 or 0
 *         if isinstance(value, int):
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)             # <<<<<<<<<<<<<<
 *         elif isinstance(value, basestring):
 *             bvalue = encode(value)
 */
    __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_bkey); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_rc = sp_setint(__pyx_v_self->sophia->__pyx_base.handle, ((char const *)__pyx_t_5), ((int)__pyx_t_6));

    /* "sophy.pyx":141
 *         if isinstance(value, bool):
 *             value = value and 1 or 0
 *         if isinstance(value, int):             # <<<<<<<<<<<<<<
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)
 *         elif isinstance(value, basestring):
 */
    goto __pyx_L7;
  }

  /* "sophy.pyx":143
 *         if isinstance(value, int):
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)
 *         elif isinstance(value, basestring):             # <<<<<<<<<<<<<<
 *             bvalue = encode(value)
 *             rc = sp_setstring(
 */
  __pyx_t_2 = __Pyx_PyBaseString_Check(__pyx_v_value); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "sophy.pyx":144
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)
 *         elif isinstance(value, basestring):
 *             bvalue = encode(value)             # <<<<<<<<<<<<<<
 *             rc = sp_setstring(
 *                 self.sophia.handle,
 */
    __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_bvalue = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "sophy.pyx":147
 *             rc = sp_setstring(
 *                 self.sophia.handle,
 *                 <const char *>bkey,             # <<<<<<<<<<<<<<
 *                 <const char *>bvalue,
 *                 0)
 */
    __pyx_t_7 = __Pyx_PyObject_AsString(__pyx_v_bkey); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":148
 *                 self.sophia.handle,
 *                 <const char *>bkey,
 *                 <const char *>bvalue,             # <<<<<<<<<<<<<<
 *                 0)
 *         _check(self.sophia.handle, rc)
 */
    __pyx_t_8 = __Pyx_PyObject_AsString(__pyx_v_bvalue); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 148; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":145
 *         elif isinstance(value, basestring):
 *             bvalue = encode(value)
 *             rc = sp_setstring(             # <<<<<<<<<<<<<<
 *                 self.sophia.handle,
 *                 <const char *>bkey,
 */
    __pyx_v_rc = sp_setstring(__pyx_v_self->sophia->__pyx_base.handle, ((char const *)__pyx_t_7), ((char const *)__pyx_t_8), 0);

    /* "sophy.pyx":143
 *         if isinstance(value, int):
 *             rc = sp_setint(self.sophia.handle, <const char *>bkey, <int>value)
 *         elif isinstance(value, basestring):             # <<<<<<<<<<<<<<
 *             bvalue = encode(value)
 *             rc = sp_setstring(
 */
  }
  __pyx_L7:;

  /* "sophy.pyx":150
 *                 <const char *>bvalue,
 *                 0)
 *         _check(self.sophia.handle, rc)             # <<<<<<<<<<<<<<
 * 
 *     cdef apply_all(self):
 */
  __pyx_t_1 = __pyx_f_5sophy__check(__pyx_v_self->sophia->__pyx_base.handle, __pyx_v_rc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":134
 *         del self._config[key]
 * 
 *     cdef apply_config(self, key, value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes bkey = encode(key)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sophy._ConfigManager.apply_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bkey);
  __Pyx_XDECREF(__pyx_v_bvalue);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":152
 *         _check(self.sophia.handle, rc)
 * 
 *     cdef apply_all(self):             # <<<<<<<<<<<<<<
 *         for key, value in self._config.items():
 *             self.apply_config(key, value)
 */

static PyObject *__pyx_f_5sophy_14_ConfigManager_apply_all(struct __pyx_obj_5sophy__ConfigManager *__pyx_v_self) {
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_all", 0);

  /* "sophy.pyx":153
 * 
 *     cdef apply_all(self):
 *         for key, value in self._config.items():             # <<<<<<<<<<<<<<
 *             self.apply_config(key, value)
 * 
 */
  if (unlikely(__pyx_v_self->_config == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%s'", "items");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_Items(__pyx_v_self->_config); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "sophy.pyx":154
 *     cdef apply_all(self):
 *         for key, value in self._config.items():
 *             self.apply_config(key, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__ConfigManager *)__pyx_v_self->__pyx_vtab)->apply_config(__pyx_v_self, __pyx_v_key, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sophy.pyx":153
 * 
 *     cdef apply_all(self):
 *         for key, value in self._config.items():             # <<<<<<<<<<<<<<
 *             self.apply_config(key, value)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":152
 *         _check(self.sophia.handle, rc)
 * 
 *     cdef apply_all(self):             # <<<<<<<<<<<<<<
 *         for key, value in self._config.items():
 *             self.apply_config(key, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("sophy._ConfigManager.apply_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":161
 *         void *handle
 * 
 *     def __cinit__(self, *_, **__):             # <<<<<<<<<<<<<<
 *         self.handle = <void *>0
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_13_SophiaObject_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_13_SophiaObject_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  CYTHON_UNUSED PyObject *__pyx_v___ = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v__ = __pyx_args;
  __pyx_r = __pyx_pf_5sophy_13_SophiaObject___cinit__(((struct __pyx_obj_5sophy__SophiaObject *)__pyx_v_self), __pyx_v__, __pyx_v___);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v___);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_13_SophiaObject___cinit__(struct __pyx_obj_5sophy__SophiaObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__, CYTHON_UNUSED PyObject *__pyx_v___) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":162
 * 
 *     def __cinit__(self, *_, **__):
 *         self.handle = <void *>0             # <<<<<<<<<<<<<<
 * 
 *     cdef destroy(self):
 */
  __pyx_v_self->handle = ((void *)0);

  /* "sophy.pyx":161
 *         void *handle
 * 
 *     def __cinit__(self, *_, **__):             # <<<<<<<<<<<<<<
 *         self.handle = <void *>0
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":164
 *         self.handle = <void *>0
 * 
 *     cdef destroy(self):             # <<<<<<<<<<<<<<
 *         sp_destroy(self.handle)
 *         self.handle = <void *>0
 */

static PyObject *__pyx_f_5sophy_13_SophiaObject_destroy(struct __pyx_obj_5sophy__SophiaObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("destroy", 0);

  /* "sophy.pyx":165
 * 
 *     cdef destroy(self):
 *         sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 *         self.handle = <void *>0
 * 
 */
  sp_destroy(__pyx_v_self->handle);

  /* "sophy.pyx":166
 *     cdef destroy(self):
 *         sp_destroy(self.handle)
 *         self.handle = <void *>0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->handle = ((void *)0);

  /* "sophy.pyx":164
 *         self.handle = <void *>0
 * 
 *     cdef destroy(self):             # <<<<<<<<<<<<<<
 *         sp_destroy(self.handle)
 *         self.handle = <void *>0
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":179
 *         readonly object path
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.handle = sp_env()
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_6Sophia_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_6Sophia_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_5sophy_6Sophia___cinit__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_6Sophia___cinit__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":180
 * 
 *     def __cinit__(self):
 *         self.handle = sp_env()             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, path, databases=None, auto_open=True,
 */
  __pyx_v_self->__pyx_base.handle = sp_env();

  /* "sophy.pyx":179
 *         readonly object path
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.handle = sp_env()
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":182
 *         self.handle = sp_env()
 * 
 *     def __init__(self, path, databases=None, auto_open=True,             # <<<<<<<<<<<<<<
 *                  backup_path=None, memory_limit=None, threads=None,
 *                  log_path=None):
 */

/* Python wrapper */
static int __pyx_pw_5sophy_6Sophia_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_6Sophia_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_path = 0;
  PyObject *__pyx_v_databases = 0;
  PyObject *__pyx_v_auto_open = 0;
  PyObject *__pyx_v_backup_path = 0;
  PyObject *__pyx_v_memory_limit = 0;
  PyObject *__pyx_v_threads = 0;
  PyObject *__pyx_v_log_path = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_path,&__pyx_n_s_databases,&__pyx_n_s_auto_open,&__pyx_n_s_backup_path,&__pyx_n_s_memory_limit,&__pyx_n_s_threads,&__pyx_n_s_log_path,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_True);

    /* "sophy.pyx":183
 * 
 *     def __init__(self, path, databases=None, auto_open=True,
 *                  backup_path=None, memory_limit=None, threads=None,             # <<<<<<<<<<<<<<
 *                  log_path=None):
 *         self.path = path
 */
    values[3] = ((PyObject *)Py_None);
    values[4] = ((PyObject *)Py_None);
    values[5] = ((PyObject *)Py_None);

    /* "sophy.pyx":184
 *     def __init__(self, path, databases=None, auto_open=True,
 *                  backup_path=None, memory_limit=None, threads=None,
 *                  log_path=None):             # <<<<<<<<<<<<<<
 *         self.path = path
 *         self.auto_open = auto_open
 */
    values[6] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_path)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_databases);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_auto_open);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_backup_path);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_memory_limit);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_threads);
          if (value) { values[5] = value; kw_args--; }
        }
        case  6:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_log_path);
          if (value) { values[6] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_path = values[0];
    __pyx_v_databases = values[1];
    __pyx_v_auto_open = values[2];
    __pyx_v_backup_path = values[3];
    __pyx_v_memory_limit = values[4];
    __pyx_v_threads = values[5];
    __pyx_v_log_path = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 7, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Sophia.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_6Sophia_2__init__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self), __pyx_v_path, __pyx_v_databases, __pyx_v_auto_open, __pyx_v_backup_path, __pyx_v_memory_limit, __pyx_v_threads, __pyx_v_log_path);

  /* "sophy.pyx":182
 *         self.handle = sp_env()
 * 
 *     def __init__(self, path, databases=None, auto_open=True,             # <<<<<<<<<<<<<<
 *                  backup_path=None, memory_limit=None, threads=None,
 *                  log_path=None):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_6Sophia_2__init__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, PyObject *__pyx_v_path, PyObject *__pyx_v_databases, PyObject *__pyx_v_auto_open, PyObject *__pyx_v_backup_path, PyObject *__pyx_v_memory_limit, PyObject *__pyx_v_threads, PyObject *__pyx_v_log_path) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "sophy.pyx":185
 *                  backup_path=None, memory_limit=None, threads=None,
 *                  log_path=None):
 *         self.path = path             # <<<<<<<<<<<<<<
 *         self.auto_open = auto_open
 * 
 */
  __Pyx_INCREF(__pyx_v_path);
  __Pyx_GIVEREF(__pyx_v_path);
  __Pyx_GOTREF(__pyx_v_self->path);
  __Pyx_DECREF(__pyx_v_self->path);
  __pyx_v_self->path = __pyx_v_path;

  /* "sophy.pyx":186
 *                  log_path=None):
 *         self.path = path
 *         self.auto_open = auto_open             # <<<<<<<<<<<<<<
 * 
 *         self.config = _ConfigManager(self)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_auto_open); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->auto_open = __pyx_t_1;

  /* "sophy.pyx":188
 *         self.auto_open = auto_open
 * 
 *         self.config = _ConfigManager(self)             # <<<<<<<<<<<<<<
 *         self.b_path = encode(path)
 *         self.db_defs = databases or []
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy__ConfigManager), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->config);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->config));
  __pyx_v_self->config = ((struct __pyx_obj_5sophy__ConfigManager *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "sophy.pyx":189
 * 
 *         self.config = _ConfigManager(self)
 *         self.b_path = encode(path)             # <<<<<<<<<<<<<<
 *         self.db_defs = databases or []
 *         self.dbs = {}
 */
  __pyx_t_3 = __pyx_f_5sophy_encode(__pyx_v_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->b_path);
  __Pyx_DECREF(__pyx_v_self->b_path);
  __pyx_v_self->b_path = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "sophy.pyx":190
 *         self.config = _ConfigManager(self)
 *         self.b_path = encode(path)
 *         self.db_defs = databases or []             # <<<<<<<<<<<<<<
 *         self.dbs = {}
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_databases); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (!__pyx_t_1) {
  } else {
    if (!(likely(PyList_CheckExact(__pyx_v_databases))||((__pyx_v_databases) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_databases)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_INCREF(__pyx_v_databases);
    __pyx_t_3 = __pyx_v_databases;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = __pyx_t_2;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_L3_bool_binop_done:;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->db_defs);
  __Pyx_DECREF(__pyx_v_self->db_defs);
  __pyx_v_self->db_defs = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "sophy.pyx":191
 *         self.b_path = encode(path)
 *         self.db_defs = databases or []
 *         self.dbs = {}             # <<<<<<<<<<<<<<
 * 
 *         if backup_path is not None:
 */
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->dbs);
  __Pyx_DECREF(__pyx_v_self->dbs);
  __pyx_v_self->dbs = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "sophy.pyx":193
 *         self.dbs = {}
 * 
 *         if backup_path is not None:             # <<<<<<<<<<<<<<
 *             self.backup_path = backup_path
 *         if memory_limit is not None:
 */
  __pyx_t_1 = (__pyx_v_backup_path != Py_None);
  __pyx_t_4 = (__pyx_t_1 != 0);
  if (__pyx_t_4) {

    /* "sophy.pyx":194
 * 
 *         if backup_path is not None:
 *             self.backup_path = backup_path             # <<<<<<<<<<<<<<
 *         if memory_limit is not None:
 *             self.memory_limit = memory_limit
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_backup_path, __pyx_v_backup_path) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 194; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":193
 *         self.dbs = {}
 * 
 *         if backup_path is not None:             # <<<<<<<<<<<<<<
 *             self.backup_path = backup_path
 *         if memory_limit is not None:
 */
  }

  /* "sophy.pyx":195
 *         if backup_path is not None:
 *             self.backup_path = backup_path
 *         if memory_limit is not None:             # <<<<<<<<<<<<<<
 *             self.memory_limit = memory_limit
 *         if threads is not None:
 */
  __pyx_t_4 = (__pyx_v_memory_limit != Py_None);
  __pyx_t_1 = (__pyx_t_4 != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":196
 *             self.backup_path = backup_path
 *         if memory_limit is not None:
 *             self.memory_limit = memory_limit             # <<<<<<<<<<<<<<
 *         if threads is not None:
 *             self.scheduler_threads = threads
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memory_limit, __pyx_v_memory_limit) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 196; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":195
 *         if backup_path is not None:
 *             self.backup_path = backup_path
 *         if memory_limit is not None:             # <<<<<<<<<<<<<<
 *             self.memory_limit = memory_limit
 *         if threads is not None:
 */
  }

  /* "sophy.pyx":197
 *         if memory_limit is not None:
 *             self.memory_limit = memory_limit
 *         if threads is not None:             # <<<<<<<<<<<<<<
 *             self.scheduler_threads = threads
 *         if log_path is not None:
 */
  __pyx_t_1 = (__pyx_v_threads != Py_None);
  __pyx_t_4 = (__pyx_t_1 != 0);
  if (__pyx_t_4) {

    /* "sophy.pyx":198
 *             self.memory_limit = memory_limit
 *         if threads is not None:
 *             self.scheduler_threads = threads             # <<<<<<<<<<<<<<
 *         if log_path is not None:
 *             self.log_path = log_path
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_scheduler_threads, __pyx_v_threads) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":197
 *         if memory_limit is not None:
 *             self.memory_limit = memory_limit
 *         if threads is not None:             # <<<<<<<<<<<<<<
 *             self.scheduler_threads = threads
 *         if log_path is not None:
 */
  }

  /* "sophy.pyx":199
 *         if threads is not None:
 *             self.scheduler_threads = threads
 *         if log_path is not None:             # <<<<<<<<<<<<<<
 *             self.log_path = log_path
 * 
 */
  __pyx_t_4 = (__pyx_v_log_path != Py_None);
  __pyx_t_1 = (__pyx_t_4 != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":200
 *             self.scheduler_threads = threads
 *         if log_path is not None:
 *             self.log_path = log_path             # <<<<<<<<<<<<<<
 * 
 *         if self.auto_open:
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_log_path, __pyx_v_log_path) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":199
 *         if threads is not None:
 *             self.scheduler_threads = threads
 *         if log_path is not None:             # <<<<<<<<<<<<<<
 *             self.log_path = log_path
 * 
 */
  }

  /* "sophy.pyx":202
 *             self.log_path = log_path
 * 
 *         if self.auto_open:             # <<<<<<<<<<<<<<
 *             self.open()
 * 
 */
  __pyx_t_1 = (__pyx_v_self->auto_open != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":203
 * 
 *         if self.auto_open:
 *             self.open()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
    ((struct __pyx_vtabstruct_5sophy_Sophia *)__pyx_v_self->__pyx_base.__pyx_vtab)->open(__pyx_v_self, 0);

    /* "sophy.pyx":202
 *             self.log_path = log_path
 * 
 *         if self.auto_open:             # <<<<<<<<<<<<<<
 *             self.open()
 * 
 */
  }

  /* "sophy.pyx":182
 *         self.handle = sp_env()
 * 
 *     def __init__(self, path, databases=None, auto_open=True,             # <<<<<<<<<<<<<<
 *                  backup_path=None, memory_limit=None, threads=None,
 *                  log_path=None):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.Sophia.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":205
 *             self.open()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.handle:
 *             sp_destroy(self.handle)
 */

/* Python wrapper */
static void __pyx_pw_5sophy_6Sophia_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5sophy_6Sophia_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5sophy_6Sophia_4__dealloc__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5sophy_6Sophia_4__dealloc__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sophy.pyx":206
 * 
 *     def __dealloc__(self):
 *         if self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->__pyx_base.handle != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":207
 *     def __dealloc__(self):
 *         if self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint open(self):
 */
    sp_destroy(__pyx_v_self->__pyx_base.handle);

    /* "sophy.pyx":206
 * 
 *     def __dealloc__(self):
 *         if self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  }

  /* "sophy.pyx":205
 *             self.open()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.handle:
 *             sp_destroy(self.handle)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sophy.pyx":209
 *             sp_destroy(self.handle)
 * 
 *     cpdef bint open(self):             # <<<<<<<<<<<<<<
 *         if self.is_open:
 *             return False
 */

static PyObject *__pyx_pw_5sophy_6Sophia_7open(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_5sophy_6Sophia_open(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5sophy_Database *__pyx_v_db_obj = 0;
  PyObject *__pyx_v_db_def = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  char const *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  struct __pyx_opt_args_5sophy_13_BaseDBObject_open __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_6Sophia_7open)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":210
 * 
 *     cpdef bint open(self):
 *         if self.is_open:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  __pyx_t_5 = (__pyx_v_self->is_open != 0);
  if (__pyx_t_5) {

    /* "sophy.pyx":211
 *     cpdef bint open(self):
 *         if self.is_open:
 *             return False             # <<<<<<<<<<<<<<
 * 
 *         cdef:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "sophy.pyx":210
 * 
 *     cpdef bint open(self):
 *         if self.is_open:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  }

  /* "sophy.pyx":218
 * 
 *         # Configure the environment, databases and indexes.
 *         sp_setstring(self.handle, 'sophia.path', <const char *>self.b_path, 0)             # <<<<<<<<<<<<<<
 *         for db_def in self.db_defs:
 *             self._add_db(Database(self, *db_def))
 */
  __pyx_t_6 = __Pyx_PyObject_AsString(__pyx_v_self->b_path); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_self->__pyx_base.handle, __pyx_k_sophia_path, ((char const *)__pyx_t_6), 0);

  /* "sophy.pyx":219
 *         # Configure the environment, databases and indexes.
 *         sp_setstring(self.handle, 'sophia.path', <const char *>self.b_path, 0)
 *         for db_def in self.db_defs:             # <<<<<<<<<<<<<<
 *             self._add_db(Database(self, *db_def))
 *         self.config.apply_all()
 */
  if (unlikely(__pyx_v_self->db_defs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_self->db_defs; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
  for (;;) {
    if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_2)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_db_def, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "sophy.pyx":220
 *         sp_setstring(self.handle, 'sophia.path', <const char *>self.b_path, 0)
 *         for db_def in self.db_defs:
 *             self._add_db(Database(self, *db_def))             # <<<<<<<<<<<<<<
 *         self.config.apply_all()
 * 
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
    if (unlikely(__pyx_v_db_def == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_v_db_def); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Database), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy_Sophia *)__pyx_v_self->__pyx_base.__pyx_vtab)->_add_db(__pyx_v_self, ((struct __pyx_obj_5sophy_Database *)__pyx_t_2), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sophy.pyx":219
 *         # Configure the environment, databases and indexes.
 *         sp_setstring(self.handle, 'sophia.path', <const char *>self.b_path, 0)
 *         for db_def in self.db_defs:             # <<<<<<<<<<<<<<
 *             self._add_db(Database(self, *db_def))
 *         self.config.apply_all()
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":221
 *         for db_def in self.db_defs:
 *             self._add_db(Database(self, *db_def))
 *         self.config.apply_all()             # <<<<<<<<<<<<<<
 * 
 *         # Open the environment.
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__ConfigManager *)__pyx_v_self->config->__pyx_vtab)->apply_all(__pyx_v_self->config); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 221; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":224
 * 
 *         # Open the environment.
 *         _check(self.handle, sp_open(self.handle))             # <<<<<<<<<<<<<<
 * 
 *         # Open all databases.
 */
  __pyx_t_1 = __pyx_f_5sophy__check(__pyx_v_self->__pyx_base.handle, sp_open(__pyx_v_self->__pyx_base.handle)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":227
 * 
 *         # Open all databases.
 *         for db_obj in self.dbs.values():             # <<<<<<<<<<<<<<
 *             db_obj.open(configure=False)
 * 
 */
  if (unlikely(__pyx_v_self->dbs == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%s'", "values");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_Values(__pyx_v_self->dbs); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5sophy_Database))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_db_obj, ((struct __pyx_obj_5sophy_Database *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sophy.pyx":228
 *         # Open all databases.
 *         for db_obj in self.dbs.values():
 *             db_obj.open(configure=False)             # <<<<<<<<<<<<<<
 * 
 *         self.is_open = True
 */
    __pyx_t_9.__pyx_n = 1;
    __pyx_t_9.configure = 0;
    __pyx_t_5 = ((struct __pyx_vtabstruct_5sophy_Database *)__pyx_v_db_obj->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.open(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_db_obj), 0, &__pyx_t_9); 

    /* "sophy.pyx":227
 * 
 *         # Open all databases.
 *         for db_obj in self.dbs.values():             # <<<<<<<<<<<<<<
 *             db_obj.open(configure=False)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":230
 *             db_obj.open(configure=False)
 * 
 *         self.is_open = True             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_v_self->is_open = 1;

  /* "sophy.pyx":231
 * 
 *         self.is_open = True
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cdef _add_db(self, Database db_obj, open_database=False):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "sophy.pyx":209
 *             sp_destroy(self.handle)
 * 
 *     cpdef bint open(self):             # <<<<<<<<<<<<<<
 *         if self.is_open:
 *             return False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("sophy.Sophia.open", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_db_obj);
  __Pyx_XDECREF(__pyx_v_db_def);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_7open(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_7open(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("open (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_6open(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_6open(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_5sophy_6Sophia_open(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Sophia.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":233
 *         return True
 * 
 *     cdef _add_db(self, Database db_obj, open_database=False):             # <<<<<<<<<<<<<<
 *         self.dbs[db_obj.name] = db_obj
 *         db_obj.configure()
 */

static PyObject *__pyx_f_5sophy_6Sophia__add_db(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, struct __pyx_obj_5sophy_Database *__pyx_v_db_obj, struct __pyx_opt_args_5sophy_6Sophia__add_db *__pyx_optional_args) {
  PyObject *__pyx_v_open_database = ((PyObject *)Py_False);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  struct __pyx_opt_args_5sophy_13_BaseDBObject_open __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_add_db", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_open_database = __pyx_optional_args->open_database;
    }
  }

  /* "sophy.pyx":234
 * 
 *     cdef _add_db(self, Database db_obj, open_database=False):
 *         self.dbs[db_obj.name] = db_obj             # <<<<<<<<<<<<<<
 *         db_obj.configure()
 *         if open_database:
 */
  if (unlikely(__pyx_v_self->dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 234; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  if (unlikely(PyDict_SetItem(__pyx_v_self->dbs, __pyx_v_db_obj->name, ((PyObject *)__pyx_v_db_obj)) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 234; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":235
 *     cdef _add_db(self, Database db_obj, open_database=False):
 *         self.dbs[db_obj.name] = db_obj
 *         db_obj.configure()             # <<<<<<<<<<<<<<
 *         if open_database:
 *             db_obj.open(configure=False)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy_Database *)__pyx_v_db_obj->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.configure(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_db_obj)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":236
 *         self.dbs[db_obj.name] = db_obj
 *         db_obj.configure()
 *         if open_database:             # <<<<<<<<<<<<<<
 *             db_obj.open(configure=False)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_open_database); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "sophy.pyx":237
 *         db_obj.configure()
 *         if open_database:
 *             db_obj.open(configure=False)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint close(self):
 */
    __pyx_t_3.__pyx_n = 1;
    __pyx_t_3.configure = 0;
    __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy_Database *)__pyx_v_db_obj->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.open(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_db_obj), 0, &__pyx_t_3); 

    /* "sophy.pyx":236
 *         self.dbs[db_obj.name] = db_obj
 *         db_obj.configure()
 *         if open_database:             # <<<<<<<<<<<<<<
 *             db_obj.open(configure=False)
 * 
 */
  }

  /* "sophy.pyx":233
 *         return True
 * 
 *     cdef _add_db(self, Database db_obj, open_database=False):             # <<<<<<<<<<<<<<
 *         self.dbs[db_obj.name] = db_obj
 *         db_obj.configure()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Sophia._add_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":239
 *             db_obj.open(configure=False)
 * 
 *     cpdef bint close(self):             # <<<<<<<<<<<<<<
 *         if not self.is_open:
 *             return False
 */

static PyObject *__pyx_pw_5sophy_6Sophia_9close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_5sophy_6Sophia_close(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_close); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_6Sophia_9close)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":240
 * 
 *     cpdef bint close(self):
 *         if not self.is_open:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  __pyx_t_5 = ((!(__pyx_v_self->is_open != 0)) != 0);
  if (__pyx_t_5) {

    /* "sophy.pyx":241
 *     cpdef bint close(self):
 *         if not self.is_open:
 *             return False             # <<<<<<<<<<<<<<
 * 
 *         self.dbs = {}
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "sophy.pyx":240
 * 
 *     cpdef bint close(self):
 *         if not self.is_open:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  }

  /* "sophy.pyx":243
 *             return False
 * 
 *         self.dbs = {}             # <<<<<<<<<<<<<<
 *         sp_destroy(self.handle)
 *         self.handle = sp_env()
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->dbs);
  __Pyx_DECREF(__pyx_v_self->dbs);
  __pyx_v_self->dbs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":244
 * 
 *         self.dbs = {}
 *         sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 *         self.handle = sp_env()
 *         self.is_open = False
 */
  sp_destroy(__pyx_v_self->__pyx_base.handle);

  /* "sophy.pyx":245
 *         self.dbs = {}
 *         sp_destroy(self.handle)
 *         self.handle = sp_env()             # <<<<<<<<<<<<<<
 *         self.is_open = False
 *         return True
 */
  __pyx_v_self->__pyx_base.handle = sp_env();

  /* "sophy.pyx":246
 *         sp_destroy(self.handle)
 *         self.handle = sp_env()
 *         self.is_open = False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_v_self->is_open = 0;

  /* "sophy.pyx":247
 *         self.handle = sp_env()
 *         self.is_open = False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, name):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "sophy.pyx":239
 *             db_obj.open(configure=False)
 * 
 *     cpdef bint close(self):             # <<<<<<<<<<<<<<
 *         if not self.is_open:
 *             return False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("sophy.Sophia.close", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_9close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_9close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_8close(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_8close(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_5sophy_6Sophia_close(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Sophia.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":249
 *         return True
 * 
 *     def __getitem__(self, name):             # <<<<<<<<<<<<<<
 *         return self.dbs[name]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_11__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_11__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_10__getitem__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_10__getitem__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "sophy.pyx":250
 * 
 *     def __getitem__(self, name):
 *         return self.dbs[name]             # <<<<<<<<<<<<<<
 * 
 *     def create_database(self, name, index_type='string'):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 250; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->dbs, __pyx_v_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 250; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":249
 *         return True
 * 
 *     def __getitem__(self, name):             # <<<<<<<<<<<<<<
 *         return self.dbs[name]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Sophia.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":252
 *         return self.dbs[name]
 * 
 *     def create_database(self, name, index_type='string'):             # <<<<<<<<<<<<<<
 *         self._add_db(Database(self, name, index_type), self.is_open)
 *         self.db_defs.append((name, index_type))
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_13create_database(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_13create_database(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_index_type = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_database (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_index_type,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_n_s_string);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index_type);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_database") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 252; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = values[0];
    __pyx_v_index_type = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_database", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 252; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Sophia.create_database", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_6Sophia_12create_database(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self), __pyx_v_name, __pyx_v_index_type);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_12create_database(struct __pyx_obj_5sophy_Sophia *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_index_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_5sophy_6Sophia__add_db __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_database", 0);

  /* "sophy.pyx":253
 * 
 *     def create_database(self, name, index_type='string'):
 *         self._add_db(Database(self, name, index_type), self.is_open)             # <<<<<<<<<<<<<<
 *         self.db_defs.append((name, index_type))
 *         return self.dbs[name]
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_index_type);
  __Pyx_GIVEREF(__pyx_v_index_type);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_index_type);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Database), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->is_open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4.__pyx_n = 1;
  __pyx_t_4.open_database = __pyx_t_1;
  __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy_Sophia *)__pyx_v_self->__pyx_base.__pyx_vtab)->_add_db(__pyx_v_self, ((struct __pyx_obj_5sophy_Database *)__pyx_t_2), &__pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":254
 *     def create_database(self, name, index_type='string'):
 *         self._add_db(Database(self, name, index_type), self.is_open)
 *         self.db_defs.append((name, index_type))             # <<<<<<<<<<<<<<
 *         return self.dbs[name]
 * 
 */
  if (unlikely(__pyx_v_self->db_defs == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%s'", "append");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_index_type);
  __Pyx_GIVEREF(__pyx_v_index_type);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_index_type);
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->db_defs, __pyx_t_3); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":255
 *         self._add_db(Database(self, name, index_type), self.is_open)
 *         self.db_defs.append((name, index_type))
 *         return self.dbs[name]             # <<<<<<<<<<<<<<
 * 
 *     # Properties: key, is_string?, is_readonly?
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 255; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_self->dbs, __pyx_v_name); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 255; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":252
 *         return self.dbs[name]
 * 
 *     def create_database(self, name, index_type='string'):             # <<<<<<<<<<<<<<
 *         self._add_db(Database(self, name, index_type), self.is_open)
 *         self.db_defs.append((name, index_type))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.Sophia.create_database", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":173
 *         bint is_open
 *         bytes b_path
 *         readonly dict dbs             # <<<<<<<<<<<<<<
 *         list db_defs
 *         readonly bint auto_open
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_3dbs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_3dbs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_3dbs___get__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_3dbs___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->dbs);
  __pyx_r = __pyx_v_self->dbs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":175
 *         readonly dict dbs
 *         list db_defs
 *         readonly bint auto_open             # <<<<<<<<<<<<<<
 *         readonly _ConfigManager config
 *         readonly object path
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_9auto_open_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_9auto_open_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_9auto_open___get__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_9auto_open___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->auto_open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Sophia.auto_open.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":176
 *         list db_defs
 *         readonly bint auto_open
 *         readonly _ConfigManager config             # <<<<<<<<<<<<<<
 *         readonly object path
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_6config_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_6config_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_6config___get__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_6config___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->config));
  __pyx_r = ((PyObject *)__pyx_v_self->config);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":177
 *         readonly bint auto_open
 *         readonly _ConfigManager config
 *         readonly object path             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Sophia_4path_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Sophia_4path_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Sophia_4path___get__(((struct __pyx_obj_5sophy_Sophia *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Sophia_4path___get__(struct __pyx_obj_5sophy_Sophia *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->path);
  __pyx_r = __pyx_v_self->path;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":377
 *         _Index index
 * 
 *     def __cinit__(self, Sophia sophia, *_):             # <<<<<<<<<<<<<<
 *         self.sophia = sophia
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_13_BaseDBObject_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_13_BaseDBObject_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (PyTuple_GET_SIZE(__pyx_args) > 1) {
    __pyx_v__ = PyTuple_GetSlice(__pyx_args, 1, PyTuple_GET_SIZE(__pyx_args));
    if (unlikely(!__pyx_v__)) {
      __Pyx_RefNannyFinishContext();
      return -1;
    }
    __Pyx_GOTREF(__pyx_v__);
  } else {
    __pyx_v__ = __pyx_empty_tuple; __Pyx_INCREF(__pyx_empty_tuple);
  }
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        default:
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t used_pos_args = (pos_args < 1) ? pos_args : 1;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, used_pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) < 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v__); __pyx_v__ = 0;
  __Pyx_AddTraceback("sophy._BaseDBObject.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject___cinit__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_sophia, __pyx_v__);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_13_BaseDBObject___cinit__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED PyObject *__pyx_v__) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":378
 * 
 *     def __cinit__(self, Sophia sophia, *_):
 *         self.sophia = sophia             # <<<<<<<<<<<<<<
 * 
 *     cdef configure(self):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GOTREF(__pyx_v_self->sophia);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->sophia));
  __pyx_v_self->sophia = __pyx_v_sophia;

  /* "sophy.pyx":377
 *         _Index index
 * 
 *     def __cinit__(self, Sophia sophia, *_):             # <<<<<<<<<<<<<<
 *         self.sophia = sophia
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":380
 *         self.sophia = sophia
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         self.index = self._get_index()
 * 
 */

static PyObject *__pyx_f_5sophy_13_BaseDBObject_configure(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("configure", 0);

  /* "sophy.pyx":381
 * 
 *     cdef configure(self):
 *         self.index = self._get_index()             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint open(self, bint configure=True):
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->_get_index(__pyx_v_self)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 381; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->index);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->index));
  __pyx_v_self->index = ((struct __pyx_obj_5sophy__Index *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":380
 *         self.sophia = sophia
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         self.index = self._get_index()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._BaseDBObject.configure", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":383
 *         self.index = self._get_index()
 * 
 *     cpdef bint open(self, bint configure=True):             # <<<<<<<<<<<<<<
 *         if self.handle:
 *             return False
 */

static PyObject *__pyx_pw_5sophy_13_BaseDBObject_3open(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_f_5sophy_13_BaseDBObject_open(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_open *__pyx_optional_args) {
  int __pyx_v_configure = ((int)1);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_configure = __pyx_optional_args->configure;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_3open)) {
      __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_configure); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_7;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":384
 * 
 *     cpdef bint open(self, bint configure=True):
 *         if self.handle:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  __pyx_t_7 = (__pyx_v_self->__pyx_base.handle != 0);
  if (__pyx_t_7) {

    /* "sophy.pyx":385
 *     cpdef bint open(self, bint configure=True):
 *         if self.handle:
 *             return False             # <<<<<<<<<<<<<<
 * 
 *         if configure:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "sophy.pyx":384
 * 
 *     cpdef bint open(self, bint configure=True):
 *         if self.handle:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  }

  /* "sophy.pyx":387
 *             return False
 * 
 *         if configure:             # <<<<<<<<<<<<<<
 *             self.configure()
 * 
 */
  __pyx_t_7 = (__pyx_v_configure != 0);
  if (__pyx_t_7) {

    /* "sophy.pyx":388
 * 
 *         if configure:
 *             self.configure()             # <<<<<<<<<<<<<<
 * 
 *         self.handle = self._create_handle()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->configure(__pyx_v_self); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sophy.pyx":387
 *             return False
 * 
 *         if configure:             # <<<<<<<<<<<<<<
 *             self.configure()
 * 
 */
  }

  /* "sophy.pyx":390
 *             self.configure()
 * 
 *         self.handle = self._create_handle()             # <<<<<<<<<<<<<<
 *         if not self.handle:
 *             raise MemoryError('Unable to allocate object: %s.' % self)
 */
  __pyx_v_self->__pyx_base.handle = ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->_create_handle(__pyx_v_self);

  /* "sophy.pyx":391
 * 
 *         self.handle = self._create_handle()
 *         if not self.handle:             # <<<<<<<<<<<<<<
 *             raise MemoryError('Unable to allocate object: %s.' % self)
 *         return True
 */
  __pyx_t_7 = ((!(__pyx_v_self->__pyx_base.handle != 0)) != 0);
  if (__pyx_t_7) {

    /* "sophy.pyx":392
 *         self.handle = self._create_handle()
 *         if not self.handle:
 *             raise MemoryError('Unable to allocate object: %s.' % self)             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Unable_to_allocate_object_s, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":391
 * 
 *         self.handle = self._create_handle()
 *         if not self.handle:             # <<<<<<<<<<<<<<
 *             raise MemoryError('Unable to allocate object: %s.' % self)
 *         return True
 */
  }

  /* "sophy.pyx":393
 *         if not self.handle:
 *             raise MemoryError('Unable to allocate object: %s.' % self)
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint close(self):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "sophy.pyx":383
 *         self.index = self._get_index()
 * 
 *     cpdef bint open(self, bint configure=True):             # <<<<<<<<<<<<<<
 *         if self.handle:
 *             return False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("sophy._BaseDBObject.open", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_3open(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_3open(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_configure;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("open (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_configure,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_configure);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "open") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_configure = __Pyx_PyObject_IsTrue(values[0]); if (unlikely((__pyx_v_configure == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_configure = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("open", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_2open(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_configure);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_2open(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_v_configure) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_opt_args_5sophy_13_BaseDBObject_open __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.configure = __pyx_v_configure;
  __pyx_t_1 = __pyx_vtabptr_5sophy__BaseDBObject->open(__pyx_v_self, 1, &__pyx_t_2); 
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._BaseDBObject.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":395
 *         return True
 * 
 *     cpdef bint close(self):             # <<<<<<<<<<<<<<
 *         if not self.handle:
 *             return False
 */

static PyObject *__pyx_pw_5sophy_13_BaseDBObject_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_5sophy_13_BaseDBObject_close(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_close); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_5close)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":396
 * 
 *     cpdef bint close(self):
 *         if not self.handle:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  __pyx_t_5 = ((!(__pyx_v_self->__pyx_base.handle != 0)) != 0);
  if (__pyx_t_5) {

    /* "sophy.pyx":397
 *     cpdef bint close(self):
 *         if not self.handle:
 *             return False             # <<<<<<<<<<<<<<
 * 
 *         self.destroy()
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "sophy.pyx":396
 * 
 *     cpdef bint close(self):
 *         if not self.handle:             # <<<<<<<<<<<<<<
 *             return False
 * 
 */
  }

  /* "sophy.pyx":399
 *             return False
 * 
 *         self.destroy()             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.destroy(((struct __pyx_obj_5sophy__SophiaObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":400
 * 
 *         self.destroy()
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cdef void *_create_handle(self):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "sophy.pyx":395
 *         return True
 * 
 *     cpdef bint close(self):             # <<<<<<<<<<<<<<
 *         if not self.handle:
 *             return False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("sophy._BaseDBObject.close", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_4close(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_4close(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_5sophy_13_BaseDBObject_close(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._BaseDBObject.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":402
 *         return True
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError
 * 
 */

static void *__pyx_f_5sophy_13_BaseDBObject__create_handle(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_handle", 0);

  /* "sophy.pyx":403
 * 
 *     cdef void *_create_handle(self):
 *         raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *     cdef _Index _get_index(self):
 */
  __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":402
 *         return True
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("sophy._BaseDBObject._create_handle", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":405
 *         raise NotImplementedError
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError
 * 
 */

static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_13_BaseDBObject__get_index(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  struct __pyx_obj_5sophy__Index *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_index", 0);

  /* "sophy.pyx":406
 * 
 *     cdef _Index _get_index(self):
 *         raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
 */
  __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":405
 *         raise NotImplementedError
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject._get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":408
 *         raise NotImplementedError
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.open()
 *         return self
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_7__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_7__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_6__enter__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_6__enter__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "sophy.pyx":409
 * 
 *     def __enter__(self):
 *         self.open()             # <<<<<<<<<<<<<<
 *         return self
 * 
 */
  ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->open(__pyx_v_self, 0, NULL);

  /* "sophy.pyx":410
 *     def __enter__(self):
 *         self.open()
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "sophy.pyx":408
 *         raise NotImplementedError
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.open()
 *         return self
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":412
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_9__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_9__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_exc_type,&__pyx_n_s_exc_val,&__pyx_n_s_exc_tb,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exc_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exc_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exc_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__exit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_8__exit__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_8__exit__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "sophy.pyx":413
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __call__(self, fn):
 */
  ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->close(__pyx_v_self, 0);

  /* "sophy.pyx":412
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":415
 *         self.close()
 * 
 *     def __call__(self, fn):             # <<<<<<<<<<<<<<
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_11__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_11__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_fn = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__call__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_fn,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fn)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__call__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 415; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_fn = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 415; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_10__call__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_fn);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":417
 *     def __call__(self, fn):
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):             # <<<<<<<<<<<<<<
 *             with self:
 *                 return fn(*args, **kwargs)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_8__call___1wrapper(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_13_BaseDBObject_8__call___1wrapper = {"wrapper", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_8__call___1wrapper, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_8__call___1wrapper(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("wrapper (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "wrapper", 1))) return NULL;
  if (unlikely(__pyx_kwds)) {
    __pyx_v_kwargs = PyDict_Copy(__pyx_kwds); if (unlikely(!__pyx_v_kwargs)) return NULL;
    __Pyx_GOTREF(__pyx_v_kwargs);
  } else {
    __pyx_v_kwargs = NULL;
  }
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_8__call___wrapper(__pyx_self, __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_8__call___wrapper(PyObject *__pyx_self, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs) {
  struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *__pyx_cur_scope;
  struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wrapper", 0);
  __pyx_outer_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sophy.pyx":418
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 *             with self:             # <<<<<<<<<<<<<<
 *                 return fn(*args, **kwargs)
 *         return wrapper
 */
  /*with:*/ {
    if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "sophy.pyx":419
 *         def wrapper(*args, **kwargs):
 *             with self:
 *                 return fn(*args, **kwargs)             # <<<<<<<<<<<<<<
 *         return wrapper
 * 
 */
          __Pyx_XDECREF(__pyx_r);
          if (unlikely(!__pyx_cur_scope->__pyx_v_fn)) { __Pyx_RaiseClosureNameError("fn"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 419; __pyx_clineno = __LINE__; goto __pyx_L7_error;} }
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_cur_scope->__pyx_v_fn, __pyx_v_args, __pyx_v_kwargs); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 419; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L11_try_return;

          /* "sophy.pyx":418
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 *             with self:             # <<<<<<<<<<<<<<
 *                 return fn(*args, **kwargs)
 *         return wrapper
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("sophy._BaseDBObject.__call__.wrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestore(__pyx_t_2, __pyx_t_3, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__15, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__16, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L18;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L18:;
  }

  /* "sophy.pyx":417
 *     def __call__(self, fn):
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):             # <<<<<<<<<<<<<<
 *             with self:
 *                 return fn(*args, **kwargs)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sophy._BaseDBObject.__call__.wrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":415
 *         self.close()
 * 
 *     def __call__(self, fn):             # <<<<<<<<<<<<<<
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 */

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_10__call__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_fn) {
  struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *__pyx_cur_scope;
  PyObject *__pyx_v_wrapper = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__call__", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *)__pyx_tp_new_5sophy___pyx_scope_struct_4___call__(__pyx_ptype_5sophy___pyx_scope_struct_4___call__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_fn = __pyx_v_fn;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_fn);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_fn);

  /* "sophy.pyx":416
 * 
 *     def __call__(self, fn):
 *         @wraps(fn)             # <<<<<<<<<<<<<<
 *         def wrapper(*args, **kwargs):
 *             with self:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_wraps); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_cur_scope->__pyx_v_fn); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_fn);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_fn);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_cur_scope->__pyx_v_fn);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":417
 *     def __call__(self, fn):
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):             # <<<<<<<<<<<<<<
 *             with self:
 *                 return fn(*args, **kwargs)
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5sophy_13_BaseDBObject_8__call___1wrapper, 0, __pyx_n_s_call___locals_wrapper, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sophy, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_wrapper = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":420
 *             with self:
 *                 return fn(*args, **kwargs)
 *         return wrapper             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, key):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_wrapper);
  __pyx_r = __pyx_v_wrapper;
  goto __pyx_L0;

  /* "sophy.pyx":415
 *         self.close()
 * 
 *     def __call__(self, fn):             # <<<<<<<<<<<<<<
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy._BaseDBObject.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_wrapper);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":422
 *         return wrapper
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         if isinstance(key, slice):
 *             return self.get_range(key.start, key.stop, key.step)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_13__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_13__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_12__getitem__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_12__getitem__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "sophy.pyx":423
 * 
 *     def __getitem__(self, key):
 *         if isinstance(key, slice):             # <<<<<<<<<<<<<<
 *             return self.get_range(key.start, key.stop, key.step)
 *         return self.index.get(self, key)
 */
  __pyx_t_1 = PySlice_Check(__pyx_v_key); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":424
 *     def __getitem__(self, key):
 *         if isinstance(key, slice):
 *             return self.get_range(key.start, key.stop, key.step)             # <<<<<<<<<<<<<<
 *         return self.index.get(self, key)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_range); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 424; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_key, __pyx_n_s_start); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 424; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_key, __pyx_n_s_stop); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 424; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_key, __pyx_n_s_step); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 424; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_9 = 1;
      }
    }
    __pyx_t_10 = PyTuple_New(3+__pyx_t_9); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 424; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    if (__pyx_t_8) {
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 2+__pyx_t_9, __pyx_t_7);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 424; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":423
 * 
 *     def __getitem__(self, key):
 *         if isinstance(key, slice):             # <<<<<<<<<<<<<<
 *             return self.get_range(key.start, key.stop, key.step)
 *         return self.index.get(self, key)
 */
  }

  /* "sophy.pyx":425
 *         if isinstance(key, slice):
 *             return self.get_range(key.start, key.stop, key.step)
 *         return self.index.get(self, key)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, key, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->index->__pyx_vtab)->get(__pyx_v_self->index, __pyx_v_self, __pyx_v_key); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":422
 *         return wrapper
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         if isinstance(key, slice):
 *             return self.get_range(key.start, key.stop, key.step)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("sophy._BaseDBObject.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":427
 *         return self.index.get(self, key)
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         self.index.set(self, key, value)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_13_BaseDBObject_15__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5sophy_13_BaseDBObject_15__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_14__setitem__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_13_BaseDBObject_14__setitem__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "sophy.pyx":428
 * 
 *     def __setitem__(self, key, value):
 *         self.index.set(self, key, value)             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, key):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->index->__pyx_vtab)->set(__pyx_v_self->index, __pyx_v_self, __pyx_v_key, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 428; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":427
 *         return self.index.get(self, key)
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         self.index.set(self, key, value)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._BaseDBObject.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":430
 *         self.index.set(self, key, value)
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         self.index.delete(self, key)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_13_BaseDBObject_17__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_5sophy_13_BaseDBObject_17__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_16__delitem__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_13_BaseDBObject_16__delitem__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "sophy.pyx":431
 * 
 *     def __delitem__(self, key):
 *         self.index.delete(self, key)             # <<<<<<<<<<<<<<
 * 
 *     def __contains__(self, key):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->index->__pyx_vtab)->delete(__pyx_v_self->index, __pyx_v_self, __pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 431; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":430
 *         self.index.set(self, key, value)
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         self.index.delete(self, key)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._BaseDBObject.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":433
 *         self.index.delete(self, key)
 * 
 *     def __contains__(self, key):             # <<<<<<<<<<<<<<
 *         return self.index.exists(self, key)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_13_BaseDBObject_19__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_5sophy_13_BaseDBObject_19__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_18__contains__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_13_BaseDBObject_18__contains__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "sophy.pyx":434
 * 
 *     def __contains__(self, key):
 *         return self.index.exists(self, key)             # <<<<<<<<<<<<<<
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->index->__pyx_vtab)->exists(__pyx_v_self->index, __pyx_v_self, __pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "sophy.pyx":433
 *         self.index.delete(self, key)
 * 
 *     def __contains__(self, key):             # <<<<<<<<<<<<<<
 *         return self.index.exists(self, key)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._BaseDBObject.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":436
 *         return self.index.exists(self, key)
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):             # <<<<<<<<<<<<<<
 *         with db.transaction() as txn:
 *             if _data:
 */

static PyObject *__pyx_f_5sophy_13_BaseDBObject__update(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, struct __pyx_obj_5sophy_Database *__pyx_v_db, struct __pyx_opt_args_5sophy_13_BaseDBObject__update *__pyx_optional_args) {
  PyObject *__pyx_v__data = ((PyObject*)Py_None);
  PyObject *__pyx_v_k = ((PyObject*)Py_None);
  PyObject *__pyx_v_txn = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_update", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v__data = __pyx_optional_args->_data;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_k = __pyx_optional_args->k;
      }
    }
  }

  /* "sophy.pyx":437
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):
 *         with db.transaction() as txn:             # <<<<<<<<<<<<<<
 *             if _data:
 *                 for key in _data:
 */
  /*with:*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy_Database *)__pyx_v_db->__pyx_base.__pyx_base.__pyx_vtab)->transaction(__pyx_v_db, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_exit); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_enter); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __pyx_t_3;
    __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {
          __pyx_v_txn = __pyx_t_4;
          __pyx_t_4 = 0;

          /* "sophy.pyx":438
 *     cdef _update(self, Database db, dict _data=None, dict k=None):
 *         with db.transaction() as txn:
 *             if _data:             # <<<<<<<<<<<<<<
 *                 for key in _data:
 *                     txn[key] = _data[key]
 */
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v__data); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          if (__pyx_t_9) {

            /* "sophy.pyx":439
 *         with db.transaction() as txn:
 *             if _data:
 *                 for key in _data:             # <<<<<<<<<<<<<<
 *                     txn[key] = _data[key]
 *             if k:
 */
            __pyx_t_10 = 0;
            if (unlikely(__pyx_v__data == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 439; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            }
            __pyx_t_1 = __Pyx_dict_iterator(__pyx_v__data, 1, ((PyObject *)NULL), (&__pyx_t_11), (&__pyx_t_12)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 439; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_XDECREF(__pyx_t_4);
            __pyx_t_4 = __pyx_t_1;
            __pyx_t_1 = 0;
            while (1) {
              __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_4, __pyx_t_11, &__pyx_t_10, &__pyx_t_1, NULL, NULL, __pyx_t_12);
              if (unlikely(__pyx_t_13 == 0)) break;
              if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 439; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_1);
              __pyx_t_1 = 0;

              /* "sophy.pyx":440
 *             if _data:
 *                 for key in _data:
 *                     txn[key] = _data[key]             # <<<<<<<<<<<<<<
 *             if k:
 *                 for key in k:
 */
              if (unlikely(__pyx_v__data == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
              }
              __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v__data, __pyx_v_key); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L7_error;};
              __Pyx_GOTREF(__pyx_t_1);
              if (unlikely(PyObject_SetItem(__pyx_v_txn, __pyx_v_key, __pyx_t_1) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "sophy.pyx":438
 *     cdef _update(self, Database db, dict _data=None, dict k=None):
 *         with db.transaction() as txn:
 *             if _data:             # <<<<<<<<<<<<<<
 *                 for key in _data:
 *                     txn[key] = _data[key]
 */
          }

          /* "sophy.pyx":441
 *                 for key in _data:
 *                     txn[key] = _data[key]
 *             if k:             # <<<<<<<<<<<<<<
 *                 for key in k:
 *                     txn[key] = k[key]
 */
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_k); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 441; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          if (__pyx_t_9) {

            /* "sophy.pyx":442
 *                     txn[key] = _data[key]
 *             if k:
 *                 for key in k:             # <<<<<<<<<<<<<<
 *                     txn[key] = k[key]
 * 
 */
            __pyx_t_11 = 0;
            if (unlikely(__pyx_v_k == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            }
            __pyx_t_1 = __Pyx_dict_iterator(__pyx_v_k, 1, ((PyObject *)NULL), (&__pyx_t_10), (&__pyx_t_12)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_XDECREF(__pyx_t_4);
            __pyx_t_4 = __pyx_t_1;
            __pyx_t_1 = 0;
            while (1) {
              __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_4, __pyx_t_10, &__pyx_t_11, &__pyx_t_1, NULL, NULL, __pyx_t_12);
              if (unlikely(__pyx_t_13 == 0)) break;
              if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_1);
              __pyx_t_1 = 0;

              /* "sophy.pyx":443
 *             if k:
 *                 for key in k:
 *                     txn[key] = k[key]             # <<<<<<<<<<<<<<
 * 
 *     def get_range(self, start=None, stop=None, reverse=False):
 */
              if (unlikely(__pyx_v_k == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                {__pyx_filename = __pyx_f[0]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
              }
              __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_k, __pyx_v_key); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L7_error;};
              __Pyx_GOTREF(__pyx_t_1);
              if (unlikely(PyObject_SetItem(__pyx_v_txn, __pyx_v_key, __pyx_t_1) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "sophy.pyx":441
 *                 for key in _data:
 *                     txn[key] = _data[key]
 *             if k:             # <<<<<<<<<<<<<<
 *                 for key in k:
 *                     txn[key] = k[key]
 */
          }

          /* "sophy.pyx":437
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):
 *         with db.transaction() as txn:             # <<<<<<<<<<<<<<
 *             if _data:
 *                 for key in _data:
 */
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L14_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("sophy._BaseDBObject._update", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = PyTuple_Pack(3, __pyx_t_4, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (__pyx_t_9 < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __pyx_t_15 = ((!(__pyx_t_9 != 0)) != 0);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestore(__pyx_t_4, __pyx_t_1, __pyx_t_3);
            __pyx_t_4 = 0; __pyx_t_1 = 0; __pyx_t_3 = 0; 
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L14_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__19, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L24;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L24:;
  }

  /* "sophy.pyx":436
 *         return self.index.exists(self, key)
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):             # <<<<<<<<<<<<<<
 *         with db.transaction() as txn:
 *             if _data:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy._BaseDBObject._update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_txn);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_5sophy_13_BaseDBObject_22generator(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "sophy.pyx":445
 *                     txn[key] = k[key]
 * 
 *     def get_range(self, start=None, stop=None, reverse=False):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Cursor cursor
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_21get_range(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_21get_range(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_reverse = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_range (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_reverse,0};
    PyObject* values[3] = {0,0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[0] = value; kw_args--; }
        }
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_stop);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_reverse);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_range") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_start = values[0];
    __pyx_v_stop = values[1];
    __pyx_v_reverse = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_range", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.get_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_20get_range(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_reverse);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_20get_range(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_reverse) {
  struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_range", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *)__pyx_tp_new_5sophy___pyx_scope_struct_5_get_range(__pyx_ptype_5sophy___pyx_scope_struct_5_get_range, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_start = __pyx_v_start;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_start);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_start);
  __pyx_cur_scope->__pyx_v_stop = __pyx_v_stop;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_stop);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_stop);
  __pyx_cur_scope->__pyx_v_reverse = __pyx_v_reverse;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_reverse);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_reverse);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_5sophy_13_BaseDBObject_22generator, (PyObject *) __pyx_cur_scope, __pyx_n_s_get_range, __pyx_n_s_BaseDBObject_get_range); if (unlikely(!gen)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.get_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_5sophy_13_BaseDBObject_22generator(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *__pyx_cur_scope = ((struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *(*__pyx_t_14)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L27_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":449
 *             Cursor cursor
 * 
 *         first = start is None             # <<<<<<<<<<<<<<
 *         last = stop is None
 * 
 */
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_start == Py_None);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_first = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "sophy.pyx":450
 * 
 *         first = start is None
 *         last = stop is None             # <<<<<<<<<<<<<<
 * 
 *         if reverse:
 */
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_stop == Py_None);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_last = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "sophy.pyx":452
 *         last = stop is None
 * 
 *         if reverse:             # <<<<<<<<<<<<<<
 *             if (first and not last) or (last and not first):
 *                 start, stop = stop, start
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_reverse); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "sophy.pyx":453
 * 
 *         if reverse:
 *             if (first and not last) or (last and not first):             # <<<<<<<<<<<<<<
 *                 start, stop = stop, start
 *             if (not first and not last) and (start < stop):
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_first); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!__pyx_t_3) {
      goto __pyx_L7_next_or;
    } else {
    }
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_last); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = ((!__pyx_t_3) != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_L7_next_or:;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_last); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_first); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = ((!__pyx_t_4) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_1) {

      /* "sophy.pyx":454
 *         if reverse:
 *             if (first and not last) or (last and not first):
 *                 start, stop = stop, start             # <<<<<<<<<<<<<<
 *             if (not first and not last) and (start < stop):
 *                 start, stop = stop, start
 */
      __pyx_t_5 = __pyx_cur_scope->__pyx_v_stop;
      __pyx_t_6 = __pyx_cur_scope->__pyx_v_start;
      __pyx_cur_scope->__pyx_v_start = __pyx_t_5;
      __pyx_t_5 = 0;
      __pyx_cur_scope->__pyx_v_stop = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "sophy.pyx":453
 * 
 *         if reverse:
 *             if (first and not last) or (last and not first):             # <<<<<<<<<<<<<<
 *                 start, stop = stop, start
 *             if (not first and not last) and (start < stop):
 */
    }

    /* "sophy.pyx":455
 *             if (first and not last) or (last and not first):
 *                 start, stop = stop, start
 *             if (not first and not last) and (start < stop):             # <<<<<<<<<<<<<<
 *                 start, stop = stop, start
 *         elif (not first and not last) and (start > stop):
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_first); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = ((!__pyx_t_3) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L11_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_last); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = ((!__pyx_t_4) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L11_bool_binop_done;
    }
    __pyx_t_2 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_start, __pyx_cur_scope->__pyx_v_stop, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __pyx_t_3;
    __pyx_L11_bool_binop_done:;
    if (__pyx_t_1) {

      /* "sophy.pyx":456
 *                 start, stop = stop, start
 *             if (not first and not last) and (start < stop):
 *                 start, stop = stop, start             # <<<<<<<<<<<<<<
 *         elif (not first and not last) and (start > stop):
 *             reverse = True
 */
      __pyx_t_6 = __pyx_cur_scope->__pyx_v_stop;
      __pyx_t_5 = __pyx_cur_scope->__pyx_v_start;
      __pyx_cur_scope->__pyx_v_start = __pyx_t_6;
      __pyx_t_6 = 0;
      __pyx_cur_scope->__pyx_v_stop = __pyx_t_5;
      __pyx_t_5 = 0;

      /* "sophy.pyx":455
 *             if (first and not last) or (last and not first):
 *                 start, stop = stop, start
 *             if (not first and not last) and (start < stop):             # <<<<<<<<<<<<<<
 *                 start, stop = stop, start
 *         elif (not first and not last) and (start > stop):
 */
    }

    /* "sophy.pyx":452
 *         last = stop is None
 * 
 *         if reverse:             # <<<<<<<<<<<<<<
 *             if (first and not last) or (last and not first):
 *                 start, stop = stop, start
 */
    goto __pyx_L4;
  }

  /* "sophy.pyx":457
 *             if (not first and not last) and (start < stop):
 *                 start, stop = stop, start
 *         elif (not first and not last) and (start > stop):             # <<<<<<<<<<<<<<
 *             reverse = True
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_first); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = ((!__pyx_t_3) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_last); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((!__pyx_t_4) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_2 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_start, __pyx_cur_scope->__pyx_v_stop, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L14_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":458
 *                 start, stop = stop, start
 *         elif (not first and not last) and (start > stop):
 *             reverse = True             # <<<<<<<<<<<<<<
 * 
 *         order = '<=' if reverse else '>='
 */
    __Pyx_INCREF(Py_True);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_reverse);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_reverse, Py_True);
    __Pyx_GIVEREF(Py_True);

    /* "sophy.pyx":457
 *             if (not first and not last) and (start < stop):
 *                 start, stop = stop, start
 *         elif (not first and not last) and (start > stop):             # <<<<<<<<<<<<<<
 *             reverse = True
 * 
 */
  }
  __pyx_L4:;

  /* "sophy.pyx":460
 *             reverse = True
 * 
 *         order = '<=' if reverse else '>='             # <<<<<<<<<<<<<<
 *         cursor = self.cursor(order=order, key=start)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_reverse); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {
    __Pyx_INCREF(__pyx_kp_s__20);
    __pyx_t_2 = __pyx_kp_s__20;
  } else {
    __Pyx_INCREF(__pyx_kp_s__21);
    __pyx_t_2 = __pyx_kp_s__21;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_order = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "sophy.pyx":461
 * 
 *         order = '<=' if reverse else '>='
 *         cursor = self.cursor(order=order, key=start)             # <<<<<<<<<<<<<<
 * 
 *         for key, value in cursor:
 */
  __pyx_t_7.__pyx_n = 2;
  __pyx_t_7.order = __pyx_cur_scope->__pyx_v_order;
  __pyx_t_7.key = __pyx_cur_scope->__pyx_v_start;
  __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_vtab)->cursor(__pyx_cur_scope->__pyx_v_self, 0, &__pyx_t_7); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5sophy_Cursor))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "sophy.pyx":463
 *         cursor = self.cursor(order=order, key=start)
 * 
 *         for key, value in cursor:             # <<<<<<<<<<<<<<
 *             if stop:
 *                 if reverse and key < stop:
 */
  if (likely(PyList_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) || PyTuple_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) {
    __pyx_t_2 = ((PyObject *)__pyx_cur_scope->__pyx_v_cursor); __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_2 = PyObject_GetIter(((PyObject *)__pyx_cur_scope->__pyx_v_cursor)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_10 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_10); __pyx_t_8++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_10 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_10);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_10); __pyx_t_8++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_10 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_10);
        #endif
      }
    } else {
      __pyx_t_10 = __pyx_t_9(__pyx_t_2);
      if (unlikely(!__pyx_t_10)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_10);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_10))) || (PyList_CheckExact(__pyx_t_10))) {
      PyObject* sequence = __pyx_t_10;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_12 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_12 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_12);
      #else
      __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      #endif
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_13 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_14 = Py_TYPE(__pyx_t_13)->tp_iternext;
      index = 0; __pyx_t_11 = __pyx_t_14(__pyx_t_13); if (unlikely(!__pyx_t_11)) goto __pyx_L19_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 1; __pyx_t_12 = __pyx_t_14(__pyx_t_13); if (unlikely(!__pyx_t_12)) goto __pyx_L19_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_12);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_14(__pyx_t_13), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_14 = NULL;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      goto __pyx_L20_unpacking_done;
      __pyx_L19_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_L20_unpacking_done:;
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_value);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_value, __pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_12);
    __pyx_t_12 = 0;

    /* "sophy.pyx":464
 * 
 *         for key, value in cursor:
 *             if stop:             # <<<<<<<<<<<<<<
 *                 if reverse and key < stop:
 *                     raise StopIteration
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_stop); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_1) {

      /* "sophy.pyx":465
 *         for key, value in cursor:
 *             if stop:
 *                 if reverse and key < stop:             # <<<<<<<<<<<<<<
 *                     raise StopIteration
 *                 elif not reverse and key > stop:
 */
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_reverse); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (__pyx_t_3) {
      } else {
        __pyx_t_1 = __pyx_t_3;
        goto __pyx_L23_bool_binop_done;
      }
      __pyx_t_10 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_stop, Py_LT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_1 = __pyx_t_3;
      __pyx_L23_bool_binop_done:;
      if (__pyx_t_1) {

        /* "sophy.pyx":466
 *             if stop:
 *                 if reverse and key < stop:
 *                     raise StopIteration             # <<<<<<<<<<<<<<
 *                 elif not reverse and key > stop:
 *                     raise StopIteration
 */
        __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "sophy.pyx":465
 *         for key, value in cursor:
 *             if stop:
 *                 if reverse and key < stop:             # <<<<<<<<<<<<<<
 *                     raise StopIteration
 *                 elif not reverse and key > stop:
 */
      }

      /* "sophy.pyx":467
 *                 if reverse and key < stop:
 *                     raise StopIteration
 *                 elif not reverse and key > stop:             # <<<<<<<<<<<<<<
 *                     raise StopIteration
 * 
 */
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_reverse); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_4 = ((!__pyx_t_3) != 0);
      if (__pyx_t_4) {
      } else {
        __pyx_t_1 = __pyx_t_4;
        goto __pyx_L25_bool_binop_done;
      }
      __pyx_t_10 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_stop, Py_GT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_1 = __pyx_t_4;
      __pyx_L25_bool_binop_done:;
      if (__pyx_t_1) {

        /* "sophy.pyx":468
 *                     raise StopIteration
 *                 elif not reverse and key > stop:
 *                     raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *             yield (key, value)
 */
        __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "sophy.pyx":467
 *                 if reverse and key < stop:
 *                     raise StopIteration
 *                 elif not reverse and key > stop:             # <<<<<<<<<<<<<<
 *                     raise StopIteration
 * 
 */
      }

      /* "sophy.pyx":464
 * 
 *         for key, value in cursor:
 *             if stop:             # <<<<<<<<<<<<<<
 *                 if reverse and key < stop:
 *                     raise StopIteration
 */
    }

    /* "sophy.pyx":470
 *                     raise StopIteration
 * 
 *             yield (key, value)             # <<<<<<<<<<<<<<
 * 
 *     def keys(self):
 */
    __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_cur_scope->__pyx_v_key);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_value);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_cur_scope->__pyx_v_value);
    __pyx_r = __pyx_t_10;
    __pyx_t_10 = 0;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_8;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_9;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L27_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_8 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_9 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":463
 *         cursor = self.cursor(order=order, key=start)
 * 
 *         for key, value in cursor:             # <<<<<<<<<<<<<<
 *             if stop:
 *                 if reverse and key < stop:
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":445
 *                     txn[key] = k[key]
 * 
 *     def get_range(self, start=None, stop=None, reverse=False):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Cursor cursor
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("get_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_5sophy_13_BaseDBObject_25generator1(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "sophy.pyx":472
 *             yield (key, value)
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor(values=False)
 *         for item in cursor:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_24keys(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_24keys(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("keys (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_23keys(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_23keys(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("keys", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *)__pyx_tp_new_5sophy___pyx_scope_struct_6_keys(__pyx_ptype_5sophy___pyx_scope_struct_6_keys, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_5sophy_13_BaseDBObject_25generator1, (PyObject *) __pyx_cur_scope, __pyx_n_s_keys, __pyx_n_s_BaseDBObject_keys); if (unlikely(!gen)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 472; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.keys", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_5sophy_13_BaseDBObject_25generator1(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *__pyx_cur_scope = ((struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 472; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":473
 * 
 *     def keys(self):
 *         cdef Cursor cursor = self.cursor(values=False)             # <<<<<<<<<<<<<<
 *         for item in cursor:
 *             yield item
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_values, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5sophy_Cursor))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_cur_scope->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "sophy.pyx":474
 *     def keys(self):
 *         cdef Cursor cursor = self.cursor(values=False)
 *         for item in cursor:             # <<<<<<<<<<<<<<
 *             yield item
 * 
 */
  if (likely(PyList_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) || PyTuple_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) {
    __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_cursor); __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(((PyObject *)__pyx_cur_scope->__pyx_v_cursor)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_item);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_item, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_t_2 = 0;

    /* "sophy.pyx":475
 *         cdef Cursor cursor = self.cursor(values=False)
 *         for item in cursor:
 *             yield item             # <<<<<<<<<<<<<<
 * 
 *     def values(self):
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_item);
    __pyx_r = __pyx_cur_scope->__pyx_v_item;
    __Pyx_XGIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":474
 *     def keys(self):
 *         cdef Cursor cursor = self.cursor(values=False)
 *         for item in cursor:             # <<<<<<<<<<<<<<
 *             yield item
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":472
 *             yield (key, value)
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor(values=False)
 *         for item in cursor:
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("keys", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_5sophy_13_BaseDBObject_28generator2(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "sophy.pyx":477
 *             yield item
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor(keys=False)
 *         for item in cursor:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_27values(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_27values(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("values (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_26values(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_26values(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct_7_values *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("values", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_7_values *)__pyx_tp_new_5sophy___pyx_scope_struct_7_values(__pyx_ptype_5sophy___pyx_scope_struct_7_values, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_5sophy_13_BaseDBObject_28generator2, (PyObject *) __pyx_cur_scope, __pyx_n_s_values, __pyx_n_s_BaseDBObject_values); if (unlikely(!gen)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_5sophy_13_BaseDBObject_28generator2(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_5sophy___pyx_scope_struct_7_values *__pyx_cur_scope = ((struct __pyx_obj_5sophy___pyx_scope_struct_7_values *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":478
 * 
 *     def values(self):
 *         cdef Cursor cursor = self.cursor(keys=False)             # <<<<<<<<<<<<<<
 *         for item in cursor:
 *             yield item
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_keys, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5sophy_Cursor))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_cur_scope->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "sophy.pyx":479
 *     def values(self):
 *         cdef Cursor cursor = self.cursor(keys=False)
 *         for item in cursor:             # <<<<<<<<<<<<<<
 *             yield item
 * 
 */
  if (likely(PyList_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) || PyTuple_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) {
    __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_cursor); __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(((PyObject *)__pyx_cur_scope->__pyx_v_cursor)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 479; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_item);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_item, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_t_2 = 0;

    /* "sophy.pyx":480
 *         cdef Cursor cursor = self.cursor(keys=False)
 *         for item in cursor:
 *             yield item             # <<<<<<<<<<<<<<
 * 
 *     def items(self):
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_item);
    __pyx_r = __pyx_cur_scope->__pyx_v_item;
    __Pyx_XGIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 480; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":479
 *     def values(self):
 *         cdef Cursor cursor = self.cursor(keys=False)
 *         for item in cursor:             # <<<<<<<<<<<<<<
 *             yield item
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":477
 *             yield item
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor(keys=False)
 *         for item in cursor:
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_5sophy_13_BaseDBObject_31generator3(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "sophy.pyx":482
 *             yield item
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor()
 *         for item in cursor:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_30items(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_30items(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("items (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_29items(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_29items(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  struct __pyx_obj_5sophy___pyx_scope_struct_8_items *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("items", 0);
  __pyx_cur_scope = (struct __pyx_obj_5sophy___pyx_scope_struct_8_items *)__pyx_tp_new_5sophy___pyx_scope_struct_8_items(__pyx_ptype_5sophy___pyx_scope_struct_8_items, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __Pyx_GOTREF(__pyx_cur_scope);
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_5sophy_13_BaseDBObject_31generator3, (PyObject *) __pyx_cur_scope, __pyx_n_s_items, __pyx_n_s_BaseDBObject_items); if (unlikely(!gen)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_5sophy_13_BaseDBObject_31generator3(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_5sophy___pyx_scope_struct_8_items *__pyx_cur_scope = ((struct __pyx_obj_5sophy___pyx_scope_struct_8_items *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":483
 * 
 *     def items(self):
 *         cdef Cursor cursor = self.cursor()             # <<<<<<<<<<<<<<
 *         for item in cursor:
 *             yield item
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_vtab)->cursor(__pyx_cur_scope->__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 483; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5sophy_Cursor))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 483; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":484
 *     def items(self):
 *         cdef Cursor cursor = self.cursor()
 *         for item in cursor:             # <<<<<<<<<<<<<<
 *             yield item
 * 
 */
  if (likely(PyList_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) || PyTuple_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_v_cursor))) {
    __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_cursor); __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(((PyObject *)__pyx_cur_scope->__pyx_v_cursor)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 484; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_item);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_item, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;

    /* "sophy.pyx":485
 *         cdef Cursor cursor = self.cursor()
 *         for item in cursor:
 *             yield item             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_item);
    __pyx_r = __pyx_cur_scope->__pyx_v_item;
    __Pyx_XGIVEREF(__pyx_t_1);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":484
 *     def items(self):
 *         cdef Cursor cursor = self.cursor()
 *         for item in cursor:             # <<<<<<<<<<<<<<
 *             yield item
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":482
 *             yield item
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         cdef Cursor cursor = self.cursor()
 *         for item in cursor:
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":487
 *             yield item
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.cursor())
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_33__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_33__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_32__iter__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_32__iter__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "sophy.pyx":488
 * 
 *     def __iter__(self):
 *         return iter(self.cursor())             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__BaseDBObject *)__pyx_v_self->__pyx_base.__pyx_vtab)->cursor(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":487
 *             yield item
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.cursor())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy._BaseDBObject.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":490
 *         return iter(self.cursor())
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int i = 0
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_5sophy_13_BaseDBObject_35__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_5sophy_13_BaseDBObject_35__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_34__len__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_5sophy_13_BaseDBObject_34__len__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  int __pyx_v_i;
  PyObject *__pyx_v_curs = 0;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "sophy.pyx":492
 *     def __len__(self):
 *         cdef:
 *             int i = 0             # <<<<<<<<<<<<<<
 *             curs = self.cursor(keys=False, values=False)
 * 
 */
  __pyx_v_i = 0;

  /* "sophy.pyx":493
 *         cdef:
 *             int i = 0
 *             curs = self.cursor(keys=False, values=False)             # <<<<<<<<<<<<<<
 * 
 *         for _ in curs:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_keys, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_values, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_curs = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "sophy.pyx":495
 *             curs = self.cursor(keys=False, values=False)
 * 
 *         for _ in curs:             # <<<<<<<<<<<<<<
 *             i += 1
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_curs)) || PyTuple_CheckExact(__pyx_v_curs)) {
    __pyx_t_3 = __pyx_v_curs; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_curs); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v__, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "sophy.pyx":496
 * 
 *         for _ in curs:
 *             i += 1             # <<<<<<<<<<<<<<
 * 
 *         return i
 */
    __pyx_v_i = (__pyx_v_i + 1);

    /* "sophy.pyx":495
 *             curs = self.cursor(keys=False, values=False)
 * 
 *         for _ in curs:             # <<<<<<<<<<<<<<
 *             i += 1
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":498
 *             i += 1
 * 
 *         return i             # <<<<<<<<<<<<<<
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 */
  __pyx_r = __pyx_v_i;
  goto __pyx_L0;

  /* "sophy.pyx":490
 *         return iter(self.cursor())
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int i = 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._BaseDBObject.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_curs);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":500
 *         return i
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         raise NotImplemented
 */

static PyObject *__pyx_pw_5sophy_13_BaseDBObject_37cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_13_BaseDBObject_cursor(CYTHON_UNUSED struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor *__pyx_optional_args) {
  PyObject *__pyx_v_order = ((PyObject *)__pyx_kp_s__21);
  PyObject *__pyx_v_key = ((PyObject *)Py_None);
  PyObject *__pyx_v_prefix = ((PyObject *)Py_None);
  PyObject *__pyx_v_keys = ((PyObject *)Py_True);

  /* "sophy.pyx":501
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):             # <<<<<<<<<<<<<<
 *         raise NotImplemented
 * 
 */
  PyObject *__pyx_v_values = ((PyObject *)Py_True);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_order = __pyx_optional_args->order;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_key = __pyx_optional_args->key;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_prefix = __pyx_optional_args->prefix;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_keys = __pyx_optional_args->keys;
            if (__pyx_optional_args->__pyx_n > 4) {
              __pyx_v_values = __pyx_optional_args->values;
            }
          }
        }
      }
    }
  }

  /* "sophy.pyx":500
 *         return i
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         raise NotImplemented
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_37cursor)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(5+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_order);
      __Pyx_GIVEREF(__pyx_v_order);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_order);
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_key);
      __Pyx_INCREF(__pyx_v_prefix);
      __Pyx_GIVEREF(__pyx_v_prefix);
      PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_prefix);
      __Pyx_INCREF(__pyx_v_keys);
      __Pyx_GIVEREF(__pyx_v_keys);
      PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_keys);
      __Pyx_INCREF(__pyx_v_values);
      __Pyx_GIVEREF(__pyx_v_values);
      PyTuple_SET_ITEM(__pyx_t_6, 4+__pyx_t_5, __pyx_v_values);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":502
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):
 *         raise NotImplemented             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_Raise(__pyx_builtin_NotImplemented, 0, 0, 0);
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":500
 *         return i
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         raise NotImplemented
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy._BaseDBObject.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_37cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_37cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_order = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_keys = 0;
  PyObject *__pyx_v_values = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cursor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_order,&__pyx_n_s_key,&__pyx_n_s_prefix,&__pyx_n_s_keys,&__pyx_n_s_values,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[0] = ((PyObject *)__pyx_kp_s__21);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    values[3] = ((PyObject *)Py_True);

    /* "sophy.pyx":501
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):             # <<<<<<<<<<<<<<
 *         raise NotImplemented
 * 
 */
    values[4] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_order);
          if (value) { values[0] = value; kw_args--; }
        }
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prefix);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_keys);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_values);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cursor") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 500; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_order = values[0];
    __pyx_v_key = values[1];
    __pyx_v_prefix = values[2];
    __pyx_v_keys = values[3];
    __pyx_v_values = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cursor", 0, 0, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 500; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy._BaseDBObject.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_36cursor(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_order, __pyx_v_key, __pyx_v_prefix, __pyx_v_keys, __pyx_v_values);

  /* "sophy.pyx":500
 *         return i
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         raise NotImplemented
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_36cursor(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 5;
  __pyx_t_2.order = __pyx_v_order;
  __pyx_t_2.key = __pyx_v_key;
  __pyx_t_2.prefix = __pyx_v_prefix;
  __pyx_t_2.keys = __pyx_v_keys;
  __pyx_t_2.values = __pyx_v_values;
  __pyx_t_1 = __pyx_vtabptr_5sophy__BaseDBObject->cursor(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._BaseDBObject.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":374
 * cdef class _BaseDBObject(_SophiaObject):
 *     cdef:
 *         readonly Sophia sophia             # <<<<<<<<<<<<<<
 *         _Index index
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_6sophia_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_13_BaseDBObject_6sophia_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_13_BaseDBObject_6sophia___get__(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_13_BaseDBObject_6sophia___get__(struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->sophia));
  __pyx_r = ((PyObject *)__pyx_v_self->sophia);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":514
 *         tuple index_type
 * 
 *     def __cinit__(self, Sophia sophia, name, index_type=None):             # <<<<<<<<<<<<<<
 *         self.name = encode(name)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_8Database_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_8Database_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_index_type = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_name,&__pyx_n_s_index_type,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index_type);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_name = values[1];
    __pyx_v_index_type = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Database.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_8Database___cinit__(((struct __pyx_obj_5sophy_Database *)__pyx_v_self), __pyx_v_sophia, __pyx_v_name, __pyx_v_index_type);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_8Database___cinit__(struct __pyx_obj_5sophy_Database *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, PyObject *__pyx_v_name, PyObject *__pyx_v_index_type) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":515
 * 
 *     def __cinit__(self, Sophia sophia, name, index_type=None):
 *         self.name = encode(name)             # <<<<<<<<<<<<<<
 * 
 *         if not index_type:
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 515; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":517
 *         self.name = encode(name)
 * 
 *         if not index_type:             # <<<<<<<<<<<<<<
 *             self.index_type = ('string',)
 *         elif isinstance(index_type, basestring):
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_index_type); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 517; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {

    /* "sophy.pyx":518
 * 
 *         if not index_type:
 *             self.index_type = ('string',)             # <<<<<<<<<<<<<<
 *         elif isinstance(index_type, basestring):
 *             self.index_type = (index_type,)
 */
    __Pyx_INCREF(__pyx_tuple__22);
    __Pyx_GIVEREF(__pyx_tuple__22);
    __Pyx_GOTREF(__pyx_v_self->index_type);
    __Pyx_DECREF(__pyx_v_self->index_type);
    __pyx_v_self->index_type = __pyx_tuple__22;

    /* "sophy.pyx":517
 *         self.name = encode(name)
 * 
 *         if not index_type:             # <<<<<<<<<<<<<<
 *             self.index_type = ('string',)
 *         elif isinstance(index_type, basestring):
 */
    goto __pyx_L3;
  }

  /* "sophy.pyx":519
 *         if not index_type:
 *             self.index_type = ('string',)
 *         elif isinstance(index_type, basestring):             # <<<<<<<<<<<<<<
 *             self.index_type = (index_type,)
 *         else:
 */
  __pyx_t_3 = __Pyx_PyBaseString_Check(__pyx_v_index_type); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":520
 *             self.index_type = ('string',)
 *         elif isinstance(index_type, basestring):
 *             self.index_type = (index_type,)             # <<<<<<<<<<<<<<
 *         else:
 *             self.index_type = tuple(index_type)
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_index_type);
    __Pyx_GIVEREF(__pyx_v_index_type);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_index_type);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->index_type);
    __Pyx_DECREF(__pyx_v_self->index_type);
    __pyx_v_self->index_type = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "sophy.pyx":519
 *         if not index_type:
 *             self.index_type = ('string',)
 *         elif isinstance(index_type, basestring):             # <<<<<<<<<<<<<<
 *             self.index_type = (index_type,)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "sophy.pyx":522
 *             self.index_type = (index_type,)
 *         else:
 *             self.index_type = tuple(index_type)             # <<<<<<<<<<<<<<
 * 
 *         self.config = self.sophia.config
 */
  /*else*/ {
    __pyx_t_1 = PySequence_Tuple(__pyx_v_index_type); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 522; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->index_type);
    __Pyx_DECREF(__pyx_v_self->index_type);
    __pyx_v_self->index_type = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "sophy.pyx":524
 *             self.index_type = tuple(index_type)
 * 
 *         self.config = self.sophia.config             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->__pyx_base.sophia->config);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->config);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->config));
  __pyx_v_self->config = ((struct __pyx_obj_5sophy__ConfigManager *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":514
 *         tuple index_type
 * 
 *     def __cinit__(self, Sophia sophia, name, index_type=None):             # <<<<<<<<<<<<<<
 *         self.name = encode(name)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Database.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":526
 *         self.config = self.sophia.config
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)
 */

/* Python wrapper */
static void __pyx_pw_5sophy_8Database_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5sophy_8Database_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5sophy_8Database_2__dealloc__(((struct __pyx_obj_5sophy_Database *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5sophy_8Database_2__dealloc__(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sophy.pyx":527
 * 
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  __pyx_t_2 = (__pyx_v_self->__pyx_base.sophia->__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->__pyx_base.__pyx_base.handle != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":528
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint close(self):
 */
    sp_destroy(__pyx_v_self->__pyx_base.__pyx_base.handle);

    /* "sophy.pyx":527
 * 
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  }

  /* "sophy.pyx":526
 *         self.config = self.sophia.config
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sophy.pyx":530
 *             sp_destroy(self.handle)
 * 
 *     cpdef bint close(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.is_open:
 *             raise CannotCloseException()
 */

static PyObject *__pyx_pw_5sophy_8Database_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_5sophy_8Database_close(struct __pyx_obj_5sophy_Database *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_close); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_8Database_5close)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":531
 * 
 *     cpdef bint close(self):
 *         if self.sophia.is_open:             # <<<<<<<<<<<<<<
 *             raise CannotCloseException()
 *         return False
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.sophia->is_open != 0);
  if (__pyx_t_5) {

    /* "sophy.pyx":532
 *     cpdef bint close(self):
 *         if self.sophia.is_open:
 *             raise CannotCloseException()             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_CannotCloseException); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":531
 * 
 *     cpdef bint close(self):
 *         if self.sophia.is_open:             # <<<<<<<<<<<<<<
 *             raise CannotCloseException()
 *         return False
 */
  }

  /* "sophy.pyx":533
 *         if self.sophia.is_open:
 *             raise CannotCloseException()
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cdef destroy(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "sophy.pyx":530
 *             sp_destroy(self.handle)
 * 
 *     cpdef bint close(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.is_open:
 *             raise CannotCloseException()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("sophy.Database.close", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_8Database_4close(((struct __pyx_obj_5sophy_Database *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_4close(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_5sophy_8Database_close(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Database.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":535
 *         return False
 * 
 *     cdef destroy(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)
 */

static PyObject *__pyx_f_5sophy_8Database_destroy(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("destroy", 0);

  /* "sophy.pyx":536
 * 
 *     cdef destroy(self):
 *         if self.sophia.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 *             self.handle = <void *>0
 */
  __pyx_t_2 = (__pyx_v_self->__pyx_base.sophia->__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->__pyx_base.__pyx_base.handle != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":537
 *     cdef destroy(self):
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 *             self.handle = <void *>0
 * 
 */
    sp_destroy(__pyx_v_self->__pyx_base.__pyx_base.handle);

    /* "sophy.pyx":538
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)
 *             self.handle = <void *>0             # <<<<<<<<<<<<<<
 * 
 *     cdef configure(self):
 */
    __pyx_v_self->__pyx_base.__pyx_base.handle = ((void *)0);

    /* "sophy.pyx":536
 * 
 *     cdef destroy(self):
 *         if self.sophia.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 *             self.handle = <void *>0
 */
  }

  /* "sophy.pyx":535
 *         return False
 * 
 *     cdef destroy(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.handle:
 *             sp_destroy(self.handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":540
 *             self.handle = <void *>0
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         sp_setstring(self.sophia.handle, 'db', <const char *>self.name, 0)
 *         self.index = self._get_index()
 */

static PyObject *__pyx_f_5sophy_8Database_configure(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("configure", 0);

  /* "sophy.pyx":541
 * 
 *     cdef configure(self):
 *         sp_setstring(self.sophia.handle, 'db', <const char *>self.name, 0)             # <<<<<<<<<<<<<<
 *         self.index = self._get_index()
 *         self.index.configure()
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->name); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 541; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, __pyx_k_db, ((char const *)__pyx_t_1), 0);

  /* "sophy.pyx":542
 *     cdef configure(self):
 *         sp_setstring(self.sophia.handle, 'db', <const char *>self.name, 0)
 *         self.index = self._get_index()             # <<<<<<<<<<<<<<
 *         self.index.configure()
 *         self.mmap = 1
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5sophy_Database *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base._get_index(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self))); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 542; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.index);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.index));
  __pyx_v_self->__pyx_base.index = ((struct __pyx_obj_5sophy__Index *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "sophy.pyx":543
 *         sp_setstring(self.sophia.handle, 'db', <const char *>self.name, 0)
 *         self.index = self._get_index()
 *         self.index.configure()             # <<<<<<<<<<<<<<
 *         self.mmap = 1
 * 
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->__pyx_base.index->__pyx_vtab)->configure(__pyx_v_self->__pyx_base.index); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":544
 *         self.index = self._get_index()
 *         self.index.configure()
 *         self.mmap = 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void *_create_handle(self):
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_mmap, __pyx_int_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 544; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":540
 *             self.handle = <void *>0
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         sp_setstring(self.sophia.handle, 'db', <const char *>self.name, 0)
 *         self.index = self._get_index()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy.Database.configure", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":546
 *         self.mmap = 1
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *handle
 */

static void *__pyx_f_5sophy_8Database__create_handle(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  void *__pyx_v_handle;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  char const *__pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_handle", 0);

  /* "sophy.pyx":549
 *         cdef:
 *             void *handle
 *         handle = sp_getobject(self.sophia.handle, encode('db.%s' % self.name))             # <<<<<<<<<<<<<<
 *         if self.sophia.is_open:
 *             sp_open(handle)
 */
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_db_s, __pyx_v_self->name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 549; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_5sophy_encode(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 549; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 549; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_handle = sp_getobject(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":550
 *             void *handle
 *         handle = sp_getobject(self.sophia.handle, encode('db.%s' % self.name))
 *         if self.sophia.is_open:             # <<<<<<<<<<<<<<
 *             sp_open(handle)
 *         return handle
 */
  __pyx_t_4 = (__pyx_v_self->__pyx_base.sophia->is_open != 0);
  if (__pyx_t_4) {

    /* "sophy.pyx":551
 *         handle = sp_getobject(self.sophia.handle, encode('db.%s' % self.name))
 *         if self.sophia.is_open:
 *             sp_open(handle)             # <<<<<<<<<<<<<<
 *         return handle
 * 
 */
    sp_open(__pyx_v_handle);

    /* "sophy.pyx":550
 *             void *handle
 *         handle = sp_getobject(self.sophia.handle, encode('db.%s' % self.name))
 *         if self.sophia.is_open:             # <<<<<<<<<<<<<<
 *             sp_open(handle)
 *         return handle
 */
  }

  /* "sophy.pyx":552
 *         if self.sophia.is_open:
 *             sp_open(handle)
 *         return handle             # <<<<<<<<<<<<<<
 * 
 *     cdef _Index _get_index(self):
 */
  __pyx_r = __pyx_v_handle;
  goto __pyx_L0;

  /* "sophy.pyx":546
 *         self.mmap = 1
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *handle
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("sophy.Database._create_handle", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":554
 *         return handle
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index index
 */

static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_8Database__get_index(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  struct __pyx_obj_5sophy__Index *__pyx_v_index = 0;
  PyObject *__pyx_v_IndexType = NULL;
  struct __pyx_obj_5sophy__Index *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_index", 0);

  /* "sophy.pyx":558
 *             _Index index
 * 
 *         if len(self.index_type) == 1:             # <<<<<<<<<<<<<<
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 */
  __pyx_t_1 = __pyx_v_self->index_type;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 558; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 558; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = ((__pyx_t_2 == 1) != 0);
  if (__pyx_t_3) {

    /* "sophy.pyx":559
 * 
 *         if len(self.index_type) == 1:
 *             try:             # <<<<<<<<<<<<<<
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 *             except KeyError:
 */
    {
      __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      /*try:*/ {

        /* "sophy.pyx":560
 *         if len(self.index_type) == 1:
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]             # <<<<<<<<<<<<<<
 *             except KeyError:
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 */
        if (unlikely(__pyx_v_5sophy_INDEX_TYPE_MAP == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        }
        if (unlikely(__pyx_v_self->index_type == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        }
        __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v_self->index_type, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L4_error;};
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_5sophy_INDEX_TYPE_MAP, __pyx_t_1); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L4_error;};
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_v_IndexType = __pyx_t_7;
        __pyx_t_7 = 0;

        /* "sophy.pyx":559
 * 
 *         if len(self.index_type) == 1:
 *             try:             # <<<<<<<<<<<<<<
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 *             except KeyError:
 */
      }

      /* "sophy.pyx":565
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 *             else:
 *                 index = IndexType(self.sophia, self)             # <<<<<<<<<<<<<<
 *         else:
 *             index = _MultiIndex(self.sophia, self, index_types=self.index_type)
 */
      /*else:*/ {
        __Pyx_INCREF(__pyx_v_IndexType);
        __pyx_t_1 = __pyx_v_IndexType; __pyx_t_8 = NULL;
        __pyx_t_2 = 0;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
            __pyx_t_2 = 1;
          }
        }
        __pyx_t_9 = PyTuple_New(2+__pyx_t_2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 565; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_2, ((PyObject *)__pyx_v_self->__pyx_base.sophia));
        __Pyx_INCREF(((PyObject *)__pyx_v_self));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_2, ((PyObject *)__pyx_v_self));
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 565; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 565; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __pyx_v_index = ((struct __pyx_obj_5sophy__Index *)__pyx_t_7);
        __pyx_t_7 = 0;
      }
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L11_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "sophy.pyx":561
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 *             except KeyError:             # <<<<<<<<<<<<<<
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 */
      __pyx_t_10 = PyErr_ExceptionMatches(__pyx_builtin_KeyError);
      if (__pyx_t_10) {
        __Pyx_AddTraceback("sophy.Database._get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_1, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 561; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GOTREF(__pyx_t_9);

        /* "sophy.pyx":563
 *             except KeyError:
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))             # <<<<<<<<<<<<<<
 *             else:
 *                 index = IndexType(self.sophia, self)
 */
        __pyx_t_11 = __pyx_v_5sophy_INDEX_TYPE_MAP;
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_12 = PySequence_List(__pyx_t_11); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 563; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_8 = ((PyObject*)__pyx_t_12);
        __pyx_t_12 = 0;
        __pyx_t_13 = PyList_Sort(__pyx_t_8); if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 563; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __pyx_t_12 = __Pyx_PyString_Join(__pyx_kp_s__23, __pyx_t_8); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 563; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Unrecognized_index_type_must_be, __pyx_t_12); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 563; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "sophy.pyx":562
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 *             except KeyError:
 *                 raise ValueError('Unrecognized index type, must be one of: %s'             # <<<<<<<<<<<<<<
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 *             else:
 */
        __pyx_t_12 = PyTuple_New(1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 562; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_12, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 562; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 562; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "sophy.pyx":559
 * 
 *         if len(self.index_type) == 1:
 *             try:             # <<<<<<<<<<<<<<
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 *             except KeyError:
 */
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
      goto __pyx_L1_error;
      __pyx_L11_try_end:;
    }

    /* "sophy.pyx":558
 *             _Index index
 * 
 *         if len(self.index_type) == 1:             # <<<<<<<<<<<<<<
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[self.index_type[0]]
 */
    goto __pyx_L3;
  }

  /* "sophy.pyx":567
 *                 index = IndexType(self.sophia, self)
 *         else:
 *             index = _MultiIndex(self.sophia, self, index_types=self.index_type)             # <<<<<<<<<<<<<<
 * 
 *         return index
 */
  /*else*/ {
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
    PyTuple_SET_ITEM(__pyx_t_9, 0, ((PyObject *)__pyx_v_self->__pyx_base.sophia));
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_9, 1, ((PyObject *)__pyx_v_self));
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_index_types, __pyx_v_self->index_type) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy__MultiIndex), __pyx_t_9, __pyx_t_1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_index = ((struct __pyx_obj_5sophy__Index *)__pyx_t_7);
    __pyx_t_7 = 0;
  }
  __pyx_L3:;

  /* "sophy.pyx":569
 *             index = _MultiIndex(self.sophia, self, index_types=self.index_type)
 * 
 *         return index             # <<<<<<<<<<<<<<
 * 
 *     def update(self, dict _data=None, **k):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_index));
  __pyx_r = __pyx_v_index;
  goto __pyx_L0;

  /* "sophy.pyx":554
 *         return handle
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index index
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("sophy.Database._get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_IndexType);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":571
 *         return index
 * 
 *     def update(self, dict _data=None, **k):             # <<<<<<<<<<<<<<
 *         self._update(self, _data, k)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_7update(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_7update(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v__data = 0;
  PyObject *__pyx_v_k = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("update (wrapper)", 0);
  __pyx_v_k = PyDict_New(); if (unlikely(!__pyx_v_k)) return NULL;
  __Pyx_GOTREF(__pyx_v_k);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_k, values, pos_args, "update") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 571; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v__data = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("update", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 571; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_k); __pyx_v_k = 0;
  __Pyx_AddTraceback("sophy.Database.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v__data), (&PyDict_Type), 1, "_data", 1))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_8Database_6update(((struct __pyx_obj_5sophy_Database *)__pyx_v_self), __pyx_v__data, __pyx_v_k);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_6update(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v__data, PyObject *__pyx_v_k) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_13_BaseDBObject__update __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("update", 0);

  /* "sophy.pyx":572
 * 
 *     def update(self, dict _data=None, **k):
 *         self._update(self, _data, k)             # <<<<<<<<<<<<<<
 * 
 *     cpdef transaction(self):
 */
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2._data = __pyx_v__data;
  __pyx_t_2.k = __pyx_v_k;
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy_Database *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base._update(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), __pyx_v_self, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":571
 *         return index
 * 
 *     def update(self, dict _data=None, **k):             # <<<<<<<<<<<<<<
 *         self._update(self, _data, k)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Database.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":574
 *         self._update(self, _data, k)
 * 
 *     cpdef transaction(self):             # <<<<<<<<<<<<<<
 *         return Transaction(self.sophia, self)
 * 
 */

static PyObject *__pyx_pw_5sophy_8Database_9transaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5sophy_8Database_transaction(struct __pyx_obj_5sophy_Database *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transaction", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_transaction); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_8Database_9transaction)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":575
 * 
 *     cpdef transaction(self):
 *         return Transaction(self.sophia, self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef view(self, name):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 575; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->__pyx_base.sophia));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Transaction), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 575; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":574
 *         self._update(self, _data, k)
 * 
 *     cpdef transaction(self):             # <<<<<<<<<<<<<<
 *         return Transaction(self.sophia, self)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sophy.Database.transaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_9transaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_9transaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transaction (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_8Database_8transaction(((struct __pyx_obj_5sophy_Database *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_8transaction(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transaction", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5sophy_8Database_transaction(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Database.transaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":577
 *         return Transaction(self.sophia, self)
 * 
 *     cpdef view(self, name):             # <<<<<<<<<<<<<<
 *         return View(self.sophia, self, name)
 * 
 */

static PyObject *__pyx_pw_5sophy_8Database_11view(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_f_5sophy_8Database_view(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("view", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_view); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 577; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_8Database_11view)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 577; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 577; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_name);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 577; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":578
 * 
 *     cpdef view(self, name):
 *         return View(self.sophia, self, name)             # <<<<<<<<<<<<<<
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 578; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->__pyx_base.sophia));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_name);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_View), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 578; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":577
 *         return Transaction(self.sophia, self)
 * 
 *     cpdef view(self, name):             # <<<<<<<<<<<<<<
 *         return View(self.sophia, self, name)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy.Database.view", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_11view(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_11view(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("view (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_8Database_10view(((struct __pyx_obj_5sophy_Database *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_10view(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("view", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5sophy_8Database_view(__pyx_v_self, __pyx_v_name, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 577; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Database.view", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":580
 *         return View(self.sophia, self, name)
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */

static PyObject *__pyx_pw_5sophy_8Database_13cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_8Database_cursor(struct __pyx_obj_5sophy_Database *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_8Database_cursor *__pyx_optional_args) {
  PyObject *__pyx_v_order = ((PyObject *)__pyx_kp_s__21);
  PyObject *__pyx_v_key = ((PyObject *)Py_None);
  PyObject *__pyx_v_prefix = ((PyObject *)Py_None);
  PyObject *__pyx_v_keys = ((PyObject *)Py_True);

  /* "sophy.pyx":581
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):             # <<<<<<<<<<<<<<
 *         return Cursor(
 *             sophia=self.sophia,
 */
  PyObject *__pyx_v_values = ((PyObject *)Py_True);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_order = __pyx_optional_args->order;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_key = __pyx_optional_args->key;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_prefix = __pyx_optional_args->prefix;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_keys = __pyx_optional_args->keys;
            if (__pyx_optional_args->__pyx_n > 4) {
              __pyx_v_values = __pyx_optional_args->values;
            }
          }
        }
      }
    }
  }

  /* "sophy.pyx":580
 *         return View(self.sophia, self, name)
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 580; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_8Database_13cursor)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(5+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 580; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_order);
      __Pyx_GIVEREF(__pyx_v_order);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_order);
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_key);
      __Pyx_INCREF(__pyx_v_prefix);
      __Pyx_GIVEREF(__pyx_v_prefix);
      PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_prefix);
      __Pyx_INCREF(__pyx_v_keys);
      __Pyx_GIVEREF(__pyx_v_keys);
      PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_keys);
      __Pyx_INCREF(__pyx_v_values);
      __Pyx_GIVEREF(__pyx_v_values);
      PyTuple_SET_ITEM(__pyx_t_6, 4+__pyx_t_5, __pyx_v_values);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 580; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":582
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):
 *         return Cursor(             # <<<<<<<<<<<<<<
 *             sophia=self.sophia,
 *             db=self,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "sophy.pyx":583
 *                  values=True):
 *         return Cursor(
 *             sophia=self.sophia,             # <<<<<<<<<<<<<<
 *             db=self,
 *             target=self.sophia,
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sophia, ((PyObject *)__pyx_v_self->__pyx_base.sophia)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":584
 *         return Cursor(
 *             sophia=self.sophia,
 *             db=self,             # <<<<<<<<<<<<<<
 *             target=self.sophia,
 *             order=order,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_db, ((PyObject *)__pyx_v_self)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":585
 *             sophia=self.sophia,
 *             db=self,
 *             target=self.sophia,             # <<<<<<<<<<<<<<
 *             order=order,
 *             key=key,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_target, ((PyObject *)__pyx_v_self->__pyx_base.sophia)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":586
 *             db=self,
 *             target=self.sophia,
 *             order=order,             # <<<<<<<<<<<<<<
 *             key=key,
 *             prefix=prefix,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_v_order) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":587
 *             target=self.sophia,
 *             order=order,
 *             key=key,             # <<<<<<<<<<<<<<
 *             prefix=prefix,
 *             keys=keys,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_v_key) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":588
 *             order=order,
 *             key=key,
 *             prefix=prefix,             # <<<<<<<<<<<<<<
 *             keys=keys,
 *             values=values)
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_prefix, __pyx_v_prefix) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":589
 *             key=key,
 *             prefix=prefix,
 *             keys=keys,             # <<<<<<<<<<<<<<
 *             values=values)
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_keys, __pyx_v_keys) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":590
 *             prefix=prefix,
 *             keys=keys,
 *             values=values)             # <<<<<<<<<<<<<<
 * 
 *     # key, is string?, is readonly?
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_values, __pyx_v_values) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":582
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):
 *         return Cursor(             # <<<<<<<<<<<<<<
 *             sophia=self.sophia,
 *             db=self,
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Cursor), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":580
 *         return View(self.sophia, self, name)
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy.Database.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_13cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_13cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_order = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_keys = 0;
  PyObject *__pyx_v_values = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cursor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_order,&__pyx_n_s_key,&__pyx_n_s_prefix,&__pyx_n_s_keys,&__pyx_n_s_values,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[0] = ((PyObject *)__pyx_kp_s__21);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    values[3] = ((PyObject *)Py_True);

    /* "sophy.pyx":581
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):             # <<<<<<<<<<<<<<
 *         return Cursor(
 *             sophia=self.sophia,
 */
    values[4] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_order);
          if (value) { values[0] = value; kw_args--; }
        }
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prefix);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_keys);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_values);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cursor") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 580; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_order = values[0];
    __pyx_v_key = values[1];
    __pyx_v_prefix = values[2];
    __pyx_v_keys = values[3];
    __pyx_v_values = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cursor", 0, 0, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 580; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Database.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_8Database_12cursor(((struct __pyx_obj_5sophy_Database *)__pyx_v_self), __pyx_v_order, __pyx_v_key, __pyx_v_prefix, __pyx_v_keys, __pyx_v_values);

  /* "sophy.pyx":580
 *         return View(self.sophia, self, name)
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_12cursor(struct __pyx_obj_5sophy_Database *__pyx_v_self, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 5;
  __pyx_t_2.order = __pyx_v_order;
  __pyx_t_2.key = __pyx_v_key;
  __pyx_t_2.prefix = __pyx_v_prefix;
  __pyx_t_2.keys = __pyx_v_keys;
  __pyx_t_2.values = __pyx_v_values;
  __pyx_t_1 = __pyx_vtabptr_5sophy_Database->__pyx_base.cursor(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 580; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Database.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":510
 * cdef class Database(_BaseDBObject):
 *     cdef:
 *         readonly bytes name             # <<<<<<<<<<<<<<
 *         readonly _ConfigManager config
 *         tuple index_type
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_8Database_4name___get__(((struct __pyx_obj_5sophy_Database *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_4name___get__(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":511
 *     cdef:
 *         readonly bytes name
 *         readonly _ConfigManager config             # <<<<<<<<<<<<<<
 *         tuple index_type
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_8Database_6config_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_8Database_6config_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_8Database_6config___get__(((struct __pyx_obj_5sophy_Database *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8Database_6config___get__(struct __pyx_obj_5sophy_Database *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->config));
  __pyx_r = ((PyObject *)__pyx_v_self->config);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":637
 *         bytes name
 * 
 *     def __cinit__(self, Sophia sophia, Database db, name):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.name = encode(name)
 */

/* Python wrapper */
static int __pyx_pw_5sophy_4View_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_4View_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  PyObject *__pyx_v_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,&__pyx_n_s_name,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
    __pyx_v_name = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.View.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_4View___cinit__(((struct __pyx_obj_5sophy_View *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db, __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_4View___cinit__(struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":638
 * 
 *     def __cinit__(self, Sophia sophia, Database db, name):
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.name = encode(name)
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "sophy.pyx":639
 *     def __cinit__(self, Sophia sophia, Database db, name):
 *         self.db = db
 *         self.name = encode(name)             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, Sophia sophia, Database db, name):
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 639; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":637
 *         bytes name
 * 
 *     def __cinit__(self, Sophia sophia, Database db, name):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.name = encode(name)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.View.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":641
 *         self.name = encode(name)
 * 
 *     def __init__(self, Sophia sophia, Database db, name):             # <<<<<<<<<<<<<<
 *         self.open()
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_4View_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_4View_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  CYTHON_UNUSED PyObject *__pyx_v_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,&__pyx_n_s_name,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
    __pyx_v_name = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.View.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_4View_2__init__(((struct __pyx_obj_5sophy_View *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db, __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_4View_2__init__(struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db, CYTHON_UNUSED PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "sophy.pyx":642
 * 
 *     def __init__(self, Sophia sophia, Database db, name):
 *         self.open()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  ((struct __pyx_vtabstruct_5sophy_View *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.open(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), 0, NULL);

  /* "sophy.pyx":641
 *         self.name = encode(name)
 * 
 *     def __init__(self, Sophia sophia, Database db, name):             # <<<<<<<<<<<<<<
 *         self.open()
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":644
 *         self.open()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 */

/* Python wrapper */
static void __pyx_pw_5sophy_4View_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5sophy_4View_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5sophy_4View_4__dealloc__(((struct __pyx_obj_5sophy_View *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5sophy_4View_4__dealloc__(struct __pyx_obj_5sophy_View *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sophy.pyx":645
 * 
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  __pyx_t_2 = (__pyx_v_self->__pyx_base.sophia->__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->db->__pyx_base.__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->__pyx_base.__pyx_base.handle != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":646
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 * 
 *     cdef destroy(self):
 */
    sp_destroy(__pyx_v_self->__pyx_base.__pyx_base.handle);

    /* "sophy.pyx":645
 * 
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  }

  /* "sophy.pyx":644
 *         self.open()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sophy.pyx":648
 *             sp_destroy(self.handle)
 * 
 *     cdef destroy(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 */

static PyObject *__pyx_f_5sophy_4View_destroy(struct __pyx_obj_5sophy_View *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("destroy", 0);

  /* "sophy.pyx":649
 * 
 *     cdef destroy(self):
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 *             self.handle = <void *>0
 */
  __pyx_t_2 = (__pyx_v_self->__pyx_base.sophia->__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->db->__pyx_base.__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->__pyx_base.__pyx_base.handle != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":650
 *     cdef destroy(self):
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 *             self.handle = <void *>0
 * 
 */
    sp_destroy(__pyx_v_self->__pyx_base.__pyx_base.handle);

    /* "sophy.pyx":651
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 *             self.handle = <void *>0             # <<<<<<<<<<<<<<
 * 
 *     cdef void *_create_handle(self):
 */
    __pyx_v_self->__pyx_base.__pyx_base.handle = ((void *)0);

    /* "sophy.pyx":649
 * 
 *     cdef destroy(self):
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 *             self.handle = <void *>0
 */
  }

  /* "sophy.pyx":648
 *             sp_destroy(self.handle)
 * 
 *     cdef destroy(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":653
 *             self.handle = <void *>0
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         sp_setstring(self.sophia.handle, 'view', <char *>self.name, 0)
 *         return sp_getobject(self.sophia.handle, encode('view.%s' % self.name))
 */

static void *__pyx_f_5sophy_4View__create_handle(struct __pyx_obj_5sophy_View *__pyx_v_self) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char const *__pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_handle", 0);

  /* "sophy.pyx":654
 * 
 *     cdef void *_create_handle(self):
 *         sp_setstring(self.sophia.handle, 'view', <char *>self.name, 0)             # <<<<<<<<<<<<<<
 *         return sp_getobject(self.sophia.handle, encode('view.%s' % self.name))
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->name); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, __pyx_k_view, ((char *)__pyx_t_1), 0);

  /* "sophy.pyx":655
 *     cdef void *_create_handle(self):
 *         sp_setstring(self.sophia.handle, 'view', <char *>self.name, 0)
 *         return sp_getobject(self.sophia.handle, encode('view.%s' % self.name))             # <<<<<<<<<<<<<<
 * 
 *     cdef _Index _get_index(self):
 */
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_view_s, __pyx_v_self->name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 655; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_5sophy_encode(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 655; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_t_3); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 655; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = sp_getobject(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":653
 *             self.handle = <void *>0
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         sp_setstring(self.sophia.handle, 'view', <char *>self.name, 0)
 *         return sp_getobject(self.sophia.handle, encode('view.%s' % self.name))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("sophy.View._create_handle", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":657
 *         return sp_getobject(self.sophia.handle, encode('view.%s' % self.name))
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         return self.db.index
 * 
 */

static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_4View__get_index(struct __pyx_obj_5sophy_View *__pyx_v_self) {
  struct __pyx_obj_5sophy__Index *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_index", 0);

  /* "sophy.pyx":658
 * 
 *     cdef _Index _get_index(self):
 *         return self.db.index             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, key, value):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->db->__pyx_base.index));
  __pyx_r = __pyx_v_self->db->__pyx_base.index;
  goto __pyx_L0;

  /* "sophy.pyx":657
 *         return sp_getobject(self.sophia.handle, encode('view.%s' % self.name))
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         return self.db.index
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":660
 *         return self.db.index
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         raise ValueError('Views are read-only.')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_4View_7__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5sophy_4View_7__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_4View_6__setitem__(((struct __pyx_obj_5sophy_View *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_4View_6__setitem__(CYTHON_UNUSED struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_key, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "sophy.pyx":661
 * 
 *     def __setitem__(self, key, value):
 *         raise ValueError('Views are read-only.')             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, key):
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":660
 *         return self.db.index
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         raise ValueError('Views are read-only.')
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.View.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":663
 *         raise ValueError('Views are read-only.')
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         raise ValueError('Views are read-only.')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_4View_9__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_5sophy_4View_9__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_4View_8__delitem__(((struct __pyx_obj_5sophy_View *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_4View_8__delitem__(CYTHON_UNUSED struct __pyx_obj_5sophy_View *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "sophy.pyx":664
 * 
 *     def __delitem__(self, key):
 *         raise ValueError('Views are read-only.')             # <<<<<<<<<<<<<<
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 664; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 664; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":663
 *         raise ValueError('Views are read-only.')
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         raise ValueError('Views are read-only.')
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.View.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":666
 *         raise ValueError('Views are read-only.')
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */

static PyObject *__pyx_pw_5sophy_4View_11cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_4View_cursor(struct __pyx_obj_5sophy_View *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_4View_cursor *__pyx_optional_args) {
  PyObject *__pyx_v_order = ((PyObject *)__pyx_kp_s__21);
  PyObject *__pyx_v_key = ((PyObject *)Py_None);
  PyObject *__pyx_v_prefix = ((PyObject *)Py_None);
  PyObject *__pyx_v_keys = ((PyObject *)Py_True);

  /* "sophy.pyx":667
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):             # <<<<<<<<<<<<<<
 *         return Cursor(
 *             sophia=self.sophia,
 */
  PyObject *__pyx_v_values = ((PyObject *)Py_True);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_order = __pyx_optional_args->order;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_key = __pyx_optional_args->key;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_prefix = __pyx_optional_args->prefix;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_keys = __pyx_optional_args->keys;
            if (__pyx_optional_args->__pyx_n > 4) {
              __pyx_v_values = __pyx_optional_args->values;
            }
          }
        }
      }
    }
  }

  /* "sophy.pyx":666
 *         raise ValueError('Views are read-only.')
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_4View_11cursor)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(5+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_order);
      __Pyx_GIVEREF(__pyx_v_order);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_order);
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_key);
      __Pyx_INCREF(__pyx_v_prefix);
      __Pyx_GIVEREF(__pyx_v_prefix);
      PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_prefix);
      __Pyx_INCREF(__pyx_v_keys);
      __Pyx_GIVEREF(__pyx_v_keys);
      PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_keys);
      __Pyx_INCREF(__pyx_v_values);
      __Pyx_GIVEREF(__pyx_v_values);
      PyTuple_SET_ITEM(__pyx_t_6, 4+__pyx_t_5, __pyx_v_values);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":668
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):
 *         return Cursor(             # <<<<<<<<<<<<<<
 *             sophia=self.sophia,
 *             db=self.db,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "sophy.pyx":669
 *                  values=True):
 *         return Cursor(
 *             sophia=self.sophia,             # <<<<<<<<<<<<<<
 *             db=self.db,
 *             target=self,
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sophia, ((PyObject *)__pyx_v_self->__pyx_base.sophia)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":670
 *         return Cursor(
 *             sophia=self.sophia,
 *             db=self.db,             # <<<<<<<<<<<<<<
 *             target=self,
 *             order=order,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_db, ((PyObject *)__pyx_v_self->db)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":671
 *             sophia=self.sophia,
 *             db=self.db,
 *             target=self,             # <<<<<<<<<<<<<<
 *             order=order,
 *             key=key,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_target, ((PyObject *)__pyx_v_self)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":672
 *             db=self.db,
 *             target=self,
 *             order=order,             # <<<<<<<<<<<<<<
 *             key=key,
 *             prefix=prefix,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_v_order) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":673
 *             target=self,
 *             order=order,
 *             key=key,             # <<<<<<<<<<<<<<
 *             prefix=prefix,
 *             keys=keys,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_v_key) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":674
 *             order=order,
 *             key=key,
 *             prefix=prefix,             # <<<<<<<<<<<<<<
 *             keys=keys,
 *             values=values)
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_prefix, __pyx_v_prefix) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":675
 *             key=key,
 *             prefix=prefix,
 *             keys=keys,             # <<<<<<<<<<<<<<
 *             values=values)
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_keys, __pyx_v_keys) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":676
 *             prefix=prefix,
 *             keys=keys,
 *             values=values)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_values, __pyx_v_values) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 669; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":668
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):
 *         return Cursor(             # <<<<<<<<<<<<<<
 *             sophia=self.sophia,
 *             db=self.db,
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Cursor), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":666
 *         raise ValueError('Views are read-only.')
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("sophy.View.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_4View_11cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_4View_11cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_order = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_keys = 0;
  PyObject *__pyx_v_values = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cursor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_order,&__pyx_n_s_key,&__pyx_n_s_prefix,&__pyx_n_s_keys,&__pyx_n_s_values,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[0] = ((PyObject *)__pyx_kp_s__21);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    values[3] = ((PyObject *)Py_True);

    /* "sophy.pyx":667
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 *                  values=True):             # <<<<<<<<<<<<<<
 *         return Cursor(
 *             sophia=self.sophia,
 */
    values[4] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_order);
          if (value) { values[0] = value; kw_args--; }
        }
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prefix);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_keys);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_values);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cursor") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 666; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_order = values[0];
    __pyx_v_key = values[1];
    __pyx_v_prefix = values[2];
    __pyx_v_keys = values[3];
    __pyx_v_values = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cursor", 0, 0, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 666; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.View.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_4View_10cursor(((struct __pyx_obj_5sophy_View *)__pyx_v_self), __pyx_v_order, __pyx_v_key, __pyx_v_prefix, __pyx_v_keys, __pyx_v_values);

  /* "sophy.pyx":666
 *         raise ValueError('Views are read-only.')
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,             # <<<<<<<<<<<<<<
 *                  values=True):
 *         return Cursor(
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_4View_10cursor(struct __pyx_obj_5sophy_View *__pyx_v_self, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 5;
  __pyx_t_2.order = __pyx_v_order;
  __pyx_t_2.key = __pyx_v_key;
  __pyx_t_2.prefix = __pyx_v_prefix;
  __pyx_t_2.keys = __pyx_v_keys;
  __pyx_t_2.values = __pyx_v_values;
  __pyx_t_1 = __pyx_vtabptr_5sophy_View->__pyx_base.cursor(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.View.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":683
 *         Database db
 * 
 *     def __cinit__(self, Sophia sophia, Database db):             # <<<<<<<<<<<<<<
 *         self.db = db
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_11Transaction_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_11Transaction_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Transaction.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_11Transaction___cinit__(((struct __pyx_obj_5sophy_Transaction *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_11Transaction___cinit__(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":684
 * 
 *     def __cinit__(self, Sophia sophia, Database db):
 *         self.db = db             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "sophy.pyx":683
 *         Database db
 * 
 *     def __cinit__(self, Sophia sophia, Database db):             # <<<<<<<<<<<<<<
 *         self.db = db
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":686
 *         self.db = db
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 */

/* Python wrapper */
static void __pyx_pw_5sophy_11Transaction_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5sophy_11Transaction_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5sophy_11Transaction_2__dealloc__(((struct __pyx_obj_5sophy_Transaction *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5sophy_11Transaction_2__dealloc__(struct __pyx_obj_5sophy_Transaction *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sophy.pyx":687
 * 
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  __pyx_t_2 = (__pyx_v_self->__pyx_base.sophia->__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->db->__pyx_base.__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->__pyx_base.__pyx_base.handle != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":688
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 * 
 *     cdef void *_create_handle(self):
 */
    sp_destroy(__pyx_v_self->__pyx_base.__pyx_base.handle);

    /* "sophy.pyx":687
 * 
 *     def __dealloc__(self):
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  }

  /* "sophy.pyx":686
 *         self.db = db
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sophy.pyx":690
 *             sp_destroy(self.handle)
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         return sp_begin(self.sophia.handle)
 * 
 */

static void *__pyx_f_5sophy_11Transaction__create_handle(struct __pyx_obj_5sophy_Transaction *__pyx_v_self) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_handle", 0);

  /* "sophy.pyx":691
 * 
 *     cdef void *_create_handle(self):
 *         return sp_begin(self.sophia.handle)             # <<<<<<<<<<<<<<
 * 
 *     cdef _Index _get_index(self):
 */
  __pyx_r = sp_begin(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle);
  goto __pyx_L0;

  /* "sophy.pyx":690
 *             sp_destroy(self.handle)
 * 
 *     cdef void *_create_handle(self):             # <<<<<<<<<<<<<<
 *         return sp_begin(self.sophia.handle)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":693
 *         return sp_begin(self.sophia.handle)
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         return self.db.index
 * 
 */

static struct __pyx_obj_5sophy__Index *__pyx_f_5sophy_11Transaction__get_index(struct __pyx_obj_5sophy_Transaction *__pyx_v_self) {
  struct __pyx_obj_5sophy__Index *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_index", 0);

  /* "sophy.pyx":694
 * 
 *     cdef _Index _get_index(self):
 *         return self.db.index             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->db->__pyx_base.index));
  __pyx_r = __pyx_v_self->db->__pyx_base.index;
  goto __pyx_L0;

  /* "sophy.pyx":693
 *         return sp_begin(self.sophia.handle)
 * 
 *     cdef _Index _get_index(self):             # <<<<<<<<<<<<<<
 *         return self.db.index
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":696
 *         return self.db.index
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if exc_type:
 *             self.rollback(False)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_11Transaction_5__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_11Transaction_5__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_exc_type,&__pyx_n_s_exc_val,&__pyx_n_s_exc_tb,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exc_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exc_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exc_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__exit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Transaction.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_11Transaction_4__exit__(((struct __pyx_obj_5sophy_Transaction *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_11Transaction_4__exit__(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  struct __pyx_opt_args_5sophy_11Transaction_rollback __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  struct __pyx_opt_args_5sophy_11Transaction_commit __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "sophy.pyx":697
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if exc_type:             # <<<<<<<<<<<<<<
 *             self.rollback(False)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_exc_type); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 697; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "sophy.pyx":698
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if exc_type:
 *             self.rollback(False)             # <<<<<<<<<<<<<<
 *         else:
 *             try:
 */
    __pyx_t_3.__pyx_n = 1;
    __pyx_t_3.begin = Py_False;
    __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy_Transaction *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->rollback(__pyx_v_self, 0, &__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 698; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "sophy.pyx":697
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if exc_type:             # <<<<<<<<<<<<<<
 *             self.rollback(False)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "sophy.pyx":700
 *             self.rollback(False)
 *         else:
 *             try:             # <<<<<<<<<<<<<<
 *                 self.commit(False)
 *             except:
 */
  /*else*/ {
    {
      __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      /*try:*/ {

        /* "sophy.pyx":701
 *         else:
 *             try:
 *                 self.commit(False)             # <<<<<<<<<<<<<<
 *             except:
 *                 self.rollback(False)
 */
        __pyx_t_7.__pyx_n = 1;
        __pyx_t_7.begin = Py_False;
        __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy_Transaction *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->commit(__pyx_v_self, 0, &__pyx_t_7); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 701; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sophy.pyx":700
 *             self.rollback(False)
 *         else:
 *             try:             # <<<<<<<<<<<<<<
 *                 self.commit(False)
 *             except:
 */
      }
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L11_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "sophy.pyx":702
 *             try:
 *                 self.commit(False)
 *             except:             # <<<<<<<<<<<<<<
 *                 self.rollback(False)
 *                 raise
 */
      /*except:*/ {
        __Pyx_AddTraceback("sophy.Transaction.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 702; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GOTREF(__pyx_t_9);

        /* "sophy.pyx":703
 *                 self.commit(False)
 *             except:
 *                 self.rollback(False)             # <<<<<<<<<<<<<<
 *                 raise
 * 
 */
        __pyx_t_3.__pyx_n = 1;
        __pyx_t_3.begin = Py_False;
        __pyx_t_10 = ((struct __pyx_vtabstruct_5sophy_Transaction *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->rollback(__pyx_v_self, 0, &__pyx_t_3); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "sophy.pyx":704
 *             except:
 *                 self.rollback(False)
 *                 raise             # <<<<<<<<<<<<<<
 * 
 *     cpdef commit(self, begin=True):
 */
        __Pyx_GIVEREF(__pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ErrRestore(__pyx_t_2, __pyx_t_8, __pyx_t_9);
        __pyx_t_2 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; 
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      }
      __pyx_L6_except_error:;

      /* "sophy.pyx":700
 *             self.rollback(False)
 *         else:
 *             try:             # <<<<<<<<<<<<<<
 *                 self.commit(False)
 *             except:
 */
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
      goto __pyx_L1_error;
      __pyx_L11_try_end:;
    }
  }
  __pyx_L3:;

  /* "sophy.pyx":696
 *         return self.db.index
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if exc_type:
 *             self.rollback(False)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("sophy.Transaction.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":706
 *                 raise
 * 
 *     cpdef commit(self, begin=True):             # <<<<<<<<<<<<<<
 *         cdef int rc = sp_commit(self.handle)
 *         _check(self.sophia.handle, rc)
 */

static PyObject *__pyx_pw_5sophy_11Transaction_7commit(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_11Transaction_commit(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_commit *__pyx_optional_args) {
  PyObject *__pyx_v_begin = ((PyObject *)Py_True);
  int __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("commit", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_begin = __pyx_optional_args->begin;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_commit); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_11Transaction_7commit)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_begin); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_begin);
        __Pyx_GIVEREF(__pyx_v_begin);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_begin);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":707
 * 
 *     cpdef commit(self, begin=True):
 *         cdef int rc = sp_commit(self.handle)             # <<<<<<<<<<<<<<
 *         _check(self.sophia.handle, rc)
 *         if rc == 1:
 */
  __pyx_v_rc = sp_commit(__pyx_v_self->__pyx_base.__pyx_base.handle);

  /* "sophy.pyx":708
 *     cpdef commit(self, begin=True):
 *         cdef int rc = sp_commit(self.handle)
 *         _check(self.sophia.handle, rc)             # <<<<<<<<<<<<<<
 *         if rc == 1:
 *             raise Exception('transaction was rolled back by another '
 */
  __pyx_t_1 = __pyx_f_5sophy__check(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, __pyx_v_rc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 708; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":709
 *         cdef int rc = sp_commit(self.handle)
 *         _check(self.sophia.handle, rc)
 *         if rc == 1:             # <<<<<<<<<<<<<<
 *             raise Exception('transaction was rolled back by another '
 *                             'concurrent transaction.')
 */
  switch (__pyx_v_rc) {
    case 1:

    /* "sophy.pyx":710
 *         _check(self.sophia.handle, rc)
 *         if rc == 1:
 *             raise Exception('transaction was rolled back by another '             # <<<<<<<<<<<<<<
 *                             'concurrent transaction.')
 *         elif rc == 2:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 710; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 710; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":709
 *         cdef int rc = sp_commit(self.handle)
 *         _check(self.sophia.handle, rc)
 *         if rc == 1:             # <<<<<<<<<<<<<<
 *             raise Exception('transaction was rolled back by another '
 *                             'concurrent transaction.')
 */
    break;

    /* "sophy.pyx":712
 *             raise Exception('transaction was rolled back by another '
 *                             'concurrent transaction.')
 *         elif rc == 2:             # <<<<<<<<<<<<<<
 *             raise Exception('transaction is not finished, waiting for a '
 *                             'concurrent transaction to finish.')
 */
    case 2:

    /* "sophy.pyx":713
 *                             'concurrent transaction.')
 *         elif rc == 2:
 *             raise Exception('transaction is not finished, waiting for a '             # <<<<<<<<<<<<<<
 *                             'concurrent transaction to finish.')
 *         self.handle = <void *>0
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":712
 *             raise Exception('transaction was rolled back by another '
 *                             'concurrent transaction.')
 *         elif rc == 2:             # <<<<<<<<<<<<<<
 *             raise Exception('transaction is not finished, waiting for a '
 *                             'concurrent transaction to finish.')
 */
    break;
    default: break;
  }

  /* "sophy.pyx":715
 *             raise Exception('transaction is not finished, waiting for a '
 *                             'concurrent transaction to finish.')
 *         self.handle = <void *>0             # <<<<<<<<<<<<<<
 *         if begin:
 *             self.open()
 */
  __pyx_v_self->__pyx_base.__pyx_base.handle = ((void *)0);

  /* "sophy.pyx":716
 *                             'concurrent transaction to finish.')
 *         self.handle = <void *>0
 *         if begin:             # <<<<<<<<<<<<<<
 *             self.open()
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_begin); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 716; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "sophy.pyx":717
 *         self.handle = <void *>0
 *         if begin:
 *             self.open()             # <<<<<<<<<<<<<<
 * 
 *     cpdef rollback(self, begin=True):
 */
    ((struct __pyx_vtabstruct_5sophy_Transaction *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.open(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), 0, NULL);

    /* "sophy.pyx":716
 *                             'concurrent transaction to finish.')
 *         self.handle = <void *>0
 *         if begin:             # <<<<<<<<<<<<<<
 *             self.open()
 * 
 */
  }

  /* "sophy.pyx":706
 *                 raise
 * 
 *     cpdef commit(self, begin=True):             # <<<<<<<<<<<<<<
 *         cdef int rc = sp_commit(self.handle)
 *         _check(self.sophia.handle, rc)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy.Transaction.commit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_11Transaction_7commit(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_11Transaction_7commit(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_begin = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("commit (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_begin);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "commit") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_begin = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("commit", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Transaction.commit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_11Transaction_6commit(((struct __pyx_obj_5sophy_Transaction *)__pyx_v_self), __pyx_v_begin);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_11Transaction_6commit(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, PyObject *__pyx_v_begin) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_11Transaction_commit __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("commit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.begin = __pyx_v_begin;
  __pyx_t_1 = __pyx_vtabptr_5sophy_Transaction->commit(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Transaction.commit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":719
 *             self.open()
 * 
 *     cpdef rollback(self, begin=True):             # <<<<<<<<<<<<<<
 *         self.destroy()
 *         if begin:
 */

static PyObject *__pyx_pw_5sophy_11Transaction_9rollback(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5sophy_11Transaction_rollback(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_rollback *__pyx_optional_args) {
  PyObject *__pyx_v_begin = ((PyObject *)Py_True);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("rollback", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_begin = __pyx_optional_args->begin;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rollback); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_11Transaction_9rollback)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_begin); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_begin);
        __Pyx_GIVEREF(__pyx_v_begin);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_begin);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":720
 * 
 *     cpdef rollback(self, begin=True):
 *         self.destroy()             # <<<<<<<<<<<<<<
 *         if begin:
 *             self.open()
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy_Transaction *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.destroy(((struct __pyx_obj_5sophy__SophiaObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":721
 *     cpdef rollback(self, begin=True):
 *         self.destroy()
 *         if begin:             # <<<<<<<<<<<<<<
 *             self.open()
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_begin); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_6) {

    /* "sophy.pyx":722
 *         self.destroy()
 *         if begin:
 *             self.open()             # <<<<<<<<<<<<<<
 * 
 * 
 */
    ((struct __pyx_vtabstruct_5sophy_Transaction *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.open(((struct __pyx_obj_5sophy__BaseDBObject *)__pyx_v_self), 0, NULL);

    /* "sophy.pyx":721
 *     cpdef rollback(self, begin=True):
 *         self.destroy()
 *         if begin:             # <<<<<<<<<<<<<<
 *             self.open()
 * 
 */
  }

  /* "sophy.pyx":719
 *             self.open()
 * 
 *     cpdef rollback(self, begin=True):             # <<<<<<<<<<<<<<
 *         self.destroy()
 *         if begin:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy.Transaction.rollback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_11Transaction_9rollback(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_11Transaction_9rollback(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_begin = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rollback (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_begin);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rollback") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_begin = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rollback", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Transaction.rollback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_11Transaction_8rollback(((struct __pyx_obj_5sophy_Transaction *)__pyx_v_self), __pyx_v_begin);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_11Transaction_8rollback(struct __pyx_obj_5sophy_Transaction *__pyx_v_self, PyObject *__pyx_v_begin) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_5sophy_11Transaction_rollback __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("rollback", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.begin = __pyx_v_begin;
  __pyx_t_1 = __pyx_vtabptr_5sophy_Transaction->rollback(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Transaction.rollback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":738
 *         void *current_item
 * 
 *     def __cinit__(self, Sophia sophia, Database db, _SophiaObject target,             # <<<<<<<<<<<<<<
 *                   order='>=', key=None, prefix=None, keys=True, values=True):
 *         self.sophia = sophia
 */

/* Python wrapper */
static int __pyx_pw_5sophy_6Cursor_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_6Cursor_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  struct __pyx_obj_5sophy__SophiaObject *__pyx_v_target = 0;
  PyObject *__pyx_v_order = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_keys = 0;
  PyObject *__pyx_v_values = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,&__pyx_n_s_target,&__pyx_n_s_order,&__pyx_n_s_key,&__pyx_n_s_prefix,&__pyx_n_s_keys,&__pyx_n_s_values,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    values[3] = ((PyObject *)__pyx_kp_s__21);

    /* "sophy.pyx":739
 * 
 *     def __cinit__(self, Sophia sophia, Database db, _SophiaObject target,
 *                   order='>=', key=None, prefix=None, keys=True, values=True):             # <<<<<<<<<<<<<<
 *         self.sophia = sophia
 *         self.db = db
 */
    values[4] = ((PyObject *)Py_None);
    values[5] = ((PyObject *)Py_None);
    values[6] = ((PyObject *)Py_True);
    values[7] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 8, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 8, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_order);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prefix);
          if (value) { values[5] = value; kw_args--; }
        }
        case  6:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_keys);
          if (value) { values[6] = value; kw_args--; }
        }
        case  7:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_values);
          if (value) { values[7] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
    __pyx_v_target = ((struct __pyx_obj_5sophy__SophiaObject *)values[2]);
    __pyx_v_order = values[3];
    __pyx_v_key = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_keys = values[6];
    __pyx_v_values = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 8, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.Cursor.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_target), __pyx_ptype_5sophy__SophiaObject, 1, "target", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_6Cursor___cinit__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db, __pyx_v_target, __pyx_v_order, __pyx_v_key, __pyx_v_prefix, __pyx_v_keys, __pyx_v_values);

  /* "sophy.pyx":738
 *         void *current_item
 * 
 *     def __cinit__(self, Sophia sophia, Database db, _SophiaObject target,             # <<<<<<<<<<<<<<
 *                   order='>=', key=None, prefix=None, keys=True, values=True):
 *         self.sophia = sophia
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_6Cursor___cinit__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db, struct __pyx_obj_5sophy__SophiaObject *__pyx_v_target, PyObject *__pyx_v_order, PyObject *__pyx_v_key, PyObject *__pyx_v_prefix, PyObject *__pyx_v_keys, PyObject *__pyx_v_values) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":740
 *     def __cinit__(self, Sophia sophia, Database db, _SophiaObject target,
 *                   order='>=', key=None, prefix=None, keys=True, values=True):
 *         self.sophia = sophia             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.target = target
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GOTREF(__pyx_v_self->sophia);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->sophia));
  __pyx_v_self->sophia = __pyx_v_sophia;

  /* "sophy.pyx":741
 *                   order='>=', key=None, prefix=None, keys=True, values=True):
 *         self.sophia = sophia
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.target = target
 *         self.order = encode(order)
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "sophy.pyx":742
 *         self.sophia = sophia
 *         self.db = db
 *         self.target = target             # <<<<<<<<<<<<<<
 *         self.order = encode(order)
 *         self.keys = keys
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_target));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_target));
  __Pyx_GOTREF(__pyx_v_self->target);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->target));
  __pyx_v_self->target = __pyx_v_target;

  /* "sophy.pyx":743
 *         self.db = db
 *         self.target = target
 *         self.order = encode(order)             # <<<<<<<<<<<<<<
 *         self.keys = keys
 *         self.values = values
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_order); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->order);
  __Pyx_DECREF(__pyx_v_self->order);
  __pyx_v_self->order = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":744
 *         self.target = target
 *         self.order = encode(order)
 *         self.keys = keys             # <<<<<<<<<<<<<<
 *         self.values = values
 *         if key:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_keys); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 744; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->keys = __pyx_t_2;

  /* "sophy.pyx":745
 *         self.order = encode(order)
 *         self.keys = keys
 *         self.values = values             # <<<<<<<<<<<<<<
 *         if key:
 *             self.key = key
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_values); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 745; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->values = __pyx_t_2;

  /* "sophy.pyx":746
 *         self.keys = keys
 *         self.values = values
 *         if key:             # <<<<<<<<<<<<<<
 *             self.key = key
 *         if prefix:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_key); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 746; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "sophy.pyx":747
 *         self.values = values
 *         if key:
 *             self.key = key             # <<<<<<<<<<<<<<
 *         if prefix:
 *             self.prefix = encode(prefix)
 */
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    __Pyx_GOTREF(__pyx_v_self->key);
    __Pyx_DECREF(__pyx_v_self->key);
    __pyx_v_self->key = __pyx_v_key;

    /* "sophy.pyx":746
 *         self.keys = keys
 *         self.values = values
 *         if key:             # <<<<<<<<<<<<<<
 *             self.key = key
 *         if prefix:
 */
  }

  /* "sophy.pyx":748
 *         if key:
 *             self.key = key
 *         if prefix:             # <<<<<<<<<<<<<<
 *             self.prefix = encode(prefix)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_prefix); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 748; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "sophy.pyx":749
 *             self.key = key
 *         if prefix:
 *             self.prefix = encode(prefix)             # <<<<<<<<<<<<<<
 * 
 *         self.indexes = self.db.index.keys
 */
    __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_prefix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 749; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->prefix);
    __Pyx_DECREF(__pyx_v_self->prefix);
    __pyx_v_self->prefix = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "sophy.pyx":748
 *         if key:
 *             self.key = key
 *         if prefix:             # <<<<<<<<<<<<<<
 *             self.prefix = encode(prefix)
 * 
 */
  }

  /* "sophy.pyx":751
 *             self.prefix = encode(prefix)
 * 
 *         self.indexes = self.db.index.keys             # <<<<<<<<<<<<<<
 *         self.current_item = <void *>0
 * 
 */
  __pyx_t_1 = __pyx_v_self->db->__pyx_base.index->keys;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->indexes);
  __Pyx_DECREF(__pyx_v_self->indexes);
  __pyx_v_self->indexes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":752
 * 
 *         self.indexes = self.db.index.keys
 *         self.current_item = <void *>0             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->current_item = ((void *)0);

  /* "sophy.pyx":738
 *         void *current_item
 * 
 *     def __cinit__(self, Sophia sophia, Database db, _SophiaObject target,             # <<<<<<<<<<<<<<
 *                   order='>=', key=None, prefix=None, keys=True, values=True):
 *         self.sophia = sophia
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Cursor.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":754
 *         self.current_item = <void *>0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.current_item:
 *             sp_destroy(self.current_item)
 */

/* Python wrapper */
static void __pyx_pw_5sophy_6Cursor_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5sophy_6Cursor_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5sophy_6Cursor_2__dealloc__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5sophy_6Cursor_2__dealloc__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sophy.pyx":755
 * 
 *     def __dealloc__(self):
 *         if self.current_item:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.current_item)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->current_item != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":756
 *     def __dealloc__(self):
 *         if self.current_item:
 *             sp_destroy(self.current_item)             # <<<<<<<<<<<<<<
 * 
 *         if self.sophia.handle and self.db.handle and self.handle:
 */
    sp_destroy(__pyx_v_self->current_item);

    /* "sophy.pyx":755
 * 
 *     def __dealloc__(self):
 *         if self.current_item:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.current_item)
 * 
 */
  }

  /* "sophy.pyx":758
 *             sp_destroy(self.current_item)
 * 
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  __pyx_t_2 = (__pyx_v_self->sophia->__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->db->__pyx_base.__pyx_base.handle != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->__pyx_base.handle != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":759
 * 
 *         if self.sophia.handle and self.db.handle and self.handle:
 *             sp_destroy(self.handle)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
    sp_destroy(__pyx_v_self->__pyx_base.handle);

    /* "sophy.pyx":758
 *             sp_destroy(self.current_item)
 * 
 *         if self.sophia.handle and self.db.handle and self.handle:             # <<<<<<<<<<<<<<
 *             sp_destroy(self.handle)
 * 
 */
  }

  /* "sophy.pyx":754
 *         self.current_item = <void *>0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.current_item:
 *             sp_destroy(self.current_item)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sophy.pyx":761
 *             sp_destroy(self.handle)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *kbuf
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_5__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_5__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_4__iter__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_4__iter__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "sophy.pyx":766
 *             Py_ssize_t klen
 * 
 *         if self.handle:             # <<<<<<<<<<<<<<
 *             self.destroy()
 *         self.handle = sp_cursor(self.target.handle)
 */
  __pyx_t_1 = (__pyx_v_self->__pyx_base.handle != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":767
 * 
 *         if self.handle:
 *             self.destroy()             # <<<<<<<<<<<<<<
 *         self.handle = sp_cursor(self.target.handle)
 *         self.current_item = sp_document(self.db.handle)
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy_Cursor *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.destroy(((struct __pyx_obj_5sophy__SophiaObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "sophy.pyx":766
 *             Py_ssize_t klen
 * 
 *         if self.handle:             # <<<<<<<<<<<<<<
 *             self.destroy()
 *         self.handle = sp_cursor(self.target.handle)
 */
  }

  /* "sophy.pyx":768
 *         if self.handle:
 *             self.destroy()
 *         self.handle = sp_cursor(self.target.handle)             # <<<<<<<<<<<<<<
 *         self.current_item = sp_document(self.db.handle)
 *         if self.key:
 */
  __pyx_v_self->__pyx_base.handle = sp_cursor(__pyx_v_self->target->handle);

  /* "sophy.pyx":769
 *             self.destroy()
 *         self.handle = sp_cursor(self.target.handle)
 *         self.current_item = sp_document(self.db.handle)             # <<<<<<<<<<<<<<
 *         if self.key:
 *             self.db.index.set_key(self.current_item, self.key)
 */
  __pyx_v_self->current_item = sp_document(__pyx_v_self->db->__pyx_base.__pyx_base.handle);

  /* "sophy.pyx":770
 *         self.handle = sp_cursor(self.target.handle)
 *         self.current_item = sp_document(self.db.handle)
 *         if self.key:             # <<<<<<<<<<<<<<
 *             self.db.index.set_key(self.current_item, self.key)
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->key); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "sophy.pyx":771
 *         self.current_item = sp_document(self.db.handle)
 *         if self.key:
 *             self.db.index.set_key(self.current_item, self.key)             # <<<<<<<<<<<<<<
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)
 *         if self.prefix:
 */
    __pyx_t_2 = __pyx_v_self->key;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->db->__pyx_base.index->__pyx_vtab)->set_key(__pyx_v_self->db->__pyx_base.index, __pyx_v_self->current_item, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sophy.pyx":770
 *         self.handle = sp_cursor(self.target.handle)
 *         self.current_item = sp_document(self.db.handle)
 *         if self.key:             # <<<<<<<<<<<<<<
 *             self.db.index.set_key(self.current_item, self.key)
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)
 */
  }

  /* "sophy.pyx":772
 *         if self.key:
 *             self.db.index.set_key(self.current_item, self.key)
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)             # <<<<<<<<<<<<<<
 *         if self.prefix:
 *             sp_setstring(
 */
  __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_self->order); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 772; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_self->current_item, __pyx_k_order, ((char *)__pyx_t_4), 0);

  /* "sophy.pyx":773
 *             self.db.index.set_key(self.current_item, self.key)
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)
 *         if self.prefix:             # <<<<<<<<<<<<<<
 *             sp_setstring(
 *                 self.current_item,
 */
  __pyx_t_1 = (__pyx_v_self->prefix != Py_None) && (PyBytes_GET_SIZE(__pyx_v_self->prefix) != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":777
 *                 self.current_item,
 *                 'prefix',
 *                 <char *>self.prefix,             # <<<<<<<<<<<<<<
 *                 (sizeof(char) * len(self.prefix)))
 *         return self
 */
    __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_self->prefix); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":778
 *                 'prefix',
 *                 <char *>self.prefix,
 *                 (sizeof(char) * len(self.prefix)))             # <<<<<<<<<<<<<<
 *         return self
 * 
 */
    __pyx_t_3 = __pyx_v_self->prefix;
    __Pyx_INCREF(__pyx_t_3);
    if (unlikely(__pyx_t_3 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 778; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_5 = PyBytes_GET_SIZE(__pyx_t_3); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 778; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sophy.pyx":774
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)
 *         if self.prefix:
 *             sp_setstring(             # <<<<<<<<<<<<<<
 *                 self.current_item,
 *                 'prefix',
 */
    sp_setstring(__pyx_v_self->current_item, __pyx_k_prefix, ((char *)__pyx_t_4), ((sizeof(char)) * __pyx_t_5));

    /* "sophy.pyx":773
 *             self.db.index.set_key(self.current_item, self.key)
 *         sp_setstring(self.current_item, 'order', <char *>self.order, 0)
 *         if self.prefix:             # <<<<<<<<<<<<<<
 *             sp_setstring(
 *                 self.current_item,
 */
  }

  /* "sophy.pyx":779
 *                 <char *>self.prefix,
 *                 (sizeof(char) * len(self.prefix)))
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __next__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "sophy.pyx":761
 *             sp_destroy(self.handle)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *kbuf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.Cursor.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":781
 *         return self
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         self.current_item = sp_get(self.handle, self.current_item)
 *         if not self.current_item:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_7__next__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_7__next__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__next__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_6__next__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_6__next__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_v_bkey = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__", 0);

  /* "sophy.pyx":782
 * 
 *     def __next__(self):
 *         self.current_item = sp_get(self.handle, self.current_item)             # <<<<<<<<<<<<<<
 *         if not self.current_item:
 *             #self.destroy()
 */
  __pyx_v_self->current_item = sp_get(__pyx_v_self->__pyx_base.handle, __pyx_v_self->current_item);

  /* "sophy.pyx":783
 *     def __next__(self):
 *         self.current_item = sp_get(self.handle, self.current_item)
 *         if not self.current_item:             # <<<<<<<<<<<<<<
 *             #self.destroy()
 *             raise StopIteration
 */
  __pyx_t_1 = ((!(__pyx_v_self->current_item != 0)) != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":785
 *         if not self.current_item:
 *             #self.destroy()
 *             raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *         cdef bkey
 */
    __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 785; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":783
 *     def __next__(self):
 *         self.current_item = sp_get(self.handle, self.current_item)
 *         if not self.current_item:             # <<<<<<<<<<<<<<
 *             #self.destroy()
 *             raise StopIteration
 */
  }

  /* "sophy.pyx":788
 * 
 *         cdef bkey
 *         if self.keys and self.values:             # <<<<<<<<<<<<<<
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return (bkey,
 */
  __pyx_t_2 = (__pyx_v_self->keys != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->values != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sophy.pyx":789
 *         cdef bkey
 *         if self.keys and self.values:
 *             bkey = self.db.index.extract_key(self.current_item)             # <<<<<<<<<<<<<<
 *             return (bkey,
 *                     _getstring(self.current_item, 'value'))
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->db->__pyx_base.index->__pyx_vtab)->extract_key(__pyx_v_self->db->__pyx_base.index, __pyx_v_self->current_item); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_bkey = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "sophy.pyx":790
 *         if self.keys and self.values:
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return (bkey,             # <<<<<<<<<<<<<<
 *                     _getstring(self.current_item, 'value'))
 *         elif self.keys:
 */
    __Pyx_XDECREF(__pyx_r);

    /* "sophy.pyx":791
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return (bkey,
 *                     _getstring(self.current_item, 'value'))             # <<<<<<<<<<<<<<
 *         elif self.keys:
 *             bkey = self.db.index.extract_key(self.current_item)
 */
    __pyx_t_3 = __pyx_f_5sophy__getstring(__pyx_v_self->current_item, __pyx_k_value); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "sophy.pyx":790
 *         if self.keys and self.values:
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return (bkey,             # <<<<<<<<<<<<<<
 *                     _getstring(self.current_item, 'value'))
 *         elif self.keys:
 */
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 790; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_bkey);
    __Pyx_GIVEREF(__pyx_v_bkey);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_bkey);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":788
 * 
 *         cdef bkey
 *         if self.keys and self.values:             # <<<<<<<<<<<<<<
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return (bkey,
 */
  }

  /* "sophy.pyx":792
 *             return (bkey,
 *                     _getstring(self.current_item, 'value'))
 *         elif self.keys:             # <<<<<<<<<<<<<<
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return bkey
 */
  __pyx_t_1 = (__pyx_v_self->keys != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":793
 *                     _getstring(self.current_item, 'value'))
 *         elif self.keys:
 *             bkey = self.db.index.extract_key(self.current_item)             # <<<<<<<<<<<<<<
 *             return bkey
 *         elif self.values:
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->db->__pyx_base.index->__pyx_vtab)->extract_key(__pyx_v_self->db->__pyx_base.index, __pyx_v_self->current_item); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_bkey = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "sophy.pyx":794
 *         elif self.keys:
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return bkey             # <<<<<<<<<<<<<<
 *         elif self.values:
 *             return _getstring(self.current_item, 'value')
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_bkey);
    __pyx_r = __pyx_v_bkey;
    goto __pyx_L0;

    /* "sophy.pyx":792
 *             return (bkey,
 *                     _getstring(self.current_item, 'value'))
 *         elif self.keys:             # <<<<<<<<<<<<<<
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return bkey
 */
  }

  /* "sophy.pyx":795
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return bkey
 *         elif self.values:             # <<<<<<<<<<<<<<
 *             return _getstring(self.current_item, 'value')
 * 
 */
  __pyx_t_1 = (__pyx_v_self->values != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":796
 *             return bkey
 *         elif self.values:
 *             return _getstring(self.current_item, 'value')             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __pyx_f_5sophy__getstring(__pyx_v_self->current_item, __pyx_k_value); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":795
 *             bkey = self.db.index.extract_key(self.current_item)
 *             return bkey
 *         elif self.values:             # <<<<<<<<<<<<<<
 *             return _getstring(self.current_item, 'value')
 * 
 */
  }

  /* "sophy.pyx":781
 *         return self
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         self.current_item = sp_get(self.handle, self.current_item)
 *         if not self.current_item:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sophy.Cursor.__next__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bkey);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":730
 *         Database db
 *         _SophiaObject target
 *         readonly bytes order             # <<<<<<<<<<<<<<
 *         readonly bytes prefix
 *         readonly bint keys
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_5order_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_5order_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_5order___get__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_5order___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->order);
  __pyx_r = __pyx_v_self->order;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":731
 *         _SophiaObject target
 *         readonly bytes order
 *         readonly bytes prefix             # <<<<<<<<<<<<<<
 *         readonly bint keys
 *         readonly bint values
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_6prefix_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_6prefix_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_6prefix___get__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_6prefix___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __pyx_r = __pyx_v_self->prefix;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":732
 *         readonly bytes order
 *         readonly bytes prefix
 *         readonly bint keys             # <<<<<<<<<<<<<<
 *         readonly bint values
 *         readonly object key
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_4keys_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_4keys_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_4keys___get__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_4keys___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->keys); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Cursor.keys.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":733
 *         readonly bytes prefix
 *         readonly bint keys
 *         readonly bint values             # <<<<<<<<<<<<<<
 *         readonly object key
 *         tuple indexes
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_6values_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_6values_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_6values___get__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_6values___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->values); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.Cursor.values.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":734
 *         readonly bint keys
 *         readonly bint values
 *         readonly object key             # <<<<<<<<<<<<<<
 *         tuple indexes
 *         void *current_item
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_6Cursor_3key_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_6Cursor_3key_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_6Cursor_3key___get__(((struct __pyx_obj_5sophy_Cursor *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_6Cursor_3key___get__(struct __pyx_obj_5sophy_Cursor *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->key);
  __pyx_r = __pyx_v_self->key;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":810
 *     index_type = 'string'
 * 
 *     def __cinit__(self, Sophia sophia, Database db, key='key', **_):             # <<<<<<<<<<<<<<
 *         self.sophia = sophia
 *         self.db = db
 */

/* Python wrapper */
static int __pyx_pw_5sophy_6_Index_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_6_Index_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  PyObject *__pyx_v_key = 0;
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  __pyx_v__ = PyDict_New(); if (unlikely(!__pyx_v__)) return -1;
  __Pyx_GOTREF(__pyx_v__);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,&__pyx_n_s_key,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)__pyx_n_s_key);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v__, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
    __pyx_v_key = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v__); __pyx_v__ = 0;
  __Pyx_AddTraceback("sophy._Index.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_6_Index___cinit__(((struct __pyx_obj_5sophy__Index *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db, __pyx_v_key, __pyx_v__);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_6_Index___cinit__(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, struct __pyx_obj_5sophy_Database *__pyx_v_db, PyObject *__pyx_v_key, CYTHON_UNUSED PyObject *__pyx_v__) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":811
 * 
 *     def __cinit__(self, Sophia sophia, Database db, key='key', **_):
 *         self.sophia = sophia             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.key = encode(key)
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_sophia));
  __Pyx_GOTREF(__pyx_v_self->sophia);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->sophia));
  __pyx_v_self->sophia = __pyx_v_sophia;

  /* "sophy.pyx":812
 *     def __cinit__(self, Sophia sophia, Database db, key='key', **_):
 *         self.sophia = sophia
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.key = encode(key)
 *         self.keys = (self.key,)
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "sophy.pyx":813
 *         self.sophia = sophia
 *         self.db = db
 *         self.key = encode(key)             # <<<<<<<<<<<<<<
 *         self.keys = (self.key,)
 *         self.b_path = encode('db.%s.index.%s' % (db.name, key))
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->key);
  __Pyx_DECREF(__pyx_v_self->key);
  __pyx_v_self->key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":814
 *         self.db = db
 *         self.key = encode(key)
 *         self.keys = (self.key,)             # <<<<<<<<<<<<<<
 *         self.b_path = encode('db.%s.index.%s' % (db.name, key))
 *         self.b_type = encode(self.index_type)
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 814; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->key);
  __Pyx_GIVEREF(__pyx_v_self->key);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->key);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->keys);
  __Pyx_DECREF(__pyx_v_self->keys);
  __pyx_v_self->keys = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":815
 *         self.key = encode(key)
 *         self.keys = (self.key,)
 *         self.b_path = encode('db.%s.index.%s' % (db.name, key))             # <<<<<<<<<<<<<<
 *         self.b_type = encode(self.index_type)
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 815; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_db->name);
  __Pyx_GIVEREF(__pyx_v_db->name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_db->name);
  __Pyx_INCREF(__pyx_v_key);
  __Pyx_GIVEREF(__pyx_v_key);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_key);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_db_s_index_s, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 815; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 815; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->b_path);
  __Pyx_DECREF(__pyx_v_self->b_path);
  __pyx_v_self->b_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":816
 *         self.keys = (self.key,)
 *         self.b_path = encode('db.%s.index.%s' % (db.name, key))
 *         self.b_type = encode(self.index_type)             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, Sophia sophia, Database db, key='key', **_):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index_type); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_5sophy_encode(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->b_type);
  __Pyx_DECREF(__pyx_v_self->b_type);
  __pyx_v_self->b_type = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "sophy.pyx":810
 *     index_type = 'string'
 * 
 *     def __cinit__(self, Sophia sophia, Database db, key='key', **_):             # <<<<<<<<<<<<<<
 *         self.sophia = sophia
 *         self.db = db
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy._Index.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":818
 *         self.b_type = encode(self.index_type)
 * 
 *     def __init__(self, Sophia sophia, Database db, key='key', **_):             # <<<<<<<<<<<<<<
 *         self.empty_value = self._get_empty_value()
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_6_Index_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_6_Index_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  CYTHON_UNUSED PyObject *__pyx_v_key = 0;
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v__ = PyDict_New(); if (unlikely(!__pyx_v__)) return -1;
  __Pyx_GOTREF(__pyx_v__);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,&__pyx_n_s_key,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)__pyx_n_s_key);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v__, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
    __pyx_v_key = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v__); __pyx_v__ = 0;
  __Pyx_AddTraceback("sophy._Index.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_6_Index_2__init__(((struct __pyx_obj_5sophy__Index *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db, __pyx_v_key, __pyx_v__);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_6_Index_2__init__(struct __pyx_obj_5sophy__Index *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db, CYTHON_UNUSED PyObject *__pyx_v_key, CYTHON_UNUSED PyObject *__pyx_v__) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "sophy.pyx":819
 * 
 *     def __init__(self, Sophia sophia, Database db, key='key', **_):
 *         self.empty_value = self._get_empty_value()             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_empty_value(self):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->__pyx_vtab)->_get_empty_value(__pyx_v_self); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 819; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->empty_value);
  __Pyx_DECREF(__pyx_v_self->empty_value);
  __pyx_v_self->empty_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":818
 *         self.b_type = encode(self.index_type)
 * 
 *     def __init__(self, Sophia sophia, Database db, key='key', **_):             # <<<<<<<<<<<<<<
 *         self.empty_value = self._get_empty_value()
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._Index.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":821
 *         self.empty_value = self._get_empty_value()
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         return ''
 * 
 */

static PyObject *__pyx_f_5sophy_6_Index__get_empty_value(CYTHON_UNUSED struct __pyx_obj_5sophy__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_empty_value", 0);

  /* "sophy.pyx":822
 * 
 *     cdef _get_empty_value(self):
 *         return ''             # <<<<<<<<<<<<<<
 * 
 *     cdef configure(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_kp_s__28);
  __pyx_r = __pyx_kp_s__28;
  goto __pyx_L0;

  /* "sophy.pyx":821
 *         self.empty_value = self._get_empty_value()
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         return ''
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":824
 *         return ''
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         sp_setstring(
 *             self.sophia.handle,
 */

static PyObject *__pyx_f_5sophy_6_Index_configure(struct __pyx_obj_5sophy__Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  char *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("configure", 0);

  /* "sophy.pyx":827
 *         sp_setstring(
 *             self.sophia.handle,
 *             <char *>self.b_path,             # <<<<<<<<<<<<<<
 *             <char *>self.b_type,
 *             0)
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->b_path); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":828
 *             self.sophia.handle,
 *             <char *>self.b_path,
 *             <char *>self.b_type,             # <<<<<<<<<<<<<<
 *             0)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_self->b_type); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":825
 * 
 *     cdef configure(self):
 *         sp_setstring(             # <<<<<<<<<<<<<<
 *             self.sophia.handle,
 *             <char *>self.b_path,
 */
  sp_setstring(__pyx_v_self->sophia->__pyx_base.handle, ((char *)__pyx_t_1), ((char *)__pyx_t_2), 0);

  /* "sophy.pyx":824
 *         return ''
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         sp_setstring(
 *             self.sophia.handle,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._Index.configure", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":831
 *             0)
 * 
 *     cdef set(self, _BaseDBObject target, key, value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *kbuf
 */

static PyObject *__pyx_f_5sophy_6_Index_set(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  char *__pyx_v_vbuf;
  Py_ssize_t __pyx_v_vlen;
  void *__pyx_v_obj;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "sophy.pyx":836
 *             char *vbuf
 *             Py_ssize_t klen, vlen
 *             void *obj = sp_document(self.db.handle)             # <<<<<<<<<<<<<<
 * 
 *         self.set_key(obj, key)
 */
  __pyx_v_obj = sp_document(__pyx_v_self->db->__pyx_base.__pyx_base.handle);

  /* "sophy.pyx":838
 *             void *obj = sp_document(self.db.handle)
 * 
 *         self.set_key(obj, key)             # <<<<<<<<<<<<<<
 * 
 *         if IS_PY3K:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->__pyx_vtab)->set_key(__pyx_v_self, __pyx_v_obj, __pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":840
 *         self.set_key(obj, key)
 * 
 *         if IS_PY3K:             # <<<<<<<<<<<<<<
 *             value = encode(value)
 *         PyBytes_AsStringAndSize(value, &vbuf, &vlen)
 */
  __pyx_t_2 = (__pyx_v_5sophy_IS_PY3K != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":841
 * 
 *         if IS_PY3K:
 *             value = encode(value)             # <<<<<<<<<<<<<<
 *         PyBytes_AsStringAndSize(value, &vbuf, &vlen)
 *         sp_setstring(obj, 'value', vbuf, vlen + 1)
 */
    __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sophy.pyx":840
 *         self.set_key(obj, key)
 * 
 *         if IS_PY3K:             # <<<<<<<<<<<<<<
 *             value = encode(value)
 *         PyBytes_AsStringAndSize(value, &vbuf, &vlen)
 */
  }

  /* "sophy.pyx":842
 *         if IS_PY3K:
 *             value = encode(value)
 *         PyBytes_AsStringAndSize(value, &vbuf, &vlen)             # <<<<<<<<<<<<<<
 *         sp_setstring(obj, 'value', vbuf, vlen + 1)
 * 
 */
  __pyx_t_3 = PyBytes_AsStringAndSize(__pyx_v_value, (&__pyx_v_vbuf), (&__pyx_v_vlen)); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":843
 *             value = encode(value)
 *         PyBytes_AsStringAndSize(value, &vbuf, &vlen)
 *         sp_setstring(obj, 'value', vbuf, vlen + 1)             # <<<<<<<<<<<<<<
 * 
 *         _check(self.sophia.handle, sp_set(target.handle, obj))
 */
  sp_setstring(__pyx_v_obj, __pyx_k_value, __pyx_v_vbuf, (__pyx_v_vlen + 1));

  /* "sophy.pyx":845
 *         sp_setstring(obj, 'value', vbuf, vlen + 1)
 * 
 *         _check(self.sophia.handle, sp_set(target.handle, obj))             # <<<<<<<<<<<<<<
 * 
 *     cdef set_key(self, void *obj, key):
 */
  __pyx_t_1 = __pyx_f_5sophy__check(__pyx_v_self->sophia->__pyx_base.handle, sp_set(__pyx_v_target->__pyx_base.handle, __pyx_v_obj)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 845; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":831
 *             0)
 * 
 *     cdef set(self, _BaseDBObject target, key, value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *kbuf
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._Index.set", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":847
 *         _check(self.sophia.handle, sp_set(target.handle, obj))
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *kbuf
 */

static PyObject *__pyx_f_5sophy_6_Index_set_key(struct __pyx_obj_5sophy__Index *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key) {
  char *__pyx_v_kbuf;
  Py_ssize_t __pyx_v_klen;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_key", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "sophy.pyx":852
 *             Py_ssize_t klen
 * 
 *         if IS_PY3K:             # <<<<<<<<<<<<<<
 *             key = encode(key)
 *         PyBytes_AsStringAndSize(key, &kbuf, &klen)
 */
  __pyx_t_1 = (__pyx_v_5sophy_IS_PY3K != 0);
  if (__pyx_t_1) {

    /* "sophy.pyx":853
 * 
 *         if IS_PY3K:
 *             key = encode(key)             # <<<<<<<<<<<<<<
 *         PyBytes_AsStringAndSize(key, &kbuf, &klen)
 *         sp_setstring(obj, <char *>self.key, kbuf, klen + 1)
 */
    __pyx_t_2 = __pyx_f_5sophy_encode(__pyx_v_key); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 853; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "sophy.pyx":852
 *             Py_ssize_t klen
 * 
 *         if IS_PY3K:             # <<<<<<<<<<<<<<
 *             key = encode(key)
 *         PyBytes_AsStringAndSize(key, &kbuf, &klen)
 */
  }

  /* "sophy.pyx":854
 *         if IS_PY3K:
 *             key = encode(key)
 *         PyBytes_AsStringAndSize(key, &kbuf, &klen)             # <<<<<<<<<<<<<<
 *         sp_setstring(obj, <char *>self.key, kbuf, klen + 1)
 * 
 */
  __pyx_t_3 = PyBytes_AsStringAndSize(__pyx_v_key, (&__pyx_v_kbuf), (&__pyx_v_klen)); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 854; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":855
 *             key = encode(key)
 *         PyBytes_AsStringAndSize(key, &kbuf, &klen)
 *         sp_setstring(obj, <char *>self.key, kbuf, klen + 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef extract_key(self, void *obj):
 */
  __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_self->key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 855; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_obj, ((char *)__pyx_t_4), __pyx_v_kbuf, (__pyx_v_klen + 1));

  /* "sophy.pyx":847
 *         _check(self.sophia.handle, sp_set(target.handle, obj))
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *kbuf
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy._Index.set_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":857
 *         sp_setstring(obj, <char *>self.key, kbuf, klen + 1)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         return _getstring(obj, self.key)
 * 
 */

static PyObject *__pyx_f_5sophy_6_Index_extract_key(struct __pyx_obj_5sophy__Index *__pyx_v_self, void *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extract_key", 0);

  /* "sophy.pyx":858
 * 
 *     cdef extract_key(self, void *obj):
 *         return _getstring(obj, self.key)             # <<<<<<<<<<<<<<
 * 
 *     cdef get(self, _BaseDBObject target, key):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->key); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 858; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __pyx_f_5sophy__getstring(__pyx_v_obj, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 858; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":857
 *         sp_setstring(obj, <char *>self.key, kbuf, klen + 1)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         return _getstring(obj, self.key)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy._Index.extract_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":860
 *         return _getstring(obj, self.key)
 * 
 *     cdef get(self, _BaseDBObject target, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *obj = sp_document(self.db.handle)
 */

static PyObject *__pyx_f_5sophy_6_Index_get(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key) {
  void *__pyx_v_obj;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);

  /* "sophy.pyx":862
 *     cdef get(self, _BaseDBObject target, key):
 *         cdef:
 *             void *obj = sp_document(self.db.handle)             # <<<<<<<<<<<<<<
 * 
 *         self.set_key(obj, key)
 */
  __pyx_v_obj = sp_document(__pyx_v_self->db->__pyx_base.__pyx_base.handle);

  /* "sophy.pyx":864
 *             void *obj = sp_document(self.db.handle)
 * 
 *         self.set_key(obj, key)             # <<<<<<<<<<<<<<
 * 
 *         obj = sp_get(target.handle, obj)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->__pyx_vtab)->set_key(__pyx_v_self, __pyx_v_obj, __pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":866
 *         self.set_key(obj, key)
 * 
 *         obj = sp_get(target.handle, obj)             # <<<<<<<<<<<<<<
 *         if not obj:
 *             raise KeyError(key)
 */
  __pyx_v_obj = sp_get(__pyx_v_target->__pyx_base.handle, __pyx_v_obj);

  /* "sophy.pyx":867
 * 
 *         obj = sp_get(target.handle, obj)
 *         if not obj:             # <<<<<<<<<<<<<<
 *             raise KeyError(key)
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_obj != 0)) != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":868
 *         obj = sp_get(target.handle, obj)
 *         if not obj:
 *             raise KeyError(key)             # <<<<<<<<<<<<<<
 * 
 *         return _getstring(obj, 'value')
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 868; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_key);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_KeyError, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 868; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 868; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":867
 * 
 *         obj = sp_get(target.handle, obj)
 *         if not obj:             # <<<<<<<<<<<<<<
 *             raise KeyError(key)
 * 
 */
  }

  /* "sophy.pyx":870
 *             raise KeyError(key)
 * 
 *         return _getstring(obj, 'value')             # <<<<<<<<<<<<<<
 * 
 *     cdef delete(self, _BaseDBObject target, key):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_f_5sophy__getstring(__pyx_v_obj, __pyx_k_value); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":860
 *         return _getstring(obj, self.key)
 * 
 *     cdef get(self, _BaseDBObject target, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *obj = sp_document(self.db.handle)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._Index.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":872
 *         return _getstring(obj, 'value')
 * 
 *     cdef delete(self, _BaseDBObject target, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *obj = sp_document(self.db.handle)
 */

static PyObject *__pyx_f_5sophy_6_Index_delete(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key) {
  void *__pyx_v_obj;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "sophy.pyx":874
 *     cdef delete(self, _BaseDBObject target, key):
 *         cdef:
 *             void *obj = sp_document(self.db.handle)             # <<<<<<<<<<<<<<
 * 
 *         self.set_key(obj, key)
 */
  __pyx_v_obj = sp_document(__pyx_v_self->db->__pyx_base.__pyx_base.handle);

  /* "sophy.pyx":876
 *             void *obj = sp_document(self.db.handle)
 * 
 *         self.set_key(obj, key)             # <<<<<<<<<<<<<<
 *         _check(self.sophia.handle, sp_delete(target.handle, obj))
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->__pyx_vtab)->set_key(__pyx_v_self, __pyx_v_obj, __pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 876; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":877
 * 
 *         self.set_key(obj, key)
 *         _check(self.sophia.handle, sp_delete(target.handle, obj))             # <<<<<<<<<<<<<<
 * 
 *     cdef exists(self, _BaseDBObject target, key):
 */
  __pyx_t_1 = __pyx_f_5sophy__check(__pyx_v_self->sophia->__pyx_base.handle, sp_delete(__pyx_v_target->__pyx_base.handle, __pyx_v_obj)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 877; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":872
 *         return _getstring(obj, 'value')
 * 
 *     cdef delete(self, _BaseDBObject target, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *obj = sp_document(self.db.handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._Index.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":879
 *         _check(self.sophia.handle, sp_delete(target.handle, obj))
 * 
 *     cdef exists(self, _BaseDBObject target, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *obj = sp_document(self.db.handle)
 */

static PyObject *__pyx_f_5sophy_6_Index_exists(struct __pyx_obj_5sophy__Index *__pyx_v_self, struct __pyx_obj_5sophy__BaseDBObject *__pyx_v_target, PyObject *__pyx_v_key) {
  void *__pyx_v_obj;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("exists", 0);

  /* "sophy.pyx":881
 *     cdef exists(self, _BaseDBObject target, key):
 *         cdef:
 *             void *obj = sp_document(self.db.handle)             # <<<<<<<<<<<<<<
 * 
 *         self.set_key(obj, key)
 */
  __pyx_v_obj = sp_document(__pyx_v_self->db->__pyx_base.__pyx_base.handle);

  /* "sophy.pyx":883
 *             void *obj = sp_document(self.db.handle)
 * 
 *         self.set_key(obj, key)             # <<<<<<<<<<<<<<
 * 
 *         obj = sp_get(target.handle, obj)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_self->__pyx_vtab)->set_key(__pyx_v_self, __pyx_v_obj, __pyx_v_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 883; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":885
 *         self.set_key(obj, key)
 * 
 *         obj = sp_get(target.handle, obj)             # <<<<<<<<<<<<<<
 *         if not obj:
 *             return False
 */
  __pyx_v_obj = sp_get(__pyx_v_target->__pyx_base.handle, __pyx_v_obj);

  /* "sophy.pyx":886
 * 
 *         obj = sp_get(target.handle, obj)
 *         if not obj:             # <<<<<<<<<<<<<<
 *             return False
 *         return True
 */
  __pyx_t_2 = ((!(__pyx_v_obj != 0)) != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":887
 *         obj = sp_get(target.handle, obj)
 *         if not obj:
 *             return False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "sophy.pyx":886
 * 
 *         obj = sp_get(target.handle, obj)
 *         if not obj:             # <<<<<<<<<<<<<<
 *             return False
 *         return True
 */
  }

  /* "sophy.pyx":888
 *         if not obj:
 *             return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_True);
  __pyx_r = Py_True;
  goto __pyx_L0;

  /* "sophy.pyx":879
 *         _check(self.sophia.handle, sp_delete(target.handle, obj))
 * 
 *     cdef exists(self, _BaseDBObject target, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *obj = sp_document(self.db.handle)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy._Index.exists", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":894
 *     index_type = 'u32'
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */

static PyObject *__pyx_f_5sophy_12_UInt32Index__get_empty_value(CYTHON_UNUSED struct __pyx_obj_5sophy__UInt32Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_empty_value", 0);

  /* "sophy.pyx":895
 * 
 *     cdef _get_empty_value(self):
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef set_key(self, void *obj, key):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_0);
  __pyx_r = __pyx_int_0;
  goto __pyx_L0;

  /* "sophy.pyx":894
 *     index_type = 'u32'
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":897
 *         return 0
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint32_t* key_ptr = <uint32_t*>malloc(sizeof(uint32_t))
 */

static PyObject *__pyx_f_5sophy_12_UInt32Index_set_key(struct __pyx_obj_5sophy__UInt32Index *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key) {
  uint32_t *__pyx_v_key_ptr;
  Py_ssize_t __pyx_v_klen;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32_t __pyx_t_1;
  char *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_key", 0);

  /* "sophy.pyx":899
 *     cdef set_key(self, void *obj, key):
 *         cdef:
 *             uint32_t* key_ptr = <uint32_t*>malloc(sizeof(uint32_t))             # <<<<<<<<<<<<<<
 *             Py_ssize_t klen = sizeof(uint32_t)
 * 
 */
  __pyx_v_key_ptr = ((uint32_t *)malloc((sizeof(uint32_t))));

  /* "sophy.pyx":900
 *         cdef:
 *             uint32_t* key_ptr = <uint32_t*>malloc(sizeof(uint32_t))
 *             Py_ssize_t klen = sizeof(uint32_t)             # <<<<<<<<<<<<<<
 * 
 *         key_ptr[0] = <uint32_t>key
 */
  __pyx_v_klen = (sizeof(uint32_t));

  /* "sophy.pyx":902
 *             Py_ssize_t klen = sizeof(uint32_t)
 * 
 *         key_ptr[0] = <uint32_t>key             # <<<<<<<<<<<<<<
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint32_t(__pyx_v_key); if (unlikely((__pyx_t_1 == (uint32_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 902; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_key_ptr[0]) = ((uint32_t)__pyx_t_1);

  /* "sophy.pyx":903
 * 
 *         key_ptr[0] = <uint32_t>key
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef extract_key(self, void *obj):
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_self->__pyx_base.key); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_obj, ((char *)__pyx_t_2), __pyx_v_key_ptr, (__pyx_v_klen + 1));

  /* "sophy.pyx":897
 *         return 0
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint32_t* key_ptr = <uint32_t*>malloc(sizeof(uint32_t))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._UInt32Index.set_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":905
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int nlen
 */

static PyObject *__pyx_f_5sophy_12_UInt32Index_extract_key(struct __pyx_obj_5sophy__UInt32Index *__pyx_v_self, void *__pyx_v_obj) {
  int __pyx_v_nlen;
  void *__pyx_v_ptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extract_key", 0);

  /* "sophy.pyx":910
 *             void *ptr
 * 
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)             # <<<<<<<<<<<<<<
 *         if ptr:
 *             return <uint32_t>((<uint32_t *>ptr)[0])
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->__pyx_base.key); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ptr = sp_getstring(__pyx_v_obj, ((char *)__pyx_t_1), (&__pyx_v_nlen));

  /* "sophy.pyx":911
 * 
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)
 *         if ptr:             # <<<<<<<<<<<<<<
 *             return <uint32_t>((<uint32_t *>ptr)[0])
 * 
 */
  __pyx_t_2 = (__pyx_v_ptr != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":912
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)
 *         if ptr:
 *             return <uint32_t>((<uint32_t *>ptr)[0])             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint32_t(((uint32_t)(((uint32_t *)__pyx_v_ptr)[0]))); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":911
 * 
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)
 *         if ptr:             # <<<<<<<<<<<<<<
 *             return <uint32_t>((<uint32_t *>ptr)[0])
 * 
 */
  }

  /* "sophy.pyx":905
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int nlen
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._UInt32Index.extract_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":922
 *     index_type = 'u64'
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */

static PyObject *__pyx_f_5sophy_12_UInt64Index__get_empty_value(CYTHON_UNUSED struct __pyx_obj_5sophy__UInt64Index *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_empty_value", 0);

  /* "sophy.pyx":923
 * 
 *     cdef _get_empty_value(self):
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef set_key(self, void *obj, key):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_0);
  __pyx_r = __pyx_int_0;
  goto __pyx_L0;

  /* "sophy.pyx":922
 *     index_type = 'u64'
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":925
 *         return 0
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint64_t* key_ptr = <uint64_t*>malloc(sizeof(uint64_t))
 */

static PyObject *__pyx_f_5sophy_12_UInt64Index_set_key(struct __pyx_obj_5sophy__UInt64Index *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key) {
  uint64_t *__pyx_v_key_ptr;
  Py_ssize_t __pyx_v_klen;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint64_t __pyx_t_1;
  char *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_key", 0);

  /* "sophy.pyx":927
 *     cdef set_key(self, void *obj, key):
 *         cdef:
 *             uint64_t* key_ptr = <uint64_t*>malloc(sizeof(uint64_t))             # <<<<<<<<<<<<<<
 *             Py_ssize_t klen = sizeof(uint64_t)
 * 
 */
  __pyx_v_key_ptr = ((uint64_t *)malloc((sizeof(uint64_t))));

  /* "sophy.pyx":928
 *         cdef:
 *             uint64_t* key_ptr = <uint64_t*>malloc(sizeof(uint64_t))
 *             Py_ssize_t klen = sizeof(uint64_t)             # <<<<<<<<<<<<<<
 * 
 *         key_ptr[0] = <uint64_t>key
 */
  __pyx_v_klen = (sizeof(uint64_t));

  /* "sophy.pyx":930
 *             Py_ssize_t klen = sizeof(uint64_t)
 * 
 *         key_ptr[0] = <uint64_t>key             # <<<<<<<<<<<<<<
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint64_t(__pyx_v_key); if (unlikely((__pyx_t_1 == (uint64_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 930; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_key_ptr[0]) = ((uint64_t)__pyx_t_1);

  /* "sophy.pyx":931
 * 
 *         key_ptr[0] = <uint64_t>key
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef extract_key(self, void *obj):
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_self->__pyx_base.key); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 931; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  sp_setstring(__pyx_v_obj, ((char *)__pyx_t_2), __pyx_v_key_ptr, (__pyx_v_klen + 1));

  /* "sophy.pyx":925
 *         return 0
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint64_t* key_ptr = <uint64_t*>malloc(sizeof(uint64_t))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sophy._UInt64Index.set_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":933
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int nlen
 */

static PyObject *__pyx_f_5sophy_12_UInt64Index_extract_key(struct __pyx_obj_5sophy__UInt64Index *__pyx_v_self, void *__pyx_v_obj) {
  int __pyx_v_nlen;
  void *__pyx_v_ptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extract_key", 0);

  /* "sophy.pyx":938
 *             void *ptr
 * 
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)             # <<<<<<<<<<<<<<
 *         if ptr:
 *             return <uint64_t>((<uint64_t *>ptr)[0])
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->__pyx_base.key); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 938; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ptr = sp_getstring(__pyx_v_obj, ((char *)__pyx_t_1), (&__pyx_v_nlen));

  /* "sophy.pyx":939
 * 
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)
 *         if ptr:             # <<<<<<<<<<<<<<
 *             return <uint64_t>((<uint64_t *>ptr)[0])
 * 
 */
  __pyx_t_2 = (__pyx_v_ptr != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":940
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)
 *         if ptr:
 *             return <uint64_t>((<uint64_t *>ptr)[0])             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint64_t(((uint64_t)(((uint64_t *)__pyx_v_ptr)[0]))); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sophy.pyx":939
 * 
 *         ptr = sp_getstring(obj, <char *>self.key, &nlen)
 *         if ptr:             # <<<<<<<<<<<<<<
 *             return <uint64_t>((<uint64_t *>ptr)[0])
 * 
 */
  }

  /* "sophy.pyx":933
 *         sp_setstring(obj, <char *>self.key, key_ptr, klen + 1)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int nlen
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._UInt64Index.extract_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":951
 *         tuple indexes
 * 
 *     def __cinit__(self, Sophia sophia, Database db, key='key', index_types=None,             # <<<<<<<<<<<<<<
 *                   **_):
 *         if not index_types:
 */

/* Python wrapper */
static int __pyx_pw_5sophy_11_MultiIndex_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_11_MultiIndex_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db = 0;
  CYTHON_UNUSED PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_index_types = 0;
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  __pyx_v__ = PyDict_New(); if (unlikely(!__pyx_v__)) return -1;
  __Pyx_GOTREF(__pyx_v__);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sophia,&__pyx_n_s_db,&__pyx_n_s_key,&__pyx_n_s_index_types,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = ((PyObject *)__pyx_n_s_key);
    values[3] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sophia)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_key);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index_types);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v__, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)values[0]);
    __pyx_v_db = ((struct __pyx_obj_5sophy_Database *)values[1]);
    __pyx_v_key = values[2];
    __pyx_v_index_types = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v__); __pyx_v__ = 0;
  __Pyx_AddTraceback("sophy._MultiIndex.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sophia), __pyx_ptype_5sophy_Sophia, 1, "sophia", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_5sophy_Database, 1, "db", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 951; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_11_MultiIndex___cinit__(((struct __pyx_obj_5sophy__MultiIndex *)__pyx_v_self), __pyx_v_sophia, __pyx_v_db, __pyx_v_key, __pyx_v_index_types, __pyx_v__);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_11_MultiIndex___cinit__(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia, CYTHON_UNUSED struct __pyx_obj_5sophy_Database *__pyx_v_db, CYTHON_UNUSED PyObject *__pyx_v_key, PyObject *__pyx_v_index_types, CYTHON_UNUSED PyObject *__pyx_v__) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sophy.pyx":953
 *     def __cinit__(self, Sophia sophia, Database db, key='key', index_types=None,
 *                   **_):
 *         if not index_types:             # <<<<<<<<<<<<<<
 *             raise ValueError('index_types is a required parameter.')
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_index_types); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 953; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "sophy.pyx":954
 *                   **_):
 *         if not index_types:
 *             raise ValueError('index_types is a required parameter.')             # <<<<<<<<<<<<<<
 * 
 *         self.initialize_indexes(index_types)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":953
 *     def __cinit__(self, Sophia sophia, Database db, key='key', index_types=None,
 *                   **_):
 *         if not index_types:             # <<<<<<<<<<<<<<
 *             raise ValueError('index_types is a required parameter.')
 * 
 */
  }

  /* "sophy.pyx":956
 *             raise ValueError('index_types is a required parameter.')
 * 
 *         self.initialize_indexes(index_types)             # <<<<<<<<<<<<<<
 * 
 *     cdef initialize_indexes(self, tuple index_types):
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v_index_types))||((__pyx_v_index_types) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_index_types)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy__MultiIndex *)__pyx_v_self->__pyx_base.__pyx_vtab)->initialize_indexes(__pyx_v_self, ((PyObject*)__pyx_v_index_types)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 956; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sophy.pyx":951
 *         tuple indexes
 * 
 *     def __cinit__(self, Sophia sophia, Database db, key='key', index_types=None,             # <<<<<<<<<<<<<<
 *                   **_):
 *         if not index_types:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._MultiIndex.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":958
 *         self.initialize_indexes(index_types)
 * 
 *     cdef initialize_indexes(self, tuple index_types):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes bkey = encode('key')
 */

static PyObject *__pyx_f_5sophy_11_MultiIndex_initialize_indexes(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, PyObject *__pyx_v_index_types) {
  PyObject *__pyx_v_bkey = 0;
  PyObject *__pyx_v_suffixes = 0;
  struct __pyx_obj_5sophy__Index *__pyx_v_index = 0;
  int __pyx_v_i;
  PyObject *__pyx_v_accum = 0;
  PyObject *__pyx_v_subindex = NULL;
  PyObject *__pyx_v_IndexType = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("initialize_indexes", 0);

  /* "sophy.pyx":960
 *     cdef initialize_indexes(self, tuple index_types):
 *         cdef:
 *             bytes bkey = encode('key')             # <<<<<<<<<<<<<<
 *             bytes suffixes = encode(' bcdefgh')
 *             _Index index
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_n_s_key); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 960; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bkey = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":961
 *         cdef:
 *             bytes bkey = encode('key')
 *             bytes suffixes = encode(' bcdefgh')             # <<<<<<<<<<<<<<
 *             _Index index
 *             int i
 */
  __pyx_t_1 = __pyx_f_5sophy_encode(__pyx_kp_s_bcdefgh); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 961; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_suffixes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":964
 *             _Index index
 *             int i
 *             list accum = []             # <<<<<<<<<<<<<<
 * 
 *         for i, subindex in enumerate(index_types):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 964; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_accum = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":966
 *             list accum = []
 * 
 *         for i, subindex in enumerate(index_types):             # <<<<<<<<<<<<<<
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 */
  __pyx_t_2 = 0;
  __pyx_t_1 = __pyx_v_index_types; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 966; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 966; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_subindex, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_v_i = __pyx_t_2;
    __pyx_t_2 = (__pyx_t_2 + 1);

    /* "sophy.pyx":967
 * 
 *         for i, subindex in enumerate(index_types):
 *             try:             # <<<<<<<<<<<<<<
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 *             except KeyError:
 */
    {
      __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      /*try:*/ {

        /* "sophy.pyx":968
 *         for i, subindex in enumerate(index_types):
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[subindex]             # <<<<<<<<<<<<<<
 *             except KeyError:
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 */
        if (unlikely(__pyx_v_5sophy_INDEX_TYPE_MAP == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 968; __pyx_clineno = __LINE__; goto __pyx_L5_error;}
        }
        __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_5sophy_INDEX_TYPE_MAP, __pyx_v_subindex); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 968; __pyx_clineno = __LINE__; goto __pyx_L5_error;};
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_XDECREF_SET(__pyx_v_IndexType, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "sophy.pyx":967
 * 
 *         for i, subindex in enumerate(index_types):
 *             try:             # <<<<<<<<<<<<<<
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 *             except KeyError:
 */
      }
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L12_try_end;
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sophy.pyx":969
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 *             except KeyError:             # <<<<<<<<<<<<<<
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 */
      __pyx_t_8 = PyErr_ExceptionMatches(__pyx_builtin_KeyError);
      if (__pyx_t_8) {
        __Pyx_AddTraceback("sophy._MultiIndex.initialize_indexes", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_9, &__pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 969; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GOTREF(__pyx_t_10);

        /* "sophy.pyx":971
 *             except KeyError:
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))             # <<<<<<<<<<<<<<
 *             if i > 0:
 *                 bkey = encode('key_%s' % suffixes[i])
 */
        __pyx_t_12 = __pyx_v_5sophy_INDEX_TYPE_MAP;
        __Pyx_INCREF(__pyx_t_12);
        __pyx_t_13 = PySequence_List(__pyx_t_12); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 971; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_11 = ((PyObject*)__pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_14 = PyList_Sort(__pyx_t_11); if (unlikely(__pyx_t_14 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 971; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __pyx_t_13 = __Pyx_PyString_Join(__pyx_kp_s__23, __pyx_t_11); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 971; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = __Pyx_PyString_Format(__pyx_kp_s_Unrecognized_index_type_must_be, __pyx_t_13); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 971; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

        /* "sophy.pyx":970
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 *             except KeyError:
 *                 raise ValueError('Unrecognized index type, must be one of: %s'             # <<<<<<<<<<<<<<
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 *             if i > 0:
 */
        __pyx_t_13 = PyTuple_New(1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11);
        __pyx_t_11 = 0;
        __pyx_t_11 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_13, NULL); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_Raise(__pyx_t_11, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 970; __pyx_clineno = __LINE__; goto __pyx_L7_except_error;}
      }
      goto __pyx_L7_except_error;
      __pyx_L7_except_error:;

      /* "sophy.pyx":967
 * 
 *         for i, subindex in enumerate(index_types):
 *             try:             # <<<<<<<<<<<<<<
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 *             except KeyError:
 */
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      goto __pyx_L1_error;
      __pyx_L12_try_end:;
    }

    /* "sophy.pyx":972
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 *             if i > 0:             # <<<<<<<<<<<<<<
 *                 bkey = encode('key_%s' % suffixes[i])
 * 
 */
    __pyx_t_15 = ((__pyx_v_i > 0) != 0);
    if (__pyx_t_15) {

      /* "sophy.pyx":973
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 *             if i > 0:
 *                 bkey = encode('key_%s' % suffixes[i])             # <<<<<<<<<<<<<<
 * 
 *             accum.append(IndexType(self.sophia, self.db, key=bkey))
 */
      __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_suffixes, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(__pyx_t_10 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 973; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_key__s, __pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 973; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = __pyx_f_5sophy_encode(__pyx_t_9); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 973; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF_SET(__pyx_v_bkey, ((PyObject*)__pyx_t_10));
      __pyx_t_10 = 0;

      /* "sophy.pyx":972
 *                 raise ValueError('Unrecognized index type, must be one of: %s'
 *                                  % ', '.join(sorted(INDEX_TYPE_MAP)))
 *             if i > 0:             # <<<<<<<<<<<<<<
 *                 bkey = encode('key_%s' % suffixes[i])
 * 
 */
    }

    /* "sophy.pyx":975
 *                 bkey = encode('key_%s' % suffixes[i])
 * 
 *             accum.append(IndexType(self.sophia, self.db, key=bkey))             # <<<<<<<<<<<<<<
 * 
 *         self.indexes = tuple(accum)
 */
    __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 975; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.sophia));
    PyTuple_SET_ITEM(__pyx_t_10, 0, ((PyObject *)__pyx_v_self->__pyx_base.sophia));
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.db));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.db));
    PyTuple_SET_ITEM(__pyx_t_10, 1, ((PyObject *)__pyx_v_self->__pyx_base.db));
    __pyx_t_9 = PyDict_New(); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 975; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_key, __pyx_v_bkey) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 975; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_v_IndexType, __pyx_t_10, __pyx_t_9); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 975; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_accum, __pyx_t_4); if (unlikely(__pyx_t_14 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 975; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sophy.pyx":966
 *             list accum = []
 * 
 *         for i, subindex in enumerate(index_types):             # <<<<<<<<<<<<<<
 *             try:
 *                 IndexType = INDEX_TYPE_MAP[subindex]
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":977
 *             accum.append(IndexType(self.sophia, self.db, key=bkey))
 * 
 *         self.indexes = tuple(accum)             # <<<<<<<<<<<<<<
 *         self.keys = tuple([index.key for index in accum])
 * 
 */
  __pyx_t_1 = PyList_AsTuple(__pyx_v_accum); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 977; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->indexes);
  __Pyx_DECREF(__pyx_v_self->indexes);
  __pyx_v_self->indexes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":978
 * 
 *         self.indexes = tuple(accum)
 *         self.keys = tuple([index.key for index in accum])             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_empty_value(self):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_v_accum; __Pyx_INCREF(__pyx_t_4); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_4)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_9 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_9); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_9 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    #endif
    if (!(likely(((__pyx_t_9) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_9, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_index, ((struct __pyx_obj_5sophy__Index *)__pyx_t_9));
    __pyx_t_9 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_v_index->key))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.keys);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.keys);
  __pyx_v_self->__pyx_base.keys = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "sophy.pyx":958
 *         self.initialize_indexes(index_types)
 * 
 *     cdef initialize_indexes(self, tuple index_types):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes bkey = encode('key')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("sophy._MultiIndex.initialize_indexes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bkey);
  __Pyx_XDECREF(__pyx_v_suffixes);
  __Pyx_XDECREF((PyObject *)__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_accum);
  __Pyx_XDECREF(__pyx_v_subindex);
  __Pyx_XDECREF(__pyx_v_IndexType);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":980
 *         self.keys = tuple([index.key for index in accum])
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index idx
 */

static PyObject *__pyx_f_5sophy_11_MultiIndex__get_empty_value(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self) {
  struct __pyx_obj_5sophy__Index *__pyx_v_idx = 0;
  PyObject *__pyx_v_accum = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_empty_value", 0);

  /* "sophy.pyx":983
 *         cdef:
 *             _Index idx
 *             list accum = []             # <<<<<<<<<<<<<<
 * 
 *         for idx in self.indexes:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 983; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_accum = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":985
 *             list accum = []
 * 
 *         for idx in self.indexes:             # <<<<<<<<<<<<<<
 *             accum.append(idx.empty_value)
 *         return tuple(accum)
 */
  if (unlikely(__pyx_v_self->indexes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 985; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_self->indexes; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 985; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 985; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 985; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_idx, ((struct __pyx_obj_5sophy__Index *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "sophy.pyx":986
 * 
 *         for idx in self.indexes:
 *             accum.append(idx.empty_value)             # <<<<<<<<<<<<<<
 *         return tuple(accum)
 * 
 */
    __pyx_t_3 = __pyx_v_idx->empty_value;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_accum, __pyx_t_3); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 986; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sophy.pyx":985
 *             list accum = []
 * 
 *         for idx in self.indexes:             # <<<<<<<<<<<<<<
 *             accum.append(idx.empty_value)
 *         return tuple(accum)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":987
 *         for idx in self.indexes:
 *             accum.append(idx.empty_value)
 *         return tuple(accum)             # <<<<<<<<<<<<<<
 * 
 *     cdef configure(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_AsTuple(__pyx_v_accum); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":980
 *         self.keys = tuple([index.key for index in accum])
 * 
 *     cdef _get_empty_value(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index idx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._MultiIndex._get_empty_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_idx);
  __Pyx_XDECREF(__pyx_v_accum);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":989
 *         return tuple(accum)
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes db_idx_path = encode('db.%s.index' % self.db.name)
 */

static PyObject *__pyx_f_5sophy_11_MultiIndex_configure(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self) {
  PyObject *__pyx_v_db_idx_path = 0;
  struct __pyx_obj_5sophy__Index *__pyx_v_index = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  char *__pyx_t_4;
  char *__pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("configure", 0);

  /* "sophy.pyx":991
 *     cdef configure(self):
 *         cdef:
 *             bytes db_idx_path = encode('db.%s.index' % self.db.name)             # <<<<<<<<<<<<<<
 *             _Index index
 *             bindex_type
 */
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_db_s_index, __pyx_v_self->__pyx_base.db->name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_5sophy_encode(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_db_idx_path = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "sophy.pyx":995
 *             bindex_type
 * 
 *         for index in self.indexes[1:]:             # <<<<<<<<<<<<<<
 *             sp_setstring(
 *                 self.sophia.handle,
 */
  if (unlikely(__pyx_v_self->indexes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 995; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = __Pyx_PyTuple_GetSlice(__pyx_v_self->indexes, 1, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 995; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 995; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 995; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 995; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_index, ((struct __pyx_obj_5sophy__Index *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "sophy.pyx":998
 *             sp_setstring(
 *                 self.sophia.handle,
 *                 <char *>db_idx_path,             # <<<<<<<<<<<<<<
 *                 <char *>index.key,
 *                 0)
 */
    __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_db_idx_path); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 998; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":999
 *                 self.sophia.handle,
 *                 <char *>db_idx_path,
 *                 <char *>index.key,             # <<<<<<<<<<<<<<
 *                 0)
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_index->key); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 999; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":996
 * 
 *         for index in self.indexes[1:]:
 *             sp_setstring(             # <<<<<<<<<<<<<<
 *                 self.sophia.handle,
 *                 <char *>db_idx_path,
 */
    sp_setstring(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, ((char *)__pyx_t_4), ((char *)__pyx_t_5), 0);

    /* "sophy.pyx":995
 *             bindex_type
 * 
 *         for index in self.indexes[1:]:             # <<<<<<<<<<<<<<
 *             sp_setstring(
 *                 self.sophia.handle,
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":1002
 *                 0)
 * 
 *         for index in self.indexes:             # <<<<<<<<<<<<<<
 *             sp_setstring(
 *                 self.sophia.handle,
 */
  if (unlikely(__pyx_v_self->indexes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_self->indexes; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_index, ((struct __pyx_obj_5sophy__Index *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "sophy.pyx":1005
 *             sp_setstring(
 *                 self.sophia.handle,
 *                 <char *>index.b_path,             # <<<<<<<<<<<<<<
 *                 <char *>index.b_type,
 *                 0)
 */
    __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_index->b_path); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1005; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":1006
 *                 self.sophia.handle,
 *                 <char *>index.b_path,
 *                 <char *>index.b_type,             # <<<<<<<<<<<<<<
 *                 0)
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_index->b_type); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1006; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":1003
 * 
 *         for index in self.indexes:
 *             sp_setstring(             # <<<<<<<<<<<<<<
 *                 self.sophia.handle,
 *                 <char *>index.b_path,
 */
    sp_setstring(__pyx_v_self->__pyx_base.sophia->__pyx_base.handle, ((char *)__pyx_t_5), ((char *)__pyx_t_4), 0);

    /* "sophy.pyx":1002
 *                 0)
 * 
 *         for index in self.indexes:             # <<<<<<<<<<<<<<
 *             sp_setstring(
 *                 self.sophia.handle,
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":989
 *         return tuple(accum)
 * 
 *     cdef configure(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes db_idx_path = encode('db.%s.index' % self.db.name)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy._MultiIndex.configure", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_db_idx_path);
  __Pyx_XDECREF((PyObject *)__pyx_v_index);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1009
 *                 0)
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index subindex
 */

static PyObject *__pyx_f_5sophy_11_MultiIndex_set_key(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, void *__pyx_v_obj, PyObject *__pyx_v_key) {
  struct __pyx_obj_5sophy__Index *__pyx_v_subindex = 0;
  PyObject *__pyx_v_key_part = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_key", 0);

  /* "sophy.pyx":1013
 *             _Index subindex
 * 
 *         for subindex, key_part in zip(self.indexes, key):             # <<<<<<<<<<<<<<
 *             subindex.set_key(obj, key_part)
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->indexes);
  __Pyx_GIVEREF(__pyx_v_self->indexes);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->indexes);
  __Pyx_INCREF(__pyx_v_key);
  __Pyx_GIVEREF(__pyx_v_key);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_key);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_L6_unpacking_done:;
    }
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_subindex, ((struct __pyx_obj_5sophy__Index *)__pyx_t_5));
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_key_part, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "sophy.pyx":1014
 * 
 *         for subindex, key_part in zip(self.indexes, key):
 *             subindex.set_key(obj, key_part)             # <<<<<<<<<<<<<<
 * 
 *     cdef extract_key(self, void *obj):
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_subindex->__pyx_vtab)->set_key(__pyx_v_subindex, __pyx_v_obj, __pyx_v_key_part); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1014; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "sophy.pyx":1013
 *             _Index subindex
 * 
 *         for subindex, key_part in zip(self.indexes, key):             # <<<<<<<<<<<<<<
 *             subindex.set_key(obj, key_part)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":1009
 *                 0)
 * 
 *     cdef set_key(self, void *obj, key):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index subindex
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("sophy._MultiIndex.set_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_subindex);
  __Pyx_XDECREF(__pyx_v_key_part);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1016
 *             subindex.set_key(obj, key_part)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes bkey
 */

static PyObject *__pyx_f_5sophy_11_MultiIndex_extract_key(struct __pyx_obj_5sophy__MultiIndex *__pyx_v_self, void *__pyx_v_obj) {
  PyObject *__pyx_v_bkey = 0;
  struct __pyx_obj_5sophy__Index *__pyx_v_subindex = 0;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extract_key", 0);

  /* "sophy.pyx":1020
 *             bytes bkey
 *             _Index subindex
 *             list result = []             # <<<<<<<<<<<<<<
 * 
 *         for subindex in self.indexes:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":1022
 *             list result = []
 * 
 *         for subindex in self.indexes:             # <<<<<<<<<<<<<<
 *             bkey = subindex.extract_key(obj)
 *             result.append(bkey)
 */
  if (unlikely(__pyx_v_self->indexes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_self->indexes; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5sophy__Index))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_subindex, ((struct __pyx_obj_5sophy__Index *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "sophy.pyx":1023
 * 
 *         for subindex in self.indexes:
 *             bkey = subindex.extract_key(obj)             # <<<<<<<<<<<<<<
 *             result.append(bkey)
 * 
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_5sophy__Index *)__pyx_v_subindex->__pyx_vtab)->extract_key(__pyx_v_subindex, __pyx_v_obj); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1023; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1023; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_bkey, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "sophy.pyx":1024
 *         for subindex in self.indexes:
 *             bkey = subindex.extract_key(obj)
 *             result.append(bkey)             # <<<<<<<<<<<<<<
 * 
 *         return tuple(result)
 */
    __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_result, __pyx_v_bkey); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sophy.pyx":1022
 *             list result = []
 * 
 *         for subindex in self.indexes:             # <<<<<<<<<<<<<<
 *             bkey = subindex.extract_key(obj)
 *             result.append(bkey)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":1026
 *             result.append(bkey)
 * 
 *         return tuple(result)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1026; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1016
 *             subindex.set_key(obj, key_part)
 * 
 *     cdef extract_key(self, void *obj):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bytes bkey
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy._MultiIndex.extract_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bkey);
  __Pyx_XDECREF((PyObject *)__pyx_v_subindex);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1033
 *         readonly db_name
 * 
 *     def __init__(self, filename, index_type=None, auto_open=True):             # <<<<<<<<<<<<<<
 *         path = os.path.dirname(filename)
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'
 */

/* Python wrapper */
static int __pyx_pw_5sophy_14SimpleDatabase_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5sophy_14SimpleDatabase_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_filename = 0;
  PyObject *__pyx_v_index_type = 0;
  PyObject *__pyx_v_auto_open = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_filename,&__pyx_n_s_index_type,&__pyx_n_s_auto_open,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index_type);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_auto_open);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_filename = values[0];
    __pyx_v_index_type = values[1];
    __pyx_v_auto_open = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.SimpleDatabase.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase___init__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), __pyx_v_filename, __pyx_v_index_type, __pyx_v_auto_open);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_14SimpleDatabase___init__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_filename, PyObject *__pyx_v_index_type, PyObject *__pyx_v_auto_open) {
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_databases = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "sophy.pyx":1034
 * 
 *     def __init__(self, filename, index_type=None, auto_open=True):
 *         path = os.path.dirname(filename)             # <<<<<<<<<<<<<<
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'
 *         databases = [(self.db_name, index_type or 'string')]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dirname); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_filename); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_filename);
    __Pyx_GIVEREF(__pyx_v_filename);
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_filename);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_path = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":1035
 *     def __init__(self, filename, index_type=None, auto_open=True):
 *         path = os.path.dirname(filename)
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'             # <<<<<<<<<<<<<<
 *         databases = [(self.db_name, index_type or 'string')]
 *         super(SimpleDatabase, self).__init__(
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_basename); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_filename); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_filename);
    __Pyx_GIVEREF(__pyx_v_filename);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_filename);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_split); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__31, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (!__pyx_t_6) {
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_n_s_default);
  __pyx_t_1 = __pyx_n_s_default;
  __pyx_L3_bool_binop_done:;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->db_name);
  __Pyx_DECREF(__pyx_v_self->db_name);
  __pyx_v_self->db_name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sophy.pyx":1036
 *         path = os.path.dirname(filename)
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'
 *         databases = [(self.db_name, index_type or 'string')]             # <<<<<<<<<<<<<<
 *         super(SimpleDatabase, self).__init__(
 *             path=path,
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_index_type); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (!__pyx_t_6) {
  } else {
    __Pyx_INCREF(__pyx_v_index_type);
    __pyx_t_1 = __pyx_v_index_type;
    goto __pyx_L5_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_n_s_string);
  __pyx_t_1 = __pyx_n_s_string;
  __pyx_L5_bool_binop_done:;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->db_name);
  __Pyx_GIVEREF(__pyx_v_self->db_name);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->db_name);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_databases = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":1037
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'
 *         databases = [(self.db_name, index_type or 'string')]
 *         super(SimpleDatabase, self).__init__(             # <<<<<<<<<<<<<<
 *             path=path,
 *             databases=databases,
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_5sophy_SimpleDatabase));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_5sophy_SimpleDatabase));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_5sophy_SimpleDatabase));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sophy.pyx":1038
 *         databases = [(self.db_name, index_type or 'string')]
 *         super(SimpleDatabase, self).__init__(
 *             path=path,             # <<<<<<<<<<<<<<
 *             databases=databases,
 *             auto_open=auto_open)
 */
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1038; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_path, __pyx_v_path) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1038; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1039
 *         super(SimpleDatabase, self).__init__(
 *             path=path,
 *             databases=databases,             # <<<<<<<<<<<<<<
 *             auto_open=auto_open)
 * 
 */
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_databases, __pyx_v_databases) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1038; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1040
 *             path=path,
 *             databases=databases,
 *             auto_open=auto_open)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, key):
 */
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_auto_open, __pyx_v_auto_open) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1038; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1037
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'
 *         databases = [(self.db_name, index_type or 'string')]
 *         super(SimpleDatabase, self).__init__(             # <<<<<<<<<<<<<<
 *             path=path,
 *             databases=databases,
 */
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":1033
 *         readonly db_name
 * 
 *     def __init__(self, filename, index_type=None, auto_open=True):             # <<<<<<<<<<<<<<
 *         path = os.path.dirname(filename)
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_databases);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1042
 *             auto_open=auto_open)
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name][key]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_2__getitem__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_2__getitem__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "sophy.pyx":1043
 * 
 *     def __getitem__(self, key):
 *         return self.dbs[self.db_name][key]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, key, value):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1043; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1043; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetItem(__pyx_t_1, __pyx_v_key); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1043; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1042
 *             auto_open=auto_open)
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name][key]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1045
 *         return self.dbs[self.db_name][key]
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         self.dbs[self.db_name][key] = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_14SimpleDatabase_5__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5sophy_14SimpleDatabase_5__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_4__setitem__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_14SimpleDatabase_4__setitem__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "sophy.pyx":1046
 * 
 *     def __setitem__(self, key, value):
 *         self.dbs[self.db_name][key] = value             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, key):
 */
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1046; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1046; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_key, __pyx_v_value) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1046; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":1045
 *         return self.dbs[self.db_name][key]
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         self.dbs[self.db_name][key] = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1048
 *         self.dbs[self.db_name][key] = value
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         del self.dbs[self.db_name][key]
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_14SimpleDatabase_7__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_5sophy_14SimpleDatabase_7__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_6__delitem__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_14SimpleDatabase_6__delitem__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "sophy.pyx":1049
 * 
 *     def __delitem__(self, key):
 *         del self.dbs[self.db_name][key]             # <<<<<<<<<<<<<<
 * 
 *     def __contains__(self, key):
 */
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1049; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (PyObject_DelItem(__pyx_t_1, __pyx_v_key) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":1048
 *         self.dbs[self.db_name][key] = value
 * 
 *     def __delitem__(self, key):             # <<<<<<<<<<<<<<
 *         del self.dbs[self.db_name][key]
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1051
 *         del self.dbs[self.db_name][key]
 * 
 *     def __contains__(self, key):             # <<<<<<<<<<<<<<
 *         return key in self.dbs[self.db_name]
 * 
 */

/* Python wrapper */
static int __pyx_pw_5sophy_14SimpleDatabase_9__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static int __pyx_pw_5sophy_14SimpleDatabase_9__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_8__contains__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5sophy_14SimpleDatabase_8__contains__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "sophy.pyx":1052
 * 
 *     def __contains__(self, key):
 *         return key in self.dbs[self.db_name]             # <<<<<<<<<<<<<<
 * 
 *     def keys(self):
 */
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1052; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_key, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "sophy.pyx":1051
 *         del self.dbs[self.db_name][key]
 * 
 *     def __contains__(self, key):             # <<<<<<<<<<<<<<
 *         return key in self.dbs[self.db_name]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1054
 *         return key in self.dbs[self.db_name]
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].keys()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_11keys(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_11keys(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("keys (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_10keys(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_10keys(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("keys", 0);

  /* "sophy.pyx":1055
 * 
 *     def keys(self):
 *         return self.dbs[self.db_name].keys()             # <<<<<<<<<<<<<<
 * 
 *     def values(self):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_keys); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1054
 *         return key in self.dbs[self.db_name]
 * 
 *     def keys(self):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].keys()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.SimpleDatabase.keys", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1057
 *         return self.dbs[self.db_name].keys()
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].values()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_13values(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_13values(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("values (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_12values(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_12values(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("values", 0);

  /* "sophy.pyx":1058
 * 
 *     def values(self):
 *         return self.dbs[self.db_name].values()             # <<<<<<<<<<<<<<
 * 
 *     def items(self):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_values); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1057
 *         return self.dbs[self.db_name].keys()
 * 
 *     def values(self):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].values()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.SimpleDatabase.values", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1060
 *         return self.dbs[self.db_name].values()
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].items()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_15items(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_15items(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("items (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_14items(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_14items(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("items", 0);

  /* "sophy.pyx":1061
 * 
 *     def items(self):
 *         return self.dbs[self.db_name].items()             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1060
 *         return self.dbs[self.db_name].values()
 * 
 *     def items(self):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].items()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.SimpleDatabase.items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1063
 *         return self.dbs[self.db_name].items()
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.cursor())
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_17__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_17__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_16__iter__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_16__iter__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "sophy.pyx":1064
 * 
 *     def __iter__(self):
 *         return iter(self.cursor())             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cursor); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1063
 *         return self.dbs[self.db_name].items()
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.cursor())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1066
 *         return iter(self.cursor())
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.dbs[self.db_name])
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_5sophy_14SimpleDatabase_19__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_5sophy_14SimpleDatabase_19__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_18__len__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_5sophy_14SimpleDatabase_18__len__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "sophy.pyx":1067
 * 
 *     def __len__(self):
 *         return len(self.dbs[self.db_name])             # <<<<<<<<<<<<<<
 * 
 *     def cursor(self, *args, **kwargs):
 */
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1067; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1067; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1067; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "sophy.pyx":1066
 *         return iter(self.cursor())
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.dbs[self.db_name])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.SimpleDatabase.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1069
 *         return len(self.dbs[self.db_name])
 * 
 *     def cursor(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].cursor(*args, **kwargs)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_21cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_21cursor(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cursor (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "cursor", 1))) return NULL;
  if (unlikely(__pyx_kwds)) {
    __pyx_v_kwargs = PyDict_Copy(__pyx_kwds); if (unlikely(!__pyx_v_kwargs)) return NULL;
    __Pyx_GOTREF(__pyx_v_kwargs);
  } else {
    __pyx_v_kwargs = NULL;
  }
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_20cursor(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_20cursor(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cursor", 0);

  /* "sophy.pyx":1070
 * 
 *     def cursor(self, *args, **kwargs):
 *         return self.dbs[self.db_name].cursor(*args, **kwargs)             # <<<<<<<<<<<<<<
 * 
 *     def update(self, dict _data=None, **k):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_cursor); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_v_args, __pyx_v_kwargs); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1069
 *         return len(self.dbs[self.db_name])
 * 
 *     def cursor(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].cursor(*args, **kwargs)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy.SimpleDatabase.cursor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1072
 *         return self.dbs[self.db_name].cursor(*args, **kwargs)
 * 
 *     def update(self, dict _data=None, **k):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].update(_data, **k)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_23update(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_23update(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v__data = 0;
  PyObject *__pyx_v_k = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("update (wrapper)", 0);
  __pyx_v_k = PyDict_New(); if (unlikely(!__pyx_v_k)) return NULL;
  __Pyx_GOTREF(__pyx_v_k);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_k, values, pos_args, "update") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1072; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v__data = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("update", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1072; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_k); __pyx_v_k = 0;
  __Pyx_AddTraceback("sophy.SimpleDatabase.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v__data), (&PyDict_Type), 1, "_data", 1))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1072; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_22update(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), __pyx_v__data, __pyx_v_k);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_22update(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v__data, PyObject *__pyx_v_k) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("update", 0);

  /* "sophy.pyx":1073
 * 
 *     def update(self, dict _data=None, **k):
 *         return self.dbs[self.db_name].update(_data, **k)             # <<<<<<<<<<<<<<
 * 
 *     cpdef transaction(self):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_update); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v__data);
  __Pyx_GIVEREF(__pyx_v__data);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__data);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_v_k); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1072
 *         return self.dbs[self.db_name].cursor(*args, **kwargs)
 * 
 *     def update(self, dict _data=None, **k):             # <<<<<<<<<<<<<<
 *         return self.dbs[self.db_name].update(_data, **k)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sophy.SimpleDatabase.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1075
 *         return self.dbs[self.db_name].update(_data, **k)
 * 
 *     cpdef transaction(self):             # <<<<<<<<<<<<<<
 *         return Transaction(self, self.dbs[self.db_name])
 * 
 */

static PyObject *__pyx_pw_5sophy_14SimpleDatabase_25transaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5sophy_14SimpleDatabase_transaction(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transaction", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_transaction); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1075; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_25transaction)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1075; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1075; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":1076
 * 
 *     cpdef transaction(self):
 *         return Transaction(self, self.dbs[self.db_name])             # <<<<<<<<<<<<<<
 * 
 *     cpdef view(self, name):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1076; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1076; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1076; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Transaction), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1076; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1075
 *         return self.dbs[self.db_name].update(_data, **k)
 * 
 *     cpdef transaction(self):             # <<<<<<<<<<<<<<
 *         return Transaction(self, self.dbs[self.db_name])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sophy.SimpleDatabase.transaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_25transaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_25transaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transaction (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_24transaction(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_24transaction(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transaction", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5sophy_14SimpleDatabase_transaction(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1075; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.SimpleDatabase.transaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1078
 *         return Transaction(self, self.dbs[self.db_name])
 * 
 *     cpdef view(self, name):             # <<<<<<<<<<<<<<
 *         return View(self, self.dbs[self.db_name], name)
 * 
 */

static PyObject *__pyx_pw_5sophy_14SimpleDatabase_27view(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_f_5sophy_14SimpleDatabase_view(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("view", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_view); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1078; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_27view)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1078; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1078; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_name);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1078; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "sophy.pyx":1079
 * 
 *     cpdef view(self, name):
 *         return View(self, self.dbs[self.db_name], name)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->__pyx_base.dbs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.dbs, __pyx_v_self->db_name); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1079; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_name);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_View), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1078
 *         return Transaction(self, self.dbs[self.db_name])
 * 
 *     cpdef view(self, name):             # <<<<<<<<<<<<<<
 *         return View(self, self.dbs[self.db_name], name)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sophy.SimpleDatabase.view", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_27view(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_27view(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("view (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_26view(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_26view(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("view", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5sophy_14SimpleDatabase_view(__pyx_v_self, __pyx_v_name, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1078; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sophy.SimpleDatabase.view", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1031
 * cdef class SimpleDatabase(Sophia):
 *     cdef:
 *         readonly db_name             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, filename, index_type=None, auto_open=True):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_7db_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5sophy_14SimpleDatabase_7db_name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5sophy_14SimpleDatabase_7db_name___get__(((struct __pyx_obj_5sophy_SimpleDatabase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_14SimpleDatabase_7db_name___get__(struct __pyx_obj_5sophy_SimpleDatabase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->db_name);
  __pyx_r = __pyx_v_self->db_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sophy.pyx":1082
 * 
 * 
 * def connect(data_dir, db_name, index_type=None):             # <<<<<<<<<<<<<<
 *     cdef Sophia sophia
 *     sophia = Sophia(data_dir, [(db_name, index_type)])
 */

/* Python wrapper */
static PyObject *__pyx_pw_5sophy_9connect(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5sophy_9connect = {"connect", (PyCFunction)__pyx_pw_5sophy_9connect, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5sophy_9connect(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data_dir = 0;
  PyObject *__pyx_v_db_name = 0;
  PyObject *__pyx_v_index_type = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("connect (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data_dir,&__pyx_n_s_db_name,&__pyx_n_s_index_type,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data_dir)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_db_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("connect", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index_type);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "connect") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data_dir = values[0];
    __pyx_v_db_name = values[1];
    __pyx_v_index_type = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("connect", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sophy.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5sophy_8connect(__pyx_self, __pyx_v_data_dir, __pyx_v_db_name, __pyx_v_index_type);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5sophy_8connect(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data_dir, PyObject *__pyx_v_db_name, PyObject *__pyx_v_index_type) {
  struct __pyx_obj_5sophy_Sophia *__pyx_v_sophia = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("connect", 0);

  /* "sophy.pyx":1084
 * def connect(data_dir, db_name, index_type=None):
 *     cdef Sophia sophia
 *     sophia = Sophia(data_dir, [(db_name, index_type)])             # <<<<<<<<<<<<<<
 *     sophia.open()
 *     return sophia[db_name]
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1084; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_db_name);
  __Pyx_GIVEREF(__pyx_v_db_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_db_name);
  __Pyx_INCREF(__pyx_v_index_type);
  __Pyx_GIVEREF(__pyx_v_index_type);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_index_type);
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1084; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1084; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_data_dir);
  __Pyx_GIVEREF(__pyx_v_data_dir);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_data_dir);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5sophy_Sophia), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1084; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_sophia = ((struct __pyx_obj_5sophy_Sophia *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "sophy.pyx":1085
 *     cdef Sophia sophia
 *     sophia = Sophia(data_dir, [(db_name, index_type)])
 *     sophia.open()             # <<<<<<<<<<<<<<
 *     return sophia[db_name]
 * 
 */
  ((struct __pyx_vtabstruct_5sophy_Sophia *)__pyx_v_sophia->__pyx_base.__pyx_vtab)->open(__pyx_v_sophia, 0);

  /* "sophy.pyx":1086
 *     sophia = Sophia(data_dir, [(db_name, index_type)])
 *     sophia.open()
 *     return sophia[db_name]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyObject_GetItem(((PyObject *)__pyx_v_sophia), __pyx_v_db_name); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "sophy.pyx":1082
 * 
 * 
 * def connect(data_dir, db_name, index_type=None):             # <<<<<<<<<<<<<<
 *     cdef Sophia sophia
 *     sophia = Sophia(data_dir, [(db_name, index_type)])
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sophy.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_sophia);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_5sophy__ConfigManager __pyx_vtable_5sophy__ConfigManager;

static PyObject *__pyx_tp_new_5sophy__ConfigManager(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__ConfigManager *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__ConfigManager *)o);
  p->__pyx_vtab = __pyx_vtabptr_5sophy__ConfigManager;
  p->_config = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_14_ConfigManager_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy__ConfigManager(PyObject *o) {
  struct __pyx_obj_5sophy__ConfigManager *p = (struct __pyx_obj_5sophy__ConfigManager *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_config);
  Py_CLEAR(p->sophia);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5sophy__ConfigManager(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy__ConfigManager *p = (struct __pyx_obj_5sophy__ConfigManager *)o;
  if (p->_config) {
    e = (*v)(p->_config, a); if (e) return e;
  }
  if (p->sophia) {
    e = (*v)(((PyObject*)p->sophia), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy__ConfigManager(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy__ConfigManager *p = (struct __pyx_obj_5sophy__ConfigManager *)o;
  tmp = ((PyObject*)p->_config);
  p->_config = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->sophia);
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5sophy__ConfigManager[] = {
  {"set_option", (PyCFunction)__pyx_pw_5sophy_14_ConfigManager_3set_option, METH_VARARGS|METH_KEYWORDS, 0},
  {"get_option", (PyCFunction)__pyx_pw_5sophy_14_ConfigManager_5get_option, METH_VARARGS|METH_KEYWORDS, 0},
  {"clear_option", (PyCFunction)__pyx_pw_5sophy_14_ConfigManager_7clear_option, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy__ConfigManager = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._ConfigManager", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__ConfigManager), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__ConfigManager, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__ConfigManager, /*tp_traverse*/
  __pyx_tp_clear_5sophy__ConfigManager, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy__ConfigManager, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__ConfigManager, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__SophiaObject __pyx_vtable_5sophy__SophiaObject;

static PyObject *__pyx_tp_new_5sophy__SophiaObject(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__SophiaObject *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__SophiaObject *)o);
  p->__pyx_vtab = __pyx_vtabptr_5sophy__SophiaObject;
  if (unlikely(__pyx_pw_5sophy_13_SophiaObject_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy__SophiaObject(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5sophy__SophiaObject[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy__SophiaObject = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._SophiaObject", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__SophiaObject), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__SophiaObject, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy__SophiaObject, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__SophiaObject, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy_Sophia __pyx_vtable_5sophy_Sophia;

static PyObject *__pyx_tp_new_5sophy_Sophia(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy_Sophia *p;
  PyObject *o = __pyx_tp_new_5sophy__SophiaObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy_Sophia *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy_Sophia;
  p->b_path = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->dbs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->db_defs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->config = ((struct __pyx_obj_5sophy__ConfigManager *)Py_None); Py_INCREF(Py_None);
  p->path = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_6Sophia_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy_Sophia(PyObject *o) {
  struct __pyx_obj_5sophy_Sophia *p = (struct __pyx_obj_5sophy_Sophia *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5sophy_6Sophia_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->b_path);
  Py_CLEAR(p->dbs);
  Py_CLEAR(p->db_defs);
  Py_CLEAR(p->config);
  Py_CLEAR(p->path);
  #if CYTHON_COMPILING_IN_CPYTHON
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__SophiaObject(o);
}

static int __pyx_tp_traverse_5sophy_Sophia(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy_Sophia *p = (struct __pyx_obj_5sophy_Sophia *)o;
  e = ((likely(__pyx_ptype_5sophy__SophiaObject)) ? ((__pyx_ptype_5sophy__SophiaObject->tp_traverse) ? __pyx_ptype_5sophy__SophiaObject->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5sophy_Sophia)); if (e) return e;
  if (p->dbs) {
    e = (*v)(p->dbs, a); if (e) return e;
  }
  if (p->db_defs) {
    e = (*v)(p->db_defs, a); if (e) return e;
  }
  if (p->config) {
    e = (*v)(((PyObject*)p->config), a); if (e) return e;
  }
  if (p->path) {
    e = (*v)(p->path, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy_Sophia(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy_Sophia *p = (struct __pyx_obj_5sophy_Sophia *)o;
  if (likely(__pyx_ptype_5sophy__SophiaObject)) { if (__pyx_ptype_5sophy__SophiaObject->tp_clear) __pyx_ptype_5sophy__SophiaObject->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5sophy_Sophia);
  tmp = ((PyObject*)p->dbs);
  p->dbs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->db_defs);
  p->db_defs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->config);
  p->config = ((struct __pyx_obj_5sophy__ConfigManager *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->path);
  p->path = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_5sophy_Sophia(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_getprop_5sophy_6Sophia_dbs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Sophia_3dbs_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Sophia_auto_open(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Sophia_9auto_open_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Sophia_config(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Sophia_6config_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Sophia_path(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Sophia_4path_1__get__(o);
}

static PyMethodDef __pyx_methods_5sophy_Sophia[] = {
  {"open", (PyCFunction)__pyx_pw_5sophy_6Sophia_7open, METH_NOARGS, 0},
  {"close", (PyCFunction)__pyx_pw_5sophy_6Sophia_9close, METH_NOARGS, 0},
  {"create_database", (PyCFunction)__pyx_pw_5sophy_6Sophia_13create_database, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5sophy_Sophia[] = {
  {(char *)"dbs", __pyx_getprop_5sophy_6Sophia_dbs, 0, 0, 0},
  {(char *)"auto_open", __pyx_getprop_5sophy_6Sophia_auto_open, 0, 0, 0},
  {(char *)"config", __pyx_getprop_5sophy_6Sophia_config, 0, 0, 0},
  {(char *)"path", __pyx_getprop_5sophy_6Sophia_path, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_Sophia = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_5sophy_Sophia, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_Sophia = {
  0, /*mp_length*/
  __pyx_pw_5sophy_6Sophia_11__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_5sophy_Sophia = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.Sophia", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy_Sophia), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy_Sophia, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_Sophia, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_Sophia, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy_Sophia, /*tp_traverse*/
  __pyx_tp_clear_5sophy_Sophia, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy_Sophia, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5sophy_Sophia, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5sophy_6Sophia_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy_Sophia, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__BaseDBObject __pyx_vtable_5sophy__BaseDBObject;

static PyObject *__pyx_tp_new_5sophy__BaseDBObject(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__BaseDBObject *p;
  PyObject *o = __pyx_tp_new_5sophy__SophiaObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__BaseDBObject *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy__BaseDBObject;
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  p->index = ((struct __pyx_obj_5sophy__Index *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_13_BaseDBObject_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy__BaseDBObject(PyObject *o) {
  struct __pyx_obj_5sophy__BaseDBObject *p = (struct __pyx_obj_5sophy__BaseDBObject *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->sophia);
  Py_CLEAR(p->index);
  #if CYTHON_COMPILING_IN_CPYTHON
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__SophiaObject(o);
}

static int __pyx_tp_traverse_5sophy__BaseDBObject(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy__BaseDBObject *p = (struct __pyx_obj_5sophy__BaseDBObject *)o;
  e = ((likely(__pyx_ptype_5sophy__SophiaObject)) ? ((__pyx_ptype_5sophy__SophiaObject->tp_traverse) ? __pyx_ptype_5sophy__SophiaObject->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5sophy__BaseDBObject)); if (e) return e;
  if (p->sophia) {
    e = (*v)(((PyObject*)p->sophia), a); if (e) return e;
  }
  if (p->index) {
    e = (*v)(((PyObject*)p->index), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy__BaseDBObject(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy__BaseDBObject *p = (struct __pyx_obj_5sophy__BaseDBObject *)o;
  if (likely(__pyx_ptype_5sophy__SophiaObject)) { if (__pyx_ptype_5sophy__SophiaObject->tp_clear) __pyx_ptype_5sophy__SophiaObject->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5sophy__BaseDBObject);
  tmp = ((PyObject*)p->sophia);
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->index);
  p->index = ((struct __pyx_obj_5sophy__Index *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_5sophy__BaseDBObject(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_5sophy__BaseDBObject(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_5sophy_13_BaseDBObject_15__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_5sophy_13_BaseDBObject_17__delitem__(o, i);
  }
}

static PyObject *__pyx_getprop_5sophy_13_BaseDBObject_sophia(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_13_BaseDBObject_6sophia_1__get__(o);
}

static PyMethodDef __pyx_methods_5sophy__BaseDBObject[] = {
  {"open", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_3open, METH_VARARGS|METH_KEYWORDS, 0},
  {"close", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_5close, METH_NOARGS, 0},
  {"__enter__", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_7__enter__, METH_NOARGS, 0},
  {"__exit__", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_9__exit__, METH_VARARGS|METH_KEYWORDS, 0},
  {"get_range", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_21get_range, METH_VARARGS|METH_KEYWORDS, 0},
  {"keys", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_24keys, METH_NOARGS, 0},
  {"values", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_27values, METH_NOARGS, 0},
  {"items", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_30items, METH_NOARGS, 0},
  {"cursor", (PyCFunction)__pyx_pw_5sophy_13_BaseDBObject_37cursor, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5sophy__BaseDBObject[] = {
  {(char *)"sophia", __pyx_getprop_5sophy_13_BaseDBObject_sophia, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence__BaseDBObject = {
  __pyx_pw_5sophy_13_BaseDBObject_35__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_5sophy__BaseDBObject, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_5sophy_13_BaseDBObject_19__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__BaseDBObject = {
  __pyx_pw_5sophy_13_BaseDBObject_35__len__, /*mp_length*/
  __pyx_pw_5sophy_13_BaseDBObject_13__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_5sophy__BaseDBObject, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_5sophy__BaseDBObject = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._BaseDBObject", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__BaseDBObject), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__BaseDBObject, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence__BaseDBObject, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__BaseDBObject, /*tp_as_mapping*/
  0, /*tp_hash*/
  __pyx_pw_5sophy_13_BaseDBObject_11__call__, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__BaseDBObject, /*tp_traverse*/
  __pyx_tp_clear_5sophy__BaseDBObject, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_5sophy_13_BaseDBObject_33__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy__BaseDBObject, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5sophy__BaseDBObject, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__BaseDBObject, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy_Database __pyx_vtable_5sophy_Database;

static PyObject *__pyx_tp_new_5sophy_Database(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy_Database *p;
  PyObject *o = __pyx_tp_new_5sophy__BaseDBObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy_Database *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy_Database;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->config = ((struct __pyx_obj_5sophy__ConfigManager *)Py_None); Py_INCREF(Py_None);
  p->index_type = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_8Database_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy_Database(PyObject *o) {
  struct __pyx_obj_5sophy_Database *p = (struct __pyx_obj_5sophy_Database *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5sophy_8Database_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->name);
  Py_CLEAR(p->config);
  Py_CLEAR(p->index_type);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__BaseDBObject(o);
}

static int __pyx_tp_traverse_5sophy_Database(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy_Database *p = (struct __pyx_obj_5sophy_Database *)o;
  e = __pyx_tp_traverse_5sophy__BaseDBObject(o, v, a); if (e) return e;
  if (p->config) {
    e = (*v)(((PyObject*)p->config), a); if (e) return e;
  }
  if (p->index_type) {
    e = (*v)(p->index_type, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy_Database(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy_Database *p = (struct __pyx_obj_5sophy_Database *)o;
  __pyx_tp_clear_5sophy__BaseDBObject(o);
  tmp = ((PyObject*)p->config);
  p->config = ((struct __pyx_obj_5sophy__ConfigManager *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->index_type);
  p->index_type = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5sophy_8Database_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_8Database_4name_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_8Database_config(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_8Database_6config_1__get__(o);
}

static PyMethodDef __pyx_methods_5sophy_Database[] = {
  {"close", (PyCFunction)__pyx_pw_5sophy_8Database_5close, METH_NOARGS, 0},
  {"update", (PyCFunction)__pyx_pw_5sophy_8Database_7update, METH_VARARGS|METH_KEYWORDS, 0},
  {"transaction", (PyCFunction)__pyx_pw_5sophy_8Database_9transaction, METH_NOARGS, 0},
  {"view", (PyCFunction)__pyx_pw_5sophy_8Database_11view, METH_O, 0},
  {"cursor", (PyCFunction)__pyx_pw_5sophy_8Database_13cursor, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5sophy_Database[] = {
  {(char *)"name", __pyx_getprop_5sophy_8Database_name, 0, 0, 0},
  {(char *)"config", __pyx_getprop_5sophy_8Database_config, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy_Database = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.Database", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy_Database), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy_Database, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_11__call__, /*tp_call*/
  #else
  0, /*tp_call*/
  #endif
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy_Database, /*tp_traverse*/
  __pyx_tp_clear_5sophy_Database, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_33__iter__, /*tp_iter*/
  #else
  0, /*tp_iter*/
  #endif
  0, /*tp_iternext*/
  __pyx_methods_5sophy_Database, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5sophy_Database, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy_Database, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy_View __pyx_vtable_5sophy_View;

static PyObject *__pyx_tp_new_5sophy_View(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy_View *p;
  PyObject *o = __pyx_tp_new_5sophy__BaseDBObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy_View *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy_View;
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_4View_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy_View(PyObject *o) {
  struct __pyx_obj_5sophy_View *p = (struct __pyx_obj_5sophy_View *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5sophy_4View_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->db);
  Py_CLEAR(p->name);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__BaseDBObject(o);
}

static int __pyx_tp_traverse_5sophy_View(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy_View *p = (struct __pyx_obj_5sophy_View *)o;
  e = __pyx_tp_traverse_5sophy__BaseDBObject(o, v, a); if (e) return e;
  if (p->db) {
    e = (*v)(((PyObject*)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy_View(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy_View *p = (struct __pyx_obj_5sophy_View *)o;
  __pyx_tp_clear_5sophy__BaseDBObject(o);
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static int __pyx_mp_ass_subscript_5sophy_View(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_5sophy_4View_7__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_5sophy_4View_9__delitem__(o, i);
  }
}

static PyMethodDef __pyx_methods_5sophy_View[] = {
  {"cursor", (PyCFunction)__pyx_pw_5sophy_4View_11cursor, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static PyMappingMethods __pyx_tp_as_mapping_View = {
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_35__len__, /*mp_length*/
  #else
  0, /*mp_length*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_13__getitem__, /*mp_subscript*/
  #else
  0, /*mp_subscript*/
  #endif
  __pyx_mp_ass_subscript_5sophy_View, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_5sophy_View = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.View", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy_View), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy_View, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_View, /*tp_as_mapping*/
  0, /*tp_hash*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_11__call__, /*tp_call*/
  #else
  0, /*tp_call*/
  #endif
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy_View, /*tp_traverse*/
  __pyx_tp_clear_5sophy_View, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_33__iter__, /*tp_iter*/
  #else
  0, /*tp_iter*/
  #endif
  0, /*tp_iternext*/
  __pyx_methods_5sophy_View, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5sophy_4View_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy_View, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy_Transaction __pyx_vtable_5sophy_Transaction;

static PyObject *__pyx_tp_new_5sophy_Transaction(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy_Transaction *p;
  PyObject *o = __pyx_tp_new_5sophy__BaseDBObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy_Transaction *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy_Transaction;
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_11Transaction_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy_Transaction(PyObject *o) {
  struct __pyx_obj_5sophy_Transaction *p = (struct __pyx_obj_5sophy_Transaction *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5sophy_11Transaction_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->db);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__BaseDBObject(o);
}

static int __pyx_tp_traverse_5sophy_Transaction(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy_Transaction *p = (struct __pyx_obj_5sophy_Transaction *)o;
  e = __pyx_tp_traverse_5sophy__BaseDBObject(o, v, a); if (e) return e;
  if (p->db) {
    e = (*v)(((PyObject*)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy_Transaction(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy_Transaction *p = (struct __pyx_obj_5sophy_Transaction *)o;
  __pyx_tp_clear_5sophy__BaseDBObject(o);
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5sophy_Transaction[] = {
  {"__exit__", (PyCFunction)__pyx_pw_5sophy_11Transaction_5__exit__, METH_VARARGS|METH_KEYWORDS, 0},
  {"commit", (PyCFunction)__pyx_pw_5sophy_11Transaction_7commit, METH_VARARGS|METH_KEYWORDS, 0},
  {"rollback", (PyCFunction)__pyx_pw_5sophy_11Transaction_9rollback, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy_Transaction = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.Transaction", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy_Transaction), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy_Transaction, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_11__call__, /*tp_call*/
  #else
  0, /*tp_call*/
  #endif
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy_Transaction, /*tp_traverse*/
  __pyx_tp_clear_5sophy_Transaction, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_13_BaseDBObject_33__iter__, /*tp_iter*/
  #else
  0, /*tp_iter*/
  #endif
  0, /*tp_iternext*/
  __pyx_methods_5sophy_Transaction, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy_Transaction, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy_Cursor __pyx_vtable_5sophy_Cursor;

static PyObject *__pyx_tp_new_5sophy_Cursor(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy_Cursor *p;
  PyObject *o = __pyx_tp_new_5sophy__SophiaObject(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy_Cursor *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy_Cursor;
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  p->target = ((struct __pyx_obj_5sophy__SophiaObject *)Py_None); Py_INCREF(Py_None);
  p->order = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->key = Py_None; Py_INCREF(Py_None);
  p->indexes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_6Cursor_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy_Cursor(PyObject *o) {
  struct __pyx_obj_5sophy_Cursor *p = (struct __pyx_obj_5sophy_Cursor *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5sophy_6Cursor_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->sophia);
  Py_CLEAR(p->db);
  Py_CLEAR(p->target);
  Py_CLEAR(p->order);
  Py_CLEAR(p->prefix);
  Py_CLEAR(p->key);
  Py_CLEAR(p->indexes);
  #if CYTHON_COMPILING_IN_CPYTHON
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__SophiaObject(o);
}

static int __pyx_tp_traverse_5sophy_Cursor(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy_Cursor *p = (struct __pyx_obj_5sophy_Cursor *)o;
  e = ((likely(__pyx_ptype_5sophy__SophiaObject)) ? ((__pyx_ptype_5sophy__SophiaObject->tp_traverse) ? __pyx_ptype_5sophy__SophiaObject->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5sophy_Cursor)); if (e) return e;
  if (p->sophia) {
    e = (*v)(((PyObject*)p->sophia), a); if (e) return e;
  }
  if (p->db) {
    e = (*v)(((PyObject*)p->db), a); if (e) return e;
  }
  if (p->target) {
    e = (*v)(((PyObject*)p->target), a); if (e) return e;
  }
  if (p->key) {
    e = (*v)(p->key, a); if (e) return e;
  }
  if (p->indexes) {
    e = (*v)(p->indexes, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy_Cursor(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy_Cursor *p = (struct __pyx_obj_5sophy_Cursor *)o;
  if (likely(__pyx_ptype_5sophy__SophiaObject)) { if (__pyx_ptype_5sophy__SophiaObject->tp_clear) __pyx_ptype_5sophy__SophiaObject->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5sophy_Cursor);
  tmp = ((PyObject*)p->sophia);
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->target);
  p->target = ((struct __pyx_obj_5sophy__SophiaObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->key);
  p->key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->indexes);
  p->indexes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5sophy_6Cursor_order(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Cursor_5order_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Cursor_prefix(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Cursor_6prefix_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Cursor_keys(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Cursor_4keys_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Cursor_values(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Cursor_6values_1__get__(o);
}

static PyObject *__pyx_getprop_5sophy_6Cursor_key(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_6Cursor_3key_1__get__(o);
}

static PyMethodDef __pyx_methods_5sophy_Cursor[] = {
  {"__next__", (PyCFunction)__pyx_pw_5sophy_6Cursor_7__next__, METH_NOARGS|METH_COEXIST, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5sophy_Cursor[] = {
  {(char *)"order", __pyx_getprop_5sophy_6Cursor_order, 0, 0, 0},
  {(char *)"prefix", __pyx_getprop_5sophy_6Cursor_prefix, 0, 0, 0},
  {(char *)"keys", __pyx_getprop_5sophy_6Cursor_keys, 0, 0, 0},
  {(char *)"values", __pyx_getprop_5sophy_6Cursor_values, 0, 0, 0},
  {(char *)"key", __pyx_getprop_5sophy_6Cursor_key, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy_Cursor = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.Cursor", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy_Cursor), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy_Cursor, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy_Cursor, /*tp_traverse*/
  __pyx_tp_clear_5sophy_Cursor, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_5sophy_6Cursor_5__iter__, /*tp_iter*/
  __pyx_pw_5sophy_6Cursor_7__next__, /*tp_iternext*/
  __pyx_methods_5sophy_Cursor, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5sophy_Cursor, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy_Cursor, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__Index __pyx_vtable_5sophy__Index;

static PyObject *__pyx_tp_new_5sophy__Index(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__Index *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__Index *)o);
  p->__pyx_vtab = __pyx_vtabptr_5sophy__Index;
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  p->b_path = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->b_type = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->key = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->empty_value = Py_None; Py_INCREF(Py_None);
  p->keys = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_6_Index_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy__Index(PyObject *o) {
  struct __pyx_obj_5sophy__Index *p = (struct __pyx_obj_5sophy__Index *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->sophia);
  Py_CLEAR(p->db);
  Py_CLEAR(p->b_path);
  Py_CLEAR(p->b_type);
  Py_CLEAR(p->key);
  Py_CLEAR(p->empty_value);
  Py_CLEAR(p->keys);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5sophy__Index(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy__Index *p = (struct __pyx_obj_5sophy__Index *)o;
  if (p->sophia) {
    e = (*v)(((PyObject*)p->sophia), a); if (e) return e;
  }
  if (p->db) {
    e = (*v)(((PyObject*)p->db), a); if (e) return e;
  }
  if (p->empty_value) {
    e = (*v)(p->empty_value, a); if (e) return e;
  }
  if (p->keys) {
    e = (*v)(p->keys, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy__Index(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy__Index *p = (struct __pyx_obj_5sophy__Index *)o;
  tmp = ((PyObject*)p->sophia);
  p->sophia = ((struct __pyx_obj_5sophy_Sophia *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_5sophy_Database *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->empty_value);
  p->empty_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->keys);
  p->keys = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5sophy__Index[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy__Index = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._Index", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__Index), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__Index, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__Index, /*tp_traverse*/
  __pyx_tp_clear_5sophy__Index, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy__Index, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5sophy_6_Index_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__Index, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__UInt32Index __pyx_vtable_5sophy__UInt32Index;

static PyObject *__pyx_tp_new_5sophy__UInt32Index(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__UInt32Index *p;
  PyObject *o = __pyx_tp_new_5sophy__Index(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__UInt32Index *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__Index*)__pyx_vtabptr_5sophy__UInt32Index;
  return o;
}

static PyTypeObject __pyx_type_5sophy__UInt32Index = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._UInt32Index", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__UInt32Index), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__Index, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__Index, /*tp_traverse*/
  __pyx_tp_clear_5sophy__Index, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_6_Index_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__UInt32Index, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__UInt32RevIndex __pyx_vtable_5sophy__UInt32RevIndex;

static PyObject *__pyx_tp_new_5sophy__UInt32RevIndex(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__UInt32RevIndex *p;
  PyObject *o = __pyx_tp_new_5sophy__UInt32Index(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__UInt32RevIndex *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__Index*)__pyx_vtabptr_5sophy__UInt32RevIndex;
  return o;
}

static PyTypeObject __pyx_type_5sophy__UInt32RevIndex = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._UInt32RevIndex", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__UInt32RevIndex), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__Index, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__Index, /*tp_traverse*/
  __pyx_tp_clear_5sophy__Index, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_6_Index_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__UInt32RevIndex, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__UInt64Index __pyx_vtable_5sophy__UInt64Index;

static PyObject *__pyx_tp_new_5sophy__UInt64Index(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__UInt64Index *p;
  PyObject *o = __pyx_tp_new_5sophy__Index(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__UInt64Index *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__Index*)__pyx_vtabptr_5sophy__UInt64Index;
  return o;
}

static PyTypeObject __pyx_type_5sophy__UInt64Index = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._UInt64Index", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__UInt64Index), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__Index, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__Index, /*tp_traverse*/
  __pyx_tp_clear_5sophy__Index, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_6_Index_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__UInt64Index, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__UInt64RevIndex __pyx_vtable_5sophy__UInt64RevIndex;

static PyObject *__pyx_tp_new_5sophy__UInt64RevIndex(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__UInt64RevIndex *p;
  PyObject *o = __pyx_tp_new_5sophy__UInt64Index(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__UInt64RevIndex *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__Index*)__pyx_vtabptr_5sophy__UInt64RevIndex;
  return o;
}

static PyTypeObject __pyx_type_5sophy__UInt64RevIndex = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._UInt64RevIndex", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__UInt64RevIndex), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__Index, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__Index, /*tp_traverse*/
  __pyx_tp_clear_5sophy__Index, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_6_Index_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__UInt64RevIndex, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy__MultiIndex __pyx_vtable_5sophy__MultiIndex;

static PyObject *__pyx_tp_new_5sophy__MultiIndex(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy__MultiIndex *p;
  PyObject *o = __pyx_tp_new_5sophy__Index(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy__MultiIndex *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__Index*)__pyx_vtabptr_5sophy__MultiIndex;
  p->indexes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5sophy_11_MultiIndex_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy__MultiIndex(PyObject *o) {
  struct __pyx_obj_5sophy__MultiIndex *p = (struct __pyx_obj_5sophy__MultiIndex *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->indexes);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy__Index(o);
}

static int __pyx_tp_traverse_5sophy__MultiIndex(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy__MultiIndex *p = (struct __pyx_obj_5sophy__MultiIndex *)o;
  e = __pyx_tp_traverse_5sophy__Index(o, v, a); if (e) return e;
  if (p->indexes) {
    e = (*v)(p->indexes, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy__MultiIndex(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy__MultiIndex *p = (struct __pyx_obj_5sophy__MultiIndex *)o;
  __pyx_tp_clear_5sophy__Index(o);
  tmp = ((PyObject*)p->indexes);
  p->indexes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5sophy__MultiIndex[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5sophy__MultiIndex = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy._MultiIndex", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy__MultiIndex), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy__MultiIndex, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy__MultiIndex, /*tp_traverse*/
  __pyx_tp_clear_5sophy__MultiIndex, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy__MultiIndex, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5sophy_6_Index_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy__MultiIndex, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5sophy_SimpleDatabase __pyx_vtable_5sophy_SimpleDatabase;

static PyObject *__pyx_tp_new_5sophy_SimpleDatabase(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5sophy_SimpleDatabase *p;
  PyObject *o = __pyx_tp_new_5sophy_Sophia(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5sophy_SimpleDatabase *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5sophy__SophiaObject*)__pyx_vtabptr_5sophy_SimpleDatabase;
  p->db_name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5sophy_SimpleDatabase(PyObject *o) {
  struct __pyx_obj_5sophy_SimpleDatabase *p = (struct __pyx_obj_5sophy_SimpleDatabase *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->db_name);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5sophy_Sophia(o);
}

static int __pyx_tp_traverse_5sophy_SimpleDatabase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy_SimpleDatabase *p = (struct __pyx_obj_5sophy_SimpleDatabase *)o;
  e = __pyx_tp_traverse_5sophy_Sophia(o, v, a); if (e) return e;
  if (p->db_name) {
    e = (*v)(p->db_name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy_SimpleDatabase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy_SimpleDatabase *p = (struct __pyx_obj_5sophy_SimpleDatabase *)o;
  __pyx_tp_clear_5sophy_Sophia(o);
  tmp = ((PyObject*)p->db_name);
  p->db_name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_5sophy_SimpleDatabase(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_5sophy_SimpleDatabase(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_5sophy_14SimpleDatabase_5__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_5sophy_14SimpleDatabase_7__delitem__(o, i);
  }
}

static PyObject *__pyx_getprop_5sophy_14SimpleDatabase_db_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5sophy_14SimpleDatabase_7db_name_1__get__(o);
}

static PyMethodDef __pyx_methods_5sophy_SimpleDatabase[] = {
  {"keys", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_11keys, METH_NOARGS, 0},
  {"values", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_13values, METH_NOARGS, 0},
  {"items", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_15items, METH_NOARGS, 0},
  {"cursor", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_21cursor, METH_VARARGS|METH_KEYWORDS, 0},
  {"update", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_23update, METH_VARARGS|METH_KEYWORDS, 0},
  {"transaction", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_25transaction, METH_NOARGS, 0},
  {"view", (PyCFunction)__pyx_pw_5sophy_14SimpleDatabase_27view, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5sophy_SimpleDatabase[] = {
  {(char *)"db_name", __pyx_getprop_5sophy_14SimpleDatabase_db_name, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_SimpleDatabase = {
  __pyx_pw_5sophy_14SimpleDatabase_19__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_5sophy_SimpleDatabase, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_5sophy_14SimpleDatabase_9__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_SimpleDatabase = {
  __pyx_pw_5sophy_14SimpleDatabase_19__len__, /*mp_length*/
  __pyx_pw_5sophy_14SimpleDatabase_3__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_5sophy_SimpleDatabase, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_5sophy_SimpleDatabase = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.SimpleDatabase", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy_SimpleDatabase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy_SimpleDatabase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_SimpleDatabase, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_SimpleDatabase, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy_SimpleDatabase, /*tp_traverse*/
  __pyx_tp_clear_5sophy_SimpleDatabase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_5sophy_14SimpleDatabase_17__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5sophy_SimpleDatabase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5sophy_SimpleDatabase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5sophy_14SimpleDatabase_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy_SimpleDatabase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct___sp *__pyx_freelist_5sophy___pyx_scope_struct___sp[8];
static int __pyx_freecount_5sophy___pyx_scope_struct___sp = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct___sp(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct___sp > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct___sp)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct___sp[--__pyx_freecount_5sophy___pyx_scope_struct___sp];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct___sp));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct___sp(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *p = (struct __pyx_obj_5sophy___pyx_scope_struct___sp *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_propname);
  Py_CLEAR(p->__pyx_v_string);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct___sp < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct___sp)))) {
    __pyx_freelist_5sophy___pyx_scope_struct___sp[__pyx_freecount_5sophy___pyx_scope_struct___sp++] = ((struct __pyx_obj_5sophy___pyx_scope_struct___sp *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct___sp(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *p = (struct __pyx_obj_5sophy___pyx_scope_struct___sp *)o;
  if (p->__pyx_v_propname) {
    e = (*v)(p->__pyx_v_propname, a); if (e) return e;
  }
  if (p->__pyx_v_string) {
    e = (*v)(p->__pyx_v_string, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct___sp(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct___sp *p = (struct __pyx_obj_5sophy___pyx_scope_struct___sp *)o;
  tmp = ((PyObject*)p->__pyx_v_propname);
  p->__pyx_v_propname = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_string);
  p->__pyx_v_string = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct___sp = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct___sp", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct___sp), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct___sp, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct___sp, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct___sp, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct___sp, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *__pyx_freelist_5sophy___pyx_scope_struct_1__sm[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_1__sm = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_1__sm(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_1__sm > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_1__sm)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_1__sm[--__pyx_freecount_5sophy___pyx_scope_struct_1__sm];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_1__sm));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_1__sm(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *p = (struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_propname);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_1__sm < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_1__sm)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_1__sm[__pyx_freecount_5sophy___pyx_scope_struct_1__sm++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_1__sm(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *p = (struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *)o;
  if (p->__pyx_v_propname) {
    e = (*v)(p->__pyx_v_propname, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_1__sm(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *p = (struct __pyx_obj_5sophy___pyx_scope_struct_1__sm *)o;
  tmp = ((PyObject*)p->__pyx_v_propname);
  p->__pyx_v_propname = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_1__sm = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_1__sm", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_1__sm), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_1__sm, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_1__sm, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_1__sm, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_1__sm, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *__pyx_freelist_5sophy___pyx_scope_struct_2__dbp[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_2__dbp = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_2__dbp(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_2__dbp > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_2__dbp[--__pyx_freecount_5sophy___pyx_scope_struct_2__dbp];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_2__dbp(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *p = (struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_prop);
  Py_CLEAR(p->__pyx_v_string);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_2__dbp < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_2__dbp[__pyx_freecount_5sophy___pyx_scope_struct_2__dbp++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_2__dbp(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *p = (struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *)o;
  if (p->__pyx_v_prop) {
    e = (*v)(p->__pyx_v_prop, a); if (e) return e;
  }
  if (p->__pyx_v_string) {
    e = (*v)(p->__pyx_v_string, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_2__dbp(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *p = (struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp *)o;
  tmp = ((PyObject*)p->__pyx_v_prop);
  p->__pyx_v_prop = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_string);
  p->__pyx_v_string = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_2__dbp = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_2__dbp", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_2__dbp), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_2__dbp, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_2__dbp, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_2__dbp, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_2__dbp, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *__pyx_freelist_5sophy___pyx_scope_struct_3__dbm[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_3__dbm = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_3__dbm(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_3__dbm > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_3__dbm[--__pyx_freecount_5sophy___pyx_scope_struct_3__dbm];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_3__dbm(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *p = (struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_propname);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_3__dbm < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_3__dbm[__pyx_freecount_5sophy___pyx_scope_struct_3__dbm++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_3__dbm(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *p = (struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *)o;
  if (p->__pyx_v_propname) {
    e = (*v)(p->__pyx_v_propname, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_3__dbm(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *p = (struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm *)o;
  tmp = ((PyObject*)p->__pyx_v_propname);
  p->__pyx_v_propname = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_3__dbm = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_3__dbm", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_3__dbm), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_3__dbm, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_3__dbm, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_3__dbm, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_3__dbm, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *__pyx_freelist_5sophy___pyx_scope_struct_4___call__[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_4___call__ = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_4___call__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_4___call__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_4___call__)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_4___call__[--__pyx_freecount_5sophy___pyx_scope_struct_4___call__];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_4___call__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_4___call__(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *p = (struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_fn);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_4___call__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_4___call__)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_4___call__[__pyx_freecount_5sophy___pyx_scope_struct_4___call__++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_4___call__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *p = (struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *)o;
  if (p->__pyx_v_fn) {
    e = (*v)(p->__pyx_v_fn, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject*)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_4___call__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *p = (struct __pyx_obj_5sophy___pyx_scope_struct_4___call__ *)o;
  tmp = ((PyObject*)p->__pyx_v_fn);
  p->__pyx_v_fn = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_5sophy__BaseDBObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_4___call__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_4___call__", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_4___call__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_4___call__, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_4___call__, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_4___call__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_4___call__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *__pyx_freelist_5sophy___pyx_scope_struct_5_get_range[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_5_get_range = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_5_get_range(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_5_get_range > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_5_get_range[--__pyx_freecount_5sophy___pyx_scope_struct_5_get_range];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_5_get_range(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *p = (struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_cursor);
  Py_CLEAR(p->__pyx_v_first);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_last);
  Py_CLEAR(p->__pyx_v_order);
  Py_CLEAR(p->__pyx_v_reverse);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_start);
  Py_CLEAR(p->__pyx_v_stop);
  Py_CLEAR(p->__pyx_v_value);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_5_get_range < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_5_get_range[__pyx_freecount_5sophy___pyx_scope_struct_5_get_range++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_5_get_range(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *p = (struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *)o;
  if (p->__pyx_v_cursor) {
    e = (*v)(((PyObject*)p->__pyx_v_cursor), a); if (e) return e;
  }
  if (p->__pyx_v_first) {
    e = (*v)(p->__pyx_v_first, a); if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_last) {
    e = (*v)(p->__pyx_v_last, a); if (e) return e;
  }
  if (p->__pyx_v_order) {
    e = (*v)(p->__pyx_v_order, a); if (e) return e;
  }
  if (p->__pyx_v_reverse) {
    e = (*v)(p->__pyx_v_reverse, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject*)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_start) {
    e = (*v)(p->__pyx_v_start, a); if (e) return e;
  }
  if (p->__pyx_v_stop) {
    e = (*v)(p->__pyx_v_stop, a); if (e) return e;
  }
  if (p->__pyx_v_value) {
    e = (*v)(p->__pyx_v_value, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_5_get_range(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *p = (struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range *)o;
  tmp = ((PyObject*)p->__pyx_v_cursor);
  p->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_first);
  p->__pyx_v_first = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_key);
  p->__pyx_v_key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_last);
  p->__pyx_v_last = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_order);
  p->__pyx_v_order = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_reverse);
  p->__pyx_v_reverse = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_5sophy__BaseDBObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_start);
  p->__pyx_v_start = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_stop);
  p->__pyx_v_stop = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_value);
  p->__pyx_v_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_5_get_range = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_5_get_range", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_5_get_range), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_5_get_range, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_5_get_range, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_5_get_range, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_5_get_range, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *__pyx_freelist_5sophy___pyx_scope_struct_6_keys[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_6_keys = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_6_keys(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_6_keys > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_6_keys)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_6_keys[--__pyx_freecount_5sophy___pyx_scope_struct_6_keys];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_6_keys));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_6_keys(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *p = (struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_cursor);
  Py_CLEAR(p->__pyx_v_item);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_6_keys < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_6_keys)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_6_keys[__pyx_freecount_5sophy___pyx_scope_struct_6_keys++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_6_keys(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *p = (struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *)o;
  if (p->__pyx_v_cursor) {
    e = (*v)(((PyObject*)p->__pyx_v_cursor), a); if (e) return e;
  }
  if (p->__pyx_v_item) {
    e = (*v)(p->__pyx_v_item, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject*)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_6_keys(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *p = (struct __pyx_obj_5sophy___pyx_scope_struct_6_keys *)o;
  tmp = ((PyObject*)p->__pyx_v_cursor);
  p->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_item);
  p->__pyx_v_item = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_5sophy__BaseDBObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_6_keys = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_6_keys", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_6_keys), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_6_keys, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_6_keys, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_6_keys, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_6_keys, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_7_values *__pyx_freelist_5sophy___pyx_scope_struct_7_values[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_7_values = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_7_values(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_7_values > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_7_values)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_7_values[--__pyx_freecount_5sophy___pyx_scope_struct_7_values];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_7_values));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_7_values(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_7_values *p = (struct __pyx_obj_5sophy___pyx_scope_struct_7_values *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_cursor);
  Py_CLEAR(p->__pyx_v_item);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_7_values < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_7_values)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_7_values[__pyx_freecount_5sophy___pyx_scope_struct_7_values++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_7_values *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_7_values(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_7_values *p = (struct __pyx_obj_5sophy___pyx_scope_struct_7_values *)o;
  if (p->__pyx_v_cursor) {
    e = (*v)(((PyObject*)p->__pyx_v_cursor), a); if (e) return e;
  }
  if (p->__pyx_v_item) {
    e = (*v)(p->__pyx_v_item, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject*)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_7_values(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_7_values *p = (struct __pyx_obj_5sophy___pyx_scope_struct_7_values *)o;
  tmp = ((PyObject*)p->__pyx_v_cursor);
  p->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_item);
  p->__pyx_v_item = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_5sophy__BaseDBObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_7_values = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_7_values", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_7_values), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_7_values, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_7_values, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_7_values, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_7_values, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5sophy___pyx_scope_struct_8_items *__pyx_freelist_5sophy___pyx_scope_struct_8_items[8];
static int __pyx_freecount_5sophy___pyx_scope_struct_8_items = 0;

static PyObject *__pyx_tp_new_5sophy___pyx_scope_struct_8_items(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5sophy___pyx_scope_struct_8_items > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_8_items)))) {
    o = (PyObject*)__pyx_freelist_5sophy___pyx_scope_struct_8_items[--__pyx_freecount_5sophy___pyx_scope_struct_8_items];
    memset(o, 0, sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_8_items));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5sophy___pyx_scope_struct_8_items(PyObject *o) {
  struct __pyx_obj_5sophy___pyx_scope_struct_8_items *p = (struct __pyx_obj_5sophy___pyx_scope_struct_8_items *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_cursor);
  Py_CLEAR(p->__pyx_v_item);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5sophy___pyx_scope_struct_8_items < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_8_items)))) {
    __pyx_freelist_5sophy___pyx_scope_struct_8_items[__pyx_freecount_5sophy___pyx_scope_struct_8_items++] = ((struct __pyx_obj_5sophy___pyx_scope_struct_8_items *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5sophy___pyx_scope_struct_8_items(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5sophy___pyx_scope_struct_8_items *p = (struct __pyx_obj_5sophy___pyx_scope_struct_8_items *)o;
  if (p->__pyx_v_cursor) {
    e = (*v)(((PyObject*)p->__pyx_v_cursor), a); if (e) return e;
  }
  if (p->__pyx_v_item) {
    e = (*v)(p->__pyx_v_item, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject*)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5sophy___pyx_scope_struct_8_items(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5sophy___pyx_scope_struct_8_items *p = (struct __pyx_obj_5sophy___pyx_scope_struct_8_items *)o;
  tmp = ((PyObject*)p->__pyx_v_cursor);
  p->__pyx_v_cursor = ((struct __pyx_obj_5sophy_Cursor *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_item);
  p->__pyx_v_item = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_5sophy__BaseDBObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5sophy___pyx_scope_struct_8_items = {
  PyVarObject_HEAD_INIT(0, 0)
  "sophy.__pyx_scope_struct_8_items", /*tp_name*/
  sizeof(struct __pyx_obj_5sophy___pyx_scope_struct_8_items), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5sophy___pyx_scope_struct_8_items, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5sophy___pyx_scope_struct_8_items, /*tp_traverse*/
  __pyx_tp_clear_5sophy___pyx_scope_struct_8_items, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5sophy___pyx_scope_struct_8_items, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "sophy",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_BaseDBObject_get_range, __pyx_k_BaseDBObject_get_range, sizeof(__pyx_k_BaseDBObject_get_range), 0, 0, 1, 1},
  {&__pyx_n_s_BaseDBObject_items, __pyx_k_BaseDBObject_items, sizeof(__pyx_k_BaseDBObject_items), 0, 0, 1, 1},
  {&__pyx_n_s_BaseDBObject_keys, __pyx_k_BaseDBObject_keys, sizeof(__pyx_k_BaseDBObject_keys), 0, 0, 1, 1},
  {&__pyx_n_s_BaseDBObject_values, __pyx_k_BaseDBObject_values, sizeof(__pyx_k_BaseDBObject_values), 0, 0, 1, 1},
  {&__pyx_n_s_CannotCloseException, __pyx_k_CannotCloseException, sizeof(__pyx_k_CannotCloseException), 0, 0, 1, 1},
  {&__pyx_n_s_Exception, __pyx_k_Exception, sizeof(__pyx_k_Exception), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s_NotImplemented, __pyx_k_NotImplemented, sizeof(__pyx_k_NotImplemented), 0, 0, 1, 1},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_StopIteration, __pyx_k_StopIteration, sizeof(__pyx_k_StopIteration), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_allocate_object_s, __pyx_k_Unable_to_allocate_object_s, sizeof(__pyx_k_Unable_to_allocate_object_s), 0, 0, 1, 0},
  {&__pyx_n_s_UnicodeDecodeError, __pyx_k_UnicodeDecodeError, sizeof(__pyx_k_UnicodeDecodeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unrecognized_index_type_must_be, __pyx_k_Unrecognized_index_type_must_be, sizeof(__pyx_k_Unrecognized_index_type_must_be), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s_Views_are_read_only, __pyx_k_Views_are_read_only, sizeof(__pyx_k_Views_are_read_only), 0, 0, 1, 0},
  {&__pyx_kp_s__20, __pyx_k__20, sizeof(__pyx_k__20), 0, 0, 1, 0},
  {&__pyx_kp_s__21, __pyx_k__21, sizeof(__pyx_k__21), 0, 0, 1, 0},
  {&__pyx_kp_s__23, __pyx_k__23, sizeof(__pyx_k__23), 0, 0, 1, 0},
  {&__pyx_kp_s__28, __pyx_k__28, sizeof(__pyx_k__28), 0, 0, 1, 0},
  {&__pyx_kp_s__30, __pyx_k__30, sizeof(__pyx_k__30), 0, 0, 1, 0},
  {&__pyx_n_s_amqf, __pyx_k_amqf, sizeof(__pyx_k_amqf), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_auto_open, __pyx_k_auto_open, sizeof(__pyx_k_auto_open), 0, 0, 1, 1},
  {&__pyx_kp_s_backup_active, __pyx_k_backup_active, sizeof(__pyx_k_backup_active), 0, 0, 1, 0},
  {&__pyx_n_s_backup_active_2, __pyx_k_backup_active_2, sizeof(__pyx_k_backup_active_2), 0, 0, 1, 1},
  {&__pyx_kp_s_backup_last, __pyx_k_backup_last, sizeof(__pyx_k_backup_last), 0, 0, 1, 0},
  {&__pyx_n_s_backup_last_2, __pyx_k_backup_last_2, sizeof(__pyx_k_backup_last_2), 0, 0, 1, 1},
  {&__pyx_kp_s_backup_last_complete, __pyx_k_backup_last_complete, sizeof(__pyx_k_backup_last_complete), 0, 0, 1, 0},
  {&__pyx_n_s_backup_last_complete_2, __pyx_k_backup_last_complete_2, sizeof(__pyx_k_backup_last_complete_2), 0, 0, 1, 1},
  {&__pyx_n_s_backup_path, __pyx_k_backup_path, sizeof(__pyx_k_backup_path), 0, 0, 1, 1},
  {&__pyx_kp_s_backup_path_2, __pyx_k_backup_path_2, sizeof(__pyx_k_backup_path_2), 0, 0, 1, 0},
  {&__pyx_kp_s_backup_run, __pyx_k_backup_run, sizeof(__pyx_k_backup_run), 0, 0, 1, 0},
  {&__pyx_n_s_backup_run_2, __pyx_k_backup_run_2, sizeof(__pyx_k_backup_run_2), 0, 0, 1, 1},
  {&__pyx_n_s_basename, __pyx_k_basename, sizeof(__pyx_k_basename), 0, 0, 1, 1},
  {&__pyx_kp_s_bcdefgh, __pyx_k_bcdefgh, sizeof(__pyx_k_bcdefgh), 0, 0, 1, 0},
  {&__pyx_n_s_begin, __pyx_k_begin, sizeof(__pyx_k_begin), 0, 0, 1, 1},
  {&__pyx_n_s_branch, __pyx_k_branch, sizeof(__pyx_k_branch), 0, 0, 1, 1},
  {&__pyx_n_s_build, __pyx_k_build, sizeof(__pyx_k_build), 0, 0, 1, 1},
  {&__pyx_n_s_cache, __pyx_k_cache, sizeof(__pyx_k_cache), 0, 0, 1, 1},
  {&__pyx_n_s_cache_mode, __pyx_k_cache_mode, sizeof(__pyx_k_cache_mode), 0, 0, 1, 1},
  {&__pyx_n_s_call___locals_wrapper, __pyx_k_call___locals_wrapper, sizeof(__pyx_k_call___locals_wrapper), 0, 0, 1, 1},
  {&__pyx_n_s_clear_option, __pyx_k_clear_option, sizeof(__pyx_k_clear_option), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_commit, __pyx_k_commit, sizeof(__pyx_k_commit), 0, 0, 1, 1},
  {&__pyx_n_s_compact, __pyx_k_compact, sizeof(__pyx_k_compact), 0, 0, 1, 1},
  {&__pyx_n_s_compact_index, __pyx_k_compact_index, sizeof(__pyx_k_compact_index), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone, __pyx_k_compaction_redzone, sizeof(__pyx_k_compaction_redzone), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_2, __pyx_k_compaction_redzone_2, sizeof(__pyx_k_compaction_redzone_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_anticache_per, __pyx_k_compaction_redzone_anticache_per, sizeof(__pyx_k_compaction_redzone_anticache_per), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_anticache_per_2, __pyx_k_compaction_redzone_anticache_per_2, sizeof(__pyx_k_compaction_redzone_anticache_per_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_async, __pyx_k_compaction_redzone_async, sizeof(__pyx_k_compaction_redzone_async), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_async_2, __pyx_k_compaction_redzone_async_2, sizeof(__pyx_k_compaction_redzone_async_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_backup_prio, __pyx_k_compaction_redzone_backup_prio, sizeof(__pyx_k_compaction_redzone_backup_prio), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_backup_prio_2, __pyx_k_compaction_redzone_backup_prio_2, sizeof(__pyx_k_compaction_redzone_backup_prio_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_branch_age, __pyx_k_compaction_redzone_branch_age, sizeof(__pyx_k_compaction_redzone_branch_age), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_branch_age_2, __pyx_k_compaction_redzone_branch_age_2, sizeof(__pyx_k_compaction_redzone_branch_age_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_branch_age_pe, __pyx_k_compaction_redzone_branch_age_pe, sizeof(__pyx_k_compaction_redzone_branch_age_pe), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_branch_age_pe_2, __pyx_k_compaction_redzone_branch_age_pe_2, sizeof(__pyx_k_compaction_redzone_branch_age_pe_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_branch_age_wm, __pyx_k_compaction_redzone_branch_age_wm, sizeof(__pyx_k_compaction_redzone_branch_age_wm), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_branch_age_wm_2, __pyx_k_compaction_redzone_branch_age_wm_2, sizeof(__pyx_k_compaction_redzone_branch_age_wm_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_branch_prio, __pyx_k_compaction_redzone_branch_prio, sizeof(__pyx_k_compaction_redzone_branch_prio), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_branch_prio_2, __pyx_k_compaction_redzone_branch_prio_2, sizeof(__pyx_k_compaction_redzone_branch_prio_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_branch_wm, __pyx_k_compaction_redzone_branch_wm, sizeof(__pyx_k_compaction_redzone_branch_wm), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_branch_wm_2, __pyx_k_compaction_redzone_branch_wm_2, sizeof(__pyx_k_compaction_redzone_branch_wm_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_compact_mode, __pyx_k_compaction_redzone_compact_mode, sizeof(__pyx_k_compaction_redzone_compact_mode), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_compact_mode_2, __pyx_k_compaction_redzone_compact_mode_2, sizeof(__pyx_k_compaction_redzone_compact_mode_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_compact_wm, __pyx_k_compaction_redzone_compact_wm, sizeof(__pyx_k_compaction_redzone_compact_wm), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_compact_wm_2, __pyx_k_compaction_redzone_compact_wm_2, sizeof(__pyx_k_compaction_redzone_compact_wm_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_gc_db_prio, __pyx_k_compaction_redzone_gc_db_prio, sizeof(__pyx_k_compaction_redzone_gc_db_prio), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_gc_db_prio_2, __pyx_k_compaction_redzone_gc_db_prio_2, sizeof(__pyx_k_compaction_redzone_gc_db_prio_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_gc_period, __pyx_k_compaction_redzone_gc_period, sizeof(__pyx_k_compaction_redzone_gc_period), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_gc_period_2, __pyx_k_compaction_redzone_gc_period_2, sizeof(__pyx_k_compaction_redzone_gc_period_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_gc_wm, __pyx_k_compaction_redzone_gc_wm, sizeof(__pyx_k_compaction_redzone_gc_wm), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_gc_wm_2, __pyx_k_compaction_redzone_gc_wm_2, sizeof(__pyx_k_compaction_redzone_gc_wm_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_lru_period, __pyx_k_compaction_redzone_lru_period, sizeof(__pyx_k_compaction_redzone_lru_period), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_lru_period_2, __pyx_k_compaction_redzone_lru_period_2, sizeof(__pyx_k_compaction_redzone_lru_period_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_lru_prio, __pyx_k_compaction_redzone_lru_prio, sizeof(__pyx_k_compaction_redzone_lru_prio), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_lru_prio_2, __pyx_k_compaction_redzone_lru_prio_2, sizeof(__pyx_k_compaction_redzone_lru_prio_2), 0, 0, 1, 1},
  {&__pyx_kp_s_compaction_redzone_mode, __pyx_k_compaction_redzone_mode, sizeof(__pyx_k_compaction_redzone_mode), 0, 0, 1, 0},
  {&__pyx_n_s_compaction_redzone_mode_2, __pyx_k_compaction_redzone_mode_2, sizeof(__pyx_k_compaction_redzone_mode_2), 0, 0, 1, 1},
  {&__pyx_n_s_compression, __pyx_k_compression, sizeof(__pyx_k_compression), 0, 0, 1, 1},
  {&__pyx_n_s_compression_branch, __pyx_k_compression_branch, sizeof(__pyx_k_compression_branch), 0, 0, 1, 1},
  {&__pyx_n_s_compression_key, __pyx_k_compression_key, sizeof(__pyx_k_compression_key), 0, 0, 1, 1},
  {&__pyx_n_s_config, __pyx_k_config, sizeof(__pyx_k_config), 0, 0, 1, 1},
  {&__pyx_n_s_configure, __pyx_k_configure, sizeof(__pyx_k_configure), 0, 0, 1, 1},
  {&__pyx_n_s_connect, __pyx_k_connect, sizeof(__pyx_k_connect), 0, 0, 1, 1},
  {&__pyx_n_s_cursor, __pyx_k_cursor, sizeof(__pyx_k_cursor), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_data_dir, __pyx_k_data_dir, sizeof(__pyx_k_data_dir), 0, 0, 1, 1},
  {&__pyx_n_s_database_id, __pyx_k_database_id, sizeof(__pyx_k_database_id), 0, 0, 1, 1},
  {&__pyx_n_s_database_path, __pyx_k_database_path, sizeof(__pyx_k_database_path), 0, 0, 1, 1},
  {&__pyx_n_s_databases, __pyx_k_databases, sizeof(__pyx_k_databases), 0, 0, 1, 1},
  {&__pyx_n_s_db, __pyx_k_db, sizeof(__pyx_k_db), 0, 0, 1, 1},
  {&__pyx_n_s_db_name, __pyx_k_db_name, sizeof(__pyx_k_db_name), 0, 0, 1, 1},
  {&__pyx_kp_s_db_s, __pyx_k_db_s, sizeof(__pyx_k_db_s), 0, 0, 1, 0},
  {&__pyx_kp_s_db_s_index, __pyx_k_db_s_index, sizeof(__pyx_k_db_s_index), 0, 0, 1, 0},
  {&__pyx_kp_s_db_s_index_s, __pyx_k_db_s_index_s, sizeof(__pyx_k_db_s_index_s), 0, 0, 1, 0},
  {&__pyx_kp_s_db_s_s, __pyx_k_db_s_s, sizeof(__pyx_k_db_s_s), 0, 0, 1, 0},
  {&__pyx_n_s_dbm, __pyx_k_dbm, sizeof(__pyx_k_dbm), 0, 0, 1, 1},
  {&__pyx_n_s_dbm_locals__method, __pyx_k_dbm_locals__method, sizeof(__pyx_k_dbm_locals__method), 0, 0, 1, 1},
  {&__pyx_n_s_dbp, __pyx_k_dbp, sizeof(__pyx_k_dbp), 0, 0, 1, 1},
  {&__pyx_n_s_dbp_locals__getter, __pyx_k_dbp_locals__getter, sizeof(__pyx_k_dbp_locals__getter), 0, 0, 1, 1},
  {&__pyx_n_s_dbp_locals__setter, __pyx_k_dbp_locals__setter, sizeof(__pyx_k_dbp_locals__setter), 0, 0, 1, 1},
  {&__pyx_n_s_default, __pyx_k_default, sizeof(__pyx_k_default), 0, 0, 1, 1},
  {&__pyx_n_s_dirname, __pyx_k_dirname, sizeof(__pyx_k_dirname), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_exc_tb, __pyx_k_exc_tb, sizeof(__pyx_k_exc_tb), 0, 0, 1, 1},
  {&__pyx_n_s_exc_type, __pyx_k_exc_type, sizeof(__pyx_k_exc_type), 0, 0, 1, 1},
  {&__pyx_n_s_exc_val, __pyx_k_exc_val, sizeof(__pyx_k_exc_val), 0, 0, 1, 1},
  {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
  {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
  {&__pyx_n_s_fn, __pyx_k_fn, sizeof(__pyx_k_fn), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_functools, __pyx_k_functools, sizeof(__pyx_k_functools), 0, 0, 1, 1},
  {&__pyx_n_s_get_option, __pyx_k_get_option, sizeof(__pyx_k_get_option), 0, 0, 1, 1},
  {&__pyx_n_s_get_range, __pyx_k_get_range, sizeof(__pyx_k_get_range), 0, 0, 1, 1},
  {&__pyx_n_s_getter, __pyx_k_getter, sizeof(__pyx_k_getter), 0, 0, 1, 1},
  {&__pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_k_home_charles_Dropbox_code_sophy, sizeof(__pyx_k_home_charles_Dropbox_code_sophy), 0, 0, 1, 0},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_kp_s_index_branch_count, __pyx_k_index_branch_count, sizeof(__pyx_k_index_branch_count), 0, 0, 1, 0},
  {&__pyx_n_s_index_branch_count_2, __pyx_k_index_branch_count_2, sizeof(__pyx_k_index_branch_count_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_branch_max, __pyx_k_index_branch_max, sizeof(__pyx_k_index_branch_max), 0, 0, 1, 0},
  {&__pyx_n_s_index_branch_max_2, __pyx_k_index_branch_max_2, sizeof(__pyx_k_index_branch_max_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_count, __pyx_k_index_count, sizeof(__pyx_k_index_count), 0, 0, 1, 0},
  {&__pyx_n_s_index_count_2, __pyx_k_index_count_2, sizeof(__pyx_k_index_count_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_count_dup, __pyx_k_index_count_dup, sizeof(__pyx_k_index_count_dup), 0, 0, 1, 0},
  {&__pyx_n_s_index_count_dup_2, __pyx_k_index_count_dup_2, sizeof(__pyx_k_index_count_dup_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_memory_used, __pyx_k_index_memory_used, sizeof(__pyx_k_index_memory_used), 0, 0, 1, 0},
  {&__pyx_n_s_index_memory_used_2, __pyx_k_index_memory_used_2, sizeof(__pyx_k_index_memory_used_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_node_count, __pyx_k_index_node_count, sizeof(__pyx_k_index_node_count), 0, 0, 1, 0},
  {&__pyx_n_s_index_node_count_2, __pyx_k_index_node_count_2, sizeof(__pyx_k_index_node_count_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_page_count, __pyx_k_index_page_count, sizeof(__pyx_k_index_page_count), 0, 0, 1, 0},
  {&__pyx_n_s_index_page_count_2, __pyx_k_index_page_count_2, sizeof(__pyx_k_index_page_count_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_read_cache, __pyx_k_index_read_cache, sizeof(__pyx_k_index_read_cache), 0, 0, 1, 0},
  {&__pyx_n_s_index_read_cache_2, __pyx_k_index_read_cache_2, sizeof(__pyx_k_index_read_cache_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_read_disk, __pyx_k_index_read_disk, sizeof(__pyx_k_index_read_disk), 0, 0, 1, 0},
  {&__pyx_n_s_index_read_disk_2, __pyx_k_index_read_disk_2, sizeof(__pyx_k_index_read_disk_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_size, __pyx_k_index_size, sizeof(__pyx_k_index_size), 0, 0, 1, 0},
  {&__pyx_n_s_index_size_2, __pyx_k_index_size_2, sizeof(__pyx_k_index_size_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_size_amqf, __pyx_k_index_size_amqf, sizeof(__pyx_k_index_size_amqf), 0, 0, 1, 0},
  {&__pyx_n_s_index_size_amqf_2, __pyx_k_index_size_amqf_2, sizeof(__pyx_k_index_size_amqf_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_size_snapshot, __pyx_k_index_size_snapshot, sizeof(__pyx_k_index_size_snapshot), 0, 0, 1, 0},
  {&__pyx_n_s_index_size_snapshot_2, __pyx_k_index_size_snapshot_2, sizeof(__pyx_k_index_size_snapshot_2), 0, 0, 1, 1},
  {&__pyx_kp_s_index_size_uncompressed, __pyx_k_index_size_uncompressed, sizeof(__pyx_k_index_size_uncompressed), 0, 0, 1, 0},
  {&__pyx_n_s_index_size_uncompressed_2, __pyx_k_index_size_uncompressed_2, sizeof(__pyx_k_index_size_uncompressed_2), 0, 0, 1, 1},
  {&__pyx_n_s_index_type, __pyx_k_index_type, sizeof(__pyx_k_index_type), 0, 0, 1, 1},
  {&__pyx_n_s_index_types, __pyx_k_index_types, sizeof(__pyx_k_index_types), 0, 0, 1, 1},
  {&__pyx_kp_s_index_types_is_a_required_parame, __pyx_k_index_types_is_a_required_parame, sizeof(__pyx_k_index_types_is_a_required_parame), 0, 0, 1, 0},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_kp_s_key__s, __pyx_k_key__s, sizeof(__pyx_k_key__s), 0, 0, 1, 0},
  {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_kp_s_log_enable, __pyx_k_log_enable, sizeof(__pyx_k_log_enable), 0, 0, 1, 0},
  {&__pyx_n_s_log_enable_2, __pyx_k_log_enable_2, sizeof(__pyx_k_log_enable_2), 0, 0, 1, 1},
  {&__pyx_kp_s_log_files, __pyx_k_log_files, sizeof(__pyx_k_log_files), 0, 0, 1, 0},
  {&__pyx_n_s_log_files_2, __pyx_k_log_files_2, sizeof(__pyx_k_log_files_2), 0, 0, 1, 1},
  {&__pyx_kp_s_log_gc, __pyx_k_log_gc, sizeof(__pyx_k_log_gc), 0, 0, 1, 0},
  {&__pyx_n_s_log_gc_2, __pyx_k_log_gc_2, sizeof(__pyx_k_log_gc_2), 0, 0, 1, 1},
  {&__pyx_n_s_log_path, __pyx_k_log_path, sizeof(__pyx_k_log_path), 0, 0, 1, 1},
  {&__pyx_kp_s_log_path_2, __pyx_k_log_path_2, sizeof(__pyx_k_log_path_2), 0, 0, 1, 0},
  {&__pyx_kp_s_log_rotate, __pyx_k_log_rotate, sizeof(__pyx_k_log_rotate), 0, 0, 1, 0},
  {&__pyx_n_s_log_rotate_2, __pyx_k_log_rotate_2, sizeof(__pyx_k_log_rotate_2), 0, 0, 1, 1},
  {&__pyx_kp_s_log_rotate_sync, __pyx_k_log_rotate_sync, sizeof(__pyx_k_log_rotate_sync), 0, 0, 1, 0},
  {&__pyx_n_s_log_rotate_sync_2, __pyx_k_log_rotate_sync_2, sizeof(__pyx_k_log_rotate_sync_2), 0, 0, 1, 1},
  {&__pyx_kp_s_log_rotate_wm, __pyx_k_log_rotate_wm, sizeof(__pyx_k_log_rotate_wm), 0, 0, 1, 0},
  {&__pyx_n_s_log_rotate_wm_2, __pyx_k_log_rotate_wm_2, sizeof(__pyx_k_log_rotate_wm_2), 0, 0, 1, 1},
  {&__pyx_kp_s_log_sync, __pyx_k_log_sync, sizeof(__pyx_k_log_sync), 0, 0, 1, 0},
  {&__pyx_n_s_log_sync_2, __pyx_k_log_sync_2, sizeof(__pyx_k_log_sync_2), 0, 0, 1, 1},
  {&__pyx_n_s_lru, __pyx_k_lru, sizeof(__pyx_k_lru), 0, 0, 1, 1},
  {&__pyx_n_s_lru_step, __pyx_k_lru_step, sizeof(__pyx_k_lru_step), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_kp_s_memory_anticache, __pyx_k_memory_anticache, sizeof(__pyx_k_memory_anticache), 0, 0, 1, 0},
  {&__pyx_n_s_memory_anticache_2, __pyx_k_memory_anticache_2, sizeof(__pyx_k_memory_anticache_2), 0, 0, 1, 1},
  {&__pyx_n_s_memory_limit, __pyx_k_memory_limit, sizeof(__pyx_k_memory_limit), 0, 0, 1, 1},
  {&__pyx_kp_s_memory_limit_2, __pyx_k_memory_limit_2, sizeof(__pyx_k_memory_limit_2), 0, 0, 1, 0},
  {&__pyx_kp_s_memory_pager_page_size, __pyx_k_memory_pager_page_size, sizeof(__pyx_k_memory_pager_page_size), 0, 0, 1, 0},
  {&__pyx_n_s_memory_pager_page_size_2, __pyx_k_memory_pager_page_size_2, sizeof(__pyx_k_memory_pager_page_size_2), 0, 0, 1, 1},
  {&__pyx_kp_s_memory_pager_pool_size, __pyx_k_memory_pager_pool_size, sizeof(__pyx_k_memory_pager_pool_size), 0, 0, 1, 0},
  {&__pyx_n_s_memory_pager_pool_size_2, __pyx_k_memory_pager_pool_size_2, sizeof(__pyx_k_memory_pager_pool_size_2), 0, 0, 1, 1},
  {&__pyx_kp_s_memory_pager_pools, __pyx_k_memory_pager_pools, sizeof(__pyx_k_memory_pager_pools), 0, 0, 1, 0},
  {&__pyx_n_s_memory_pager_pools_2, __pyx_k_memory_pager_pools_2, sizeof(__pyx_k_memory_pager_pools_2), 0, 0, 1, 1},
  {&__pyx_kp_s_memory_used, __pyx_k_memory_used, sizeof(__pyx_k_memory_used), 0, 0, 1, 0},
  {&__pyx_n_s_memory_used_2, __pyx_k_memory_used_2, sizeof(__pyx_k_memory_used_2), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_method, __pyx_k_method, sizeof(__pyx_k_method), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_asn, __pyx_k_metric_asn, sizeof(__pyx_k_metric_asn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_asn_2, __pyx_k_metric_asn_2, sizeof(__pyx_k_metric_asn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_bsn, __pyx_k_metric_bsn, sizeof(__pyx_k_metric_bsn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_bsn_2, __pyx_k_metric_bsn_2, sizeof(__pyx_k_metric_bsn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_dsn, __pyx_k_metric_dsn, sizeof(__pyx_k_metric_dsn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_dsn_2, __pyx_k_metric_dsn_2, sizeof(__pyx_k_metric_dsn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_lfsn, __pyx_k_metric_lfsn, sizeof(__pyx_k_metric_lfsn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_lfsn_2, __pyx_k_metric_lfsn_2, sizeof(__pyx_k_metric_lfsn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_lsn, __pyx_k_metric_lsn, sizeof(__pyx_k_metric_lsn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_lsn_2, __pyx_k_metric_lsn_2, sizeof(__pyx_k_metric_lsn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_nsn, __pyx_k_metric_nsn, sizeof(__pyx_k_metric_nsn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_nsn_2, __pyx_k_metric_nsn_2, sizeof(__pyx_k_metric_nsn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_ssn, __pyx_k_metric_ssn, sizeof(__pyx_k_metric_ssn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_ssn_2, __pyx_k_metric_ssn_2, sizeof(__pyx_k_metric_ssn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_tsn, __pyx_k_metric_tsn, sizeof(__pyx_k_metric_tsn), 0, 0, 1, 0},
  {&__pyx_n_s_metric_tsn_2, __pyx_k_metric_tsn_2, sizeof(__pyx_k_metric_tsn_2), 0, 0, 1, 1},
  {&__pyx_n_s_mmap, __pyx_k_mmap, sizeof(__pyx_k_mmap), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_node_preload, __pyx_k_node_preload, sizeof(__pyx_k_node_preload), 0, 0, 1, 1},
  {&__pyx_n_s_node_size, __pyx_k_node_size, sizeof(__pyx_k_node_size), 0, 0, 1, 1},
  {&__pyx_n_s_open, __pyx_k_open, sizeof(__pyx_k_open), 0, 0, 1, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_s_os, __pyx_k_os, sizeof(__pyx_k_os), 0, 0, 1, 1},
  {&__pyx_n_s_page_checksum, __pyx_k_page_checksum, sizeof(__pyx_k_page_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_page_size, __pyx_k_page_size, sizeof(__pyx_k_page_size), 0, 0, 1, 1},
  {&__pyx_n_s_path, __pyx_k_path, sizeof(__pyx_k_path), 0, 0, 1, 1},
  {&__pyx_n_s_path_fail_on_drop, __pyx_k_path_fail_on_drop, sizeof(__pyx_k_path_fail_on_drop), 0, 0, 1, 1},
  {&__pyx_n_s_path_fail_on_exists, __pyx_k_path_fail_on_exists, sizeof(__pyx_k_path_fail_on_exists), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_cursor, __pyx_k_performance_cursor, sizeof(__pyx_k_performance_cursor), 0, 0, 1, 0},
  {&__pyx_n_s_performance_cursor_2, __pyx_k_performance_cursor_2, sizeof(__pyx_k_performance_cursor_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_cursor_latency, __pyx_k_performance_cursor_latency, sizeof(__pyx_k_performance_cursor_latency), 0, 0, 1, 0},
  {&__pyx_n_s_performance_cursor_latency_2, __pyx_k_performance_cursor_latency_2, sizeof(__pyx_k_performance_cursor_latency_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_cursor_ops, __pyx_k_performance_cursor_ops, sizeof(__pyx_k_performance_cursor_ops), 0, 0, 1, 0},
  {&__pyx_n_s_performance_cursor_ops_2, __pyx_k_performance_cursor_ops_2, sizeof(__pyx_k_performance_cursor_ops_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_cursor_read_cache, __pyx_k_performance_cursor_read_cache, sizeof(__pyx_k_performance_cursor_read_cache), 0, 0, 1, 0},
  {&__pyx_n_s_performance_cursor_read_cache_2, __pyx_k_performance_cursor_read_cache_2, sizeof(__pyx_k_performance_cursor_read_cache_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_cursor_read_disk, __pyx_k_performance_cursor_read_disk, sizeof(__pyx_k_performance_cursor_read_disk), 0, 0, 1, 0},
  {&__pyx_n_s_performance_cursor_read_disk_2, __pyx_k_performance_cursor_read_disk_2, sizeof(__pyx_k_performance_cursor_read_disk_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_delete, __pyx_k_performance_delete, sizeof(__pyx_k_performance_delete), 0, 0, 1, 0},
  {&__pyx_n_s_performance_delete_2, __pyx_k_performance_delete_2, sizeof(__pyx_k_performance_delete_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_delete_latency, __pyx_k_performance_delete_latency, sizeof(__pyx_k_performance_delete_latency), 0, 0, 1, 0},
  {&__pyx_n_s_performance_delete_latency_2, __pyx_k_performance_delete_latency_2, sizeof(__pyx_k_performance_delete_latency_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_documents, __pyx_k_performance_documents, sizeof(__pyx_k_performance_documents), 0, 0, 1, 0},
  {&__pyx_n_s_performance_documents_2, __pyx_k_performance_documents_2, sizeof(__pyx_k_performance_documents_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_documents_used, __pyx_k_performance_documents_used, sizeof(__pyx_k_performance_documents_used), 0, 0, 1, 0},
  {&__pyx_n_s_performance_documents_used_2, __pyx_k_performance_documents_used_2, sizeof(__pyx_k_performance_documents_used_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_get, __pyx_k_performance_get, sizeof(__pyx_k_performance_get), 0, 0, 1, 0},
  {&__pyx_n_s_performance_get_2, __pyx_k_performance_get_2, sizeof(__pyx_k_performance_get_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_get_latency, __pyx_k_performance_get_latency, sizeof(__pyx_k_performance_get_latency), 0, 0, 1, 0},
  {&__pyx_n_s_performance_get_latency_2, __pyx_k_performance_get_latency_2, sizeof(__pyx_k_performance_get_latency_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_get_read_cache, __pyx_k_performance_get_read_cache, sizeof(__pyx_k_performance_get_read_cache), 0, 0, 1, 0},
  {&__pyx_n_s_performance_get_read_cache_2, __pyx_k_performance_get_read_cache_2, sizeof(__pyx_k_performance_get_read_cache_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_get_read_disk, __pyx_k_performance_get_read_disk, sizeof(__pyx_k_performance_get_read_disk), 0, 0, 1, 0},
  {&__pyx_n_s_performance_get_read_disk_2, __pyx_k_performance_get_read_disk_2, sizeof(__pyx_k_performance_get_read_disk_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_key, __pyx_k_performance_key, sizeof(__pyx_k_performance_key), 0, 0, 1, 0},
  {&__pyx_n_s_performance_key_2, __pyx_k_performance_key_2, sizeof(__pyx_k_performance_key_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_set, __pyx_k_performance_set, sizeof(__pyx_k_performance_set), 0, 0, 1, 0},
  {&__pyx_n_s_performance_set_2, __pyx_k_performance_set_2, sizeof(__pyx_k_performance_set_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_set_latency, __pyx_k_performance_set_latency, sizeof(__pyx_k_performance_set_latency), 0, 0, 1, 0},
  {&__pyx_n_s_performance_set_latency_2, __pyx_k_performance_set_latency_2, sizeof(__pyx_k_performance_set_latency_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx, __pyx_k_performance_tx, sizeof(__pyx_k_performance_tx), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_2, __pyx_k_performance_tx_2, sizeof(__pyx_k_performance_tx_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_active_ro, __pyx_k_performance_tx_active_ro, sizeof(__pyx_k_performance_tx_active_ro), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_active_ro_2, __pyx_k_performance_tx_active_ro_2, sizeof(__pyx_k_performance_tx_active_ro_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_active_rw, __pyx_k_performance_tx_active_rw, sizeof(__pyx_k_performance_tx_active_rw), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_active_rw_2, __pyx_k_performance_tx_active_rw_2, sizeof(__pyx_k_performance_tx_active_rw_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_conflict, __pyx_k_performance_tx_conflict, sizeof(__pyx_k_performance_tx_conflict), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_conflict_2, __pyx_k_performance_tx_conflict_2, sizeof(__pyx_k_performance_tx_conflict_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_gc_queue, __pyx_k_performance_tx_gc_queue, sizeof(__pyx_k_performance_tx_gc_queue), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_gc_queue_2, __pyx_k_performance_tx_gc_queue_2, sizeof(__pyx_k_performance_tx_gc_queue_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_latency, __pyx_k_performance_tx_latency, sizeof(__pyx_k_performance_tx_latency), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_latency_2, __pyx_k_performance_tx_latency_2, sizeof(__pyx_k_performance_tx_latency_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_lock, __pyx_k_performance_tx_lock, sizeof(__pyx_k_performance_tx_lock), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_lock_2, __pyx_k_performance_tx_lock_2, sizeof(__pyx_k_performance_tx_lock_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_ops, __pyx_k_performance_tx_ops, sizeof(__pyx_k_performance_tx_ops), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_ops_2, __pyx_k_performance_tx_ops_2, sizeof(__pyx_k_performance_tx_ops_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_tx_rollback, __pyx_k_performance_tx_rollback, sizeof(__pyx_k_performance_tx_rollback), 0, 0, 1, 0},
  {&__pyx_n_s_performance_tx_rollback_2, __pyx_k_performance_tx_rollback_2, sizeof(__pyx_k_performance_tx_rollback_2), 0, 0, 1, 1},
  {&__pyx_kp_s_performance_value, __pyx_k_performance_value, sizeof(__pyx_k_performance_value), 0, 0, 1, 0},
  {&__pyx_n_s_performance_value_2, __pyx_k_performance_value_2, sizeof(__pyx_k_performance_value_2), 0, 0, 1, 1},
  {&__pyx_n_s_prefix, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_prop, __pyx_k_prop, sizeof(__pyx_k_prop), 0, 0, 1, 1},
  {&__pyx_n_s_property, __pyx_k_property, sizeof(__pyx_k_property), 0, 0, 1, 1},
  {&__pyx_n_s_propname, __pyx_k_propname, sizeof(__pyx_k_propname), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_readonly, __pyx_k_readonly, sizeof(__pyx_k_readonly), 0, 0, 1, 1},
  {&__pyx_n_s_recover, __pyx_k_recover, sizeof(__pyx_k_recover), 0, 0, 1, 1},
  {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_rollback, __pyx_k_rollback, sizeof(__pyx_k_rollback), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_anticache, __pyx_k_scheduler_anticache, sizeof(__pyx_k_scheduler_anticache), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_anticache_2, __pyx_k_scheduler_anticache_2, sizeof(__pyx_k_scheduler_anticache_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_anticache_active, __pyx_k_scheduler_anticache_active, sizeof(__pyx_k_scheduler_anticache_active), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_anticache_active_2, __pyx_k_scheduler_anticache_active_2, sizeof(__pyx_k_scheduler_anticache_active_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_anticache_asn, __pyx_k_scheduler_anticache_asn, sizeof(__pyx_k_scheduler_anticache_asn), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_anticache_asn_2, __pyx_k_scheduler_anticache_asn_2, sizeof(__pyx_k_scheduler_anticache_asn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_anticache_asn_last, __pyx_k_scheduler_anticache_asn_last, sizeof(__pyx_k_scheduler_anticache_asn_last), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_anticache_asn_last_2, __pyx_k_scheduler_anticache_asn_last_2, sizeof(__pyx_k_scheduler_anticache_asn_last_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_checkpoint, __pyx_k_scheduler_checkpoint, sizeof(__pyx_k_scheduler_checkpoint), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_checkpoint_2, __pyx_k_scheduler_checkpoint_2, sizeof(__pyx_k_scheduler_checkpoint_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_checkpoint_active, __pyx_k_scheduler_checkpoint_active, sizeof(__pyx_k_scheduler_checkpoint_active), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_checkpoint_active_2, __pyx_k_scheduler_checkpoint_active_2, sizeof(__pyx_k_scheduler_checkpoint_active_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_checkpoint_lsn, __pyx_k_scheduler_checkpoint_lsn, sizeof(__pyx_k_scheduler_checkpoint_lsn), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_checkpoint_lsn_2, __pyx_k_scheduler_checkpoint_lsn_2, sizeof(__pyx_k_scheduler_checkpoint_lsn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_checkpoint_lsn_last, __pyx_k_scheduler_checkpoint_lsn_last, sizeof(__pyx_k_scheduler_checkpoint_lsn_last), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_checkpoint_lsn_last_2, __pyx_k_scheduler_checkpoint_lsn_last_2, sizeof(__pyx_k_scheduler_checkpoint_lsn_last_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_gc, __pyx_k_scheduler_gc, sizeof(__pyx_k_scheduler_gc), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_gc_2, __pyx_k_scheduler_gc_2, sizeof(__pyx_k_scheduler_gc_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_gc_active, __pyx_k_scheduler_gc_active, sizeof(__pyx_k_scheduler_gc_active), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_gc_active_2, __pyx_k_scheduler_gc_active_2, sizeof(__pyx_k_scheduler_gc_active_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_lru, __pyx_k_scheduler_lru, sizeof(__pyx_k_scheduler_lru), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_lru_2, __pyx_k_scheduler_lru_2, sizeof(__pyx_k_scheduler_lru_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_lru_active, __pyx_k_scheduler_lru_active, sizeof(__pyx_k_scheduler_lru_active), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_lru_active_2, __pyx_k_scheduler_lru_active_2, sizeof(__pyx_k_scheduler_lru_active_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_run, __pyx_k_scheduler_run, sizeof(__pyx_k_scheduler_run), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_run_2, __pyx_k_scheduler_run_2, sizeof(__pyx_k_scheduler_run_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_snapshot, __pyx_k_scheduler_snapshot, sizeof(__pyx_k_scheduler_snapshot), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_snapshot_2, __pyx_k_scheduler_snapshot_2, sizeof(__pyx_k_scheduler_snapshot_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_snapshot_active, __pyx_k_scheduler_snapshot_active, sizeof(__pyx_k_scheduler_snapshot_active), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_snapshot_active_2, __pyx_k_scheduler_snapshot_active_2, sizeof(__pyx_k_scheduler_snapshot_active_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_snapshot_ssn, __pyx_k_scheduler_snapshot_ssn, sizeof(__pyx_k_scheduler_snapshot_ssn), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_snapshot_ssn_2, __pyx_k_scheduler_snapshot_ssn_2, sizeof(__pyx_k_scheduler_snapshot_ssn_2), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_snapshot_ssn_last, __pyx_k_scheduler_snapshot_ssn_last, sizeof(__pyx_k_scheduler_snapshot_ssn_last), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_snapshot_ssn_last_2, __pyx_k_scheduler_snapshot_ssn_last_2, sizeof(__pyx_k_scheduler_snapshot_ssn_last_2), 0, 0, 1, 1},
  {&__pyx_n_s_scheduler_threads, __pyx_k_scheduler_threads, sizeof(__pyx_k_scheduler_threads), 0, 0, 1, 1},
  {&__pyx_kp_s_scheduler_threads_2, __pyx_k_scheduler_threads_2, sizeof(__pyx_k_scheduler_threads_2), 0, 0, 1, 0},
  {&__pyx_kp_s_scheduler_zone, __pyx_k_scheduler_zone, sizeof(__pyx_k_scheduler_zone), 0, 0, 1, 0},
  {&__pyx_n_s_scheduler_zone_2, __pyx_k_scheduler_zone_2, sizeof(__pyx_k_scheduler_zone_2), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_set_option, __pyx_k_set_option, sizeof(__pyx_k_set_option), 0, 0, 1, 1},
  {&__pyx_n_s_setter, __pyx_k_setter, sizeof(__pyx_k_setter), 0, 0, 1, 1},
  {&__pyx_n_s_sm, __pyx_k_sm, sizeof(__pyx_k_sm), 0, 0, 1, 1},
  {&__pyx_n_s_sm_locals__method, __pyx_k_sm_locals__method, sizeof(__pyx_k_sm_locals__method), 0, 0, 1, 1},
  {&__pyx_n_s_sophia, __pyx_k_sophia, sizeof(__pyx_k_sophia), 0, 0, 1, 1},
  {&__pyx_kp_s_sophia_build, __pyx_k_sophia_build, sizeof(__pyx_k_sophia_build), 0, 0, 1, 0},
  {&__pyx_kp_s_sophia_error, __pyx_k_sophia_error, sizeof(__pyx_k_sophia_error), 0, 0, 1, 0},
  {&__pyx_kp_s_sophia_path, __pyx_k_sophia_path, sizeof(__pyx_k_sophia_path), 0, 0, 1, 0},
  {&__pyx_n_s_sophia_path_2, __pyx_k_sophia_path_2, sizeof(__pyx_k_sophia_path_2), 0, 0, 1, 1},
  {&__pyx_kp_s_sophia_path_create, __pyx_k_sophia_path_create, sizeof(__pyx_k_sophia_path_create), 0, 0, 1, 0},
  {&__pyx_n_s_sophia_path_create_2, __pyx_k_sophia_path_create_2, sizeof(__pyx_k_sophia_path_create_2), 0, 0, 1, 1},
  {&__pyx_kp_s_sophia_recover, __pyx_k_sophia_recover, sizeof(__pyx_k_sophia_recover), 0, 0, 1, 0},
  {&__pyx_kp_s_sophia_version, __pyx_k_sophia_version, sizeof(__pyx_k_sophia_version), 0, 0, 1, 0},
  {&__pyx_kp_s_sophia_version_storage, __pyx_k_sophia_version_storage, sizeof(__pyx_k_sophia_version_storage), 0, 0, 1, 0},
  {&__pyx_n_s_sophy, __pyx_k_sophy, sizeof(__pyx_k_sophy), 0, 0, 1, 1},
  {&__pyx_n_s_sp, __pyx_k_sp, sizeof(__pyx_k_sp), 0, 0, 1, 1},
  {&__pyx_n_s_sp_locals__getter, __pyx_k_sp_locals__getter, sizeof(__pyx_k_sp_locals__getter), 0, 0, 1, 1},
  {&__pyx_n_s_sp_locals__setter, __pyx_k_sp_locals__setter, sizeof(__pyx_k_sp_locals__setter), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_status, __pyx_k_status, sizeof(__pyx_k_status), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_n_s_storage, __pyx_k_storage, sizeof(__pyx_k_storage), 0, 0, 1, 1},
  {&__pyx_n_s_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 0, 1, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_sync, __pyx_k_sync, sizeof(__pyx_k_sync), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_target, __pyx_k_target, sizeof(__pyx_k_target), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_threads, __pyx_k_threads, sizeof(__pyx_k_threads), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_transaction, __pyx_k_transaction, sizeof(__pyx_k_transaction), 0, 0, 1, 1},
  {&__pyx_kp_s_transaction_is_not_finished_wait, __pyx_k_transaction_is_not_finished_wait, sizeof(__pyx_k_transaction_is_not_finished_wait), 0, 0, 1, 0},
  {&__pyx_kp_s_transaction_was_rolled_back_by_a, __pyx_k_transaction_was_rolled_back_by_a, sizeof(__pyx_k_transaction_was_rolled_back_by_a), 0, 0, 1, 0},
  {&__pyx_n_s_u32, __pyx_k_u32, sizeof(__pyx_k_u32), 0, 0, 1, 1},
  {&__pyx_n_s_u32rev, __pyx_k_u32rev, sizeof(__pyx_k_u32rev), 0, 0, 1, 1},
  {&__pyx_n_s_u64, __pyx_k_u64, sizeof(__pyx_k_u64), 0, 0, 1, 1},
  {&__pyx_n_s_u64rev, __pyx_k_u64rev, sizeof(__pyx_k_u64rev), 0, 0, 1, 1},
  {&__pyx_kp_s_unknown_error_occurred, __pyx_k_unknown_error_occurred, sizeof(__pyx_k_unknown_error_occurred), 0, 0, 1, 0},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_kp_s_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 0, 1, 0},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
  {&__pyx_n_s_version, __pyx_k_version, sizeof(__pyx_k_version), 0, 0, 1, 1},
  {&__pyx_n_s_version_info, __pyx_k_version_info, sizeof(__pyx_k_version_info), 0, 0, 1, 1},
  {&__pyx_n_s_version_storage, __pyx_k_version_storage, sizeof(__pyx_k_version_storage), 0, 0, 1, 1},
  {&__pyx_n_s_view, __pyx_k_view, sizeof(__pyx_k_view), 0, 0, 1, 1},
  {&__pyx_kp_s_view_s, __pyx_k_view_s, sizeof(__pyx_k_view_s), 0, 0, 1, 0},
  {&__pyx_n_s_wrapper, __pyx_k_wrapper, sizeof(__pyx_k_wrapper), 0, 0, 1, 1},
  {&__pyx_n_s_wraps, __pyx_k_wraps, sizeof(__pyx_k_wraps), 0, 0, 1, 1},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_Exception = __Pyx_GetBuiltinName(__pyx_n_s_Exception); if (!__pyx_builtin_Exception) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_UnicodeDecodeError = __Pyx_GetBuiltinName(__pyx_n_s_UnicodeDecodeError); if (!__pyx_builtin_UnicodeDecodeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_property = __Pyx_GetBuiltinName(__pyx_n_s_property); if (!__pyx_builtin_property) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 82; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_StopIteration = __Pyx_GetBuiltinName(__pyx_n_s_StopIteration); if (!__pyx_builtin_StopIteration) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_NotImplemented = __Pyx_GetBuiltinName(__pyx_n_s_NotImplemented); if (!__pyx_builtin_NotImplemented) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 561; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 562; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 966; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1013; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "sophy.pyx":45
 * cdef bytes encode(obj):
 *     if isinstance(obj, unicode):
 *         return obj.encode('utf-8')             # <<<<<<<<<<<<<<
 *     elif isinstance(obj, bytes):
 *         return obj
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_utf_8); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "sophy.pyx":75
 *             raise Exception(error)
 *         else:
 *             raise Exception('unknown error occurred.')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_unknown_error_occurred); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 75; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "sophy.pyx":79
 * 
 * def _sp(propname, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option(propname, string)
 *     if readonly:
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);
  __pyx_codeobj__4 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__3, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_getter, 79, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":84
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option(propname, value)
 *         return property(_getter, _setter)
 */
  __pyx_tuple__5 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_value); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);
  __pyx_codeobj__6 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__5, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_setter, 84, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":89
 * 
 * def _sm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option(propname, 0)
 *     return _method
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_method, 89, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":94
 * 
 * def _dbp(prop, string=False, readonly=False):
 *     def _getter(self):             # <<<<<<<<<<<<<<
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 *     if readonly:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_getter, 94, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":99
 *         return property(_getter)
 *     else:
 *         def _setter(self, value):             # <<<<<<<<<<<<<<
 *             self.config.set_option('db.%s.%s' % (self.name, prop), value)
 *         return property(_getter, _setter)
 */
  __pyx_tuple__11 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_value); if (unlikely(!__pyx_tuple__11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_setter, 99, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":104
 * 
 * def _dbm(propname):
 *     def _method(self):             # <<<<<<<<<<<<<<
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 *     return _method
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 104; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_method, 104, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 104; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":418
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):
 *             with self:             # <<<<<<<<<<<<<<
 *                 return fn(*args, **kwargs)
 *         return wrapper
 */
  __pyx_tuple__15 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_tuple__16 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "sophy.pyx":417
 *     def __call__(self, fn):
 *         @wraps(fn)
 *         def wrapper(*args, **kwargs):             # <<<<<<<<<<<<<<
 *             with self:
 *                 return fn(*args, **kwargs)
 */
  __pyx_tuple__17 = PyTuple_Pack(2, __pyx_n_s_args, __pyx_n_s_kwargs); if (unlikely(!__pyx_tuple__17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(0, 0, 2, 0, CO_VARARGS|CO_VARKEYWORDS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_wrapper, 417, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":437
 * 
 *     cdef _update(self, Database db, dict _data=None, dict k=None):
 *         with db.transaction() as txn:             # <<<<<<<<<<<<<<
 *             if _data:
 *                 for key in _data:
 */
  __pyx_tuple__19 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "sophy.pyx":518
 * 
 *         if not index_type:
 *             self.index_type = ('string',)             # <<<<<<<<<<<<<<
 *         elif isinstance(index_type, basestring):
 *             self.index_type = (index_type,)
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_n_s_string); if (unlikely(!__pyx_tuple__22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 518; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "sophy.pyx":661
 * 
 *     def __setitem__(self, key, value):
 *         raise ValueError('Views are read-only.')             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, key):
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_Views_are_read_only); if (unlikely(!__pyx_tuple__24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "sophy.pyx":664
 * 
 *     def __delitem__(self, key):
 *         raise ValueError('Views are read-only.')             # <<<<<<<<<<<<<<
 * 
 *     cpdef cursor(self, order='>=', key=None, prefix=None, keys=True,
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_Views_are_read_only); if (unlikely(!__pyx_tuple__25)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 664; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "sophy.pyx":710
 *         _check(self.sophia.handle, rc)
 *         if rc == 1:
 *             raise Exception('transaction was rolled back by another '             # <<<<<<<<<<<<<<
 *                             'concurrent transaction.')
 *         elif rc == 2:
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_transaction_was_rolled_back_by_a); if (unlikely(!__pyx_tuple__26)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 710; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "sophy.pyx":713
 *                             'concurrent transaction.')
 *         elif rc == 2:
 *             raise Exception('transaction is not finished, waiting for a '             # <<<<<<<<<<<<<<
 *                             'concurrent transaction to finish.')
 *         self.handle = <void *>0
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_transaction_is_not_finished_wait); if (unlikely(!__pyx_tuple__27)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "sophy.pyx":954
 *                   **_):
 *         if not index_types:
 *             raise ValueError('index_types is a required parameter.')             # <<<<<<<<<<<<<<
 * 
 *         self.initialize_indexes(index_types)
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_index_types_is_a_required_parame); if (unlikely(!__pyx_tuple__29)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "sophy.pyx":1035
 *     def __init__(self, filename, index_type=None, auto_open=True):
 *         path = os.path.dirname(filename)
 *         self.db_name = os.path.basename(filename).split('.')[0] or 'default'             # <<<<<<<<<<<<<<
 *         databases = [(self.db_name, index_type or 'string')]
 *         super(SimpleDatabase, self).__init__(
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_kp_s__30); if (unlikely(!__pyx_tuple__31)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "sophy.pyx":78
 * 
 * 
 * def _sp(propname, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 */
  __pyx_tuple__32 = PyTuple_Pack(7, __pyx_n_s_propname, __pyx_n_s_string, __pyx_n_s_readonly, __pyx_n_s_getter, __pyx_n_s_getter, __pyx_n_s_setter, __pyx_n_s_setter); if (unlikely(!__pyx_tuple__32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(3, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_sp, 78, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":88
 *         return property(_getter, _setter)
 * 
 * def _sm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 */
  __pyx_tuple__34 = PyTuple_Pack(3, __pyx_n_s_propname, __pyx_n_s_method, __pyx_n_s_method); if (unlikely(!__pyx_tuple__34)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_sm, 88, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":93
 *     return _method
 * 
 * def _dbp(prop, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 */
  __pyx_tuple__36 = PyTuple_Pack(7, __pyx_n_s_prop, __pyx_n_s_string, __pyx_n_s_readonly, __pyx_n_s_getter, __pyx_n_s_getter, __pyx_n_s_setter, __pyx_n_s_setter); if (unlikely(!__pyx_tuple__36)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(3, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_dbp, 93, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":103
 *         return property(_getter, _setter)
 * 
 * def _dbm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 */
  __pyx_tuple__38 = PyTuple_Pack(3, __pyx_n_s_propname, __pyx_n_s_method, __pyx_n_s_method); if (unlikely(!__pyx_tuple__38)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_dbm, 103, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":260
 * 
 *     # Sophia Environment.
 *     version = _sp('sophia.version', True, True)             # <<<<<<<<<<<<<<
 *     version_storage = _sp('sophia.version_storage', True, True)
 *     build = _sp('sophia.build', True, True)
 */
  __pyx_tuple__40 = PyTuple_Pack(3, __pyx_kp_s_sophia_version, Py_True, Py_True); if (unlikely(!__pyx_tuple__40)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "sophy.pyx":261
 *     # Sophia Environment.
 *     version = _sp('sophia.version', True, True)
 *     version_storage = _sp('sophia.version_storage', True, True)             # <<<<<<<<<<<<<<
 *     build = _sp('sophia.build', True, True)
 *     error = _sp('sophia.error', True, True)
 */
  __pyx_tuple__41 = PyTuple_Pack(3, __pyx_kp_s_sophia_version_storage, Py_True, Py_True); if (unlikely(!__pyx_tuple__41)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "sophy.pyx":262
 *     version = _sp('sophia.version', True, True)
 *     version_storage = _sp('sophia.version_storage', True, True)
 *     build = _sp('sophia.build', True, True)             # <<<<<<<<<<<<<<
 *     error = _sp('sophia.error', True, True)
 *     sophia_path = _sp('sophia.path', True)
 */
  __pyx_tuple__42 = PyTuple_Pack(3, __pyx_kp_s_sophia_build, Py_True, Py_True); if (unlikely(!__pyx_tuple__42)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 262; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "sophy.pyx":263
 *     version_storage = _sp('sophia.version_storage', True, True)
 *     build = _sp('sophia.build', True, True)
 *     error = _sp('sophia.error', True, True)             # <<<<<<<<<<<<<<
 *     sophia_path = _sp('sophia.path', True)
 *     sophia_path_create = _sp('sophia.path_create')
 */
  __pyx_tuple__43 = PyTuple_Pack(3, __pyx_kp_s_sophia_error, Py_True, Py_True); if (unlikely(!__pyx_tuple__43)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "sophy.pyx":264
 *     build = _sp('sophia.build', True, True)
 *     error = _sp('sophia.error', True, True)
 *     sophia_path = _sp('sophia.path', True)             # <<<<<<<<<<<<<<
 *     sophia_path_create = _sp('sophia.path_create')
 *     recover = _sp('sophia.recover')
 */
  __pyx_tuple__44 = PyTuple_Pack(2, __pyx_kp_s_sophia_path, Py_True); if (unlikely(!__pyx_tuple__44)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "sophy.pyx":265
 *     error = _sp('sophia.error', True, True)
 *     sophia_path = _sp('sophia.path', True)
 *     sophia_path_create = _sp('sophia.path_create')             # <<<<<<<<<<<<<<
 *     recover = _sp('sophia.recover')
 * 
 */
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_kp_s_sophia_path_create); if (unlikely(!__pyx_tuple__45)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "sophy.pyx":266
 *     sophia_path = _sp('sophia.path', True)
 *     sophia_path_create = _sp('sophia.path_create')
 *     recover = _sp('sophia.recover')             # <<<<<<<<<<<<<<
 * 
 *     # Memory Control.
 */
  __pyx_tuple__46 = PyTuple_Pack(1, __pyx_kp_s_sophia_recover); if (unlikely(!__pyx_tuple__46)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "sophy.pyx":269
 * 
 *     # Memory Control.
 *     memory_limit = _sp('memory.limit')             # <<<<<<<<<<<<<<
 *     memory_used = _sp('memory.used', readonly=True)
 *     memory_anticache = _sp('memory.anticache')
 */
  __pyx_tuple__47 = PyTuple_Pack(1, __pyx_kp_s_memory_limit_2); if (unlikely(!__pyx_tuple__47)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "sophy.pyx":270
 *     # Memory Control.
 *     memory_limit = _sp('memory.limit')
 *     memory_used = _sp('memory.used', readonly=True)             # <<<<<<<<<<<<<<
 *     memory_anticache = _sp('memory.anticache')
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 */
  __pyx_tuple__48 = PyTuple_Pack(1, __pyx_kp_s_memory_used); if (unlikely(!__pyx_tuple__48)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "sophy.pyx":271
 *     memory_limit = _sp('memory.limit')
 *     memory_used = _sp('memory.used', readonly=True)
 *     memory_anticache = _sp('memory.anticache')             # <<<<<<<<<<<<<<
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)
 */
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_kp_s_memory_anticache); if (unlikely(!__pyx_tuple__49)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "sophy.pyx":272
 *     memory_used = _sp('memory.used', readonly=True)
 *     memory_anticache = _sp('memory.anticache')
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)             # <<<<<<<<<<<<<<
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)
 *     memory_pager_pools = _sp('memory.pager_pools', readonly=True)
 */
  __pyx_tuple__50 = PyTuple_Pack(1, __pyx_kp_s_memory_pager_pool_size); if (unlikely(!__pyx_tuple__50)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "sophy.pyx":273
 *     memory_anticache = _sp('memory.anticache')
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)             # <<<<<<<<<<<<<<
 *     memory_pager_pools = _sp('memory.pager_pools', readonly=True)
 * 
 */
  __pyx_tuple__51 = PyTuple_Pack(1, __pyx_kp_s_memory_pager_page_size); if (unlikely(!__pyx_tuple__51)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "sophy.pyx":274
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)
 *     memory_pager_pools = _sp('memory.pager_pools', readonly=True)             # <<<<<<<<<<<<<<
 * 
 *     # Scheduler.
 */
  __pyx_tuple__52 = PyTuple_Pack(1, __pyx_kp_s_memory_pager_pools); if (unlikely(!__pyx_tuple__52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "sophy.pyx":277
 * 
 *     # Scheduler.
 *     scheduler_threads = _sp('scheduler.threads')             # <<<<<<<<<<<<<<
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 */
  __pyx_tuple__53 = PyTuple_Pack(1, __pyx_kp_s_scheduler_threads_2); if (unlikely(!__pyx_tuple__53)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);

  /* "sophy.pyx":278
 *     # Scheduler.
 *     scheduler_threads = _sp('scheduler.threads')
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 */
  __pyx_tuple__54 = PyTuple_Pack(1, __pyx_kp_s_scheduler_zone); if (unlikely(!__pyx_tuple__54)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "sophy.pyx":279
 *     scheduler_threads = _sp('scheduler.threads')
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 */
  __pyx_tuple__55 = PyTuple_Pack(1, __pyx_kp_s_scheduler_checkpoint_active); if (unlikely(!__pyx_tuple__55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "sophy.pyx":280
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 */
  __pyx_tuple__56 = PyTuple_Pack(1, __pyx_kp_s_scheduler_checkpoint_lsn); if (unlikely(!__pyx_tuple__56)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "sophy.pyx":281
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 */
  __pyx_tuple__57 = PyTuple_Pack(1, __pyx_kp_s_scheduler_checkpoint_lsn_last); if (unlikely(!__pyx_tuple__57)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);

  /* "sophy.pyx":282
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')             # <<<<<<<<<<<<<<
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 */
  __pyx_tuple__58 = PyTuple_Pack(1, __pyx_kp_s_scheduler_checkpoint); if (unlikely(!__pyx_tuple__58)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);

  /* "sophy.pyx":283
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 */
  __pyx_tuple__59 = PyTuple_Pack(1, __pyx_kp_s_scheduler_anticache_active); if (unlikely(!__pyx_tuple__59)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "sophy.pyx":284
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 *     scheduler_anticache = _sp('scheduler.anticache')
 */
  __pyx_tuple__60 = PyTuple_Pack(1, __pyx_kp_s_scheduler_anticache_asn); if (unlikely(!__pyx_tuple__60)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);

  /* "sophy.pyx":285
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_anticache = _sp('scheduler.anticache')
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 */
  __pyx_tuple__61 = PyTuple_Pack(1, __pyx_kp_s_scheduler_anticache_asn_last); if (unlikely(!__pyx_tuple__61)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "sophy.pyx":286
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 *     scheduler_anticache = _sp('scheduler.anticache')             # <<<<<<<<<<<<<<
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 */
  __pyx_tuple__62 = PyTuple_Pack(1, __pyx_kp_s_scheduler_anticache); if (unlikely(!__pyx_tuple__62)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "sophy.pyx":287
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 *     scheduler_anticache = _sp('scheduler.anticache')
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 */
  __pyx_tuple__63 = PyTuple_Pack(1, __pyx_kp_s_scheduler_snapshot_active); if (unlikely(!__pyx_tuple__63)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);

  /* "sophy.pyx":288
 *     scheduler_anticache = _sp('scheduler.anticache')
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 */
  __pyx_tuple__64 = PyTuple_Pack(1, __pyx_kp_s_scheduler_snapshot_ssn); if (unlikely(!__pyx_tuple__64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "sophy.pyx":289
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 */
  __pyx_tuple__65 = PyTuple_Pack(1, __pyx_kp_s_scheduler_snapshot_ssn_last); if (unlikely(!__pyx_tuple__65)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "sophy.pyx":290
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 *     scheduler_snapshot = _sm('scheduler.snapshot')             # <<<<<<<<<<<<<<
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 *     scheduler_gc = _sm('scheduler.gc')
 */
  __pyx_tuple__66 = PyTuple_Pack(1, __pyx_kp_s_scheduler_snapshot); if (unlikely(!__pyx_tuple__66)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "sophy.pyx":291
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 *     scheduler_gc_active = _sm('scheduler.gc_active')             # <<<<<<<<<<<<<<
 *     scheduler_gc = _sm('scheduler.gc')
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 */
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_kp_s_scheduler_gc_active); if (unlikely(!__pyx_tuple__67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "sophy.pyx":292
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 *     scheduler_gc = _sm('scheduler.gc')             # <<<<<<<<<<<<<<
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 *     scheduler_lru = _sm('scheduler.lru')
 */
  __pyx_tuple__68 = PyTuple_Pack(1, __pyx_kp_s_scheduler_gc); if (unlikely(!__pyx_tuple__68)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);

  /* "sophy.pyx":293
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 *     scheduler_gc = _sm('scheduler.gc')
 *     scheduler_lru_active = _sm('scheduler.lru_active')             # <<<<<<<<<<<<<<
 *     scheduler_lru = _sm('scheduler.lru')
 *     scheduler_run = _sm('scheduler.run')
 */
  __pyx_tuple__69 = PyTuple_Pack(1, __pyx_kp_s_scheduler_lru_active); if (unlikely(!__pyx_tuple__69)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);

  /* "sophy.pyx":294
 *     scheduler_gc = _sm('scheduler.gc')
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 *     scheduler_lru = _sm('scheduler.lru')             # <<<<<<<<<<<<<<
 *     scheduler_run = _sm('scheduler.run')
 * 
 */
  __pyx_tuple__70 = PyTuple_Pack(1, __pyx_kp_s_scheduler_lru); if (unlikely(!__pyx_tuple__70)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);

  /* "sophy.pyx":295
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 *     scheduler_lru = _sm('scheduler.lru')
 *     scheduler_run = _sm('scheduler.run')             # <<<<<<<<<<<<<<
 * 
 *     # Compaction.
 */
  __pyx_tuple__71 = PyTuple_Pack(1, __pyx_kp_s_scheduler_run); if (unlikely(!__pyx_tuple__71)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);

  /* "sophy.pyx":298
 * 
 *     # Compaction.
 *     compaction_redzone = _sp('compaction.redzone')             # <<<<<<<<<<<<<<
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 */
  __pyx_tuple__72 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone); if (unlikely(!__pyx_tuple__72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);

  /* "sophy.pyx":299
 *     # Compaction.
 *     compaction_redzone = _sp('compaction.redzone')
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')             # <<<<<<<<<<<<<<
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 */
  __pyx_tuple__73 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_mode); if (unlikely(!__pyx_tuple__73)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);

  /* "sophy.pyx":300
 *     compaction_redzone = _sp('compaction.redzone')
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 */
  __pyx_tuple__74 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_compact_wm); if (unlikely(!__pyx_tuple__74)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);

  /* "sophy.pyx":301
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 */
  __pyx_tuple__75 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_compact_mode); if (unlikely(!__pyx_tuple__75)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);

  /* "sophy.pyx":302
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 */
  __pyx_tuple__76 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_branch_prio); if (unlikely(!__pyx_tuple__76)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);

  /* "sophy.pyx":303
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 */
  __pyx_tuple__77 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_branch_wm); if (unlikely(!__pyx_tuple__77)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "sophy.pyx":304
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 */
  __pyx_tuple__78 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_branch_age); if (unlikely(!__pyx_tuple__78)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);

  /* "sophy.pyx":305
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 */
  __pyx_tuple__79 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_branch_age_pe); if (unlikely(!__pyx_tuple__79)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);

  /* "sophy.pyx":306
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 */
  __pyx_tuple__80 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_branch_age_wm); if (unlikely(!__pyx_tuple__80)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "sophy.pyx":307
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 */
  __pyx_tuple__81 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_anticache_per); if (unlikely(!__pyx_tuple__81)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 307; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);

  /* "sophy.pyx":308
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 */
  __pyx_tuple__82 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_backup_prio); if (unlikely(!__pyx_tuple__82)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 308; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);

  /* "sophy.pyx":309
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 */
  __pyx_tuple__83 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_gc_wm); if (unlikely(!__pyx_tuple__83)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);

  /* "sophy.pyx":310
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 */
  __pyx_tuple__84 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_gc_db_prio); if (unlikely(!__pyx_tuple__84)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);

  /* "sophy.pyx":311
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')
 */
  __pyx_tuple__85 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_gc_period); if (unlikely(!__pyx_tuple__85)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);

  /* "sophy.pyx":312
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')
 *     compaction_redzone_async = _sp('compaction.redzone.async')
 */
  __pyx_tuple__86 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_lru_prio); if (unlikely(!__pyx_tuple__86)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "sophy.pyx":313
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_async = _sp('compaction.redzone.async')
 * 
 */
  __pyx_tuple__87 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_lru_period); if (unlikely(!__pyx_tuple__87)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "sophy.pyx":314
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')
 *     compaction_redzone_async = _sp('compaction.redzone.async')             # <<<<<<<<<<<<<<
 * 
 *     # Performance.
 */
  __pyx_tuple__88 = PyTuple_Pack(1, __pyx_kp_s_compaction_redzone_async); if (unlikely(!__pyx_tuple__88)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "sophy.pyx":317
 * 
 *     # Performance.
 *     performance_documents = _sp('performance.documents', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)
 *     performance_key = _sp('performance.key', True, readonly=True)
 */
  __pyx_tuple__89 = PyTuple_Pack(1, __pyx_kp_s_performance_documents); if (unlikely(!__pyx_tuple__89)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "sophy.pyx":318
 *     # Performance.
 *     performance_documents = _sp('performance.documents', readonly=True)
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_key = _sp('performance.key', True, readonly=True)
 *     performance_value = _sp('performance.value', True, readonly=True)
 */
  __pyx_tuple__90 = PyTuple_Pack(1, __pyx_kp_s_performance_documents_used); if (unlikely(!__pyx_tuple__90)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);

  /* "sophy.pyx":319
 *     performance_documents = _sp('performance.documents', readonly=True)
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)
 *     performance_key = _sp('performance.key', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_value = _sp('performance.value', True, readonly=True)
 *     performance_set = _sp('performance.set', readonly=True)
 */
  __pyx_tuple__91 = PyTuple_Pack(2, __pyx_kp_s_performance_key, Py_True); if (unlikely(!__pyx_tuple__91)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);

  /* "sophy.pyx":320
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)
 *     performance_key = _sp('performance.key', True, readonly=True)
 *     performance_value = _sp('performance.value', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_set = _sp('performance.set', readonly=True)
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 */
  __pyx_tuple__92 = PyTuple_Pack(2, __pyx_kp_s_performance_value, Py_True); if (unlikely(!__pyx_tuple__92)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);

  /* "sophy.pyx":321
 *     performance_key = _sp('performance.key', True, readonly=True)
 *     performance_value = _sp('performance.value', True, readonly=True)
 *     performance_set = _sp('performance.set', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 *     performance_delete = _sp('performance.delete', readonly=True)
 */
  __pyx_tuple__93 = PyTuple_Pack(1, __pyx_kp_s_performance_set); if (unlikely(!__pyx_tuple__93)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__93);
  __Pyx_GIVEREF(__pyx_tuple__93);

  /* "sophy.pyx":322
 *     performance_value = _sp('performance.value', True, readonly=True)
 *     performance_set = _sp('performance.set', readonly=True)
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_delete = _sp('performance.delete', readonly=True)
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 */
  __pyx_tuple__94 = PyTuple_Pack(2, __pyx_kp_s_performance_set_latency, Py_True); if (unlikely(!__pyx_tuple__94)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);

  /* "sophy.pyx":323
 *     performance_set = _sp('performance.set', readonly=True)
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 *     performance_delete = _sp('performance.delete', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 *     performance_get = _sp('performance.get', readonly=True)
 */
  __pyx_tuple__95 = PyTuple_Pack(1, __pyx_kp_s_performance_delete); if (unlikely(!__pyx_tuple__95)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__95);
  __Pyx_GIVEREF(__pyx_tuple__95);

  /* "sophy.pyx":324
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 *     performance_delete = _sp('performance.delete', readonly=True)
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get = _sp('performance.get', readonly=True)
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 */
  __pyx_tuple__96 = PyTuple_Pack(2, __pyx_kp_s_performance_delete_latency, Py_True); if (unlikely(!__pyx_tuple__96)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);

  /* "sophy.pyx":325
 *     performance_delete = _sp('performance.delete', readonly=True)
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 *     performance_get = _sp('performance.get', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 */
  __pyx_tuple__97 = PyTuple_Pack(1, __pyx_kp_s_performance_get); if (unlikely(!__pyx_tuple__97)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__97);
  __Pyx_GIVEREF(__pyx_tuple__97);

  /* "sophy.pyx":326
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 *     performance_get = _sp('performance.get', readonly=True)
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 */
  __pyx_tuple__98 = PyTuple_Pack(2, __pyx_kp_s_performance_get_latency, Py_True); if (unlikely(!__pyx_tuple__98)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__98);
  __Pyx_GIVEREF(__pyx_tuple__98);

  /* "sophy.pyx":327
 *     performance_get = _sp('performance.get', readonly=True)
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 */
  __pyx_tuple__99 = PyTuple_Pack(2, __pyx_kp_s_performance_get_read_disk, Py_True); if (unlikely(!__pyx_tuple__99)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__99);
  __Pyx_GIVEREF(__pyx_tuple__99);

  /* "sophy.pyx":328
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 */
  __pyx_tuple__100 = PyTuple_Pack(2, __pyx_kp_s_performance_get_read_cache, Py_True); if (unlikely(!__pyx_tuple__100)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__100);
  __Pyx_GIVEREF(__pyx_tuple__100);

  /* "sophy.pyx":329
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 *     performance_tx = _sp('performance.tx', readonly=True)
 */
  __pyx_tuple__101 = PyTuple_Pack(1, __pyx_kp_s_performance_tx_active_rw); if (unlikely(!__pyx_tuple__101)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__101);
  __Pyx_GIVEREF(__pyx_tuple__101);

  /* "sophy.pyx":330
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx = _sp('performance.tx', readonly=True)
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 */
  __pyx_tuple__102 = PyTuple_Pack(1, __pyx_kp_s_performance_tx_active_ro); if (unlikely(!__pyx_tuple__102)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);

  /* "sophy.pyx":331
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 *     performance_tx = _sp('performance.tx', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 */
  __pyx_tuple__103 = PyTuple_Pack(1, __pyx_kp_s_performance_tx); if (unlikely(!__pyx_tuple__103)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);

  /* "sophy.pyx":332
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 *     performance_tx = _sp('performance.tx', readonly=True)
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 */
  __pyx_tuple__104 = PyTuple_Pack(1, __pyx_kp_s_performance_tx_rollback); if (unlikely(!__pyx_tuple__104)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);

  /* "sophy.pyx":333
 *     performance_tx = _sp('performance.tx', readonly=True)
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 */
  __pyx_tuple__105 = PyTuple_Pack(1, __pyx_kp_s_performance_tx_conflict); if (unlikely(!__pyx_tuple__105)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);

  /* "sophy.pyx":334
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 */
  __pyx_tuple__106 = PyTuple_Pack(1, __pyx_kp_s_performance_tx_lock); if (unlikely(!__pyx_tuple__106)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);

  /* "sophy.pyx":335
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 */
  __pyx_tuple__107 = PyTuple_Pack(2, __pyx_kp_s_performance_tx_latency, Py_True); if (unlikely(!__pyx_tuple__107)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);

  /* "sophy.pyx":336
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 */
  __pyx_tuple__108 = PyTuple_Pack(2, __pyx_kp_s_performance_tx_ops, Py_True); if (unlikely(!__pyx_tuple__108)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);

  /* "sophy.pyx":337
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 */
  __pyx_tuple__109 = PyTuple_Pack(1, __pyx_kp_s_performance_tx_gc_queue); if (unlikely(!__pyx_tuple__109)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__109);
  __Pyx_GIVEREF(__pyx_tuple__109);

  /* "sophy.pyx":338
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 *     performance_cursor = _sp('performance.cursor', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 */
  __pyx_tuple__110 = PyTuple_Pack(1, __pyx_kp_s_performance_cursor); if (unlikely(!__pyx_tuple__110)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);

  /* "sophy.pyx":339
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)
 */
  __pyx_tuple__111 = PyTuple_Pack(1, __pyx_kp_s_performance_cursor_latency); if (unlikely(!__pyx_tuple__111)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);

  /* "sophy.pyx":340
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)
 *     performance_cursor_ops = _sp('performance.cursor_ops', readonly=True)
 */
  __pyx_tuple__112 = PyTuple_Pack(1, __pyx_kp_s_performance_cursor_read_disk); if (unlikely(!__pyx_tuple__112)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);

  /* "sophy.pyx":341
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_ops = _sp('performance.cursor_ops', readonly=True)
 * 
 */
  __pyx_tuple__113 = PyTuple_Pack(1, __pyx_kp_s_performance_cursor_read_cache); if (unlikely(!__pyx_tuple__113)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_tuple__113);

  /* "sophy.pyx":342
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)
 *     performance_cursor_ops = _sp('performance.cursor_ops', readonly=True)             # <<<<<<<<<<<<<<
 * 
 *     # Metric.
 */
  __pyx_tuple__114 = PyTuple_Pack(1, __pyx_kp_s_performance_cursor_ops); if (unlikely(!__pyx_tuple__114)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);

  /* "sophy.pyx":345
 * 
 *     # Metric.
 *     metric_lsn = _sp('metric.lsn')             # <<<<<<<<<<<<<<
 *     metric_tsn = _sp('metric.tsn')
 *     metric_nsn = _sp('metric.nsn')
 */
  __pyx_tuple__115 = PyTuple_Pack(1, __pyx_kp_s_metric_lsn); if (unlikely(!__pyx_tuple__115)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__115);
  __Pyx_GIVEREF(__pyx_tuple__115);

  /* "sophy.pyx":346
 *     # Metric.
 *     metric_lsn = _sp('metric.lsn')
 *     metric_tsn = _sp('metric.tsn')             # <<<<<<<<<<<<<<
 *     metric_nsn = _sp('metric.nsn')
 *     metric_ssn = _sp('metric.ssn')
 */
  __pyx_tuple__116 = PyTuple_Pack(1, __pyx_kp_s_metric_tsn); if (unlikely(!__pyx_tuple__116)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);

  /* "sophy.pyx":347
 *     metric_lsn = _sp('metric.lsn')
 *     metric_tsn = _sp('metric.tsn')
 *     metric_nsn = _sp('metric.nsn')             # <<<<<<<<<<<<<<
 *     metric_ssn = _sp('metric.ssn')
 *     metric_asn = _sp('metric.asn')
 */
  __pyx_tuple__117 = PyTuple_Pack(1, __pyx_kp_s_metric_nsn); if (unlikely(!__pyx_tuple__117)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 347; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__117);
  __Pyx_GIVEREF(__pyx_tuple__117);

  /* "sophy.pyx":348
 *     metric_tsn = _sp('metric.tsn')
 *     metric_nsn = _sp('metric.nsn')
 *     metric_ssn = _sp('metric.ssn')             # <<<<<<<<<<<<<<
 *     metric_asn = _sp('metric.asn')
 *     metric_dsn = _sp('metric.dsn')
 */
  __pyx_tuple__118 = PyTuple_Pack(1, __pyx_kp_s_metric_ssn); if (unlikely(!__pyx_tuple__118)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 348; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);

  /* "sophy.pyx":349
 *     metric_nsn = _sp('metric.nsn')
 *     metric_ssn = _sp('metric.ssn')
 *     metric_asn = _sp('metric.asn')             # <<<<<<<<<<<<<<
 *     metric_dsn = _sp('metric.dsn')
 *     metric_bsn = _sp('metric.bsn')
 */
  __pyx_tuple__119 = PyTuple_Pack(1, __pyx_kp_s_metric_asn); if (unlikely(!__pyx_tuple__119)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__119);
  __Pyx_GIVEREF(__pyx_tuple__119);

  /* "sophy.pyx":350
 *     metric_ssn = _sp('metric.ssn')
 *     metric_asn = _sp('metric.asn')
 *     metric_dsn = _sp('metric.dsn')             # <<<<<<<<<<<<<<
 *     metric_bsn = _sp('metric.bsn')
 *     metric_lfsn = _sp('metric.lfsn')
 */
  __pyx_tuple__120 = PyTuple_Pack(1, __pyx_kp_s_metric_dsn); if (unlikely(!__pyx_tuple__120)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);

  /* "sophy.pyx":351
 *     metric_asn = _sp('metric.asn')
 *     metric_dsn = _sp('metric.dsn')
 *     metric_bsn = _sp('metric.bsn')             # <<<<<<<<<<<<<<
 *     metric_lfsn = _sp('metric.lfsn')
 * 
 */
  __pyx_tuple__121 = PyTuple_Pack(1, __pyx_kp_s_metric_bsn); if (unlikely(!__pyx_tuple__121)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__121);
  __Pyx_GIVEREF(__pyx_tuple__121);

  /* "sophy.pyx":352
 *     metric_dsn = _sp('metric.dsn')
 *     metric_bsn = _sp('metric.bsn')
 *     metric_lfsn = _sp('metric.lfsn')             # <<<<<<<<<<<<<<
 * 
 *     # WAL.
 */
  __pyx_tuple__122 = PyTuple_Pack(1, __pyx_kp_s_metric_lfsn); if (unlikely(!__pyx_tuple__122)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);

  /* "sophy.pyx":355
 * 
 *     # WAL.
 *     log_enable = _sp('log.enable')             # <<<<<<<<<<<<<<
 *     log_path = _sp('log.path', True)
 *     log_sync = _sp('log.sync')
 */
  __pyx_tuple__123 = PyTuple_Pack(1, __pyx_kp_s_log_enable); if (unlikely(!__pyx_tuple__123)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__123);
  __Pyx_GIVEREF(__pyx_tuple__123);

  /* "sophy.pyx":356
 *     # WAL.
 *     log_enable = _sp('log.enable')
 *     log_path = _sp('log.path', True)             # <<<<<<<<<<<<<<
 *     log_sync = _sp('log.sync')
 *     log_rotate_wm = _sp('log.rotate_wm')
 */
  __pyx_tuple__124 = PyTuple_Pack(2, __pyx_kp_s_log_path_2, Py_True); if (unlikely(!__pyx_tuple__124)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__124);
  __Pyx_GIVEREF(__pyx_tuple__124);

  /* "sophy.pyx":357
 *     log_enable = _sp('log.enable')
 *     log_path = _sp('log.path', True)
 *     log_sync = _sp('log.sync')             # <<<<<<<<<<<<<<
 *     log_rotate_wm = _sp('log.rotate_wm')
 *     log_rotate_sync = _sp('log.rotate_sync')
 */
  __pyx_tuple__125 = PyTuple_Pack(1, __pyx_kp_s_log_sync); if (unlikely(!__pyx_tuple__125)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__125);
  __Pyx_GIVEREF(__pyx_tuple__125);

  /* "sophy.pyx":358
 *     log_path = _sp('log.path', True)
 *     log_sync = _sp('log.sync')
 *     log_rotate_wm = _sp('log.rotate_wm')             # <<<<<<<<<<<<<<
 *     log_rotate_sync = _sp('log.rotate_sync')
 *     log_rotate = _sm('log.rotate')
 */
  __pyx_tuple__126 = PyTuple_Pack(1, __pyx_kp_s_log_rotate_wm); if (unlikely(!__pyx_tuple__126)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 358; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);

  /* "sophy.pyx":359
 *     log_sync = _sp('log.sync')
 *     log_rotate_wm = _sp('log.rotate_wm')
 *     log_rotate_sync = _sp('log.rotate_sync')             # <<<<<<<<<<<<<<
 *     log_rotate = _sm('log.rotate')
 *     log_gc = _sm('log.gc')
 */
  __pyx_tuple__127 = PyTuple_Pack(1, __pyx_kp_s_log_rotate_sync); if (unlikely(!__pyx_tuple__127)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 359; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_tuple__127);

  /* "sophy.pyx":360
 *     log_rotate_wm = _sp('log.rotate_wm')
 *     log_rotate_sync = _sp('log.rotate_sync')
 *     log_rotate = _sm('log.rotate')             # <<<<<<<<<<<<<<
 *     log_gc = _sm('log.gc')
 *     log_files = _sp('log.files', readonly=True)
 */
  __pyx_tuple__128 = PyTuple_Pack(1, __pyx_kp_s_log_rotate); if (unlikely(!__pyx_tuple__128)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_tuple__128);

  /* "sophy.pyx":361
 *     log_rotate_sync = _sp('log.rotate_sync')
 *     log_rotate = _sm('log.rotate')
 *     log_gc = _sm('log.gc')             # <<<<<<<<<<<<<<
 *     log_files = _sp('log.files', readonly=True)
 * 
 */
  __pyx_tuple__129 = PyTuple_Pack(1, __pyx_kp_s_log_gc); if (unlikely(!__pyx_tuple__129)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);

  /* "sophy.pyx":362
 *     log_rotate = _sm('log.rotate')
 *     log_gc = _sm('log.gc')
 *     log_files = _sp('log.files', readonly=True)             # <<<<<<<<<<<<<<
 * 
 *     # Backup.
 */
  __pyx_tuple__130 = PyTuple_Pack(1, __pyx_kp_s_log_files); if (unlikely(!__pyx_tuple__130)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__130);
  __Pyx_GIVEREF(__pyx_tuple__130);

  /* "sophy.pyx":365
 * 
 *     # Backup.
 *     backup_path = _sp('backup.path', True)             # <<<<<<<<<<<<<<
 *     backup_run = _sm('backup.run')
 *     backup_active = _sp('backup.active')
 */
  __pyx_tuple__131 = PyTuple_Pack(2, __pyx_kp_s_backup_path_2, Py_True); if (unlikely(!__pyx_tuple__131)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 365; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__131);
  __Pyx_GIVEREF(__pyx_tuple__131);

  /* "sophy.pyx":366
 *     # Backup.
 *     backup_path = _sp('backup.path', True)
 *     backup_run = _sm('backup.run')             # <<<<<<<<<<<<<<
 *     backup_active = _sp('backup.active')
 *     backup_last = _sp('backup.last')
 */
  __pyx_tuple__132 = PyTuple_Pack(1, __pyx_kp_s_backup_run); if (unlikely(!__pyx_tuple__132)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__132);
  __Pyx_GIVEREF(__pyx_tuple__132);

  /* "sophy.pyx":367
 *     backup_path = _sp('backup.path', True)
 *     backup_run = _sm('backup.run')
 *     backup_active = _sp('backup.active')             # <<<<<<<<<<<<<<
 *     backup_last = _sp('backup.last')
 *     backup_last_complete = _sp('backup.last_complete')
 */
  __pyx_tuple__133 = PyTuple_Pack(1, __pyx_kp_s_backup_active); if (unlikely(!__pyx_tuple__133)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 367; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);

  /* "sophy.pyx":368
 *     backup_run = _sm('backup.run')
 *     backup_active = _sp('backup.active')
 *     backup_last = _sp('backup.last')             # <<<<<<<<<<<<<<
 *     backup_last_complete = _sp('backup.last_complete')
 * 
 */
  __pyx_tuple__134 = PyTuple_Pack(1, __pyx_kp_s_backup_last); if (unlikely(!__pyx_tuple__134)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__134);
  __Pyx_GIVEREF(__pyx_tuple__134);

  /* "sophy.pyx":369
 *     backup_active = _sp('backup.active')
 *     backup_last = _sp('backup.last')
 *     backup_last_complete = _sp('backup.last_complete')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__135 = PyTuple_Pack(1, __pyx_kp_s_backup_last_complete); if (unlikely(!__pyx_tuple__135)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 369; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);

  /* "sophy.pyx":593
 * 
 *     # key, is string?, is readonly?
 *     database_id = _dbp('id')             # <<<<<<<<<<<<<<
 *     status = _dbp('status', True, True)
 *     storage = _dbp('storage', True)
 */
  __pyx_tuple__136 = PyTuple_Pack(1, __pyx_n_s_id); if (unlikely(!__pyx_tuple__136)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__136);
  __Pyx_GIVEREF(__pyx_tuple__136);

  /* "sophy.pyx":594
 *     # key, is string?, is readonly?
 *     database_id = _dbp('id')
 *     status = _dbp('status', True, True)             # <<<<<<<<<<<<<<
 *     storage = _dbp('storage', True)
 *     format = _dbp('format', True)
 */
  __pyx_tuple__137 = PyTuple_Pack(3, __pyx_n_s_status, Py_True, Py_True); if (unlikely(!__pyx_tuple__137)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__137);
  __Pyx_GIVEREF(__pyx_tuple__137);

  /* "sophy.pyx":595
 *     database_id = _dbp('id')
 *     status = _dbp('status', True, True)
 *     storage = _dbp('storage', True)             # <<<<<<<<<<<<<<
 *     format = _dbp('format', True)
 *     amqf = _dbp('amqf')
 */
  __pyx_tuple__138 = PyTuple_Pack(2, __pyx_n_s_storage, Py_True); if (unlikely(!__pyx_tuple__138)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__138);
  __Pyx_GIVEREF(__pyx_tuple__138);

  /* "sophy.pyx":596
 *     status = _dbp('status', True, True)
 *     storage = _dbp('storage', True)
 *     format = _dbp('format', True)             # <<<<<<<<<<<<<<
 *     amqf = _dbp('amqf')
 *     database_path = _dbp('path', True)
 */
  __pyx_tuple__139 = PyTuple_Pack(2, __pyx_n_s_format, Py_True); if (unlikely(!__pyx_tuple__139)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__139);
  __Pyx_GIVEREF(__pyx_tuple__139);

  /* "sophy.pyx":597
 *     storage = _dbp('storage', True)
 *     format = _dbp('format', True)
 *     amqf = _dbp('amqf')             # <<<<<<<<<<<<<<
 *     database_path = _dbp('path', True)
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 */
  __pyx_tuple__140 = PyTuple_Pack(1, __pyx_n_s_amqf); if (unlikely(!__pyx_tuple__140)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 597; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__140);
  __Pyx_GIVEREF(__pyx_tuple__140);

  /* "sophy.pyx":598
 *     format = _dbp('format', True)
 *     amqf = _dbp('amqf')
 *     database_path = _dbp('path', True)             # <<<<<<<<<<<<<<
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 */
  __pyx_tuple__141 = PyTuple_Pack(2, __pyx_n_s_path, Py_True); if (unlikely(!__pyx_tuple__141)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__141);
  __Pyx_GIVEREF(__pyx_tuple__141);

  /* "sophy.pyx":599
 *     amqf = _dbp('amqf')
 *     database_path = _dbp('path', True)
 *     path_fail_on_exists = _dbp('path_fail_on_exists')             # <<<<<<<<<<<<<<
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 *     cache_mode = _dbp('cache_mode')
 */
  __pyx_tuple__142 = PyTuple_Pack(1, __pyx_n_s_path_fail_on_exists); if (unlikely(!__pyx_tuple__142)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__142);
  __Pyx_GIVEREF(__pyx_tuple__142);

  /* "sophy.pyx":600
 *     database_path = _dbp('path', True)
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 *     path_fail_on_drop = _dbp('path_fail_on_drop')             # <<<<<<<<<<<<<<
 *     cache_mode = _dbp('cache_mode')
 *     cache = _dbp('cache', True)
 */
  __pyx_tuple__143 = PyTuple_Pack(1, __pyx_n_s_path_fail_on_drop); if (unlikely(!__pyx_tuple__143)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__143);
  __Pyx_GIVEREF(__pyx_tuple__143);

  /* "sophy.pyx":601
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 *     cache_mode = _dbp('cache_mode')             # <<<<<<<<<<<<<<
 *     cache = _dbp('cache', True)
 *     mmap = _dbp('mmap')
 */
  __pyx_tuple__144 = PyTuple_Pack(1, __pyx_n_s_cache_mode); if (unlikely(!__pyx_tuple__144)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__144);
  __Pyx_GIVEREF(__pyx_tuple__144);

  /* "sophy.pyx":602
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 *     cache_mode = _dbp('cache_mode')
 *     cache = _dbp('cache', True)             # <<<<<<<<<<<<<<
 *     mmap = _dbp('mmap')
 *     sync = _dbp('sync')
 */
  __pyx_tuple__145 = PyTuple_Pack(2, __pyx_n_s_cache, Py_True); if (unlikely(!__pyx_tuple__145)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__145);
  __Pyx_GIVEREF(__pyx_tuple__145);

  /* "sophy.pyx":603
 *     cache_mode = _dbp('cache_mode')
 *     cache = _dbp('cache', True)
 *     mmap = _dbp('mmap')             # <<<<<<<<<<<<<<
 *     sync = _dbp('sync')
 *     node_preload = _dbp('node_preload')
 */
  __pyx_tuple__146 = PyTuple_Pack(1, __pyx_n_s_mmap); if (unlikely(!__pyx_tuple__146)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 603; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__146);
  __Pyx_GIVEREF(__pyx_tuple__146);

  /* "sophy.pyx":604
 *     cache = _dbp('cache', True)
 *     mmap = _dbp('mmap')
 *     sync = _dbp('sync')             # <<<<<<<<<<<<<<
 *     node_preload = _dbp('node_preload')
 *     node_size = _dbp('node_size')
 */
  __pyx_tuple__147 = PyTuple_Pack(1, __pyx_n_s_sync); if (unlikely(!__pyx_tuple__147)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__147);
  __Pyx_GIVEREF(__pyx_tuple__147);

  /* "sophy.pyx":605
 *     mmap = _dbp('mmap')
 *     sync = _dbp('sync')
 *     node_preload = _dbp('node_preload')             # <<<<<<<<<<<<<<
 *     node_size = _dbp('node_size')
 *     page_size = _dbp('page_size')
 */
  __pyx_tuple__148 = PyTuple_Pack(1, __pyx_n_s_node_preload); if (unlikely(!__pyx_tuple__148)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__148);
  __Pyx_GIVEREF(__pyx_tuple__148);

  /* "sophy.pyx":606
 *     sync = _dbp('sync')
 *     node_preload = _dbp('node_preload')
 *     node_size = _dbp('node_size')             # <<<<<<<<<<<<<<
 *     page_size = _dbp('page_size')
 *     page_checksum = _dbp('page_checksum')
 */
  __pyx_tuple__149 = PyTuple_Pack(1, __pyx_n_s_node_size); if (unlikely(!__pyx_tuple__149)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__149);
  __Pyx_GIVEREF(__pyx_tuple__149);

  /* "sophy.pyx":607
 *     node_preload = _dbp('node_preload')
 *     node_size = _dbp('node_size')
 *     page_size = _dbp('page_size')             # <<<<<<<<<<<<<<
 *     page_checksum = _dbp('page_checksum')
 *     compression_key = _dbp('compression_key')
 */
  __pyx_tuple__150 = PyTuple_Pack(1, __pyx_n_s_page_size); if (unlikely(!__pyx_tuple__150)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__150);
  __Pyx_GIVEREF(__pyx_tuple__150);

  /* "sophy.pyx":608
 *     node_size = _dbp('node_size')
 *     page_size = _dbp('page_size')
 *     page_checksum = _dbp('page_checksum')             # <<<<<<<<<<<<<<
 *     compression_key = _dbp('compression_key')
 *     compression = _dbp('compression', True)
 */
  __pyx_tuple__151 = PyTuple_Pack(1, __pyx_n_s_page_checksum); if (unlikely(!__pyx_tuple__151)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__151);
  __Pyx_GIVEREF(__pyx_tuple__151);

  /* "sophy.pyx":609
 *     page_size = _dbp('page_size')
 *     page_checksum = _dbp('page_checksum')
 *     compression_key = _dbp('compression_key')             # <<<<<<<<<<<<<<
 *     compression = _dbp('compression', True)
 *     compression_branch = _dbp('compression_branch', True)
 */
  __pyx_tuple__152 = PyTuple_Pack(1, __pyx_n_s_compression_key); if (unlikely(!__pyx_tuple__152)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__152);
  __Pyx_GIVEREF(__pyx_tuple__152);

  /* "sophy.pyx":610
 *     page_checksum = _dbp('page_checksum')
 *     compression_key = _dbp('compression_key')
 *     compression = _dbp('compression', True)             # <<<<<<<<<<<<<<
 *     compression_branch = _dbp('compression_branch', True)
 *     lru = _dbp('lru')
 */
  __pyx_tuple__153 = PyTuple_Pack(2, __pyx_n_s_compression, Py_True); if (unlikely(!__pyx_tuple__153)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 610; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__153);
  __Pyx_GIVEREF(__pyx_tuple__153);

  /* "sophy.pyx":611
 *     compression_key = _dbp('compression_key')
 *     compression = _dbp('compression', True)
 *     compression_branch = _dbp('compression_branch', True)             # <<<<<<<<<<<<<<
 *     lru = _dbp('lru')
 *     lru_step = _dbp('lru_step')
 */
  __pyx_tuple__154 = PyTuple_Pack(2, __pyx_n_s_compression_branch, Py_True); if (unlikely(!__pyx_tuple__154)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__154);
  __Pyx_GIVEREF(__pyx_tuple__154);

  /* "sophy.pyx":612
 *     compression = _dbp('compression', True)
 *     compression_branch = _dbp('compression_branch', True)
 *     lru = _dbp('lru')             # <<<<<<<<<<<<<<
 *     lru_step = _dbp('lru_step')
 *     branch = _dbm('branch')
 */
  __pyx_tuple__155 = PyTuple_Pack(1, __pyx_n_s_lru); if (unlikely(!__pyx_tuple__155)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__155);
  __Pyx_GIVEREF(__pyx_tuple__155);

  /* "sophy.pyx":613
 *     compression_branch = _dbp('compression_branch', True)
 *     lru = _dbp('lru')
 *     lru_step = _dbp('lru_step')             # <<<<<<<<<<<<<<
 *     branch = _dbm('branch')
 *     compact = _dbm('compact')
 */
  __pyx_tuple__156 = PyTuple_Pack(1, __pyx_n_s_lru_step); if (unlikely(!__pyx_tuple__156)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 613; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__156);
  __Pyx_GIVEREF(__pyx_tuple__156);

  /* "sophy.pyx":614
 *     lru = _dbp('lru')
 *     lru_step = _dbp('lru_step')
 *     branch = _dbm('branch')             # <<<<<<<<<<<<<<
 *     compact = _dbm('compact')
 *     compact_index = _dbm('compact_index')
 */
  __pyx_tuple__157 = PyTuple_Pack(1, __pyx_n_s_branch); if (unlikely(!__pyx_tuple__157)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__157);
  __Pyx_GIVEREF(__pyx_tuple__157);

  /* "sophy.pyx":615
 *     lru_step = _dbp('lru_step')
 *     branch = _dbm('branch')
 *     compact = _dbm('compact')             # <<<<<<<<<<<<<<
 *     compact_index = _dbm('compact_index')
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 */
  __pyx_tuple__158 = PyTuple_Pack(1, __pyx_n_s_compact); if (unlikely(!__pyx_tuple__158)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__158);
  __Pyx_GIVEREF(__pyx_tuple__158);

  /* "sophy.pyx":616
 *     branch = _dbm('branch')
 *     compact = _dbm('compact')
 *     compact_index = _dbm('compact_index')             # <<<<<<<<<<<<<<
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 *     index_size = _dbp('index.size', readonly=True)
 */
  __pyx_tuple__159 = PyTuple_Pack(1, __pyx_n_s_compact_index); if (unlikely(!__pyx_tuple__159)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__159);
  __Pyx_GIVEREF(__pyx_tuple__159);

  /* "sophy.pyx":617
 *     compact = _dbm('compact')
 *     compact_index = _dbm('compact_index')
 *     index_memory_used = _dbp('index.memory_used', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size = _dbp('index.size', readonly=True)
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 */
  __pyx_tuple__160 = PyTuple_Pack(1, __pyx_kp_s_index_memory_used); if (unlikely(!__pyx_tuple__160)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__160);
  __Pyx_GIVEREF(__pyx_tuple__160);

  /* "sophy.pyx":618
 *     compact_index = _dbm('compact_index')
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 *     index_size = _dbp('index.size', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 */
  __pyx_tuple__161 = PyTuple_Pack(1, __pyx_kp_s_index_size); if (unlikely(!__pyx_tuple__161)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__161);
  __Pyx_GIVEREF(__pyx_tuple__161);

  /* "sophy.pyx":619
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 *     index_size = _dbp('index.size', readonly=True)
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 */
  __pyx_tuple__162 = PyTuple_Pack(1, __pyx_kp_s_index_size_uncompressed); if (unlikely(!__pyx_tuple__162)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 619; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__162);
  __Pyx_GIVEREF(__pyx_tuple__162);

  /* "sophy.pyx":620
 *     index_size = _dbp('index.size', readonly=True)
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 *     index_count = _dbp('index.count', readonly=True)
 */
  __pyx_tuple__163 = PyTuple_Pack(1, __pyx_kp_s_index_size_snapshot); if (unlikely(!__pyx_tuple__163)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__163);
  __Pyx_GIVEREF(__pyx_tuple__163);

  /* "sophy.pyx":621
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)             # <<<<<<<<<<<<<<
 *     index_count = _dbp('index.count', readonly=True)
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 */
  __pyx_tuple__164 = PyTuple_Pack(1, __pyx_kp_s_index_size_amqf); if (unlikely(!__pyx_tuple__164)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__164);
  __Pyx_GIVEREF(__pyx_tuple__164);

  /* "sophy.pyx":622
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 *     index_count = _dbp('index.count', readonly=True)             # <<<<<<<<<<<<<<
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 */
  __pyx_tuple__165 = PyTuple_Pack(1, __pyx_kp_s_index_count); if (unlikely(!__pyx_tuple__165)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__165);
  __Pyx_GIVEREF(__pyx_tuple__165);

  /* "sophy.pyx":623
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 *     index_count = _dbp('index.count', readonly=True)
 *     index_count_dup = _dbp('index.count_dup', readonly=True)             # <<<<<<<<<<<<<<
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 */
  __pyx_tuple__166 = PyTuple_Pack(1, __pyx_kp_s_index_count_dup); if (unlikely(!__pyx_tuple__166)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__166);
  __Pyx_GIVEREF(__pyx_tuple__166);

  /* "sophy.pyx":624
 *     index_count = _dbp('index.count', readonly=True)
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 *     index_read_disk = _dbp('index.read_disk', readonly=True)             # <<<<<<<<<<<<<<
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 *     index_node_count = _dbp('index.node_count', readonly=True)
 */
  __pyx_tuple__167 = PyTuple_Pack(1, __pyx_kp_s_index_read_disk); if (unlikely(!__pyx_tuple__167)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__167);
  __Pyx_GIVEREF(__pyx_tuple__167);

  /* "sophy.pyx":625
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 *     index_read_cache = _dbp('index.read_cache', readonly=True)             # <<<<<<<<<<<<<<
 *     index_node_count = _dbp('index.node_count', readonly=True)
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 */
  __pyx_tuple__168 = PyTuple_Pack(1, __pyx_kp_s_index_read_cache); if (unlikely(!__pyx_tuple__168)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 625; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__168);
  __Pyx_GIVEREF(__pyx_tuple__168);

  /* "sophy.pyx":626
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 *     index_node_count = _dbp('index.node_count', readonly=True)             # <<<<<<<<<<<<<<
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 *     index_branch_max = _dbp('index.branch_max', readonly=True)
 */
  __pyx_tuple__169 = PyTuple_Pack(1, __pyx_kp_s_index_node_count); if (unlikely(!__pyx_tuple__169)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__169);
  __Pyx_GIVEREF(__pyx_tuple__169);

  /* "sophy.pyx":627
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 *     index_node_count = _dbp('index.node_count', readonly=True)
 *     index_branch_count = _dbp('index.branch_count', readonly=True)             # <<<<<<<<<<<<<<
 *     index_branch_max = _dbp('index.branch_max', readonly=True)
 *     index_page_count = _dbp('index.page_count', readonly=True)
 */
  __pyx_tuple__170 = PyTuple_Pack(1, __pyx_kp_s_index_branch_count); if (unlikely(!__pyx_tuple__170)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__170);
  __Pyx_GIVEREF(__pyx_tuple__170);

  /* "sophy.pyx":628
 *     index_node_count = _dbp('index.node_count', readonly=True)
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 *     index_branch_max = _dbp('index.branch_max', readonly=True)             # <<<<<<<<<<<<<<
 *     index_page_count = _dbp('index.page_count', readonly=True)
 * 
 */
  __pyx_tuple__171 = PyTuple_Pack(1, __pyx_kp_s_index_branch_max); if (unlikely(!__pyx_tuple__171)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__171);
  __Pyx_GIVEREF(__pyx_tuple__171);

  /* "sophy.pyx":629
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 *     index_branch_max = _dbp('index.branch_max', readonly=True)
 *     index_page_count = _dbp('index.page_count', readonly=True)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__172 = PyTuple_Pack(1, __pyx_kp_s_index_page_count); if (unlikely(!__pyx_tuple__172)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__172);
  __Pyx_GIVEREF(__pyx_tuple__172);

  /* "sophy.pyx":1082
 * 
 * 
 * def connect(data_dir, db_name, index_type=None):             # <<<<<<<<<<<<<<
 *     cdef Sophia sophia
 *     sophia = Sophia(data_dir, [(db_name, index_type)])
 */
  __pyx_tuple__173 = PyTuple_Pack(4, __pyx_n_s_data_dir, __pyx_n_s_db_name, __pyx_n_s_index_type, __pyx_n_s_sophia); if (unlikely(!__pyx_tuple__173)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__173);
  __Pyx_GIVEREF(__pyx_tuple__173);
  __pyx_codeobj__174 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__173, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_charles_Dropbox_code_sophy, __pyx_n_s_connect, 1082, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__174)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initsophy(void); /*proto*/
PyMODINIT_FUNC initsophy(void)
#else
PyMODINIT_FUNC PyInit_sophy(void); /*proto*/
PyMODINIT_FUNC PyInit_sophy(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_sophy(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("sophy", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_sophy) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "sophy")) {
      if (unlikely(PyDict_SetItemString(modules, "sophy", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  __pyx_v_5sophy_INDEX_TYPE_MAP = ((PyObject*)Py_None); Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_5sophy__ConfigManager = &__pyx_vtable_5sophy__ConfigManager;
  __pyx_vtable_5sophy__ConfigManager.set_option = (PyObject *(*)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5sophy_14_ConfigManager_set_option;
  __pyx_vtable_5sophy__ConfigManager.get_option = (PyObject *(*)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_14_ConfigManager_get_option *__pyx_optional_args))__pyx_f_5sophy_14_ConfigManager_get_option;
  __pyx_vtable_5sophy__ConfigManager.clear_option = (PyObject *(*)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5sophy_14_ConfigManager_clear_option;
  __pyx_vtable_5sophy__ConfigManager.apply_config = (PyObject *(*)(struct __pyx_obj_5sophy__ConfigManager *, PyObject *, PyObject *))__pyx_f_5sophy_14_ConfigManager_apply_config;
  __pyx_vtable_5sophy__ConfigManager.apply_all = (PyObject *(*)(struct __pyx_obj_5sophy__ConfigManager *))__pyx_f_5sophy_14_ConfigManager_apply_all;
  if (PyType_Ready(&__pyx_type_5sophy__ConfigManager) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__ConfigManager.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__ConfigManager.tp_dict, __pyx_vtabptr_5sophy__ConfigManager) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_ConfigManager", (PyObject *)&__pyx_type_5sophy__ConfigManager) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__ConfigManager = &__pyx_type_5sophy__ConfigManager;
  __pyx_vtabptr_5sophy__SophiaObject = &__pyx_vtable_5sophy__SophiaObject;
  __pyx_vtable_5sophy__SophiaObject.destroy = (PyObject *(*)(struct __pyx_obj_5sophy__SophiaObject *))__pyx_f_5sophy_13_SophiaObject_destroy;
  if (PyType_Ready(&__pyx_type_5sophy__SophiaObject) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__SophiaObject.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__SophiaObject.tp_dict, __pyx_vtabptr_5sophy__SophiaObject) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_SophiaObject", (PyObject *)&__pyx_type_5sophy__SophiaObject) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__SophiaObject = &__pyx_type_5sophy__SophiaObject;
  __pyx_vtabptr_5sophy_Sophia = &__pyx_vtable_5sophy_Sophia;
  __pyx_vtable_5sophy_Sophia.__pyx_base = *__pyx_vtabptr_5sophy__SophiaObject;
  __pyx_vtable_5sophy_Sophia.open = (int (*)(struct __pyx_obj_5sophy_Sophia *, int __pyx_skip_dispatch))__pyx_f_5sophy_6Sophia_open;
  __pyx_vtable_5sophy_Sophia._add_db = (PyObject *(*)(struct __pyx_obj_5sophy_Sophia *, struct __pyx_obj_5sophy_Database *, struct __pyx_opt_args_5sophy_6Sophia__add_db *__pyx_optional_args))__pyx_f_5sophy_6Sophia__add_db;
  __pyx_vtable_5sophy_Sophia.close = (int (*)(struct __pyx_obj_5sophy_Sophia *, int __pyx_skip_dispatch))__pyx_f_5sophy_6Sophia_close;
  __pyx_type_5sophy_Sophia.tp_base = __pyx_ptype_5sophy__SophiaObject;
  if (PyType_Ready(&__pyx_type_5sophy_Sophia) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy_Sophia.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy_Sophia.tp_dict, __pyx_vtabptr_5sophy_Sophia) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "Sophia", (PyObject *)&__pyx_type_5sophy_Sophia) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy_Sophia = &__pyx_type_5sophy_Sophia;
  __pyx_vtabptr_5sophy__BaseDBObject = &__pyx_vtable_5sophy__BaseDBObject;
  __pyx_vtable_5sophy__BaseDBObject.__pyx_base = *__pyx_vtabptr_5sophy__SophiaObject;
  __pyx_vtable_5sophy__BaseDBObject.configure = (PyObject *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_13_BaseDBObject_configure;
  __pyx_vtable_5sophy__BaseDBObject.open = (int (*)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_open *__pyx_optional_args))__pyx_f_5sophy_13_BaseDBObject_open;
  __pyx_vtable_5sophy__BaseDBObject.close = (int (*)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch))__pyx_f_5sophy_13_BaseDBObject_close;
  __pyx_vtable_5sophy__BaseDBObject._create_handle = (void *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_13_BaseDBObject__create_handle;
  __pyx_vtable_5sophy__BaseDBObject._get_index = (struct __pyx_obj_5sophy__Index *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_13_BaseDBObject__get_index;
  __pyx_vtable_5sophy__BaseDBObject._update = (PyObject *(*)(struct __pyx_obj_5sophy__BaseDBObject *, struct __pyx_obj_5sophy_Database *, struct __pyx_opt_args_5sophy_13_BaseDBObject__update *__pyx_optional_args))__pyx_f_5sophy_13_BaseDBObject__update;
  __pyx_vtable_5sophy__BaseDBObject.cursor = (PyObject *(*)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor *__pyx_optional_args))__pyx_f_5sophy_13_BaseDBObject_cursor;
  __pyx_type_5sophy__BaseDBObject.tp_base = __pyx_ptype_5sophy__SophiaObject;
  if (PyType_Ready(&__pyx_type_5sophy__BaseDBObject) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__BaseDBObject.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__BaseDBObject.tp_dict, __pyx_vtabptr_5sophy__BaseDBObject) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_BaseDBObject", (PyObject *)&__pyx_type_5sophy__BaseDBObject) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__BaseDBObject = &__pyx_type_5sophy__BaseDBObject;
  __pyx_vtabptr_5sophy_Database = &__pyx_vtable_5sophy_Database;
  __pyx_vtable_5sophy_Database.__pyx_base = *__pyx_vtabptr_5sophy__BaseDBObject;
  __pyx_vtable_5sophy_Database.__pyx_base.__pyx_base.destroy = (PyObject *(*)(struct __pyx_obj_5sophy__SophiaObject *))__pyx_f_5sophy_8Database_destroy;
  __pyx_vtable_5sophy_Database.__pyx_base.configure = (PyObject *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_8Database_configure;
  __pyx_vtable_5sophy_Database.__pyx_base.close = (int (*)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch))__pyx_f_5sophy_8Database_close;
  __pyx_vtable_5sophy_Database.__pyx_base._create_handle = (void *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_8Database__create_handle;
  __pyx_vtable_5sophy_Database.__pyx_base._get_index = (struct __pyx_obj_5sophy__Index *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_8Database__get_index;
  __pyx_vtable_5sophy_Database.__pyx_base.cursor = (PyObject *(*)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor *__pyx_optional_args))__pyx_f_5sophy_8Database_cursor;
  __pyx_vtable_5sophy_Database.transaction = (PyObject *(*)(struct __pyx_obj_5sophy_Database *, int __pyx_skip_dispatch))__pyx_f_5sophy_8Database_transaction;
  __pyx_vtable_5sophy_Database.view = (PyObject *(*)(struct __pyx_obj_5sophy_Database *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5sophy_8Database_view;
  __pyx_type_5sophy_Database.tp_base = __pyx_ptype_5sophy__BaseDBObject;
  if (PyType_Ready(&__pyx_type_5sophy_Database) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 508; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy_Database.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy_Database.tp_dict, __pyx_vtabptr_5sophy_Database) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 508; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "Database", (PyObject *)&__pyx_type_5sophy_Database) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 508; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy_Database = &__pyx_type_5sophy_Database;
  __pyx_vtabptr_5sophy_View = &__pyx_vtable_5sophy_View;
  __pyx_vtable_5sophy_View.__pyx_base = *__pyx_vtabptr_5sophy__BaseDBObject;
  __pyx_vtable_5sophy_View.__pyx_base.__pyx_base.destroy = (PyObject *(*)(struct __pyx_obj_5sophy__SophiaObject *))__pyx_f_5sophy_4View_destroy;
  __pyx_vtable_5sophy_View.__pyx_base._create_handle = (void *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_4View__create_handle;
  __pyx_vtable_5sophy_View.__pyx_base._get_index = (struct __pyx_obj_5sophy__Index *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_4View__get_index;
  __pyx_vtable_5sophy_View.__pyx_base.cursor = (PyObject *(*)(struct __pyx_obj_5sophy__BaseDBObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_13_BaseDBObject_cursor *__pyx_optional_args))__pyx_f_5sophy_4View_cursor;
  __pyx_type_5sophy_View.tp_base = __pyx_ptype_5sophy__BaseDBObject;
  if (PyType_Ready(&__pyx_type_5sophy_View) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy_View.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy_View.tp_dict, __pyx_vtabptr_5sophy_View) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "View", (PyObject *)&__pyx_type_5sophy_View) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy_View = &__pyx_type_5sophy_View;
  __pyx_vtabptr_5sophy_Transaction = &__pyx_vtable_5sophy_Transaction;
  __pyx_vtable_5sophy_Transaction.__pyx_base = *__pyx_vtabptr_5sophy__BaseDBObject;
  __pyx_vtable_5sophy_Transaction.__pyx_base._create_handle = (void *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_11Transaction__create_handle;
  __pyx_vtable_5sophy_Transaction.__pyx_base._get_index = (struct __pyx_obj_5sophy__Index *(*)(struct __pyx_obj_5sophy__BaseDBObject *))__pyx_f_5sophy_11Transaction__get_index;
  __pyx_vtable_5sophy_Transaction.commit = (PyObject *(*)(struct __pyx_obj_5sophy_Transaction *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_commit *__pyx_optional_args))__pyx_f_5sophy_11Transaction_commit;
  __pyx_vtable_5sophy_Transaction.rollback = (PyObject *(*)(struct __pyx_obj_5sophy_Transaction *, int __pyx_skip_dispatch, struct __pyx_opt_args_5sophy_11Transaction_rollback *__pyx_optional_args))__pyx_f_5sophy_11Transaction_rollback;
  __pyx_type_5sophy_Transaction.tp_base = __pyx_ptype_5sophy__BaseDBObject;
  if (PyType_Ready(&__pyx_type_5sophy_Transaction) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 679; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy_Transaction.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy_Transaction.tp_dict, __pyx_vtabptr_5sophy_Transaction) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 679; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "Transaction", (PyObject *)&__pyx_type_5sophy_Transaction) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 679; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy_Transaction = &__pyx_type_5sophy_Transaction;
  __pyx_vtabptr_5sophy_Cursor = &__pyx_vtable_5sophy_Cursor;
  __pyx_vtable_5sophy_Cursor.__pyx_base = *__pyx_vtabptr_5sophy__SophiaObject;
  __pyx_type_5sophy_Cursor.tp_base = __pyx_ptype_5sophy__SophiaObject;
  if (PyType_Ready(&__pyx_type_5sophy_Cursor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 725; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy_Cursor.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy_Cursor.tp_dict, __pyx_vtabptr_5sophy_Cursor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 725; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "Cursor", (PyObject *)&__pyx_type_5sophy_Cursor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 725; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy_Cursor = &__pyx_type_5sophy_Cursor;
  __pyx_vtabptr_5sophy__Index = &__pyx_vtable_5sophy__Index;
  __pyx_vtable_5sophy__Index._get_empty_value = (PyObject *(*)(struct __pyx_obj_5sophy__Index *))__pyx_f_5sophy_6_Index__get_empty_value;
  __pyx_vtable_5sophy__Index.configure = (PyObject *(*)(struct __pyx_obj_5sophy__Index *))__pyx_f_5sophy_6_Index_configure;
  __pyx_vtable_5sophy__Index.set = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *, PyObject *))__pyx_f_5sophy_6_Index_set;
  __pyx_vtable_5sophy__Index.set_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *, PyObject *))__pyx_f_5sophy_6_Index_set_key;
  __pyx_vtable_5sophy__Index.extract_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *))__pyx_f_5sophy_6_Index_extract_key;
  __pyx_vtable_5sophy__Index.get = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *))__pyx_f_5sophy_6_Index_get;
  __pyx_vtable_5sophy__Index.delete = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *))__pyx_f_5sophy_6_Index_delete;
  __pyx_vtable_5sophy__Index.exists = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, struct __pyx_obj_5sophy__BaseDBObject *, PyObject *))__pyx_f_5sophy_6_Index_exists;
  if (PyType_Ready(&__pyx_type_5sophy__Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__Index.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__Index.tp_dict, __pyx_vtabptr_5sophy__Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_Index", (PyObject *)&__pyx_type_5sophy__Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__Index = &__pyx_type_5sophy__Index;
  __pyx_vtabptr_5sophy__UInt32Index = &__pyx_vtable_5sophy__UInt32Index;
  __pyx_vtable_5sophy__UInt32Index.__pyx_base = *__pyx_vtabptr_5sophy__Index;
  __pyx_vtable_5sophy__UInt32Index.__pyx_base._get_empty_value = (PyObject *(*)(struct __pyx_obj_5sophy__Index *))__pyx_f_5sophy_12_UInt32Index__get_empty_value;
  __pyx_vtable_5sophy__UInt32Index.__pyx_base.set_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *, PyObject *))__pyx_f_5sophy_12_UInt32Index_set_key;
  __pyx_vtable_5sophy__UInt32Index.__pyx_base.extract_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *))__pyx_f_5sophy_12_UInt32Index_extract_key;
  __pyx_type_5sophy__UInt32Index.tp_base = __pyx_ptype_5sophy__Index;
  if (PyType_Ready(&__pyx_type_5sophy__UInt32Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__UInt32Index.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__UInt32Index.tp_dict, __pyx_vtabptr_5sophy__UInt32Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_UInt32Index", (PyObject *)&__pyx_type_5sophy__UInt32Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__UInt32Index = &__pyx_type_5sophy__UInt32Index;
  __pyx_vtabptr_5sophy__UInt32RevIndex = &__pyx_vtable_5sophy__UInt32RevIndex;
  __pyx_vtable_5sophy__UInt32RevIndex.__pyx_base = *__pyx_vtabptr_5sophy__UInt32Index;
  __pyx_type_5sophy__UInt32RevIndex.tp_base = __pyx_ptype_5sophy__UInt32Index;
  if (PyType_Ready(&__pyx_type_5sophy__UInt32RevIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__UInt32RevIndex.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__UInt32RevIndex.tp_dict, __pyx_vtabptr_5sophy__UInt32RevIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_UInt32RevIndex", (PyObject *)&__pyx_type_5sophy__UInt32RevIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__UInt32RevIndex = &__pyx_type_5sophy__UInt32RevIndex;
  __pyx_vtabptr_5sophy__UInt64Index = &__pyx_vtable_5sophy__UInt64Index;
  __pyx_vtable_5sophy__UInt64Index.__pyx_base = *__pyx_vtabptr_5sophy__Index;
  __pyx_vtable_5sophy__UInt64Index.__pyx_base._get_empty_value = (PyObject *(*)(struct __pyx_obj_5sophy__Index *))__pyx_f_5sophy_12_UInt64Index__get_empty_value;
  __pyx_vtable_5sophy__UInt64Index.__pyx_base.set_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *, PyObject *))__pyx_f_5sophy_12_UInt64Index_set_key;
  __pyx_vtable_5sophy__UInt64Index.__pyx_base.extract_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *))__pyx_f_5sophy_12_UInt64Index_extract_key;
  __pyx_type_5sophy__UInt64Index.tp_base = __pyx_ptype_5sophy__Index;
  if (PyType_Ready(&__pyx_type_5sophy__UInt64Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 919; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__UInt64Index.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__UInt64Index.tp_dict, __pyx_vtabptr_5sophy__UInt64Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 919; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_UInt64Index", (PyObject *)&__pyx_type_5sophy__UInt64Index) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 919; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__UInt64Index = &__pyx_type_5sophy__UInt64Index;
  __pyx_vtabptr_5sophy__UInt64RevIndex = &__pyx_vtable_5sophy__UInt64RevIndex;
  __pyx_vtable_5sophy__UInt64RevIndex.__pyx_base = *__pyx_vtabptr_5sophy__UInt64Index;
  __pyx_type_5sophy__UInt64RevIndex.tp_base = __pyx_ptype_5sophy__UInt64Index;
  if (PyType_Ready(&__pyx_type_5sophy__UInt64RevIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 943; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__UInt64RevIndex.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__UInt64RevIndex.tp_dict, __pyx_vtabptr_5sophy__UInt64RevIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 943; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_UInt64RevIndex", (PyObject *)&__pyx_type_5sophy__UInt64RevIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 943; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__UInt64RevIndex = &__pyx_type_5sophy__UInt64RevIndex;
  __pyx_vtabptr_5sophy__MultiIndex = &__pyx_vtable_5sophy__MultiIndex;
  __pyx_vtable_5sophy__MultiIndex.__pyx_base = *__pyx_vtabptr_5sophy__Index;
  __pyx_vtable_5sophy__MultiIndex.__pyx_base._get_empty_value = (PyObject *(*)(struct __pyx_obj_5sophy__Index *))__pyx_f_5sophy_11_MultiIndex__get_empty_value;
  __pyx_vtable_5sophy__MultiIndex.__pyx_base.configure = (PyObject *(*)(struct __pyx_obj_5sophy__Index *))__pyx_f_5sophy_11_MultiIndex_configure;
  __pyx_vtable_5sophy__MultiIndex.__pyx_base.set_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *, PyObject *))__pyx_f_5sophy_11_MultiIndex_set_key;
  __pyx_vtable_5sophy__MultiIndex.__pyx_base.extract_key = (PyObject *(*)(struct __pyx_obj_5sophy__Index *, void *))__pyx_f_5sophy_11_MultiIndex_extract_key;
  __pyx_vtable_5sophy__MultiIndex.initialize_indexes = (PyObject *(*)(struct __pyx_obj_5sophy__MultiIndex *, PyObject *))__pyx_f_5sophy_11_MultiIndex_initialize_indexes;
  __pyx_type_5sophy__MultiIndex.tp_base = __pyx_ptype_5sophy__Index;
  if (PyType_Ready(&__pyx_type_5sophy__MultiIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy__MultiIndex.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy__MultiIndex.tp_dict, __pyx_vtabptr_5sophy__MultiIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "_MultiIndex", (PyObject *)&__pyx_type_5sophy__MultiIndex) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy__MultiIndex = &__pyx_type_5sophy__MultiIndex;
  __pyx_vtabptr_5sophy_SimpleDatabase = &__pyx_vtable_5sophy_SimpleDatabase;
  __pyx_vtable_5sophy_SimpleDatabase.__pyx_base = *__pyx_vtabptr_5sophy_Sophia;
  __pyx_vtable_5sophy_SimpleDatabase.transaction = (PyObject *(*)(struct __pyx_obj_5sophy_SimpleDatabase *, int __pyx_skip_dispatch))__pyx_f_5sophy_14SimpleDatabase_transaction;
  __pyx_vtable_5sophy_SimpleDatabase.view = (PyObject *(*)(struct __pyx_obj_5sophy_SimpleDatabase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5sophy_14SimpleDatabase_view;
  __pyx_type_5sophy_SimpleDatabase.tp_base = __pyx_ptype_5sophy_Sophia;
  if (PyType_Ready(&__pyx_type_5sophy_SimpleDatabase) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1029; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy_SimpleDatabase.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5sophy_SimpleDatabase.tp_dict, __pyx_vtabptr_5sophy_SimpleDatabase) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1029; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "SimpleDatabase", (PyObject *)&__pyx_type_5sophy_SimpleDatabase) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1029; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5sophy_SimpleDatabase = &__pyx_type_5sophy_SimpleDatabase;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct___sp) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct___sp.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct___sp = &__pyx_type_5sophy___pyx_scope_struct___sp;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_1__sm) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_1__sm.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_1__sm = &__pyx_type_5sophy___pyx_scope_struct_1__sm;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_2__dbp) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_2__dbp.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_2__dbp = &__pyx_type_5sophy___pyx_scope_struct_2__dbp;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_3__dbm) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_3__dbm.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_3__dbm = &__pyx_type_5sophy___pyx_scope_struct_3__dbm;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_4___call__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_4___call__.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_4___call__ = &__pyx_type_5sophy___pyx_scope_struct_4___call__;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_5_get_range) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_5_get_range.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_5_get_range = &__pyx_type_5sophy___pyx_scope_struct_5_get_range;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_6_keys) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 472; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_6_keys.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_6_keys = &__pyx_type_5sophy___pyx_scope_struct_6_keys;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_7_values) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_7_values.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_7_values = &__pyx_type_5sophy___pyx_scope_struct_7_values;
  if (PyType_Ready(&__pyx_type_5sophy___pyx_scope_struct_8_items) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5sophy___pyx_scope_struct_8_items.tp_print = 0;
  __pyx_ptype_5sophy___pyx_scope_struct_8_items = &__pyx_type_5sophy___pyx_scope_struct_8_items;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "sophy.pyx":6
 * from libc.stdint cimport uint32_t
 * from libc.stdint cimport uint64_t
 * import os             # <<<<<<<<<<<<<<
 * import sys
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_os, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":7
 * from libc.stdint cimport uint64_t
 * import os
 * import sys             # <<<<<<<<<<<<<<
 * 
 * from functools import wraps
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 7; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":9
 * import sys
 * 
 * from functools import wraps             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_wraps);
  __Pyx_GIVEREF(__pyx_n_s_wraps);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_wraps);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_functools, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_wraps); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_wraps, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":41
 *     cdef int sp_commit(void *)
 * 
 * cdef bint IS_PY3K = sys.version_info[0] == 3             # <<<<<<<<<<<<<<
 * 
 * cdef bytes encode(obj):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_version_info); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_2, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_5sophy_IS_PY3K = __pyx_t_3;

  /* "sophy.pyx":78
 * 
 * 
 * def _sp(propname, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option(propname, string)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5sophy_1_sp, NULL, __pyx_n_s_sophy); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sp, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":88
 *         return property(_getter, _setter)
 * 
 * def _sm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option(propname, 0)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5sophy_3_sm, NULL, __pyx_n_s_sophy); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sm, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":93
 *     return _method
 * 
 * def _dbp(prop, string=False, readonly=False):             # <<<<<<<<<<<<<<
 *     def _getter(self):
 *         return self.config.get_option('db.%s.%s' % (self.name, prop), string)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5sophy_5_dbp, NULL, __pyx_n_s_sophy); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_dbp, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":103
 *         return property(_getter, _setter)
 * 
 * def _dbm(propname):             # <<<<<<<<<<<<<<
 *     def _method(self):
 *         self.config.set_option('db.%s.%s' % (self.name, propname), 0)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5sophy_7_dbm, NULL, __pyx_n_s_sophy); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_dbm, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":260
 * 
 *     # Sophia Environment.
 *     version = _sp('sophia.version', True, True)             # <<<<<<<<<<<<<<
 *     version_storage = _sp('sophia.version_storage', True, True)
 *     build = _sp('sophia.build', True, True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_version, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":261
 *     # Sophia Environment.
 *     version = _sp('sophia.version', True, True)
 *     version_storage = _sp('sophia.version_storage', True, True)             # <<<<<<<<<<<<<<
 *     build = _sp('sophia.build', True, True)
 *     error = _sp('sophia.error', True, True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_version_storage, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":262
 *     version = _sp('sophia.version', True, True)
 *     version_storage = _sp('sophia.version_storage', True, True)
 *     build = _sp('sophia.build', True, True)             # <<<<<<<<<<<<<<
 *     error = _sp('sophia.error', True, True)
 *     sophia_path = _sp('sophia.path', True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 262; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__42, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 262; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_build, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 262; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":263
 *     version_storage = _sp('sophia.version_storage', True, True)
 *     build = _sp('sophia.build', True, True)
 *     error = _sp('sophia.error', True, True)             # <<<<<<<<<<<<<<
 *     sophia_path = _sp('sophia.path', True)
 *     sophia_path_create = _sp('sophia.path_create')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__43, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_error, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":264
 *     build = _sp('sophia.build', True, True)
 *     error = _sp('sophia.error', True, True)
 *     sophia_path = _sp('sophia.path', True)             # <<<<<<<<<<<<<<
 *     sophia_path_create = _sp('sophia.path_create')
 *     recover = _sp('sophia.recover')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__44, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_sophia_path_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":265
 *     error = _sp('sophia.error', True, True)
 *     sophia_path = _sp('sophia.path', True)
 *     sophia_path_create = _sp('sophia.path_create')             # <<<<<<<<<<<<<<
 *     recover = _sp('sophia.recover')
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__45, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_sophia_path_create_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":266
 *     sophia_path = _sp('sophia.path', True)
 *     sophia_path_create = _sp('sophia.path_create')
 *     recover = _sp('sophia.recover')             # <<<<<<<<<<<<<<
 * 
 *     # Memory Control.
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__46, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_recover, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":269
 * 
 *     # Memory Control.
 *     memory_limit = _sp('memory.limit')             # <<<<<<<<<<<<<<
 *     memory_used = _sp('memory.used', readonly=True)
 *     memory_anticache = _sp('memory.anticache')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__47, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_memory_limit, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":270
 *     # Memory Control.
 *     memory_limit = _sp('memory.limit')
 *     memory_used = _sp('memory.used', readonly=True)             # <<<<<<<<<<<<<<
 *     memory_anticache = _sp('memory.anticache')
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__48, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_memory_used_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":271
 *     memory_limit = _sp('memory.limit')
 *     memory_used = _sp('memory.used', readonly=True)
 *     memory_anticache = _sp('memory.anticache')             # <<<<<<<<<<<<<<
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_memory_anticache_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":272
 *     memory_used = _sp('memory.used', readonly=True)
 *     memory_anticache = _sp('memory.anticache')
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)             # <<<<<<<<<<<<<<
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)
 *     memory_pager_pools = _sp('memory.pager_pools', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__50, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_memory_pager_pool_size_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":273
 *     memory_anticache = _sp('memory.anticache')
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)             # <<<<<<<<<<<<<<
 *     memory_pager_pools = _sp('memory.pager_pools', readonly=True)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__51, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_memory_pager_page_size_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":274
 *     memory_pager_pool_size = _sp('memory.pager_pool_size', readonly=True)
 *     memory_pager_page_size = _sp('memory.pager_page_size', readonly=True)
 *     memory_pager_pools = _sp('memory.pager_pools', readonly=True)             # <<<<<<<<<<<<<<
 * 
 *     # Scheduler.
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__52, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_memory_pager_pools_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":277
 * 
 *     # Scheduler.
 *     scheduler_threads = _sp('scheduler.threads')             # <<<<<<<<<<<<<<
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__53, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_threads, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":278
 *     # Scheduler.
 *     scheduler_threads = _sp('scheduler.threads')
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__54, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_zone_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":279
 *     scheduler_threads = _sp('scheduler.threads')
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__55, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_checkpoint_active_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":280
 *     scheduler_zone = _sp('scheduler.zone', readonly=True)
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__56, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_checkpoint_lsn_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":281
 *     scheduler_checkpoint_active = _sp('scheduler.checkpoint_active', readonly=True)
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__57, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_checkpoint_lsn_last_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":282
 *     scheduler_checkpoint_lsn = _sp('scheduler.checkpoint_lsn', readonly=True)
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')             # <<<<<<<<<<<<<<
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__58, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_checkpoint_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":283
 *     scheduler_checkpoint_lsn_last = _sp('scheduler.checkpoint_lsn_last', readonly=True)
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__59, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_anticache_active_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":284
 *     scheduler_checkpoint = _sm('scheduler.checkpoint')
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 *     scheduler_anticache = _sp('scheduler.anticache')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__60, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_anticache_asn_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":285
 *     scheduler_anticache_active = _sp('scheduler.anticache_active', readonly=True)
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_anticache = _sp('scheduler.anticache')
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__61, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_anticache_asn_last_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":286
 *     scheduler_anticache_asn = _sp('scheduler.anticache_asn', readonly=True)
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 *     scheduler_anticache = _sp('scheduler.anticache')             # <<<<<<<<<<<<<<
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__62, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_anticache_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":287
 *     scheduler_anticache_asn_last = _sp('scheduler.anticache_asn_last', readonly=True)
 *     scheduler_anticache = _sp('scheduler.anticache')
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__63, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_snapshot_active_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":288
 *     scheduler_anticache = _sp('scheduler.anticache')
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__64, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_snapshot_ssn_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":289
 *     scheduler_snapshot_active = _sp('scheduler.snapshot_active', readonly=True)
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)             # <<<<<<<<<<<<<<
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__65, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_snapshot_ssn_last_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":290
 *     scheduler_snapshot_ssn = _sp('scheduler.snapshot_ssn', readonly=True)
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 *     scheduler_snapshot = _sm('scheduler.snapshot')             # <<<<<<<<<<<<<<
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 *     scheduler_gc = _sm('scheduler.gc')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__66, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_snapshot_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":291
 *     scheduler_snapshot_ssn_last = _sp('scheduler.snapshot_ssn_last', readonly=True)
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 *     scheduler_gc_active = _sm('scheduler.gc_active')             # <<<<<<<<<<<<<<
 *     scheduler_gc = _sm('scheduler.gc')
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__67, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_gc_active_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":292
 *     scheduler_snapshot = _sm('scheduler.snapshot')
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 *     scheduler_gc = _sm('scheduler.gc')             # <<<<<<<<<<<<<<
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 *     scheduler_lru = _sm('scheduler.lru')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__68, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_gc_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":293
 *     scheduler_gc_active = _sm('scheduler.gc_active')
 *     scheduler_gc = _sm('scheduler.gc')
 *     scheduler_lru_active = _sm('scheduler.lru_active')             # <<<<<<<<<<<<<<
 *     scheduler_lru = _sm('scheduler.lru')
 *     scheduler_run = _sm('scheduler.run')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__69, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_lru_active_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":294
 *     scheduler_gc = _sm('scheduler.gc')
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 *     scheduler_lru = _sm('scheduler.lru')             # <<<<<<<<<<<<<<
 *     scheduler_run = _sm('scheduler.run')
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__70, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_lru_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":295
 *     scheduler_lru_active = _sm('scheduler.lru_active')
 *     scheduler_lru = _sm('scheduler.lru')
 *     scheduler_run = _sm('scheduler.run')             # <<<<<<<<<<<<<<
 * 
 *     # Compaction.
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__71, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_scheduler_run_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":298
 * 
 *     # Compaction.
 *     compaction_redzone = _sp('compaction.redzone')             # <<<<<<<<<<<<<<
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__72, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":299
 *     # Compaction.
 *     compaction_redzone = _sp('compaction.redzone')
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')             # <<<<<<<<<<<<<<
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__73, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_mode_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":300
 *     compaction_redzone = _sp('compaction.redzone')
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__74, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_compact_wm_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":301
 *     compaction_redzone_mode = _sp('compaction.redzone.mode')
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__75, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_compact_mode_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":302
 *     compaction_redzone_compact_wm = _sp('compaction.redzone.compact_wm')
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__76, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_branch_prio_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":303
 *     compaction_redzone_compact_mode = _sp('compaction.redzone.compact_mode')
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__77, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_branch_wm_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":304
 *     compaction_redzone_branch_prio = _sp('compaction.redzone.branch_prio')
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__78, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_branch_age_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":305
 *     compaction_redzone_branch_wm = _sp('compaction.redzone.branch_wm')
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__79, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_branch_age_pe_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":306
 *     compaction_redzone_branch_age = _sp('compaction.redzone.branch_age')
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__80, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_branch_age_wm_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":307
 *     compaction_redzone_branch_age_period = _sp('compaction.redzone.branch_age_period')
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 307; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__81, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 307; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_anticache_per_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 307; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":308
 *     compaction_redzone_branch_age_wm = _sp('compaction.redzone.branch_age_wm')
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 308; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__82, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 308; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_backup_prio_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 308; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":309
 *     compaction_redzone_anticache_period = _sp('compaction.redzone.anticache_period')
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')             # <<<<<<<<<<<<<<
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__83, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_gc_wm_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":310
 *     compaction_redzone_backup_prio = _sp('compaction.redzone.backup_prio')
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__84, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_gc_db_prio_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":311
 *     compaction_redzone_gc_wm = _sp('compaction.redzone.gc_wm')
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__85, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_gc_period_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":312
 *     compaction_redzone_gc_db_prio = _sp('compaction.redzone.gc_db_prio')
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')             # <<<<<<<<<<<<<<
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')
 *     compaction_redzone_async = _sp('compaction.redzone.async')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__86, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_lru_prio_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":313
 *     compaction_redzone_gc_period = _sp('compaction.redzone.gc_period')
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')             # <<<<<<<<<<<<<<
 *     compaction_redzone_async = _sp('compaction.redzone.async')
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__87, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_lru_period_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":314
 *     compaction_redzone_lru_prio = _sp('compaction.redzone.lru_prio')
 *     compaction_redzone_lru_period = _sp('compaction.redzone.lru_period')
 *     compaction_redzone_async = _sp('compaction.redzone.async')             # <<<<<<<<<<<<<<
 * 
 *     # Performance.
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__88, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_compaction_redzone_async_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":317
 * 
 *     # Performance.
 *     performance_documents = _sp('performance.documents', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)
 *     performance_key = _sp('performance.key', True, readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__89, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_documents_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":318
 *     # Performance.
 *     performance_documents = _sp('performance.documents', readonly=True)
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_key = _sp('performance.key', True, readonly=True)
 *     performance_value = _sp('performance.value', True, readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__90, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_documents_used_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":319
 *     performance_documents = _sp('performance.documents', readonly=True)
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)
 *     performance_key = _sp('performance.key', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_value = _sp('performance.value', True, readonly=True)
 *     performance_set = _sp('performance.set', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__91, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_key_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":320
 *     performance_documents_used = _sp('performance.documents_used', readonly=True)
 *     performance_key = _sp('performance.key', True, readonly=True)
 *     performance_value = _sp('performance.value', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_set = _sp('performance.set', readonly=True)
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__92, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_value_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":321
 *     performance_key = _sp('performance.key', True, readonly=True)
 *     performance_value = _sp('performance.value', True, readonly=True)
 *     performance_set = _sp('performance.set', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 *     performance_delete = _sp('performance.delete', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__93, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_set_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":322
 *     performance_value = _sp('performance.value', True, readonly=True)
 *     performance_set = _sp('performance.set', readonly=True)
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_delete = _sp('performance.delete', readonly=True)
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__94, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_set_latency_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":323
 *     performance_set = _sp('performance.set', readonly=True)
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 *     performance_delete = _sp('performance.delete', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 *     performance_get = _sp('performance.get', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__95, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_delete_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":324
 *     performance_set_latency = _sp('performance.set_latency', True, readonly=True)
 *     performance_delete = _sp('performance.delete', readonly=True)
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get = _sp('performance.get', readonly=True)
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__96, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_delete_latency_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":325
 *     performance_delete = _sp('performance.delete', readonly=True)
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 *     performance_get = _sp('performance.get', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__97, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_get_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":326
 *     performance_delete_latency = _sp('performance.delete_latency', True, readonly=True)
 *     performance_get = _sp('performance.get', readonly=True)
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__98, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_get_latency_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":327
 *     performance_get = _sp('performance.get', readonly=True)
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__99, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_get_read_disk_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":328
 *     performance_get_latency = _sp('performance.get_latency', True, readonly=True)
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__100, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_get_read_cache_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":329
 *     performance_get_read_disk = _sp('performance.get_read_disk', True, readonly=True)
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 *     performance_tx = _sp('performance.tx', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__101, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_active_rw_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":330
 *     performance_get_read_cache = _sp('performance.get_read_cache', True, readonly=True)
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx = _sp('performance.tx', readonly=True)
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__102, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_active_ro_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":331
 *     performance_tx_active_rw = _sp('performance.tx_active_rw', readonly=True)
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 *     performance_tx = _sp('performance.tx', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__103, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":332
 *     performance_tx_active_ro = _sp('performance.tx_active_ro', readonly=True)
 *     performance_tx = _sp('performance.tx', readonly=True)
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__104, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_rollback_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":333
 *     performance_tx = _sp('performance.tx', readonly=True)
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__105, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_conflict_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":334
 *     performance_tx_rollback = _sp('performance.tx_rollback', readonly=True)
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__106, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_lock_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":335
 *     performance_tx_conflict = _sp('performance.tx_conflict', readonly=True)
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__107, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_latency_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":336
 *     performance_tx_lock = _sp('performance.tx_lock', readonly=True)
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)             # <<<<<<<<<<<<<<
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__108, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_ops_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":337
 *     performance_tx_latency = _sp('performance.tx_latency', True, readonly=True)
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__109, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_tx_gc_queue_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":338
 *     performance_tx_ops = _sp('performance.tx_ops', True, readonly=True)
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 *     performance_cursor = _sp('performance.cursor', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__110, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_cursor_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":339
 *     performance_tx_gc_queue = _sp('performance.tx_gc_queue', readonly=True)
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__111, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_cursor_latency_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":340
 *     performance_cursor = _sp('performance.cursor', readonly=True)
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)
 *     performance_cursor_ops = _sp('performance.cursor_ops', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__112, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_cursor_read_disk_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":341
 *     performance_cursor_latency = _sp('performance.cursor_latency', readonly=True)
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)             # <<<<<<<<<<<<<<
 *     performance_cursor_ops = _sp('performance.cursor_ops', readonly=True)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__113, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_cursor_read_cache_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":342
 *     performance_cursor_read_disk = _sp('performance.cursor_read_disk', readonly=True)
 *     performance_cursor_read_cache = _sp('performance.cursor_read_cache', readonly=True)
 *     performance_cursor_ops = _sp('performance.cursor_ops', readonly=True)             # <<<<<<<<<<<<<<
 * 
 *     # Metric.
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__114, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_performance_cursor_ops_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":345
 * 
 *     # Metric.
 *     metric_lsn = _sp('metric.lsn')             # <<<<<<<<<<<<<<
 *     metric_tsn = _sp('metric.tsn')
 *     metric_nsn = _sp('metric.nsn')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__115, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_lsn_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":346
 *     # Metric.
 *     metric_lsn = _sp('metric.lsn')
 *     metric_tsn = _sp('metric.tsn')             # <<<<<<<<<<<<<<
 *     metric_nsn = _sp('metric.nsn')
 *     metric_ssn = _sp('metric.ssn')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__116, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_tsn_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":347
 *     metric_lsn = _sp('metric.lsn')
 *     metric_tsn = _sp('metric.tsn')
 *     metric_nsn = _sp('metric.nsn')             # <<<<<<<<<<<<<<
 *     metric_ssn = _sp('metric.ssn')
 *     metric_asn = _sp('metric.asn')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 347; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__117, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 347; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_nsn_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 347; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":348
 *     metric_tsn = _sp('metric.tsn')
 *     metric_nsn = _sp('metric.nsn')
 *     metric_ssn = _sp('metric.ssn')             # <<<<<<<<<<<<<<
 *     metric_asn = _sp('metric.asn')
 *     metric_dsn = _sp('metric.dsn')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 348; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__118, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 348; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_ssn_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 348; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":349
 *     metric_nsn = _sp('metric.nsn')
 *     metric_ssn = _sp('metric.ssn')
 *     metric_asn = _sp('metric.asn')             # <<<<<<<<<<<<<<
 *     metric_dsn = _sp('metric.dsn')
 *     metric_bsn = _sp('metric.bsn')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__119, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_asn_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":350
 *     metric_ssn = _sp('metric.ssn')
 *     metric_asn = _sp('metric.asn')
 *     metric_dsn = _sp('metric.dsn')             # <<<<<<<<<<<<<<
 *     metric_bsn = _sp('metric.bsn')
 *     metric_lfsn = _sp('metric.lfsn')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__120, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_dsn_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":351
 *     metric_asn = _sp('metric.asn')
 *     metric_dsn = _sp('metric.dsn')
 *     metric_bsn = _sp('metric.bsn')             # <<<<<<<<<<<<<<
 *     metric_lfsn = _sp('metric.lfsn')
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__121, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_bsn_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":352
 *     metric_dsn = _sp('metric.dsn')
 *     metric_bsn = _sp('metric.bsn')
 *     metric_lfsn = _sp('metric.lfsn')             # <<<<<<<<<<<<<<
 * 
 *     # WAL.
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__122, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_metric_lfsn_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":355
 * 
 *     # WAL.
 *     log_enable = _sp('log.enable')             # <<<<<<<<<<<<<<
 *     log_path = _sp('log.path', True)
 *     log_sync = _sp('log.sync')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__123, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_enable_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":356
 *     # WAL.
 *     log_enable = _sp('log.enable')
 *     log_path = _sp('log.path', True)             # <<<<<<<<<<<<<<
 *     log_sync = _sp('log.sync')
 *     log_rotate_wm = _sp('log.rotate_wm')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__124, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_path, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":357
 *     log_enable = _sp('log.enable')
 *     log_path = _sp('log.path', True)
 *     log_sync = _sp('log.sync')             # <<<<<<<<<<<<<<
 *     log_rotate_wm = _sp('log.rotate_wm')
 *     log_rotate_sync = _sp('log.rotate_sync')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__125, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_sync_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":358
 *     log_path = _sp('log.path', True)
 *     log_sync = _sp('log.sync')
 *     log_rotate_wm = _sp('log.rotate_wm')             # <<<<<<<<<<<<<<
 *     log_rotate_sync = _sp('log.rotate_sync')
 *     log_rotate = _sm('log.rotate')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 358; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__126, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 358; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_rotate_wm_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 358; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":359
 *     log_sync = _sp('log.sync')
 *     log_rotate_wm = _sp('log.rotate_wm')
 *     log_rotate_sync = _sp('log.rotate_sync')             # <<<<<<<<<<<<<<
 *     log_rotate = _sm('log.rotate')
 *     log_gc = _sm('log.gc')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 359; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__127, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 359; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_rotate_sync_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 359; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":360
 *     log_rotate_wm = _sp('log.rotate_wm')
 *     log_rotate_sync = _sp('log.rotate_sync')
 *     log_rotate = _sm('log.rotate')             # <<<<<<<<<<<<<<
 *     log_gc = _sm('log.gc')
 *     log_files = _sp('log.files', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__128, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_rotate_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":361
 *     log_rotate_sync = _sp('log.rotate_sync')
 *     log_rotate = _sm('log.rotate')
 *     log_gc = _sm('log.gc')             # <<<<<<<<<<<<<<
 *     log_files = _sp('log.files', readonly=True)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__129, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_gc_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":362
 *     log_rotate = _sm('log.rotate')
 *     log_gc = _sm('log.gc')
 *     log_files = _sp('log.files', readonly=True)             # <<<<<<<<<<<<<<
 * 
 *     # Backup.
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__130, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_log_files_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":365
 * 
 *     # Backup.
 *     backup_path = _sp('backup.path', True)             # <<<<<<<<<<<<<<
 *     backup_run = _sm('backup.run')
 *     backup_active = _sp('backup.active')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 365; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__131, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 365; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_backup_path, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 365; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":366
 *     # Backup.
 *     backup_path = _sp('backup.path', True)
 *     backup_run = _sm('backup.run')             # <<<<<<<<<<<<<<
 *     backup_active = _sp('backup.active')
 *     backup_last = _sp('backup.last')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sm); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__132, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_backup_run_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":367
 *     backup_path = _sp('backup.path', True)
 *     backup_run = _sm('backup.run')
 *     backup_active = _sp('backup.active')             # <<<<<<<<<<<<<<
 *     backup_last = _sp('backup.last')
 *     backup_last_complete = _sp('backup.last_complete')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 367; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__133, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 367; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_backup_active_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 367; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":368
 *     backup_run = _sm('backup.run')
 *     backup_active = _sp('backup.active')
 *     backup_last = _sp('backup.last')             # <<<<<<<<<<<<<<
 *     backup_last_complete = _sp('backup.last_complete')
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__134, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_backup_last_2, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":369
 *     backup_active = _sp('backup.active')
 *     backup_last = _sp('backup.last')
 *     backup_last_complete = _sp('backup.last_complete')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_sp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 369; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__135, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 369; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Sophia->tp_dict, __pyx_n_s_backup_last_complete_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 369; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Sophia);

  /* "sophy.pyx":505
 * 
 * 
 * class CannotCloseException(Exception): pass             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_builtin_Exception);
  __Pyx_GIVEREF(__pyx_builtin_Exception);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_builtin_Exception);
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_t_2, __pyx_n_s_CannotCloseException, __pyx_n_s_CannotCloseException, (PyObject *) NULL, __pyx_n_s_sophy, (PyObject *) NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_n_s_CannotCloseException, __pyx_t_2, __pyx_t_1, NULL, 0, 1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CannotCloseException, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sophy.pyx":593
 * 
 *     # key, is string?, is readonly?
 *     database_id = _dbp('id')             # <<<<<<<<<<<<<<
 *     status = _dbp('status', True, True)
 *     storage = _dbp('storage', True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__136, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_database_id, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":594
 *     # key, is string?, is readonly?
 *     database_id = _dbp('id')
 *     status = _dbp('status', True, True)             # <<<<<<<<<<<<<<
 *     storage = _dbp('storage', True)
 *     format = _dbp('format', True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__137, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_status, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":595
 *     database_id = _dbp('id')
 *     status = _dbp('status', True, True)
 *     storage = _dbp('storage', True)             # <<<<<<<<<<<<<<
 *     format = _dbp('format', True)
 *     amqf = _dbp('amqf')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__138, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_storage, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":596
 *     status = _dbp('status', True, True)
 *     storage = _dbp('storage', True)
 *     format = _dbp('format', True)             # <<<<<<<<<<<<<<
 *     amqf = _dbp('amqf')
 *     database_path = _dbp('path', True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__139, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_format, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":597
 *     storage = _dbp('storage', True)
 *     format = _dbp('format', True)
 *     amqf = _dbp('amqf')             # <<<<<<<<<<<<<<
 *     database_path = _dbp('path', True)
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 597; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__140, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 597; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_amqf, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 597; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":598
 *     format = _dbp('format', True)
 *     amqf = _dbp('amqf')
 *     database_path = _dbp('path', True)             # <<<<<<<<<<<<<<
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__141, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_database_path, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":599
 *     amqf = _dbp('amqf')
 *     database_path = _dbp('path', True)
 *     path_fail_on_exists = _dbp('path_fail_on_exists')             # <<<<<<<<<<<<<<
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 *     cache_mode = _dbp('cache_mode')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__142, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_path_fail_on_exists, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":600
 *     database_path = _dbp('path', True)
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 *     path_fail_on_drop = _dbp('path_fail_on_drop')             # <<<<<<<<<<<<<<
 *     cache_mode = _dbp('cache_mode')
 *     cache = _dbp('cache', True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__143, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_path_fail_on_drop, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":601
 *     path_fail_on_exists = _dbp('path_fail_on_exists')
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 *     cache_mode = _dbp('cache_mode')             # <<<<<<<<<<<<<<
 *     cache = _dbp('cache', True)
 *     mmap = _dbp('mmap')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__144, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_cache_mode, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":602
 *     path_fail_on_drop = _dbp('path_fail_on_drop')
 *     cache_mode = _dbp('cache_mode')
 *     cache = _dbp('cache', True)             # <<<<<<<<<<<<<<
 *     mmap = _dbp('mmap')
 *     sync = _dbp('sync')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__145, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_cache, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":603
 *     cache_mode = _dbp('cache_mode')
 *     cache = _dbp('cache', True)
 *     mmap = _dbp('mmap')             # <<<<<<<<<<<<<<
 *     sync = _dbp('sync')
 *     node_preload = _dbp('node_preload')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 603; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__146, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 603; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_mmap, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 603; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":604
 *     cache = _dbp('cache', True)
 *     mmap = _dbp('mmap')
 *     sync = _dbp('sync')             # <<<<<<<<<<<<<<
 *     node_preload = _dbp('node_preload')
 *     node_size = _dbp('node_size')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__147, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_sync, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":605
 *     mmap = _dbp('mmap')
 *     sync = _dbp('sync')
 *     node_preload = _dbp('node_preload')             # <<<<<<<<<<<<<<
 *     node_size = _dbp('node_size')
 *     page_size = _dbp('page_size')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__148, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_node_preload, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":606
 *     sync = _dbp('sync')
 *     node_preload = _dbp('node_preload')
 *     node_size = _dbp('node_size')             # <<<<<<<<<<<<<<
 *     page_size = _dbp('page_size')
 *     page_checksum = _dbp('page_checksum')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__149, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_node_size, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":607
 *     node_preload = _dbp('node_preload')
 *     node_size = _dbp('node_size')
 *     page_size = _dbp('page_size')             # <<<<<<<<<<<<<<
 *     page_checksum = _dbp('page_checksum')
 *     compression_key = _dbp('compression_key')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__150, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_page_size, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":608
 *     node_size = _dbp('node_size')
 *     page_size = _dbp('page_size')
 *     page_checksum = _dbp('page_checksum')             # <<<<<<<<<<<<<<
 *     compression_key = _dbp('compression_key')
 *     compression = _dbp('compression', True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__151, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_page_checksum, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":609
 *     page_size = _dbp('page_size')
 *     page_checksum = _dbp('page_checksum')
 *     compression_key = _dbp('compression_key')             # <<<<<<<<<<<<<<
 *     compression = _dbp('compression', True)
 *     compression_branch = _dbp('compression_branch', True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__152, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_compression_key, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":610
 *     page_checksum = _dbp('page_checksum')
 *     compression_key = _dbp('compression_key')
 *     compression = _dbp('compression', True)             # <<<<<<<<<<<<<<
 *     compression_branch = _dbp('compression_branch', True)
 *     lru = _dbp('lru')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 610; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__153, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 610; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_compression, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 610; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":611
 *     compression_key = _dbp('compression_key')
 *     compression = _dbp('compression', True)
 *     compression_branch = _dbp('compression_branch', True)             # <<<<<<<<<<<<<<
 *     lru = _dbp('lru')
 *     lru_step = _dbp('lru_step')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__154, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_compression_branch, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":612
 *     compression = _dbp('compression', True)
 *     compression_branch = _dbp('compression_branch', True)
 *     lru = _dbp('lru')             # <<<<<<<<<<<<<<
 *     lru_step = _dbp('lru_step')
 *     branch = _dbm('branch')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__155, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_lru, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":613
 *     compression_branch = _dbp('compression_branch', True)
 *     lru = _dbp('lru')
 *     lru_step = _dbp('lru_step')             # <<<<<<<<<<<<<<
 *     branch = _dbm('branch')
 *     compact = _dbm('compact')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 613; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__156, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 613; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_lru_step, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 613; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":614
 *     lru = _dbp('lru')
 *     lru_step = _dbp('lru_step')
 *     branch = _dbm('branch')             # <<<<<<<<<<<<<<
 *     compact = _dbm('compact')
 *     compact_index = _dbm('compact_index')
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbm); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__157, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_branch, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":615
 *     lru_step = _dbp('lru_step')
 *     branch = _dbm('branch')
 *     compact = _dbm('compact')             # <<<<<<<<<<<<<<
 *     compact_index = _dbm('compact_index')
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbm); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__158, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_compact, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":616
 *     branch = _dbm('branch')
 *     compact = _dbm('compact')
 *     compact_index = _dbm('compact_index')             # <<<<<<<<<<<<<<
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 *     index_size = _dbp('index.size', readonly=True)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbm); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__159, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_compact_index, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":617
 *     compact = _dbm('compact')
 *     compact_index = _dbm('compact_index')
 *     index_memory_used = _dbp('index.memory_used', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size = _dbp('index.size', readonly=True)
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__160, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_memory_used_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":618
 *     compact_index = _dbm('compact_index')
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 *     index_size = _dbp('index.size', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__161, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_size_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":619
 *     index_memory_used = _dbp('index.memory_used', readonly=True)
 *     index_size = _dbp('index.size', readonly=True)
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 619; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 619; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 619; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__162, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 619; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_size_uncompressed_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 619; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":620
 *     index_size = _dbp('index.size', readonly=True)
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)             # <<<<<<<<<<<<<<
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 *     index_count = _dbp('index.count', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__163, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_size_snapshot_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":621
 *     index_size_uncompressed = _dbp('index.size_uncompressed', readonly=True)
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)             # <<<<<<<<<<<<<<
 *     index_count = _dbp('index.count', readonly=True)
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__164, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_size_amqf_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":622
 *     index_size_snapshot = _dbp('index.size_snapshot', readonly=True)
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 *     index_count = _dbp('index.count', readonly=True)             # <<<<<<<<<<<<<<
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__165, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_count_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":623
 *     index_size_amqf = _dbp('index.size_amqf', readonly=True)
 *     index_count = _dbp('index.count', readonly=True)
 *     index_count_dup = _dbp('index.count_dup', readonly=True)             # <<<<<<<<<<<<<<
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__166, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_count_dup_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":624
 *     index_count = _dbp('index.count', readonly=True)
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 *     index_read_disk = _dbp('index.read_disk', readonly=True)             # <<<<<<<<<<<<<<
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 *     index_node_count = _dbp('index.node_count', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__167, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_read_disk_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":625
 *     index_count_dup = _dbp('index.count_dup', readonly=True)
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 *     index_read_cache = _dbp('index.read_cache', readonly=True)             # <<<<<<<<<<<<<<
 *     index_node_count = _dbp('index.node_count', readonly=True)
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 625; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 625; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 625; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__168, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 625; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_read_cache_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 625; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":626
 *     index_read_disk = _dbp('index.read_disk', readonly=True)
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 *     index_node_count = _dbp('index.node_count', readonly=True)             # <<<<<<<<<<<<<<
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 *     index_branch_max = _dbp('index.branch_max', readonly=True)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__169, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_node_count_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":627
 *     index_read_cache = _dbp('index.read_cache', readonly=True)
 *     index_node_count = _dbp('index.node_count', readonly=True)
 *     index_branch_count = _dbp('index.branch_count', readonly=True)             # <<<<<<<<<<<<<<
 *     index_branch_max = _dbp('index.branch_max', readonly=True)
 *     index_page_count = _dbp('index.page_count', readonly=True)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__170, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_branch_count_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":628
 *     index_node_count = _dbp('index.node_count', readonly=True)
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 *     index_branch_max = _dbp('index.branch_max', readonly=True)             # <<<<<<<<<<<<<<
 *     index_page_count = _dbp('index.page_count', readonly=True)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__171, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_branch_max_2, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":629
 *     index_branch_count = _dbp('index.branch_count', readonly=True)
 *     index_branch_max = _dbp('index.branch_max', readonly=True)
 *     index_page_count = _dbp('index.page_count', readonly=True)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_dbp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_readonly, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__172, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy_Database->tp_dict, __pyx_n_s_index_page_count_2, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_5sophy_Database);

  /* "sophy.pyx":808
 *         tuple keys
 * 
 *     index_type = 'string'             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self, Sophia sophia, Database db, key='key', **_):
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy__Index->tp_dict, __pyx_n_s_index_type, __pyx_n_s_string) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  PyType_Modified(__pyx_ptype_5sophy__Index);

  /* "sophy.pyx":892
 * 
 * cdef class _UInt32Index(_Index):
 *     index_type = 'u32'             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_empty_value(self):
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy__UInt32Index->tp_dict, __pyx_n_s_index_type, __pyx_n_s_u32) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 892; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  PyType_Modified(__pyx_ptype_5sophy__UInt32Index);

  /* "sophy.pyx":916
 * 
 * cdef class _UInt32RevIndex(_UInt32Index):
 *     index_type = 'u32rev'             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy__UInt32RevIndex->tp_dict, __pyx_n_s_index_type, __pyx_n_s_u32rev) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 916; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  PyType_Modified(__pyx_ptype_5sophy__UInt32RevIndex);

  /* "sophy.pyx":920
 * 
 * cdef class _UInt64Index(_Index):
 *     index_type = 'u64'             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_empty_value(self):
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy__UInt64Index->tp_dict, __pyx_n_s_index_type, __pyx_n_s_u64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 920; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  PyType_Modified(__pyx_ptype_5sophy__UInt64Index);

  /* "sophy.pyx":944
 * 
 * cdef class _UInt64RevIndex(_UInt64Index):
 *     index_type = 'u64rev'             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_5sophy__UInt64RevIndex->tp_dict, __pyx_n_s_index_type, __pyx_n_s_u64rev) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  PyType_Modified(__pyx_ptype_5sophy__UInt64RevIndex);

  /* "sophy.pyx":1082
 * 
 * 
 * def connect(data_dir, db_name, index_type=None):             # <<<<<<<<<<<<<<
 *     cdef Sophia sophia
 *     sophia = Sophia(data_dir, [(db_name, index_type)])
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5sophy_9connect, NULL, __pyx_n_s_sophy); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_connect, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sophy.pyx":1090
 * 
 * cdef dict INDEX_TYPE_MAP = {
 *     'string': _Index,             # <<<<<<<<<<<<<<
 *     'u32': _UInt32Index,
 *     'u32rev': _UInt32RevIndex,
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_string, ((PyObject *)__pyx_ptype_5sophy__Index)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1091
 * cdef dict INDEX_TYPE_MAP = {
 *     'string': _Index,
 *     'u32': _UInt32Index,             # <<<<<<<<<<<<<<
 *     'u32rev': _UInt32RevIndex,
 *     'u64': _UInt64Index,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_u32, ((PyObject *)__pyx_ptype_5sophy__UInt32Index)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1092
 *     'string': _Index,
 *     'u32': _UInt32Index,
 *     'u32rev': _UInt32RevIndex,             # <<<<<<<<<<<<<<
 *     'u64': _UInt64Index,
 *     'u64rev': _UInt64RevIndex,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_u32rev, ((PyObject *)__pyx_ptype_5sophy__UInt32RevIndex)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1093
 *     'u32': _UInt32Index,
 *     'u32rev': _UInt32RevIndex,
 *     'u64': _UInt64Index,             # <<<<<<<<<<<<<<
 *     'u64rev': _UInt64RevIndex,
 * }
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_u64, ((PyObject *)__pyx_ptype_5sophy__UInt64Index)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sophy.pyx":1094
 *     'u32rev': _UInt32RevIndex,
 *     'u64': _UInt64Index,
 *     'u64rev': _UInt64RevIndex,             # <<<<<<<<<<<<<<
 * }
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_u64rev, ((PyObject *)__pyx_ptype_5sophy__UInt64RevIndex)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_XGOTREF(__pyx_v_5sophy_INDEX_TYPE_MAP);
  __Pyx_DECREF_SET(__pyx_v_5sophy_INDEX_TYPE_MAP, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sophy.pyx":1
 * from cpython.bytes cimport PyBytes_AsStringAndSize             # <<<<<<<<<<<<<<
 * from libc.stdlib cimport malloc
 * from libc.stdint cimport int64_t
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init sophy", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init sophy");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_COMPILING_IN_CPYTHON
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CyFunctionObject, func.m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyMem_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
#if CYTHON_COMPILING_IN_PYPY
static PyObject * __Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    PyObject *self = f->m_self;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0 = PySequence_ITEM(arg, 0);
                if (unlikely(!arg0)) return NULL;
                result = (*meth)(self, arg0);
                Py_DECREF(arg0);
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
#else
static PyObject * __Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
	return PyCFunction_Call(func, arg, kw);
}
#endif
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_Call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
#if !CYTHON_COMPILING_IN_PYPY
    __pyx_CyFunctionType_type.tp_call = PyCFunction_Call;
#endif
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (__pyx_CyFunctionType == NULL) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyMem_Malloc(size);
    if (!m->defaults)
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(PyObject_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_O | METH_NOARGS);
    }
#endif
    return 0;
}

static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_items, d);
    else
        return PyDict_Items(d);
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_values, d);
    else
        return PyDict_Values(d);
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#endif
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto bad;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *function = PyMethod_GET_FUNCTION(method);
            result = __Pyx_PyObject_CallOneArg(function, self);
            Py_DECREF(method);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

static CYTHON_INLINE int __Pyx_unpack_tuple2(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                             int is_tuple, int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    if (!is_tuple && unlikely(!PyTuple_Check(tuple))) {
        iternextfunc iternext;
        iter = PyObject_GetIter(tuple);
        if (unlikely(!iter)) goto bad;
        if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
        iternext = Py_TYPE(iter)->tp_iternext;
        value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
        value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
        if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
        Py_DECREF(iter);
    } else {
        if (!has_known_size && unlikely(PyTuple_GET_SIZE(tuple) != 2)) {
            __Pyx_UnpackTupleError(tuple, 2);
            goto bad;
        }
#if CYTHON_COMPILING_IN_PYPY
        value1 = PySequence_ITEM(tuple, 0);
        if (unlikely(!value1)) goto bad;
        value2 = PySequence_ITEM(tuple, 1);
        if (unlikely(!value2)) goto bad;
#else
        value1 = PyTuple_GET_ITEM(tuple, 0);
        value2 = PyTuple_GET_ITEM(tuple, 1);
        Py_INCREF(value1);
        Py_INCREF(value2);
#endif
        if (decref_tuple) { Py_DECREF(tuple); }
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
#if !CYTHON_COMPILING_IN_PYPY
    if (is_dict) {
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
    }
#endif
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return NULL;
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_crop_slice(Py_ssize_t* _start, Py_ssize_t* _stop, Py_ssize_t* _length) {
    Py_ssize_t start = *_start, stop = *_stop, length = *_length;
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    *_length = stop - start;
    *_start = start;
    *_stop = stop;
}
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject** CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject* __Pyx_PyList_GetSlice(
            PyObject* src, Py_ssize_t start, Py_ssize_t stop) {
    PyObject* dest;
    Py_ssize_t length = PyList_GET_SIZE(src);
    __Pyx_crop_slice(&start, &stop, &length);
    if (unlikely(length <= 0))
        return PyList_New(0);
    dest = PyList_New(length);
    if (unlikely(!dest))
        return NULL;
    __Pyx_copy_object_array(
        ((PyListObject*)src)->ob_item + start,
        ((PyListObject*)dest)->ob_item,
        length);
    return dest;
}
static CYTHON_INLINE PyObject* __Pyx_PyTuple_GetSlice(
            PyObject* src, Py_ssize_t start, Py_ssize_t stop) {
    PyObject* dest;
    Py_ssize_t length = PyTuple_GET_SIZE(src);
    __Pyx_crop_slice(&start, &stop, &length);
    if (unlikely(length <= 0))
        return PyTuple_New(0);
    dest = PyTuple_New(length);
    if (unlikely(!dest))
        return NULL;
    __Pyx_copy_object_array(
        ((PyTupleObject*)src)->ob_item + start,
        ((PyTupleObject*)dest)->ob_item,
        length);
    return dest;
}
#endif

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64_t(int64_t value) {
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int64_t),
                                     little, !is_unsigned);
    }
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value) {
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(uint32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint32_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *x) {
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint64_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint64_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 2 * PyLong_SHIFT) {
                            return (uint64_t) (((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 3 * PyLong_SHIFT) {
                            return (uint64_t) (((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 4 * PyLong_SHIFT) {
                            return (uint64_t) (((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint64_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint64_t, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint64_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) ((((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, long, PyLong_AsLong(x))
            } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint64_t val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint64_t) -1;
        }
    } else {
        uint64_t val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (uint64_t) -1;
        val = __Pyx_PyInt_As_uint64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64_t");
    return (uint64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64_t");
    return (uint64_t) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value) {
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
#else
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
#endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}

static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto bad;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            args = PyTuple_New(2);
            if (unlikely(!args)) goto bad;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            Py_DECREF(method); method = NULL;
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
bad:
    Py_XDECREF(method);
    return result;
}

#include <structmember.h>
#include <frameobject.h>
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
#if 1 || PY_VERSION_HEX < 0x030300B0
static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
#if PY_VERSION_HEX >= 0x030300A0
        if (ev && Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
#endif
        if (!ev || !PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            if (!ev) {
                Py_INCREF(Py_None);
                ev = Py_None;
            } else if (PyTuple_Check(ev)) {
                if (PyTuple_GET_SIZE(ev) >= 1) {
                    PyObject *value;
#if CYTHON_COMPILING_IN_CPYTHON
                    value = PySequence_ITEM(ev, 0);
#else
                    value = PyTuple_GET_ITEM(ev, 0);
                    Py_INCREF(value);
#endif
                    Py_DECREF(ev);
                    ev = value;
                } else {
                    Py_INCREF(Py_None);
                    Py_DECREF(ev);
                    ev = Py_None;
                }
            }
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = ev;
            return 0;
        }
    } else if (!PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
#endif
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__pyx_CoroutineObject *self) {
    PyObject *exc_type = self->exc_type;
    PyObject *exc_value = self->exc_value;
    PyObject *exc_traceback = self->exc_traceback;
    self->exc_type = NULL;
    self->exc_value = NULL;
    self->exc_traceback = NULL;
    Py_XDECREF(exc_type);
    Py_XDECREF(exc_value);
    Py_XDECREF(exc_traceback);
}
static CYTHON_INLINE
int __Pyx_Coroutine_CheckRunning(__pyx_CoroutineObject *gen) {
    if (unlikely(gen->is_running)) {
        PyErr_SetString(PyExc_ValueError,
                        "generator already executing");
        return 1;
    }
    return 0;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value) {
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            PyErr_SetString(PyExc_TypeError,
                            "can't send non-None value to a "
                            "just-started generator");
            return NULL;
        }
    }
    if (unlikely(self->resume_label == -1)) {
        PyErr_SetNone(PyExc_StopIteration);
        return NULL;
    }
    if (value) {
#if CYTHON_COMPILING_IN_PYPY
#else
        if (self->exc_traceback) {
            PyThreadState *tstate = PyThreadState_GET();
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
#endif
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
    }
    self->is_running = 1;
    retval = self->body((PyObject *) self, value);
    self->is_running = 0;
    if (retval) {
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
#if CYTHON_COMPILING_IN_PYPY
#else
        if (self->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_CLEAR(f->f_back);
        }
#endif
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(PyObject *retval) {
    if (unlikely(!retval && !PyErr_Occurred())) {
        PyErr_SetNone(PyExc_StopIteration);
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen_FetchStopIterationValue(&val);
    ret = __Pyx_Coroutine_SendEx(gen, val);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = PyIter_Next(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value);
    }
    return __Pyx_Coroutine_MethodReturn(retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL);
    if (retval) {
        Py_DECREF(retval);
        PyErr_SetString(PyExc_RuntimeError,
                        "generator ignored GeneratorExit");
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (!raised_exception
        || raised_exception == PyExc_StopIteration
        || raised_exception == PyExc_GeneratorExit
        || PyErr_GivenExceptionMatches(raised_exception, PyExc_GeneratorExit)
        || PyErr_GivenExceptionMatches(raised_exception, PyExc_StopIteration))
    {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *typ;
    PyObject *tb = NULL;
    PyObject *val = NULL;
    PyObject *yf = gen->yieldfrom;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit)) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(__Pyx_Coroutine_SendEx(gen, NULL));
            goto throw_here;
        }
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Throw(yf, args);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Throw(yf, args);
        } else
        #endif
        {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            ret = PyObject_CallObject(meth, args);
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(__Pyx_Coroutine_SendEx(gen, NULL));
}
static int __Pyx_Coroutine_traverse(PyObject *self, visitproc visit, void *arg) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    Py_VISIT(gen->exc_type);
    Py_VISIT(gen->exc_value);
    Py_VISIT(gen->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    Py_CLEAR(gen->exc_type);
    Py_CLEAR(gen->exc_value);
    Py_CLEAR(gen->exc_traceback);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label > 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (self->ob_refcnt > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *res;
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    if (gen->resume_label <= 0)
        return ;
#if PY_VERSION_HEX < 0x030400a1
    assert(self->ob_refcnt == 0);
    self->ob_refcnt = 1;
#endif
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
    res = __Pyx_Coroutine_Close(self);
    if (res == NULL)
        PyErr_WriteUnraisable(self);
    else
        Py_DECREF(res);
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if PY_VERSION_HEX < 0x030400a1
    assert(self->ob_refcnt > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = self->ob_refcnt;
        _Py_NewReference(self);
        self->ob_refcnt = refcnt;
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(self->ob_type) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self)
{
    Py_INCREF(self->gi_name);
    return self->gi_name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self)
{
    Py_INCREF(self->gi_qualname);
    return self->gi_qualname;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(PyTypeObject* type, __pyx_coroutine_body_t body,
                                                   PyObject *closure, PyObject *name, PyObject *qualname) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (gen == NULL)
        return NULL;
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->exc_type = NULL;
    gen->exc_value = NULL;
    gen->exc_traceback = NULL;
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    PyObject_GC_Track(gen);
    return gen;
}

static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (!abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_VERSION_HEX >= 0x03030000) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_VERSION_HEX >= 0x03030000) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if (0) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if PY_VERSION_HEX >= 0x030400a1
    __Pyx_Coroutine_del,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = PyObject_GenericGetAttr;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
